@@ -3,6 +3,7 @@
  */
 
 #include <getopt.h>
+#include <stdlib.h>
 #include <unistd.h>
 
 #include <rte_cryptodev.h>
@@ -10,6 +11,8 @@
 #include <rte_ether.h>
 
 #include "cperf_options.h"
+#include "cperf_test_common.h"
+#include "cperf_test_vectors.h"
 
 #define AES_BLOCK_SIZE 16
 #define DES_BLOCK_SIZE 8
@@ -24,7 +27,7 @@ usage(char *progname)
 {
 	printf("%s [EAL options] --\n"
 		" --silent: disable options dump\n"
-		" --ptest throughput / latency / verify / pmd-cycleount :"
+		" --ptest throughput / latency / verify / pmd-cyclecount :"
 		" set test type\n"
 		" --pool_sz N: set the number of crypto ops/mbufs allocated\n"
 		" --total-ops N: set the number of total operations performed\n"
@@ -34,9 +37,13 @@ usage(char *progname)
 		" --segment-sz N: set the size of the segment to use\n"
 		" --desc-nb N: set number of descriptors for each crypto device\n"
 		" --devtype TYPE: set crypto device type to use\n"
-		" --optype cipher-only / auth-only / cipher-then-auth /\n"
-		"           auth-then-cipher / aead : set operation type\n"
+		" --low-prio-qp-mask mask: set low priority for queues set in mask(hex)\n"
+		" --optype cipher-only / auth-only / cipher-then-auth / auth-then-cipher /\n"
+		"        aead / pdcp / docsis / ipsec / modex / rsa / secp192r1 /\n"
+		"        secp224r1 / secp256r1 / secp384r1 / secp521r1 / eddsa / sm2 /\n"
+		"        tls-record : set operation type\n"
 		" --sessionless: enable session-less crypto operations\n"
+		" --shared-session: share 1 session across all queue pairs on crypto device\n"
 		" --out-of-place: enable out-of-place crypto operations\n"
 		" --test-file NAME: set the test vector file path\n"
 		" --test-name NAME: set specific test name section in test file\n"
@@ -57,11 +64,19 @@ usage(char *progname)
 		" --pmd-cyclecount-delay-ms N: set delay between enqueue\n"
 		"           and dequeue in pmd-cyclecount benchmarking mode\n"
 		" --csv-friendly: enable test result output CSV friendly\n"
+		" --modex-len N: modex length, supported lengths are "
+		"60, 128, 255, 448. Default: 128\n"
+		" --asym-op encrypt / decrypt / sign / verify : set asym operation type\n"
+		" --rsa-priv-keytype exp / qt : set RSA private key type\n"
+		" --rsa-modlen N: RSA modulus length, supported lengths are "
+		"1024, 2048, 4096, 8192. Default: 1024\n"
 #ifdef RTE_LIB_SECURITY
 		" --pdcp-sn-sz N: set PDCP SN size N <5/7/12/15/18>\n"
 		" --pdcp-domain DOMAIN: set PDCP domain <control/user>\n"
 		" --pdcp-ses-hfn-en: enable session based fixed HFN\n"
+		" --enable-sdap: enable sdap\n"
 		" --docsis-hdr-sz: set DOCSIS header size\n"
+		" --tls-version VER: set TLS VERSION <TLS1.2/TLS1.3/DTLS1.2>\n"
 #endif
 		" -h: prints this help\n",
 		progname);
@@ -313,6 +328,52 @@ parse_pool_sz(struct cperf_options *opts, const char *arg)
 	return ret;
 }
 
+static int
+parse_modex_len(struct cperf_options *opts, const char *arg)
+{
+	int ret =  parse_uint16_t(&opts->modex_len, arg);
+
+	if (ret)
+		RTE_LOG(ERR, USER1, "failed to parse modex len");
+	return ret;
+}
+
+static int
+parse_rsa_priv_keytype(struct cperf_options *opts, const char *arg)
+{
+	struct name_id_map rsa_keytype_namemap[] = {
+		{
+			cperf_rsa_priv_keytype_strs[RTE_RSA_KEY_TYPE_EXP],
+			RTE_RSA_KEY_TYPE_EXP
+		},
+		{
+			cperf_rsa_priv_keytype_strs[RTE_RSA_KEY_TYPE_QT],
+			RTE_RSA_KEY_TYPE_QT
+		},
+	};
+
+	opts->rsa_keytype = get_str_key_id_mapping(rsa_keytype_namemap,
+			RTE_DIM(rsa_keytype_namemap), arg);
+
+	return 0;
+}
+
+static int
+parse_rsa_modlen(struct cperf_options *opts, const char *arg)
+{
+	uint16_t modlen = 0;
+	int ret;
+
+	ret =  parse_uint16_t(&modlen, arg);
+	if (ret) {
+		RTE_LOG(ERR, USER1, "failed to parse RSA modlen");
+		return ret;
+	}
+
+	opts->rsa_modlen = modlen;
+	return ret;
+}
+
 static int
 parse_burst_sz(struct cperf_options *opts, const char *arg)
 {
@@ -457,7 +518,51 @@ parse_op_type(struct cperf_options *opts, const char *arg)
 		{
 			cperf_op_type_strs[CPERF_DOCSIS],
 			CPERF_DOCSIS
-		}
+		},
+		{
+			cperf_op_type_strs[CPERF_IPSEC],
+			CPERF_IPSEC
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_MODEX],
+			CPERF_ASYM_MODEX
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_RSA],
+			CPERF_ASYM_RSA
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SECP192R1],
+			CPERF_ASYM_SECP192R1
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SECP224R1],
+			CPERF_ASYM_SECP224R1
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SECP256R1],
+			CPERF_ASYM_SECP256R1
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SECP384R1],
+			CPERF_ASYM_SECP384R1
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SECP521R1],
+			CPERF_ASYM_SECP521R1
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_ED25519],
+			CPERF_ASYM_ED25519
+		},
+		{
+			cperf_op_type_strs[CPERF_ASYM_SM2],
+			CPERF_ASYM_SM2
+		},
+		{
+			cperf_op_type_strs[CPERF_TLS],
+			CPERF_TLS
+		},
 	};
 
 	int id = get_str_key_id_mapping(optype_namemap,
@@ -480,6 +585,14 @@ parse_sessionless(struct cperf_options *opts,
 	return 0;
 }
 
+static int
+parse_shared_session(struct cperf_options *opts,
+		const char *arg __rte_unused)
+{
+	opts->shared_session = 1;
+	return 0;
+}
+
 static int
 parse_out_of_place(struct cperf_options *opts,
 		const char *arg __rte_unused)
@@ -493,9 +606,14 @@ parse_test_file(struct cperf_options *opts,
 		const char *arg)
 {
 	opts->test_file = strdup(arg);
+	if (opts->test_file == NULL) {
+		RTE_LOG(ERR, USER1, "Dup vector file failed!\n");
+		return -1;
+	}
 	if (access(opts->test_file, F_OK) != -1)
 		return 0;
 	RTE_LOG(ERR, USER1, "Test vector file doesn't exist\n");
+	free(opts->test_file);
 
 	return -1;
 }
@@ -506,6 +624,12 @@ parse_test_name(struct cperf_options *opts,
 {
 	char *test_name = (char *) rte_zmalloc(NULL,
 		sizeof(char) * (strlen(arg) + 3), 0);
+	if (test_name == NULL) {
+		RTE_LOG(ERR, USER1, "Failed to rte zmalloc with size: %zu\n",
+			strlen(arg) + 3);
+		return -1;
+	}
+
 	snprintf(test_name, strlen(arg) + 3, "[%s]", arg);
 	opts->test_name = test_name;
 
@@ -521,6 +645,15 @@ parse_silent(struct cperf_options *opts,
 	return 0;
 }
 
+static int
+parse_enable_sdap(struct cperf_options *opts,
+		const char *arg __rte_unused)
+{
+	opts->pdcp_sdap = 1;
+
+	return 0;
+}
+
 static int
 parse_cipher_algo(struct cperf_options *opts, const char *arg)
 {
@@ -656,7 +789,8 @@ parse_pdcp_sn_sz(struct cperf_options *opts, const char *arg)
 
 const char *cperf_pdcp_domain_strs[] = {
 	[RTE_SECURITY_PDCP_MODE_CONTROL] = "control",
-	[RTE_SECURITY_PDCP_MODE_DATA] = "data"
+	[RTE_SECURITY_PDCP_MODE_DATA] = "data",
+	[RTE_SECURITY_PDCP_MODE_SHORT_MAC] = "short_mac"
 };
 
 static int
@@ -671,6 +805,11 @@ parse_pdcp_domain(struct cperf_options *opts, const char *arg)
 			cperf_pdcp_domain_strs
 			[RTE_SECURITY_PDCP_MODE_DATA],
 			RTE_SECURITY_PDCP_MODE_DATA
+		},
+		{
+			cperf_pdcp_domain_strs
+			[RTE_SECURITY_PDCP_MODE_SHORT_MAC],
+			RTE_SECURITY_PDCP_MODE_SHORT_MAC
 		}
 	};
 
@@ -687,6 +826,45 @@ parse_pdcp_domain(struct cperf_options *opts, const char *arg)
 	return 0;
 }
 
+const char *cperf_tls_version_strs[] = {
+	[RTE_SECURITY_VERSION_TLS_1_2] = "TLS1.2",
+	[RTE_SECURITY_VERSION_TLS_1_3] = "TLS1.3",
+	[RTE_SECURITY_VERSION_DTLS_1_2] = "DTLS1.2"
+};
+
+static int
+parse_tls_version(struct cperf_options *opts, const char *arg)
+{
+	struct name_id_map tls_version_namemap[] = {
+		{
+			cperf_tls_version_strs
+			[RTE_SECURITY_VERSION_TLS_1_2],
+			RTE_SECURITY_VERSION_TLS_1_2
+		},
+		{
+			cperf_tls_version_strs
+			[RTE_SECURITY_VERSION_TLS_1_3],
+			RTE_SECURITY_VERSION_TLS_1_3
+		},
+		{
+			cperf_tls_version_strs
+			[RTE_SECURITY_VERSION_DTLS_1_2],
+			RTE_SECURITY_VERSION_DTLS_1_2
+		},
+	};
+
+	int id = get_str_key_id_mapping(tls_version_namemap,
+			RTE_DIM(tls_version_namemap), arg);
+	if (id < 0) {
+		RTE_LOG(ERR, USER1, "invalid TLS version specified\n");
+		return -1;
+	}
+
+	opts->tls_version = (enum rte_security_tls_version)id;
+
+	return 0;
+}
+
 static int
 parse_pdcp_ses_hfn_en(struct cperf_options *opts, const char *arg __rte_unused)
 {
@@ -768,6 +946,45 @@ parse_aead_aad_sz(struct cperf_options *opts, const char *arg)
 	return parse_uint16_t(&opts->aead_aad_sz, arg);
 }
 
+static int
+parse_asym_op(struct cperf_options *opts, const char *arg)
+{
+	struct name_id_map asym_op_namemap[] = {
+		{
+			rte_crypto_asym_op_strings
+			[RTE_CRYPTO_ASYM_OP_ENCRYPT],
+			RTE_CRYPTO_ASYM_OP_ENCRYPT
+		},
+		{
+			rte_crypto_asym_op_strings
+			[RTE_CRYPTO_ASYM_OP_DECRYPT],
+			RTE_CRYPTO_ASYM_OP_DECRYPT
+		},
+		{
+			rte_crypto_asym_op_strings
+			[RTE_CRYPTO_ASYM_OP_SIGN],
+			RTE_CRYPTO_ASYM_OP_SIGN
+		},
+		{
+			rte_crypto_asym_op_strings
+			[RTE_CRYPTO_ASYM_OP_VERIFY],
+			RTE_CRYPTO_ASYM_OP_VERIFY
+		}
+	};
+
+	int id = get_str_key_id_mapping(asym_op_namemap,
+			RTE_DIM(asym_op_namemap), arg);
+	if (id < 0) {
+		RTE_LOG(ERR, USER1, "invalid ASYM operation specified\n");
+		return -1;
+	}
+
+	opts->asym_op_type = (enum rte_crypto_asym_op_type)id;
+
+	return 0;
+}
+
+
 static int
 parse_csv_friendly(struct cperf_options *opts, const char *arg __rte_unused)
 {
@@ -790,6 +1007,22 @@ parse_pmd_cyclecount_delay_ms(struct cperf_options *opts,
 	return 0;
 }
 
+static int
+parse_low_prio_qp_mask(struct cperf_options *opts, const char *arg)
+{
+	char *end = NULL;
+	unsigned long n;
+
+	/* parse hexadecimal string */
+	n = strtoul(arg, &end, 16);
+	if ((optarg[0] == '\0') || (end == NULL) || (*end != '\0'))
+		return -1;
+
+	opts->low_prio_qp_mask = n;
+
+	return 0;
+}
+
 typedef int (*option_parser_t)(struct cperf_options *opts,
 		const char *arg);
 
@@ -802,6 +1035,9 @@ struct long_opt_parser {
 static struct option lgopts[] = {
 
 	{ CPERF_PTEST_TYPE, required_argument, 0, 0 },
+	{ CPERF_MODEX_LEN, required_argument, 0, 0 },
+	{ CPERF_RSA_PRIV_KEYTYPE, required_argument, 0, 0 },
+	{ CPERF_RSA_MODLEN, required_argument, 0, 0 },
 
 	{ CPERF_POOL_SIZE, required_argument, 0, 0 },
 	{ CPERF_TOTAL_OPS, required_argument, 0, 0 },
@@ -810,12 +1046,15 @@ static struct option lgopts[] = {
 	{ CPERF_SEGMENT_SIZE, required_argument, 0, 0 },
 	{ CPERF_DESC_NB, required_argument, 0, 0 },
 
+	{ CPERF_LOW_PRIO_QP_MASK, required_argument, 0, 0 },
+
 	{ CPERF_IMIX, required_argument, 0, 0 },
 	{ CPERF_DEVTYPE, required_argument, 0, 0 },
 	{ CPERF_OPTYPE, required_argument, 0, 0 },
 
 	{ CPERF_SILENT, no_argument, 0, 0 },
 	{ CPERF_SESSIONLESS, no_argument, 0, 0 },
+	{ CPERF_SHARED_SESSION, no_argument, 0, 0 },
 	{ CPERF_OUT_OF_PLACE, no_argument, 0, 0 },
 	{ CPERF_TEST_FILE, required_argument, 0, 0 },
 	{ CPERF_TEST_NAME, required_argument, 0, 0 },
@@ -841,11 +1080,15 @@ static struct option lgopts[] = {
 
 	{ CPERF_DIGEST_SZ, required_argument, 0, 0 },
 
+	{ CPERF_ASYM_OP, required_argument, 0, 0 },
+
 #ifdef RTE_LIB_SECURITY
 	{ CPERF_PDCP_SN_SZ, required_argument, 0, 0 },
 	{ CPERF_PDCP_DOMAIN, required_argument, 0, 0 },
 	{ CPERF_PDCP_SES_HFN_EN, no_argument, 0, 0 },
+	{ CPERF_ENABLE_SDAP, no_argument, 0, 0 },
 	{ CPERF_DOCSIS_HDR_SZ, required_argument, 0, 0 },
+	{ CPERF_TLS_VERSION, required_argument, 0, 0 },
 #endif
 	{ CPERF_CSV, no_argument, 0, 0},
 
@@ -917,8 +1160,21 @@ cperf_options_default(struct cperf_options *opts)
 	opts->pdcp_sn_sz = 12;
 	opts->pdcp_domain = RTE_SECURITY_PDCP_MODE_CONTROL;
 	opts->pdcp_ses_hfn_en = 0;
+	opts->pdcp_sdap = 0;
 	opts->docsis_hdr_sz = 17;
 #endif
+	opts->modex_data = (struct cperf_modex_test_data *)&modex_perf_data[0];
+	opts->rsa_data = &rsa_pub_perf_data[0];
+	opts->rsa_keytype = UINT8_MAX;
+
+	opts->secp192r1_data = &secp192r1_perf_data;
+	opts->secp224r1_data = &secp224r1_perf_data;
+	opts->secp256r1_data = &secp256r1_perf_data;
+	opts->secp384r1_data = &secp384r1_perf_data;
+	opts->secp521r1_data = &secp521r1_perf_data;
+	opts->eddsa_data = &ed25519_perf_data;
+	opts->sm2_data = &sm2_perf_data;
+	opts->asym_op_type = RTE_CRYPTO_ASYM_OP_ENCRYPT;
 }
 
 static int
@@ -926,6 +1182,9 @@ cperf_opts_parse_long(int opt_idx, struct cperf_options *opts)
 {
 	struct long_opt_parser parsermap[] = {
 		{ CPERF_PTEST_TYPE,	parse_cperf_test_type },
+		{ CPERF_MODEX_LEN,	parse_modex_len },
+		{ CPERF_RSA_PRIV_KEYTYPE,	parse_rsa_priv_keytype },
+		{ CPERF_RSA_MODLEN,	parse_rsa_modlen },
 		{ CPERF_SILENT,		parse_silent },
 		{ CPERF_POOL_SIZE,	parse_pool_sz },
 		{ CPERF_TOTAL_OPS,	parse_total_ops },
@@ -933,9 +1192,11 @@ cperf_opts_parse_long(int opt_idx, struct cperf_options *opts)
 		{ CPERF_BUFFER_SIZE,	parse_buffer_sz },
 		{ CPERF_SEGMENT_SIZE,	parse_segment_sz },
 		{ CPERF_DESC_NB,	parse_desc_nb },
+		{ CPERF_LOW_PRIO_QP_MASK,	parse_low_prio_qp_mask },
 		{ CPERF_DEVTYPE,	parse_device_type },
 		{ CPERF_OPTYPE,		parse_op_type },
 		{ CPERF_SESSIONLESS,	parse_sessionless },
+		{ CPERF_SHARED_SESSION,	parse_shared_session },
 		{ CPERF_OUT_OF_PLACE,	parse_out_of_place },
 		{ CPERF_IMIX,		parse_imix },
 		{ CPERF_TEST_FILE,	parse_test_file },
@@ -954,11 +1215,14 @@ cperf_opts_parse_long(int opt_idx, struct cperf_options *opts)
 		{ CPERF_AEAD_IV_SZ,	parse_aead_iv_sz },
 		{ CPERF_AEAD_AAD_SZ,	parse_aead_aad_sz },
 		{ CPERF_DIGEST_SZ,	parse_digest_sz },
+		{ CPERF_ASYM_OP,	parse_asym_op },
 #ifdef RTE_LIB_SECURITY
 		{ CPERF_PDCP_SN_SZ,	parse_pdcp_sn_sz },
 		{ CPERF_PDCP_DOMAIN,	parse_pdcp_domain },
 		{ CPERF_PDCP_SES_HFN_EN,	parse_pdcp_ses_hfn_en },
+		{ CPERF_ENABLE_SDAP,	parse_enable_sdap },
 		{ CPERF_DOCSIS_HDR_SZ,	parse_docsis_hdr_sz },
+		{ CPERF_TLS_VERSION,	parse_tls_version },
 #endif
 		{ CPERF_CSV,		parse_csv_friendly},
 		{ CPERF_PMDCC_DELAY_MS,	parse_pmd_cyclecount_delay_ms},
@@ -983,7 +1247,7 @@ cperf_options_parse(struct cperf_options *options, int argc, char **argv)
 		switch (opt) {
 		case 'h':
 			usage(argv[0]);
-			rte_exit(EXIT_SUCCESS, "Displayed help\n");
+			exit(EXIT_SUCCESS);
 			break;
 		/* long options */
 		case 0:
@@ -1014,6 +1278,10 @@ check_cipher_buffer_length(struct cperf_options *options)
 		else
 			buffer_size = options->buffer_size_list[0];
 
+		if ((options->auth_op == RTE_CRYPTO_AUTH_OP_GENERATE) &&
+				(options->op_type == CPERF_AUTH_THEN_CIPHER))
+			buffer_size += options->digest_sz;
+
 		while (buffer_size <= options->max_buffer_size) {
 			if ((buffer_size % AES_BLOCK_SIZE) != 0) {
 				RTE_LOG(ERR, USER1, "Some of the buffer sizes are "
@@ -1040,6 +1308,10 @@ check_cipher_buffer_length(struct cperf_options *options)
 		else
 			buffer_size = options->buffer_size_list[0];
 
+		if ((options->auth_op == RTE_CRYPTO_AUTH_OP_GENERATE) &&
+				(options->op_type == CPERF_AUTH_THEN_CIPHER))
+			buffer_size += options->digest_sz;
+
 		while (buffer_size <= options->max_buffer_size) {
 			if ((buffer_size % DES_BLOCK_SIZE) != 0) {
 				RTE_LOG(ERR, USER1, "Some of the buffer sizes are "
@@ -1094,9 +1366,60 @@ check_docsis_buffer_length(struct cperf_options *options)
 }
 #endif
 
+static bool
+is_valid_chained_op(struct cperf_options *options)
+{
+	if (options->cipher_op == RTE_CRYPTO_CIPHER_OP_ENCRYPT &&
+			options->auth_op == RTE_CRYPTO_AUTH_OP_GENERATE)
+		return true;
+
+	if (options->cipher_op == RTE_CRYPTO_CIPHER_OP_DECRYPT &&
+			options->auth_op == RTE_CRYPTO_AUTH_OP_VERIFY)
+		return true;
+
+	return false;
+}
+
+static int
+cperf_rsa_options_check(struct cperf_options *options)
+{
+	if (options->rsa_keytype != UINT8_MAX) {
+		switch (options->rsa_keytype) {
+		case RTE_RSA_KEY_TYPE_QT:
+			if (options->asym_op_type != RTE_CRYPTO_ASYM_OP_SIGN &&
+				options->asym_op_type != RTE_CRYPTO_ASYM_OP_DECRYPT) {
+				RTE_LOG(ERR, USER1, "QT private key to be used in sign and decrypt op\n");
+				return -EINVAL;
+			}
+			options->rsa_data = &rsa_qt_perf_data[0];
+			break;
+		case RTE_RSA_KEY_TYPE_EXP:
+			if (options->asym_op_type != RTE_CRYPTO_ASYM_OP_ENCRYPT &&
+				options->asym_op_type != RTE_CRYPTO_ASYM_OP_VERIFY) {
+				RTE_LOG(ERR, USER1, "Exponent private key to be used in encrypt and verify op\n");
+				return -EINVAL;
+			}
+			options->rsa_data = &rsa_exp_perf_data[0];
+			break;
+		default:
+			RTE_LOG(ERR, USER1, "Invalid RSA key type specified\n");
+			return -EINVAL;
+		}
+	} else {
+		if (options->asym_op_type != RTE_CRYPTO_ASYM_OP_ENCRYPT) {
+			RTE_LOG(ERR, USER1, "Public key to be used in encrypt op\n");
+			return -EINVAL;
+		}
+	}
+	return 0;
+}
+
 int
 cperf_options_check(struct cperf_options *options)
 {
+	uint16_t modlen;
+	int i;
+
 	if (options->op_type == CPERF_CIPHER_ONLY ||
 			options->op_type == CPERF_DOCSIS)
 		options->digest_sz = 0;
@@ -1112,9 +1435,17 @@ cperf_options_check(struct cperf_options *options)
 	 * If segment size is not set, assume only one segment,
 	 * big enough to contain the largest buffer and the digest
 	 */
-	if (options->segment_sz == 0)
+	if (options->segment_sz == 0) {
 		options->segment_sz = options->max_buffer_size +
 				options->digest_sz;
+		/* In IPsec and TLS operation, packet length will be increased
+		 * by some bytes depend upon the algorithm, so increasing
+		 * the segment size by headroom to cover most of
+		 * the scenarios.
+		 */
+		if (options->op_type == CPERF_IPSEC || options->op_type == CPERF_TLS)
+			options->segment_sz += RTE_PKTMBUF_HEADROOM;
+	}
 
 	if (options->segment_sz < options->digest_sz) {
 		RTE_LOG(ERR, USER1,
@@ -1190,6 +1521,14 @@ cperf_options_check(struct cperf_options *options)
 		return -EINVAL;
 	}
 
+	if (options->op_type == CPERF_CIPHER_THEN_AUTH ||
+			options->op_type == CPERF_AUTH_THEN_CIPHER) {
+		if (!is_valid_chained_op(options)) {
+			RTE_LOG(ERR, USER1, "Invalid chained operation.\n");
+			return -EINVAL;
+		}
+	}
+
 	if (options->op_type == CPERF_CIPHER_THEN_AUTH) {
 		if (options->cipher_op != RTE_CRYPTO_CIPHER_OP_ENCRYPT &&
 				options->auth_op !=
@@ -1198,14 +1537,29 @@ cperf_options_check(struct cperf_options *options)
 					" options: encrypt and generate.\n");
 			return -EINVAL;
 		}
-	} else if (options->op_type == CPERF_AUTH_THEN_CIPHER) {
-		if (options->cipher_op != RTE_CRYPTO_CIPHER_OP_DECRYPT &&
-				options->auth_op !=
-				RTE_CRYPTO_AUTH_OP_VERIFY) {
-			RTE_LOG(ERR, USER1, "Option auth then cipher must use"
-					" options: decrypt and verify.\n");
-			return -EINVAL;
-		}
+	}
+
+	if ((options->test == CPERF_TEST_TYPE_THROUGHPUT ||
+	    options->test == CPERF_TEST_TYPE_LATENCY) &&
+	    (options->aead_op == RTE_CRYPTO_AEAD_OP_DECRYPT ||
+	     options->auth_op == RTE_CRYPTO_AUTH_OP_VERIFY) &&
+	    !options->out_of_place) {
+		RTE_LOG(ERR, USER1, "Only out-of-place is allowed in throughput and"
+			    " latency decryption.\n");
+		return -EINVAL;
+	}
+
+	if ((options->test == CPERF_TEST_TYPE_THROUGHPUT ||
+		 options->test == CPERF_TEST_TYPE_LATENCY) &&
+	    (options->aead_op == RTE_CRYPTO_AEAD_OP_DECRYPT ||
+	     options->auth_op == RTE_CRYPTO_AUTH_OP_VERIFY) &&
+		 options->test_name == NULL &&
+		 options->test_file == NULL) {
+		RTE_LOG(ERR, USER1, "Define path to the file with test"
+				" vectors.\n");
+		RTE_LOG(ERR, USER1, "Define test name to get the correct digest"
+				" from the test vectors.\n");
+		return -EINVAL;
 	}
 
 	if (options->op_type == CPERF_CIPHER_ONLY ||
@@ -1215,11 +1569,126 @@ cperf_options_check(struct cperf_options *options)
 			return -EINVAL;
 	}
 
+	if (options->modex_len) {
+		if (options->op_type != CPERF_ASYM_MODEX) {
+			RTE_LOG(ERR, USER1, "Option modex len should be used only with "
+					" optype: modex.\n");
+			return -EINVAL;
+		}
+
+		for (i = 0; i < (int)RTE_DIM(modex_perf_data); i++) {
+			if (modex_perf_data[i].modulus.len ==
+			    options->modex_len) {
+				options->modex_data =
+					(struct cperf_modex_test_data
+						 *)&modex_perf_data[i];
+				break;
+			}
+		}
+		if (i == (int)RTE_DIM(modex_perf_data)) {
+			RTE_LOG(ERR, USER1,
+				"Option modex len: %d is not supported\n",
+				options->modex_len);
+			return -EINVAL;
+		}
+	}
+
+
+	if (options->op_type == CPERF_ASYM_RSA) {
+		if (cperf_rsa_options_check(options) < 0) {
+			RTE_LOG(ERR, USER1, "Invalid RSA options\n");
+			return -EINVAL;
+		}
+
+		if (options->rsa_modlen) {
+			if (options->rsa_keytype == RTE_RSA_KEY_TYPE_QT) {
+				for (i = 0; i < (int)RTE_DIM(rsa_qt_perf_data); i++) {
+					modlen = rsa_qt_perf_data[i].n.length * 8;
+					if (options->rsa_modlen == modlen) {
+						options->rsa_data =
+							(struct cperf_rsa_test_data *)
+							&rsa_qt_perf_data[i];
+						break;
+					}
+				}
+
+				if (i == (int)RTE_DIM(rsa_qt_perf_data)) {
+					RTE_LOG(ERR, USER1,
+						"Option rsa_modlen: %d is not supported for QT private key\n",
+						options->rsa_modlen);
+					return -EINVAL;
+				}
+			} else if (options->rsa_keytype == RTE_RSA_KEY_TYPE_EXP) {
+				for (i = 0; i < (int)RTE_DIM(rsa_exp_perf_data); i++) {
+					modlen = rsa_exp_perf_data[i].n.length * 8;
+					if (options->rsa_modlen == modlen) {
+						options->rsa_data =
+							(struct cperf_rsa_test_data *)
+							&rsa_exp_perf_data[i];
+						break;
+					}
+				}
+
+				if (i == (int)RTE_DIM(rsa_exp_perf_data)) {
+					RTE_LOG(ERR, USER1,
+						"Option rsa_modlen: %d is not supported for exponent private key\n",
+						options->rsa_modlen);
+					return -EINVAL;
+				}
+			} else {
+				for (i = 0; i < (int)RTE_DIM(rsa_pub_perf_data); i++) {
+					modlen = rsa_pub_perf_data[i].n.length * 8;
+					if (options->rsa_modlen == modlen) {
+						options->rsa_data =
+							(struct cperf_rsa_test_data *)
+							&rsa_pub_perf_data[i];
+						break;
+					}
+				}
+
+				if (i == (int)RTE_DIM(rsa_pub_perf_data)) {
+					RTE_LOG(ERR, USER1,
+						"Option rsa_modlen: %d is not supported for public key\n",
+						options->rsa_modlen);
+					return -EINVAL;
+				}
+			}
+		}
+	}
+
+	if (options->op_type == CPERF_ASYM_SECP192R1 ||
+		options->op_type == CPERF_ASYM_SECP224R1 ||
+		options->op_type == CPERF_ASYM_SECP256R1 ||
+		options->op_type == CPERF_ASYM_SECP384R1 ||
+		options->op_type == CPERF_ASYM_SECP521R1) {
+
+		if (options->asym_op_type != RTE_CRYPTO_ASYM_OP_SIGN &&
+			options->asym_op_type != RTE_CRYPTO_ASYM_OP_VERIFY) {
+			RTE_LOG(ERR, USER1, "ECDSA operations only support sign and verify\n");
+			return -EINVAL;
+		}
+	}
+
 #ifdef RTE_LIB_SECURITY
 	if (options->op_type == CPERF_DOCSIS) {
 		if (check_docsis_buffer_length(options) < 0)
 			return -EINVAL;
 	}
+
+	if (options->op_type == CPERF_IPSEC || options->op_type == CPERF_TLS) {
+		if (options->aead_algo) {
+			if (options->aead_op == RTE_CRYPTO_AEAD_OP_ENCRYPT)
+				options->is_outbound = 1;
+			else
+				options->is_outbound = 0;
+		} else {
+			if (options->cipher_op == RTE_CRYPTO_CIPHER_OP_ENCRYPT &&
+			    options->auth_op == RTE_CRYPTO_AUTH_OP_GENERATE)
+				options->is_outbound = 1;
+			else
+				options->is_outbound = 0;
+		}
+	}
 #endif
 
 	return 0;
@@ -1234,6 +1703,8 @@ cperf_options_dump(struct cperf_options *opts)
 	printf("#\n");
 	printf("# cperf test: %s\n", cperf_test_type_strs[opts->test]);
 	printf("#\n");
+	printf("# cperf operation type: %s\n", cperf_op_type_strs[opts->op_type]);
+	printf("#\n");
 	printf("# size of crypto op / mbuf pool: %u\n", opts->pool_sz);
 	printf("# total number of ops: %u\n", opts->total_ops);
 	if (opts->inc_buffer_size != 0) {
@@ -1264,7 +1735,15 @@ cperf_options_dump(struct cperf_options *opts)
 	printf("#\n");
 	printf("# number of queue pairs per device: %u\n", opts->nb_qps);
 	printf("# crypto operation: %s\n", cperf_op_type_strs[opts->op_type]);
+	if (cperf_is_asym_test(opts)) {
+		if (opts->op_type != CPERF_ASYM_MODEX)
+			printf("# asym operation type: %s\n",
+				   rte_crypto_asym_op_strings[opts->asym_op_type]);
+		if (opts->op_type == CPERF_ASYM_RSA)
+			printf("# rsa test name: %s\n", opts->rsa_data->name);
+	}
 	printf("# sessionless: %s\n", opts->sessionless ? "yes" : "no");
+	printf("# shared session: %s\n", opts->shared_session ? "yes" : "no");
 	printf("# out of place: %s\n", opts->out_of_place ? "yes" : "no");
 	if (opts->test == CPERF_TEST_TYPE_PMDCC)
 		printf("# inter-burst delay: %u ms\n", opts->pmdcc_delay);
@@ -1275,7 +1754,7 @@ cperf_options_dump(struct cperf_options *opts)
 			opts->op_type == CPERF_CIPHER_THEN_AUTH ||
 			opts->op_type == CPERF_AUTH_THEN_CIPHER) {
 		printf("# auth algorithm: %s\n",
-			rte_crypto_auth_algorithm_strings[opts->auth_algo]);
+			rte_cryptodev_get_auth_algo_string(opts->auth_algo));
 		printf("# auth operation: %s\n",
 			rte_crypto_auth_operation_strings[opts->auth_op]);
 		printf("# auth key size: %u\n", opts->auth_key_sz);
@@ -1288,7 +1767,7 @@ cperf_options_dump(struct cperf_options *opts)
 			opts->op_type == CPERF_CIPHER_THEN_AUTH ||
 			opts->op_type == CPERF_AUTH_THEN_CIPHER) {
 		printf("# cipher algorithm: %s\n",
-			rte_crypto_cipher_algorithm_strings[opts->cipher_algo]);
+			rte_cryptodev_get_cipher_algo_string(opts->cipher_algo));
 		printf("# cipher operation: %s\n",
 			rte_crypto_cipher_operation_strings[opts->cipher_op]);
 		printf("# cipher key size: %u\n", opts->cipher_key_sz);
@@ -1298,7 +1777,7 @@ cperf_options_dump(struct cperf_options *opts)
 
 	if (opts->op_type == CPERF_AEAD) {
 		printf("# aead algorithm: %s\n",
-			rte_crypto_aead_algorithm_strings[opts->aead_algo]);
+			rte_cryptodev_get_aead_algo_string(opts->aead_algo));
 		printf("# aead operation: %s\n",
 			rte_crypto_aead_operation_strings[opts->aead_op]);
 		printf("# aead key size: %u\n", opts->aead_key_sz);
