@@ -1,24 +1,42 @@
 /* SPDX-License-Identifier: BSD-3-Clause
  * Copyright 2017 NXP
  */
+
+#include "test.h"
+
 #include <rte_common.h>
 #include <rte_mbuf.h>
 #include <rte_malloc.h>
 #include <rte_memcpy.h>
 #include <rte_dev.h>
+
+#ifdef RTE_EXEC_ENV_WINDOWS
+static int
+test_rawdev_selftests(void)
+{
+	printf("rawdev not supported on Windows, skipping test\n");
+	return TEST_SKIPPED;
+}
+#else
+
 #include <rte_rawdev.h>
 #include <rte_bus_vdev.h>
 
-#include "test.h"
-
 static int
 test_rawdev_selftest_impl(const char *pmd, const char *opts)
 {
-	int ret;
+	int dev_id, ret;
 
 	printf("\n### Test rawdev infrastructure using skeleton driver\n");
 	rte_vdev_init(pmd, opts);
-	ret = rte_rawdev_selftest(rte_rawdev_get_dev_id(pmd));
+	dev_id = rte_rawdev_get_dev_id(pmd);
+	if (dev_id < 0) {
+		printf("Failed to get dev_id for %s\n", pmd);
+		ret = dev_id;
+		goto exit;
+	}
+	ret = rte_rawdev_selftest(dev_id);
+exit:
 	rte_vdev_uninit(pmd);
 	return ret;
 }
@@ -54,4 +72,6 @@ test_rawdev_selftests(void)
 	return ret;
 }
 
-REGISTER_TEST_COMMAND(rawdev_autotest, test_rawdev_selftests);
+#endif /* !RTE_EXEC_ENV_WINDOWS */
+
+REGISTER_FAST_TEST(rawdev_autotest, true, true, test_rawdev_selftests);
