@@ -179,7 +179,7 @@ static int
 test_generic_trace_points(void)
 {
 	uint8_t arr[RTE_TRACE_BLOB_LEN_MAX];
-	int tmp;
+	int tmp = 0;
 	int i;
 
 	for (i = 0; i < RTE_TRACE_BLOB_LEN_MAX; i++)
@@ -245,6 +245,10 @@ static struct unit_test_suite trace_tests = {
 static int
 test_trace(void)
 {
+	if (!rte_trace_feature_is_enabled()) {
+		printf("Trace omitted at build-time, skipping test\n");
+		return TEST_SKIPPED;
+	}
 	return unit_test_suite_runner(&trace_tests);
 }
 
