@@ -5,6 +5,7 @@
 section=all
 symbol=all
 quiet=
+version=
 
 while getopts 'S:s:qV:' name; do
 	case $name in
@@ -29,7 +30,7 @@ while getopts 'S:s:qV:' name; do
 		version=$OPTARG
 	;;
 	?)
-		echo 'usage: $0 [-S section] [-s symbol] [-V version] [-q]'
+		echo 'usage: $0 [-S section] [-s symbol] [-V [version|unset]] [-q]'
 		exit 1
 	;;
 	esac
@@ -41,7 +42,6 @@ for file in $@; do
 	cat "$file" |awk '
 	BEGIN {
 		current_section = "";
-		current_version = "";
 		if ("'$section'" == "all" && "'$symbol'" == "all" && "'$version'" == "") {
 			ret = 0;
 		} else {
@@ -53,22 +53,26 @@ for file in $@; do
 			current_section = $1;
 		}
 	}
-	/.*}/ { current_section = ""; current_version = ""; }
-	/^\t# added in / {
-		current_version=$4;
-	}
+	/.*}/ { current_section = ""; }
 	/^[^}].*[^:*];/ {
 		if (current_section == "") {
 			next;
 		}
-		if ("'$version'" != "" && "'$version'" != current_version) {
-			next;
+		if (/^[^}].*[^:*]; # added in /) {
+			symbol_version = $5
+		}
+		if ("'$version'" != "") {
+			if ("'$version'" == "unset" && symbol_version != "") {
+				next;
+			} else if ("'$version'" != "unset" && "'$version'" != symbol_version) {
+				next;
+			}
 		}
 		gsub(";","");
 		if ("'$symbol'" == "all" || $1 == "'$symbol'") {
 			ret = 0;
 			if ("'$quiet'" == "") {
-				print "'$file' "current_section" "$1;
+				print "'$file' "current_section" "$1" "symbol_version;
 			}
 			if ("'$symbol'" != "all") {
 				exit 0;
