@@ -10,6 +10,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/queue.h>
+#include <eal_export.h>
 #include <rte_errno.h>
 #include <rte_interrupts.h>
 #include <rte_log.h>
@@ -32,6 +33,7 @@
 
 #define SYSFS_PCI_DEVICES "/sys/bus/pci/devices"
 
+RTE_EXPORT_INTERNAL_SYMBOL(rte_pci_get_sysfs_path)
 const char *rte_pci_get_sysfs_path(void)
 {
 	const char *path = NULL;
@@ -99,21 +101,11 @@ pci_common_set(struct rte_pci_device *dev)
 	/* Each device has its internal, canonical name set. */
 	rte_pci_device_name(&dev->addr,
 			dev->name, sizeof(dev->name));
+	dev->device.name = dev->name;
+
 	devargs = pci_devargs_lookup(&dev->addr);
 	dev->device.devargs = devargs;
 
-	/* When using a blocklist, only blocked devices will have
-	 * an rte_devargs. Allowed devices won't have one.
-	 */
-	if (devargs != NULL)
-		/* If an rte_devargs exists, the generic rte_device uses the
-		 * given name as its name.
-		 */
-		dev->device.name = dev->device.devargs->name;
-	else
-		/* Otherwise, it uses the internal, canonical form. */
-		dev->device.name = dev->name;
-
 	if (dev->bus_info != NULL ||
 			asprintf(&dev->bus_info, "vendor_id=%"PRIx16", device_id=%"PRIx16,
 				dev->id.vendor_id, dev->id.device_id) != -1)
@@ -141,13 +133,12 @@ pci_map_resource(void *requested_addr, int fd, off_t offset, size_t size,
 		RTE_PROT_READ | RTE_PROT_WRITE,
 		RTE_MAP_SHARED | additional_flags, fd, offset);
 	if (mapaddr == NULL) {
-		RTE_LOG(ERR, EAL,
-			"%s(): cannot map resource(%d, %p, 0x%zx, 0x%llx): %s (%p)\n",
+		PCI_LOG(ERR, "%s(): cannot map resource(%d, %p, 0x%zx, 0x%llx): %s (%p)",
 			__func__, fd, requested_addr, size,
 			(unsigned long long)offset,
 			rte_strerror(rte_errno), mapaddr);
 	} else
-		RTE_LOG(DEBUG, EAL, "  PCI memory mapped at %p\n", mapaddr);
+		PCI_LOG(DEBUG, "  PCI memory mapped at %p", mapaddr);
 
 	return mapaddr;
 }
@@ -161,12 +152,11 @@ pci_unmap_resource(void *requested_addr, size_t size)
 
 	/* Unmap the PCI memory resource of device */
 	if (rte_mem_unmap(requested_addr, size)) {
-		RTE_LOG(ERR, EAL, "%s(): cannot mem unmap(%p, %#zx): %s\n",
+		PCI_LOG(ERR, "%s(): cannot mem unmap(%p, %#zx): %s",
 			__func__, requested_addr, size,
 			rte_strerror(rte_errno));
 	} else
-		RTE_LOG(DEBUG, EAL, "  PCI memory unmapped at %p\n",
-				requested_addr);
+		PCI_LOG(DEBUG, "  PCI memory unmapped at %p", requested_addr);
 }
 /*
  * Match the PCI Driver and Device using the ID Table
@@ -226,28 +216,27 @@ rte_pci_probe_one_driver(struct rte_pci_driver *dr,
 		/* Match of device and driver failed */
 		return 1;
 
-	RTE_LOG(DEBUG, EAL, "PCI device "PCI_PRI_FMT" on NUMA socket %i\n",
-			loc->domain, loc->bus, loc->devid, loc->function,
-			dev->device.numa_node);
+	PCI_LOG(DEBUG, "PCI device "PCI_PRI_FMT" on NUMA socket %i",
+		loc->domain, loc->bus, loc->devid, loc->function,
+		dev->device.numa_node);
 
 	/* no initialization when marked as blocked, return without error */
 	if (dev->device.devargs != NULL &&
 		dev->device.devargs->policy == RTE_DEV_BLOCKED) {
-		RTE_LOG(INFO, EAL, "  Device is blocked, not initializing\n");
+		PCI_LOG(INFO, "  Device is blocked, not initializing");
 		return 1;
 	}
 
 	if (dev->device.numa_node < 0 && rte_socket_count() > 1)
-		RTE_LOG(INFO, EAL, "Device %s is not NUMA-aware\n", dev->name);
+		PCI_LOG(INFO, "Device %s is not NUMA-aware", dev->name);
 
 	already_probed = rte_dev_is_probed(&dev->device);
 	if (already_probed && !(dr->drv_flags & RTE_PCI_DRV_PROBE_AGAIN)) {
-		RTE_LOG(DEBUG, EAL, "Device %s is already probed\n",
-				dev->device.name);
+		PCI_LOG(DEBUG, "Device %s is already probed", dev->device.name);
 		return -EEXIST;
 	}
 
-	RTE_LOG(DEBUG, EAL, "  probe driver: %x:%x %s\n", dev->id.vendor_id,
+	PCI_LOG(DEBUG, "  probe driver: %x:%x %s", dev->id.vendor_id,
 		dev->id.device_id, dr->driver.name);
 
 	if (!already_probed) {
@@ -258,7 +247,7 @@ rte_pci_probe_one_driver(struct rte_pci_driver *dr,
 		iova_mode = rte_eal_iova_mode();
 		if (dev_iova_mode != RTE_IOVA_DC &&
 		    dev_iova_mode != iova_mode) {
-			RTE_LOG(ERR, EAL, "  Expecting '%s' IOVA mode but current mode is '%s', not initializing\n",
+			PCI_LOG(ERR, "  Expecting '%s' IOVA mode but current mode is '%s', not initializing",
 				dev_iova_mode == RTE_IOVA_PA ? "PA" : "VA",
 				iova_mode == RTE_IOVA_PA ? "PA" : "VA");
 			return -EINVAL;
@@ -268,8 +257,7 @@ rte_pci_probe_one_driver(struct rte_pci_driver *dr,
 		dev->intr_handle =
 			rte_intr_instance_alloc(RTE_INTR_INSTANCE_F_PRIVATE);
 		if (dev->intr_handle == NULL) {
-			RTE_LOG(ERR, EAL,
-				"Failed to create interrupt instance for %s\n",
+			PCI_LOG(ERR, "Failed to create interrupt instance for %s",
 				dev->device.name);
 			return -ENOMEM;
 		}
@@ -279,8 +267,7 @@ rte_pci_probe_one_driver(struct rte_pci_driver *dr,
 		if (dev->vfio_req_intr_handle == NULL) {
 			rte_intr_instance_free(dev->intr_handle);
 			dev->intr_handle = NULL;
-			RTE_LOG(ERR, EAL,
-				"Failed to create vfio req interrupt instance for %s\n",
+			PCI_LOG(ERR, "Failed to create vfio req interrupt instance for %s",
 				dev->device.name);
 			return -ENOMEM;
 		}
@@ -304,10 +291,10 @@ rte_pci_probe_one_driver(struct rte_pci_driver *dr,
 		}
 	}
 
-	RTE_LOG(INFO, EAL, "Probe PCI driver: %s (%x:%04x) device: "PCI_PRI_FMT" (socket %i)\n",
-			dr->driver.name, dev->id.vendor_id, dev->id.device_id,
-			loc->domain, loc->bus, loc->devid, loc->function,
-			dev->device.numa_node);
+	PCI_LOG(INFO, "Probe PCI driver: %s (%x:%04x) device: "PCI_PRI_FMT" (socket %i)",
+		dr->driver.name, dev->id.vendor_id, dev->id.device_id,
+		loc->domain, loc->bus, loc->devid, loc->function,
+		dev->device.numa_node);
 	/* call the driver probe() function */
 	ret = dr->probe(dr, dev);
 	if (already_probed)
@@ -349,12 +336,12 @@ rte_pci_detach_dev(struct rte_pci_device *dev)
 	dr = dev->driver;
 	loc = &dev->addr;
 
-	RTE_LOG(DEBUG, EAL, "PCI device "PCI_PRI_FMT" on NUMA socket %i\n",
-			loc->domain, loc->bus, loc->devid,
-			loc->function, dev->device.numa_node);
+	PCI_LOG(DEBUG, "PCI device "PCI_PRI_FMT" on NUMA socket %i",
+		loc->domain, loc->bus, loc->devid,
+		loc->function, dev->device.numa_node);
 
-	RTE_LOG(DEBUG, EAL, "  remove driver: %x:%x %s\n", dev->id.vendor_id,
-			dev->id.device_id, dr->driver.name);
+	PCI_LOG(DEBUG, "  remove driver: %x:%x %s", dev->id.vendor_id,
+		dev->id.device_id, dr->driver.name);
 
 	if (dr->remove) {
 		ret = dr->remove(dev);
@@ -423,8 +410,7 @@ pci_probe(void)
 		ret = pci_probe_all_drivers(dev);
 		if (ret < 0) {
 			if (ret != -EEXIST) {
-				RTE_LOG(ERR, EAL, "Requested device "
-					PCI_PRI_FMT " cannot be used\n",
+				PCI_LOG(ERR, "Requested device " PCI_PRI_FMT " cannot be used",
 					dev->addr.domain, dev->addr.bus,
 					dev->addr.devid, dev->addr.function);
 				rte_errno = errno;
@@ -465,6 +451,7 @@ free:
 		rte_intr_instance_free(dev->vfio_req_intr_handle);
 		dev->vfio_req_intr_handle = NULL;
 
+		TAILQ_REMOVE(&rte_pci_bus.device_list, dev, next);
 		pci_free(RTE_PCI_DEVICE_INTERNAL(dev));
 	}
 
@@ -492,6 +479,7 @@ pci_dump_one_device(FILE *f, struct rte_pci_device *dev)
 }
 
 /* dump devices on the bus */
+RTE_EXPORT_SYMBOL(rte_pci_dump)
 void
 rte_pci_dump(FILE *f)
 {
@@ -516,6 +504,7 @@ pci_parse(const char *name, void *addr)
 }
 
 /* register a driver */
+RTE_EXPORT_INTERNAL_SYMBOL(rte_pci_register)
 void
 rte_pci_register(struct rte_pci_driver *driver)
 {
@@ -523,6 +512,7 @@ rte_pci_register(struct rte_pci_driver *driver)
 }
 
 /* unregister a driver */
+RTE_EXPORT_INTERNAL_SYMBOL(rte_pci_unregister)
 void
 rte_pci_unregister(struct rte_pci_driver *driver)
 {
@@ -592,8 +582,8 @@ pci_find_device_by_addr(const void *failure_addr)
 			len = pdev->mem_resource[i].len;
 			end = start + len;
 			if (check_point >= start && check_point < end) {
-				RTE_LOG(DEBUG, EAL, "Failure address %16.16"
-					PRIx64" belongs to device %s!\n",
+				PCI_LOG(DEBUG, "Failure address %16.16"
+					PRIx64" belongs to device %s!",
 					check_point, pdev->device.name);
 				return pdev;
 			}
@@ -613,7 +603,6 @@ pci_hot_unplug_handler(struct rte_device *dev)
 		return -1;
 
 	switch (pdev->kdrv) {
-#ifdef HAVE_VFIO_DEV_REQ_INTERFACE
 	case RTE_PCI_KDRV_VFIO:
 		/*
 		 * vfio kernel module guaranty the pci device would not be
@@ -624,7 +613,6 @@ pci_hot_unplug_handler(struct rte_device *dev)
 		rte_dev_event_callback_process(dev->name,
 					       RTE_DEV_EVENT_REMOVE);
 		break;
-#endif
 	case RTE_PCI_KDRV_IGB_UIO:
 	case RTE_PCI_KDRV_UIO_GENERIC:
 	case RTE_PCI_KDRV_NIC_UIO:
@@ -632,8 +620,7 @@ pci_hot_unplug_handler(struct rte_device *dev)
 		ret = pci_uio_remap_resource(pdev);
 		break;
 	default:
-		RTE_LOG(DEBUG, EAL,
-			"Not managed by a supported kernel driver, skipped\n");
+		PCI_LOG(DEBUG, "Not managed by a supported kernel driver, skipped");
 		ret = -1;
 		break;
 	}
@@ -655,8 +642,7 @@ pci_sigbus_handler(const void *failure_addr)
 		/* The sigbus error is caused of hot-unplug. */
 		ret = pci_hot_unplug_handler(&pdev->device);
 		if (ret) {
-			RTE_LOG(ERR, EAL,
-				"Failed to handle hot-unplug for device %s",
+			PCI_LOG(ERR, "Failed to handle hot-unplug for device %s",
 				pdev->name);
 			ret = -1;
 		}
@@ -780,8 +766,7 @@ rte_pci_get_iommu_class(void)
 				continue;
 
 			dev_iova_mode = pci_device_iova_mode(drv, dev);
-			RTE_LOG(DEBUG, EAL, "PCI driver %s for device "
-				PCI_PRI_FMT " wants IOVA as '%s'\n",
+			PCI_LOG(DEBUG, "PCI driver %s for device "PCI_PRI_FMT" wants IOVA as '%s'",
 				drv->driver.name,
 				dev->addr.domain, dev->addr.bus,
 				dev->addr.devid, dev->addr.function,
@@ -796,8 +781,8 @@ rte_pci_get_iommu_class(void)
 	if (iommu_no_va == 1) {
 		iova_mode = RTE_IOVA_PA;
 		if (devices_want_va) {
-			RTE_LOG(WARNING, EAL, "Some devices want 'VA' but IOMMU does not support 'VA'.\n");
-			RTE_LOG(WARNING, EAL, "The devices that want 'VA' won't initialize.\n");
+			PCI_LOG(WARNING, "Some devices want 'VA' but IOMMU does not support 'VA'.");
+			PCI_LOG(WARNING, "The devices that want 'VA' won't initialize.");
 		}
 	} else if (devices_want_va && !devices_want_pa) {
 		iova_mode = RTE_IOVA_VA;
@@ -806,13 +791,14 @@ rte_pci_get_iommu_class(void)
 	} else {
 		iova_mode = RTE_IOVA_DC;
 		if (devices_want_va) {
-			RTE_LOG(WARNING, EAL, "Some devices want 'VA' but forcing 'DC' because other devices want 'PA'.\n");
-			RTE_LOG(WARNING, EAL, "Depending on the final decision by the EAL, not all devices may be able to initialize.\n");
+			PCI_LOG(WARNING, "Some devices want 'VA' but forcing 'DC' because other devices want 'PA'.");
+			PCI_LOG(WARNING, "Depending on the final decision by the EAL, not all devices may be able to initialize.");
 		}
 	}
 	return iova_mode;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_pci_has_capability_list, 23.11)
 bool
 rte_pci_has_capability_list(const struct rte_pci_device *dev)
 {
@@ -824,12 +810,14 @@ rte_pci_has_capability_list(const struct rte_pci_device *dev)
 	return (status & RTE_PCI_STATUS_CAP_LIST) != 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_pci_find_capability, 23.11)
 off_t
 rte_pci_find_capability(const struct rte_pci_device *dev, uint8_t cap)
 {
 	return rte_pci_find_next_capability(dev, cap, 0);
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_pci_find_next_capability, 23.11)
 off_t
 rte_pci_find_next_capability(const struct rte_pci_device *dev, uint8_t cap,
 	off_t offset)
@@ -867,6 +855,7 @@ rte_pci_find_next_capability(const struct rte_pci_device *dev, uint8_t cap,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_pci_find_ext_capability, 20.11)
 off_t
 rte_pci_find_ext_capability(const struct rte_pci_device *dev, uint32_t cap)
 {
@@ -878,7 +867,7 @@ rte_pci_find_ext_capability(const struct rte_pci_device *dev, uint32_t cap)
 	ttl = (RTE_PCI_CFG_SPACE_EXP_SIZE - RTE_PCI_CFG_SPACE_SIZE) / 8;
 
 	if (rte_pci_read_config(dev, &header, 4, offset) < 0) {
-		RTE_LOG(ERR, EAL, "error in reading extended capabilities\n");
+		PCI_LOG(ERR, "error in reading extended capabilities");
 		return -1;
 	}
 
@@ -899,8 +888,7 @@ rte_pci_find_ext_capability(const struct rte_pci_device *dev, uint32_t cap)
 			break;
 
 		if (rte_pci_read_config(dev, &header, 4, offset) < 0) {
-			RTE_LOG(ERR, EAL,
-				"error in reading extended capabilities\n");
+			PCI_LOG(ERR, "error in reading extended capabilities");
 			return -1;
 		}
 
@@ -910,6 +898,7 @@ rte_pci_find_ext_capability(const struct rte_pci_device *dev, uint32_t cap)
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_pci_set_bus_master, 21.08)
 int
 rte_pci_set_bus_master(const struct rte_pci_device *dev, bool enable)
 {
@@ -917,7 +906,7 @@ rte_pci_set_bus_master(const struct rte_pci_device *dev, bool enable)
 
 	if (rte_pci_read_config(dev, &old_cmd, sizeof(old_cmd),
 				RTE_PCI_COMMAND) < 0) {
-		RTE_LOG(ERR, EAL, "error in reading PCI command register\n");
+		PCI_LOG(ERR, "error in reading PCI command register");
 		return -1;
 	}
 
@@ -931,13 +920,14 @@ rte_pci_set_bus_master(const struct rte_pci_device *dev, bool enable)
 
 	if (rte_pci_write_config(dev, &cmd, sizeof(cmd),
 				 RTE_PCI_COMMAND) < 0) {
-		RTE_LOG(ERR, EAL, "error in writing PCI command register\n");
+		PCI_LOG(ERR, "error in writing PCI command register");
 		return -1;
 	}
 
 	return 0;
 }
 
+RTE_EXPORT_INTERNAL_SYMBOL(rte_pci_pasid_set_state)
 int
 rte_pci_pasid_set_state(const struct rte_pci_device *dev,
 		off_t offset, bool enable)
@@ -969,3 +959,4 @@ struct rte_pci_bus rte_pci_bus = {
 };
 
 RTE_REGISTER_BUS(pci, rte_pci_bus.bus);
+RTE_LOG_REGISTER_DEFAULT(pci_bus_logtype, NOTICE);
