@@ -16,6 +16,7 @@
 #define ROC_NIX_SQB_THRESH	      30U
 #define ROC_NIX_SQB_SLACK	      12U
 #define ROC_NIX_AURA_THRESH	      95U
+#define ROC_NIX_LINK_SPEED_ALL	      0xFFFFF
 
 /* Reserved interface types for BPID allocation */
 #define ROC_NIX_INTF_TYPE_CGX  0
@@ -28,6 +29,11 @@
 #define ROC_NIX_INTF_TYPE_CPT_NIX 254
 #define ROC_NIX_INTF_TYPE_SSO     253
 
+/* Software defined LSO base format IDX */
+#define ROC_NIX_LSO_FORMAT_IDX_TSOV4 0
+#define ROC_NIX_LSO_FORMAT_IDX_TSOV6 1
+#define ROC_NIX_LSO_FORMAT_IDX_IPV4  2
+
 enum roc_nix_rss_reta_sz {
 	ROC_NIX_RSS_RETA_SZ_64 = 64,
 	ROC_NIX_RSS_RETA_SZ_128 = 128,
@@ -109,6 +115,89 @@ enum roc_nix_bpf_stats {
 	ROC_NIX_BPF_RED_OCTS_F_DROP = BIT_ULL(11),
 };
 
+enum roc_nix_link_duplex {
+	ROC_NIX_LINK_DUPLEX_FULL = 0,
+	ROC_NIX_LINK_DUPLEX_HALF = 1
+};
+
+enum roc_nix_link_speed_bits {
+	ROC_NIX_LINK_SPEED_NONE_BIT = BIT_ULL(0),
+	ROC_NIX_LINK_SPEED_10M_HD_BIT = BIT_ULL(1),
+	ROC_NIX_LINK_SPEED_10M_BIT = BIT_ULL(2),
+	ROC_NIX_LINK_SPEED_100M_HD_BIT = BIT_ULL(3),
+	ROC_NIX_LINK_SPEED_100M_BIT = BIT_ULL(4),
+	ROC_NIX_LINK_SPEED_1G_BIT = BIT_ULL(5),
+	ROC_NIX_LINK_SPEED_2_5G_BIT = BIT_ULL(6),
+	ROC_NIX_LINK_SPEED_5G_BIT = BIT_ULL(7),
+	ROC_NIX_LINK_SPEED_10G_BIT = BIT_ULL(8),
+	ROC_NIX_LINK_SPEED_20G_BIT = BIT_ULL(9),
+	ROC_NIX_LINK_SPEED_25G_BIT = BIT_ULL(10),
+	ROC_NIX_LINK_SPEED_40G_BIT = BIT_ULL(11),
+	ROC_NIX_LINK_SPEED_50G_BIT = BIT_ULL(12),
+	ROC_NIX_LINK_SPEED_56G_BIT = BIT_ULL(13),
+	ROC_NIX_LINK_SPEED_100G_BIT = BIT_ULL(14),
+	ROC_NIX_LINK_SPEED_200G_BIT = BIT_ULL(15),
+	ROC_NIX_LINK_SPEED_400G_BIT = BIT_ULL(16),
+	ROC_NIX_LINK_SPEED_MAX = 17
+};
+
+enum roc_nix_link_mode {
+	ROC_NIX_LINK_MODE_10BASET_HD = BIT_ULL(0),
+	ROC_NIX_LINK_MODE_10BASET_FD = BIT_ULL(1),
+	ROC_NIX_LINK_MODE_100BASET_HD = BIT_ULL(2),
+	ROC_NIX_LINK_MODE_100BASET_FD = BIT_ULL(3),
+	ROC_NIX_LINK_MODE_1000BASET_HD = BIT_ULL(4),
+	ROC_NIX_LINK_MODE_1000BASET_FD = BIT_ULL(5),
+	ROC_NIX_LINK_MODE_AUTONEG = BIT_ULL(6),
+	ROC_NIX_LINK_MODE_10000BASET_FD = BIT_ULL(12),
+	ROC_NIX_LINK_MODE_2500BASEX_FD = BIT_ULL(15),
+	ROC_NIX_LINK_MODE_1000BASEKX_FD = BIT_ULL(17),
+	ROC_NIX_LINK_MODE_10000BASEKX4_FD = BIT_ULL(18),
+	ROC_NIX_LINK_MODE_10000BASEKR_FD = BIT_ULL(19),
+	ROC_NIX_LINK_MODE_10000BASER_FEC = BIT_ULL(20),
+	ROC_NIX_LINK_MODE_20000BASEMLD2_FD = BIT_ULL(21),
+	ROC_NIX_LINK_MODE_20000BASEKR2_FD = BIT_ULL(22),
+	ROC_NIX_LINK_MODE_40000BASEKR4_FD = BIT_ULL(23),
+	ROC_NIX_LINK_MODE_40000BASECR4_FD = BIT_ULL(24),
+	ROC_NIX_LINK_MODE_40000BASESR4_FD = BIT_ULL(25),
+	ROC_NIX_LINK_MODE_40000BASELR4_FD = BIT_ULL(26),
+	ROC_NIX_LINK_MODE_56000BASEKR4_FD = BIT_ULL(27),
+	ROC_NIX_LINK_MODE_56000BASECR4_FD = BIT_ULL(28),
+	ROC_NIX_LINK_MODE_56000BASESR4_FD = BIT_ULL(29),
+	ROC_NIX_LINK_MODE_56000BASELR4_FD = BIT_ULL(30),
+	ROC_NIX_LINK_MODE_25000BASECR_FD = BIT_ULL(31),
+	ROC_NIX_LINK_MODE_25000BASEKR_FD = BIT_ULL(32),
+	ROC_NIX_LINK_MODE_25000BASESR_FD = BIT_ULL(33),
+	ROC_NIX_LINK_MODE_50000BASECR2_FD = BIT_ULL(34),
+	ROC_NIX_LINK_MODE_50000BASEKR2_FD = BIT_ULL(35),
+	ROC_NIX_LINK_MODE_100000BASEKR4_FD = BIT_ULL(36),
+	ROC_NIX_LINK_MODE_100000BASESR4_FD = BIT_ULL(37),
+	ROC_NIX_LINK_MODE_100000BASECR4_FD = BIT_ULL(38),
+	ROC_NIX_LINK_MODE_100000BASELR4_ER4_FD = BIT_ULL(39),
+	ROC_NIX_LINK_MODE_50000BASESR2_FD = BIT_ULL(40),
+	ROC_NIX_LINK_MODE_1000BASEX_FD = BIT_ULL(41),
+	ROC_NIX_LINK_MODE_10000BASECR_FD = BIT_ULL(42),
+	ROC_NIX_LINK_MODE_10000BASESR_FD = BIT_ULL(43),
+	ROC_NIX_LINK_MODE_10000BASELR_FD = BIT_ULL(44),
+	ROC_NIX_LINK_MODE_10000BASELRM_FD = BIT_ULL(45),
+	ROC_NIX_LINK_MODE_10000BASEER_FD = BIT_ULL(46),
+	ROC_NIX_LINK_MODE_2500BASET_FD = BIT_ULL(47),
+	ROC_NIX_LINK_MODE_5000BASET_FD = BIT_ULL(48),
+	ROC_NIX_LINK_MODE_FEC_NONE = BIT_ULL(49),
+	ROC_NIX_LINK_MODE_FEC_RS = BIT_ULL(50),
+	ROC_NIX_LINK_MODE_FEC_BASER = BIT_ULL(51),
+	ROC_NIX_LINK_MODE_50000BASEKR_FD = BIT_ULL(52),
+	ROC_NIX_LINK_MODE_50000BASESR_FD = BIT_ULL(53),
+	ROC_NIX_LINK_MODE_50000BASECR_FD = BIT_ULL(54),
+	ROC_NIX_LINK_MODE_50000BASELR_ER_FR_FD = BIT_ULL(55),
+	ROC_NIX_LINK_MODE_50000BASEDR_FD = BIT_ULL(56),
+	ROC_NIX_LINK_MODE_100000BASEKR2_FD = BIT_ULL(57),
+	ROC_NIX_LINK_MODE_100000BASESR2_FD = BIT_ULL(58),
+	ROC_NIX_LINK_MODE_100000BASECR2_FD = BIT_ULL(59),
+	ROC_NIX_LINK_MODE_100000BASELR2_ER2_FR2_FD = BIT_ULL(60),
+	ROC_NIX_LINK_MODE_100000BASEDR2_FD = BIT_ULL(61),
+};
+
 struct roc_nix_bpf_cfg {
 	enum roc_nix_bpf_algo alg;
 	enum roc_nix_bpf_lmode lmode;
@@ -189,6 +278,7 @@ struct roc_nix_fc_cfg {
 			uint32_t rq;
 			uint16_t tc;
 			uint16_t cq_drop;
+			uint16_t cq_bp;
 			bool enable;
 		} cq_cfg;
 
@@ -196,6 +286,7 @@ struct roc_nix_fc_cfg {
 			uint32_t rq;
 			uint16_t tc;
 			uint16_t cq_drop;
+			uint16_t cq_bp;
 			uint64_t pool;
 			uint64_t spb_pool;
 			uint64_t pool_drop_pct;
@@ -243,6 +334,7 @@ struct roc_nix_eeprom_info {
 #define ROC_NIX_LF_RX_CFG_LEN_IL3     BIT_ULL(39)
 #define ROC_NIX_LF_RX_CFG_LEN_OL4     BIT_ULL(40)
 #define ROC_NIX_LF_RX_CFG_LEN_OL3     BIT_ULL(41)
+#define ROC_NIX_LF_RX_CFG_APAD_MODE   BIT_ULL(42)
 
 #define ROC_NIX_LF_RX_CFG_RX_ERROR_MASK 0xFFFFFFFFFFF80000
 #define ROC_NIX_RE_PARTIAL		BIT_ULL(1)
@@ -267,8 +359,6 @@ struct roc_nix_eeprom_info {
 #define ROC_NIX_RSS_KEY_LEN	     48 /* 352 Bits */
 #define ROC_NIX_RSS_MCAM_IDX_DEFAULT (-1)
 
-#define ROC_NIX_DEFAULT_HW_FRS 1514
-
 #define ROC_NIX_VWQE_MAX_SIZE_LOG2 11
 #define ROC_NIX_VWQE_MIN_SIZE_LOG2 2
 
@@ -356,6 +446,8 @@ struct roc_nix_rq {
 	bool lpb_drop_ena;
 	/* SPB aura drop enable */
 	bool spb_drop_ena;
+	/* XQE drop enable */
+	bool xqe_drop_ena;
 	/* End of Input parameters */
 	struct roc_nix *roc_nix;
 	uint64_t meta_aura_handle;
@@ -370,6 +462,7 @@ struct roc_nix_cq {
 	uint8_t stash_thresh;
 	/* End of Input parameters */
 	uint16_t drop_thresh;
+	uint16_t bp_thresh;
 	struct roc_nix *roc_nix;
 	uintptr_t door;
 	int64_t *status;
@@ -390,6 +483,7 @@ struct roc_nix_sq {
 	bool cq_ena;
 	uint8_t fc_hyst_bits;
 	/* End of Input parameters */
+	uint16_t sqes_per_sqb;
 	uint16_t sqes_per_sqb_log2;
 	struct roc_nix *roc_nix;
 	uint64_t aura_handle;
@@ -400,7 +494,10 @@ struct roc_nix_sq {
 	void *lmt_addr;
 	void *sqe_mem;
 	void *fc;
+	void *sq_cnt_ptr;
+	uint8_t update_sq_cnt;
 	uint8_t tc;
+	bool enable;
 };
 
 struct roc_nix_link_info {
@@ -408,9 +505,18 @@ struct roc_nix_link_info {
 	uint64_t full_duplex : 1;
 	uint64_t lmac_type_id : 4;
 	uint64_t speed : 20;
+	uint64_t speed_bitmask : 16;
 	uint64_t autoneg : 1;
 	uint64_t fec : 2;
 	uint64_t port : 8;
+	uint64_t advertising;
+};
+
+struct roc_nix_mac_fwdata {
+	uint64_t advertised_link_modes;
+	uint64_t supported_link_modes;
+	uint64_t port_type;
+	uint64_t supported_an;
 };
 
 /** Maximum name length for extended statistics counters */
@@ -431,6 +537,7 @@ struct roc_nix_ipsec_cfg {
 	plt_iova_t iova;
 	uint16_t max_sa;
 	uint8_t tt;
+	int8_t res_addr_offset;
 };
 
 /* Link status update callback */
@@ -469,10 +576,20 @@ struct roc_nix {
 	uint32_t dwrr_mtu;
 	bool ipsec_out_sso_pffunc;
 	bool custom_sa_action;
+	int8_t res_addr_offset;
 	bool local_meta_aura_ena;
 	uint32_t meta_buf_sz;
 	bool force_rx_aura_bp;
 	bool custom_meta_aura_ena;
+	bool rx_inj_ena;
+	bool custom_inb_sa;
+	bool use_write_sa;
+	uint32_t root_sched_weight;
+	uint16_t inb_cfg_param1;
+	uint16_t inb_cfg_param2;
+	bool force_tail_drop;
+	bool dis_xqe_drop;
+	bool sq_resize_ena;
 	/* End of input parameters */
 	/* LMT line base for "Per Core Tx LMT line" mode*/
 	uintptr_t lmt_base;
@@ -482,9 +599,13 @@ struct roc_nix {
 	uint32_t buf_sz;
 	uint64_t meta_aura_handle;
 	uintptr_t meta_mempool;
+	uint16_t rep_cnt;
+	uint16_t rep_pfvf_map[MAX_PFVF_REP];
+	bool reass_ena;
+	bool use_multi_bpids;
 	TAILQ_ENTRY(roc_nix) next;
 
-#define ROC_NIX_MEM_SZ (6 * 1070)
+#define ROC_NIX_MEM_SZ (6 * 1131)
 	uint8_t reserved[ROC_NIX_MEM_SZ] __plt_cache_aligned;
 } __plt_cache_aligned;
 
@@ -524,6 +645,7 @@ int __roc_api roc_nix_dev_fini(struct roc_nix *roc_nix);
 
 /* Type */
 bool __roc_api roc_nix_is_lbk(struct roc_nix *roc_nix);
+bool __roc_api roc_nix_is_esw(struct roc_nix *roc_nix);
 bool __roc_api roc_nix_is_sdp(struct roc_nix *roc_nix);
 bool __roc_api roc_nix_is_pf(struct roc_nix *roc_nix);
 bool __roc_api roc_nix_is_vf_or_sdp(struct roc_nix *roc_nix);
@@ -548,10 +670,12 @@ int __roc_api roc_nix_rx_drop_re_set(struct roc_nix *roc_nix, bool ena);
 int __roc_api roc_nix_lf_get_reg_count(struct roc_nix *roc_nix);
 int __roc_api roc_nix_lf_reg_dump(struct roc_nix *roc_nix, uint64_t *data);
 int __roc_api roc_nix_queues_ctx_dump(struct roc_nix *roc_nix, FILE *file);
-void __roc_api roc_nix_cqe_dump(const struct nix_cqe_hdr_s *cq);
+void __roc_api roc_nix_cqe_dump(FILE *file, const struct nix_cqe_hdr_s *cq);
 void __roc_api roc_nix_rq_dump(struct roc_nix_rq *rq, FILE *file);
 void __roc_api roc_nix_cq_dump(struct roc_nix_cq *cq, FILE *file);
 void __roc_api roc_nix_sq_dump(struct roc_nix_sq *sq, FILE *file);
+int __roc_api roc_nix_sq_desc_dump(struct roc_nix *roc_nix, uint16_t q, uint16_t offset,
+				   uint16_t num, FILE *file);
 void __roc_api roc_nix_tm_dump(struct roc_nix *roc_nix, FILE *file);
 void __roc_api roc_nix_dump(struct roc_nix *roc_nix, FILE *file);
 
@@ -575,6 +699,7 @@ enum roc_nix_tm_tree {
 	ROC_NIX_TM_DEFAULT = 0,
 	ROC_NIX_TM_RLIMIT,
 	ROC_NIX_TM_PFC,
+	ROC_NIX_TM_SDP,
 	ROC_NIX_TM_USER,
 	ROC_NIX_TM_TREE_MAX,
 };
@@ -747,6 +872,7 @@ int __roc_api roc_nix_tm_lvl_cnt_get(struct roc_nix *roc_nix);
 int __roc_api roc_nix_tm_lvl_have_link_access(struct roc_nix *roc_nix, int lvl);
 int __roc_api roc_nix_tm_prepare_rate_limited_tree(struct roc_nix *roc_nix);
 int __roc_api roc_nix_tm_pfc_prepare_tree(struct roc_nix *roc_nix);
+int __roc_api roc_nix_tm_sdp_prepare_tree(struct roc_nix *roc_nix);
 bool __roc_api roc_nix_tm_is_user_hierarchy_enabled(struct roc_nix *nix);
 int __roc_api roc_nix_tm_tree_type_get(struct roc_nix *nix);
 int __roc_api roc_nix_tm_mark_config(struct roc_nix *roc_nix,
@@ -754,6 +880,8 @@ int __roc_api roc_nix_tm_mark_config(struct roc_nix *roc_nix,
 				     int mark_red);
 uint64_t __roc_api roc_nix_tm_mark_format_get(struct roc_nix *roc_nix,
 					      uint64_t *flags);
+int __roc_api roc_nix_tm_egress_link_cfg_set(struct roc_nix *roc_nix, uint64_t dst_pf_func,
+					     bool enable);
 
 /* Ingress Policer API */
 int __roc_api roc_nix_bpf_timeunit_get(struct roc_nix *roc_nix,
@@ -841,6 +969,8 @@ int __roc_api roc_nix_mac_link_info_get_cb_register(
 void __roc_api roc_nix_mac_link_info_get_cb_unregister(struct roc_nix *roc_nix);
 int __roc_api roc_nix_q_err_cb_register(struct roc_nix *roc_nix, q_err_get_t sq_err_handle);
 void __roc_api roc_nix_q_err_cb_unregister(struct roc_nix *roc_nix);
+int __roc_api roc_nix_mac_stats_reset(struct roc_nix *roc_nix);
+int __roc_api roc_nix_mac_fwdata_get(struct roc_nix *roc_nix, struct roc_nix_mac_fwdata *fwdata);
 
 /* Ops */
 int __roc_api roc_nix_switch_hdr_set(struct roc_nix *roc_nix,
@@ -852,9 +982,12 @@ int __roc_api roc_nix_lso_fmt_setup(struct roc_nix *roc_nix);
 int __roc_api roc_nix_lso_fmt_get(struct roc_nix *roc_nix,
 				  uint8_t udp_tun[ROC_NIX_LSO_TUN_MAX],
 				  uint8_t tun[ROC_NIX_LSO_TUN_MAX]);
+int __roc_api roc_nix_lso_fmt_ipv4_frag_get(struct roc_nix *roc_nix);
 int __roc_api roc_nix_lso_custom_fmt_setup(struct roc_nix *roc_nix,
 					   struct nix_lso_format *fields,
 					   uint16_t nb_fields);
+int __roc_api roc_nix_lso_alt_flags_profile_setup(struct roc_nix *roc_nix,
+						  nix_lso_alt_flg_format_t *fmt);
 
 int __roc_api roc_nix_eeprom_info_get(struct roc_nix *roc_nix,
 				      struct roc_nix_eeprom_info *info);
@@ -952,8 +1085,11 @@ void __roc_api roc_nix_cq_head_tail_get(struct roc_nix *roc_nix, uint16_t qid,
 					uint32_t *head, uint32_t *tail);
 int __roc_api roc_nix_sq_init(struct roc_nix *roc_nix, struct roc_nix_sq *sq);
 int __roc_api roc_nix_sq_fini(struct roc_nix_sq *sq);
+int __roc_api roc_nix_sq_ena_dis(struct roc_nix_sq *sq, bool enable);
 void __roc_api roc_nix_sq_head_tail_get(struct roc_nix *roc_nix, uint16_t qid,
 					uint32_t *head, uint32_t *tail);
+int __roc_api roc_nix_sq_cnt_update(struct roc_nix_sq *sq, bool enable);
+int __roc_api roc_nix_sq_resize(struct roc_nix_sq *sq, uint32_t nb_desc);
 
 /* PTP */
 int __roc_api roc_nix_ptp_rx_ena_dis(struct roc_nix *roc_nix, int enable);
@@ -970,15 +1106,11 @@ bool __roc_api roc_nix_ptp_is_enable(struct roc_nix *roc_nix);
 /* VLAN */
 int __roc_api
 roc_nix_vlan_mcam_entry_read(struct roc_nix *roc_nix, uint32_t index,
-			     struct npc_mcam_read_entry_rsp **rsp);
-int __roc_api roc_nix_vlan_mcam_entry_write(struct roc_nix *roc_nix,
-					    uint32_t index,
-					    struct mcam_entry *entry,
+			     void **rsp);
+int __roc_api roc_nix_vlan_mcam_entry_write(struct roc_nix *roc_nix, uint32_t index, void *entry,
 					    uint8_t intf, uint8_t enable);
-int __roc_api roc_nix_vlan_mcam_entry_alloc_and_write(struct roc_nix *roc_nix,
-						      struct mcam_entry *entry,
-						      uint8_t intf,
-						      uint8_t priority,
+int __roc_api roc_nix_vlan_mcam_entry_alloc_and_write(struct roc_nix *roc_nix, void *entry,
+						      uint8_t intf, uint8_t priority,
 						      uint8_t ref_entry);
 int __roc_api roc_nix_vlan_mcam_entry_free(struct roc_nix *roc_nix,
 					   uint32_t index);
@@ -999,10 +1131,13 @@ int __roc_api roc_nix_mcast_mcam_entry_alloc(struct roc_nix *roc_nix,
 					     uint16_t index[]);
 int __roc_api roc_nix_mcast_mcam_entry_free(struct roc_nix *roc_nix,
 					    uint32_t index);
-int __roc_api roc_nix_mcast_mcam_entry_write(struct roc_nix *roc_nix,
-					     struct mcam_entry *entry,
-					     uint32_t index, uint8_t intf,
-					     uint64_t action);
-int __roc_api roc_nix_mcast_mcam_entry_ena_dis(struct roc_nix *roc_nix,
-					       uint32_t index, bool enable);
+int __roc_api roc_nix_mcast_mcam_entry_write(struct roc_nix *roc_nix, void *entry, uint32_t index,
+					     uint8_t intf, uint64_t action);
+int __roc_api roc_nix_mcast_mcam_entry_ena_dis(struct roc_nix *roc_nix, uint32_t index,
+					       bool enable);
+int __roc_api roc_nix_mcast_list_setup(struct mbox *mbox, uint8_t intf, int nb_entries,
+				       uint16_t *pf_funcs, uint16_t *channels, uint32_t *rqs,
+				       uint32_t *grp_index, uint32_t *start_index);
+int __roc_api roc_nix_mcast_list_free(struct mbox *mbox, uint32_t mcast_grp_index);
+int __roc_api roc_nix_max_rep_count(struct roc_nix *roc_nix);
 #endif /* _ROC_NIX_H_ */
