@@ -4,6 +4,28 @@
 #include "roc_api.h"
 #include "roc_priv.h"
 
+uint8_t
+npc_get_key_type(struct npc *npc, struct roc_npc_flow *flow)
+{
+	int i;
+
+	/* KEX is configured just for X2 */
+	if (npc->keyw[ROC_NPC_INTF_RX] == 1)
+		return NPC_CN20K_MCAM_KEY_X2;
+
+	/* KEX is configured just for X4 */
+	if (npc->keyw[ROC_NPC_INTF_RX] == 2)
+		return NPC_CN20K_MCAM_KEY_X4;
+
+	/* KEX is configured for both X2 and X4 */
+	/* Check mask for upper 256 bits. if mask is set, then it is X4 entry */
+	for (i = 4; i < ROC_NPC_MAX_MCAM_WIDTH_DWORDS; i++) {
+		if (flow->mcam_mask[i] != 0)
+			return NPC_CN20K_MCAM_KEY_X4;
+	}
+	return NPC_CN20K_MCAM_KEY_X2;
+}
+
 static void
 npc_prep_mcam_ldata(uint8_t *ptr, const uint8_t *data, int len)
 {
@@ -57,9 +79,19 @@ npc_set_hw_mask(struct npc_parse_item_info *info, struct npc_xtract_info *xinfo,
 		hw_mask[j] = 0xff;
 }
 
-void
-npc_get_hw_supp_mask(struct npc_parse_state *pst,
-		     struct npc_parse_item_info *info, int lid, int lt)
+static void
+npc_ipv6_hash_mask_get(struct npc_xtract_info *xinfo, struct npc_parse_item_info *info)
+{
+	int offset = 0;
+	uint8_t *hw_mask = info->hw_mask;
+
+	offset = xinfo->hdr_off - info->hw_hdr_len;
+	memset(&hw_mask[offset], 0xFF, NPC_HASH_FIELD_LEN);
+}
+
+static void
+npc_get_hw_supp_mask_legacy(struct npc_parse_state *pst, struct npc_parse_item_info *info, int lid,
+			    int lt)
 {
 	struct npc_xtract_info *xinfo, *lfinfo;
 	char *hw_mask = info->hw_mask;
@@ -71,8 +103,12 @@ npc_get_hw_supp_mask(struct npc_parse_state *pst,
 	xinfo = pst->npc->prx_dxcfg[intf][lid][lt].xtract;
 	memset(hw_mask, 0, info->len);
 
-	for (i = 0; i < NPC_MAX_LD; i++)
-		npc_set_hw_mask(info, &xinfo[i], hw_mask);
+	for (i = 0; i < NPC_MAX_LD; i++) {
+		if (pst->npc->hash_extract_cap && xinfo[i].use_hash)
+			npc_ipv6_hash_mask_get(&xinfo[i], info);
+		else
+			npc_set_hw_mask(info, &xinfo[i], hw_mask);
+	}
 
 	for (i = 0; i < NPC_MAX_LD; i++) {
 		if (xinfo[i].flags_enable == 0)
@@ -88,7 +124,41 @@ npc_get_hw_supp_mask(struct npc_parse_state *pst,
 	}
 }
 
-static inline int
+static void
+npc_get_hw_supp_mask_o20k(struct npc_parse_state *pst, struct npc_parse_item_info *info, int lid,
+			  int lt)
+{
+	struct npc_xtract_info *xinfo;
+	union npc_kex_ldata_flags_cfg *lid_info;
+	char *hw_mask = info->hw_mask;
+	int ex;
+	int intf;
+
+	intf = pst->nix_intf;
+	memset(hw_mask, 0, info->len);
+	for (ex = 0; ex < NPC_MAX_EXTRACTTORS; ex++) {
+		lid_info = &pst->npc->lid_cfg[intf][ex];
+		if (lid_info->s.lid != lid)
+			continue;
+		xinfo = &pst->npc->prx_dxcfg_cn20k[intf][ex][lt].xtract;
+
+		if (pst->npc->hash_extract_cap && xinfo->use_hash)
+			npc_ipv6_hash_mask_get(xinfo, info);
+		else
+			npc_set_hw_mask(info, xinfo, hw_mask);
+	}
+}
+
+void
+npc_get_hw_supp_mask(struct npc_parse_state *pst, struct npc_parse_item_info *info, int lid, int lt)
+{
+	if (!roc_model_is_cn20k())
+		return npc_get_hw_supp_mask_legacy(pst, info, lid, lt);
+	else
+		return npc_get_hw_supp_mask_o20k(pst, info, lid, lt);
+}
+
+inline int
 npc_mask_is_supported(const char *mask, const char *hw_mask, int len)
 {
 	/*
@@ -145,6 +215,9 @@ npc_parse_item_basic(const struct roc_npc_item_info *item,
 			info->mask = item->mask;
 	}
 
+	if (info->mask == NULL)
+		return NPC_ERR_INVALID_MASK;
+
 	/* mask specified must be subset of hw supported mask
 	 * mask | hw_mask == hw_mask
 	 */
@@ -166,6 +239,9 @@ npc_update_extraction_data(struct npc_parse_state *pst,
 	int len = 0;
 
 	x = xinfo;
+	if (x->len > NPC_MAX_EXTRACT_DATA_LEN)
+		return NPC_ERR_INVALID_SIZE;
+
 	len = x->len;
 	hdr_off = x->hdr_off;
 
@@ -202,10 +278,260 @@ npc_update_extraction_data(struct npc_parse_state *pst,
 	return 0;
 }
 
+static int
+npc_field_hash_secret_get(struct npc *npc, struct npc_hash_cfg *hash_cfg)
+{
+	struct npc_get_field_hash_info_req *req;
+	struct npc_get_field_hash_info_rsp *rsp;
+	struct mbox *mbox = mbox_get(npc->mbox);
+	int rc = 0;
+
+	req = mbox_alloc_msg_npc_get_field_hash_info(mbox);
+	if (req == NULL)
+		return -ENOSPC;
+	rc = mbox_process_msg(mbox, (void *)&rsp);
+	if (rc) {
+		plt_err("Failed to fetch field hash secret key");
+		goto done;
+	}
+
+	mbox_memcpy(hash_cfg->secret_key, rsp->secret_key, sizeof(rsp->secret_key));
+	mbox_memcpy(hash_cfg->hash_mask, rsp->hash_mask, sizeof(rsp->hash_mask));
+	mbox_memcpy(hash_cfg->hash_ctrl, rsp->hash_ctrl, sizeof(rsp->hash_ctrl));
+
+done:
+	mbox_put(mbox);
+	return rc;
+}
+
+static inline void
+be32_to_cpu_array(uint32_t *dst, const uint32_t *src, size_t len)
+{
+	size_t i;
+
+	for (i = 0; i < len; i++)
+		dst[i] = plt_be_to_cpu_32(src[i]);
+}
+
+static uint64_t
+npc_wide_extract(const uint64_t input[], size_t start_bit, size_t width_bits)
+{
+	const uint64_t mask = ~(uint64_t)((~(__uint128_t)0) << width_bits);
+	const size_t msb = start_bit + width_bits - 1;
+	const size_t lword = start_bit >> 6;
+	const size_t uword = msb >> 6;
+	size_t lbits;
+	uint64_t hi, lo;
+
+	if (lword == uword)
+		return (input[lword] >> (start_bit & 63)) & mask;
+
+	lbits = 64 - (start_bit & 63);
+	hi = input[uword];
+	lo = (input[lword] >> (start_bit & 63));
+	return ((hi << lbits) | lo) & mask;
+}
+
+static void
+npc_lshift_key(uint64_t *key, size_t key_bit_len)
+{
+	uint64_t prev_orig_word = 0;
+	uint64_t cur_orig_word = 0;
+	size_t extra = key_bit_len % 64;
+	size_t max_idx = key_bit_len / 64;
+	size_t i;
+
+	if (extra)
+		max_idx++;
+
+	for (i = 0; i < max_idx; i++) {
+		cur_orig_word = key[i];
+		key[i] = key[i] << 1;
+		key[i] |= ((prev_orig_word >> 63) & 0x1);
+		prev_orig_word = cur_orig_word;
+	}
+}
+
+static uint32_t
+npc_toeplitz_hash(const uint64_t *data, uint64_t *key, size_t data_bit_len, size_t key_bit_len)
+{
+	uint32_t hash_out = 0;
+	uint64_t temp_data = 0;
+	int i;
+
+	for (i = data_bit_len - 1; i >= 0; i--) {
+		temp_data = (data[i / 64]);
+		temp_data = temp_data >> (i % 64);
+		temp_data &= 0x1;
+		if (temp_data)
+			hash_out ^= (uint32_t)(npc_wide_extract(key, key_bit_len - 32, 32));
+
+		npc_lshift_key(key, key_bit_len);
+	}
+
+	return hash_out;
+}
+
+static uint32_t
+npc_field_hash_calc(uint64_t *ldata, struct npc_hash_cfg *hash_cfg, uint8_t intf, uint8_t hash_idx)
+{
+	uint64_t hash_key[3];
+	uint64_t data_padded[2];
+	uint32_t field_hash;
+
+	hash_key[0] = hash_cfg->secret_key[1] << 31;
+	hash_key[0] |= hash_cfg->secret_key[2];
+	hash_key[1] = hash_cfg->secret_key[1] >> 33;
+	hash_key[1] |= hash_cfg->secret_key[0] << 31;
+	hash_key[2] = hash_cfg->secret_key[0] >> 33;
+
+	data_padded[0] = hash_cfg->hash_mask[intf][hash_idx][0] & ldata[0];
+	data_padded[1] = hash_cfg->hash_mask[intf][hash_idx][1] & ldata[1];
+	field_hash = npc_toeplitz_hash(data_padded, hash_key, 128, 159);
+
+	field_hash &= hash_cfg->hash_ctrl[intf][hash_idx] >> 32;
+	field_hash |= hash_cfg->hash_ctrl[intf][hash_idx];
+	return field_hash;
+}
+
+static int
+npc_ipv6_field_hash_get(struct npc *npc, const uint32_t *ip6addr, uint8_t intf, int hash_idx,
+			uint32_t *hash)
+{
+#define IPV6_WORDS 4
+	uint32_t ipv6_addr[IPV6_WORDS];
+	struct npc_hash_cfg hash_cfg;
+	uint64_t ldata[2];
+	int rc = 0;
+
+	rc = npc_field_hash_secret_get(npc, &hash_cfg);
+	if (rc)
+		return -1;
+
+	be32_to_cpu_array(ipv6_addr, ip6addr, IPV6_WORDS);
+	ldata[0] = (uint64_t)ipv6_addr[2] << 32 | ipv6_addr[3];
+	ldata[1] = (uint64_t)ipv6_addr[0] << 32 | ipv6_addr[1];
+	*hash = npc_field_hash_calc(ldata, &hash_cfg, intf, hash_idx);
+
+	return 0;
+}
+
+static int
+npc_hash_field_get(struct npc_xtract_info *xinfo, const struct roc_npc_flow_item_ipv6 *ipv6_spec,
+		   const struct roc_npc_flow_item_ipv6 *ipv6_mask, uint8_t *hash_field)
+{
+	const uint8_t *ipv6_hdr_spec, *ipv6_hdr_mask;
+	struct roc_ipv6_hdr ipv6_buf;
+	int offset = xinfo->hdr_off;
+
+	memset(&ipv6_buf, 0, sizeof(ipv6_buf));
+
+	ipv6_hdr_spec = (const uint8_t *)&ipv6_spec->hdr;
+	ipv6_hdr_mask = (const uint8_t *)&ipv6_mask->hdr;
+
+	/* Check if mask is set for the field to be hashed */
+	if (memcmp(ipv6_hdr_mask + offset, &ipv6_buf, ROC_IPV6_ADDR_LEN) == 0)
+		return 0;
+
+	/* Extract the field to be hashed from item spec */
+	memcpy(hash_field, ipv6_hdr_spec + offset, ROC_IPV6_ADDR_LEN);
+	return 1;
+}
+
+static int
+npc_process_ipv6_field_hash_legacy(const struct roc_npc_flow_item_ipv6 *ipv6_spec,
+				   const struct roc_npc_flow_item_ipv6 *ipv6_mask,
+				   struct npc_parse_state *pst, uint8_t ltype)
+{
+	struct npc_lid_lt_xtract_info *lid_lt_xinfo;
+	uint8_t hash_field[ROC_IPV6_ADDR_LEN];
+	struct npc_xtract_info *xinfo;
+	uint32_t hash = 0, mask;
+	int intf, i, rc = 0;
+
+	memset(hash_field, 0, sizeof(hash_field));
+
+	intf = pst->nix_intf;
+	lid_lt_xinfo = &pst->npc->prx_dxcfg[intf][NPC_LID_LC][ltype];
+
+	for (i = 0; i < NPC_MAX_LD; i++) {
+		xinfo = &lid_lt_xinfo->xtract[i];
+		if (!xinfo->use_hash)
+			continue;
+
+		rc = npc_hash_field_get(xinfo, ipv6_spec, ipv6_mask, hash_field);
+		if (rc == 0)
+			continue;
+
+		rc = npc_ipv6_field_hash_get(pst->npc, (const uint32_t *)hash_field, intf, i,
+					     &hash);
+		if (rc)
+			return rc;
+
+		mask = GENMASK(31, 0);
+		memcpy(pst->mcam_mask + xinfo->key_off, (uint8_t *)&mask, 4);
+		memcpy(pst->mcam_data + xinfo->key_off, (uint8_t *)&hash, 4);
+	}
+
+	return 0;
+}
+
+static int
+npc_process_ipv6_field_hash_o20k(const struct roc_npc_flow_item_ipv6 *ipv6_spec,
+				 const struct roc_npc_flow_item_ipv6 *ipv6_mask,
+				 struct npc_parse_state *pst, uint8_t ltype)
+{
+	struct npc_lid_lt_xtract_info_cn20k *lid_lt_xinfo;
+	union npc_kex_ldata_flags_cfg *lid_cfg;
+	uint8_t hash_field[ROC_IPV6_ADDR_LEN];
+	struct npc_xtract_info *xinfo;
+	uint32_t hash = 0, mask;
+	int intf, i, rc = 0;
+
+	memset(hash_field, 0, sizeof(hash_field));
+
+	intf = pst->nix_intf;
+	for (i = 0; i < NPC_MAX_EXTRACTTORS; i++) {
+		lid_cfg = &pst->npc->lid_cfg[intf][i];
+		if (lid_cfg->s.lid != NPC_LID_LC)
+			continue;
+		lid_lt_xinfo = &pst->npc->prx_dxcfg_cn20k[intf][NPC_LID_LC][ltype];
+
+		xinfo = &lid_lt_xinfo->xtract;
+		if (!xinfo->use_hash)
+			continue;
+
+		rc = npc_hash_field_get(xinfo, ipv6_spec, ipv6_mask, hash_field);
+		if (rc == 0)
+			continue;
+
+		rc = npc_ipv6_field_hash_get(pst->npc, (const uint32_t *)hash_field, intf, i,
+					     &hash);
+		if (rc)
+			return rc;
+
+		mask = GENMASK(31, 0);
+		memcpy(pst->mcam_mask + xinfo->key_off, (uint8_t *)&mask, 4);
+		memcpy(pst->mcam_data + xinfo->key_off, (uint8_t *)&hash, 4);
+	}
+
+	return 0;
+}
+
 int
-npc_update_parse_state(struct npc_parse_state *pst,
-		       struct npc_parse_item_info *info, int lid, int lt,
-		       uint8_t flags)
+npc_process_ipv6_field_hash(const struct roc_npc_flow_item_ipv6 *ipv6_spec,
+			    const struct roc_npc_flow_item_ipv6 *ipv6_mask,
+			    struct npc_parse_state *pst, uint8_t ltype)
+{
+	if (!roc_model_is_cn20k())
+		return npc_process_ipv6_field_hash_legacy(ipv6_spec, ipv6_mask, pst, ltype);
+	else
+		return npc_process_ipv6_field_hash_o20k(ipv6_spec, ipv6_mask, pst, ltype);
+}
+
+static int
+npc_update_parse_state_legacy(struct npc_parse_state *pst, struct npc_parse_item_info *info,
+			      int lid, int lt, uint8_t flags)
 {
 	struct npc_lid_lt_xtract_info *xinfo;
 	struct roc_npc_flow_dump_data *dump;
@@ -226,6 +552,8 @@ npc_update_parse_state(struct npc_parse_state *pst,
 		goto done;
 
 	for (i = 0; i < NPC_MAX_LD; i++) {
+		if (xinfo->xtract[i].use_hash)
+			continue;
 		rc = npc_update_extraction_data(pst, info, &xinfo->xtract[i]);
 		if (rc != 0)
 			return rc;
@@ -234,13 +562,14 @@ npc_update_parse_state(struct npc_parse_state *pst,
 	for (i = 0; i < NPC_MAX_LD; i++) {
 		if (xinfo->xtract[i].flags_enable == 0)
 			continue;
+		if (xinfo->xtract[i].use_hash)
+			continue;
 
 		lf_cfg = pst->npc->prx_lfcfg[i].i;
 		if (lf_cfg == lid) {
 			for (j = 0; j < NPC_MAX_LFL; j++) {
 				lfinfo = pst->npc->prx_fxcfg[intf][i][j].xtract;
-				rc = npc_update_extraction_data(pst, info,
-								&lfinfo[0]);
+				rc = npc_update_extraction_data(pst, info, &lfinfo[0]);
 				if (rc != 0)
 					return rc;
 
@@ -259,51 +588,141 @@ done:
 }
 
 static int
-npc_initialise_mcam_entry(struct npc *npc, struct roc_npc_flow *flow,
-			  int mcam_id)
+npc_update_parse_state_o20(struct npc_parse_state *pst, struct npc_parse_item_info *info, int lid,
+			   int lt, uint8_t flags)
 {
-	struct npc_mcam_write_entry_req *req;
-	struct npc_mcam_write_entry_rsq *rsp;
-	int rc = 0, idx;
+	struct npc_lid_lt_xtract_info_cn20k *xinfo;
+	union npc_kex_ldata_flags_cfg *lid_cfg;
+	struct roc_npc_flow_dump_data *dump;
+	int intf;
+	int i, rc = 0;
 
-	req = mbox_alloc_msg_npc_mcam_write_entry(npc->mbox);
-	if (req == NULL)
-		return -ENOSPC;
-	req->set_cntr = 0;
-	req->cntr = 0;
-	req->entry = mcam_id;
+	pst->layer_mask |= lid;
+	pst->lt[lid] = lt;
+	pst->flags[lid] = flags;
 
-	req->intf = (flow->nix_intf == NIX_INTF_RX) ? NPC_MCAM_RX : NPC_MCAM_TX;
-	req->enable_entry = 1;
-	req->entry_data.action = flow->npc_action;
-	req->entry_data.vtag_action = flow->vtag_action;
+	intf = pst->nix_intf;
+	if (info->spec == NULL)
+		goto done;
 
-	for (idx = 0; idx < ROC_NPC_MAX_MCAM_WIDTH_DWORDS; idx++) {
-		req->entry_data.kw[idx] = 0x0;
-		req->entry_data.kw_mask[idx] = 0x0;
+	for (i = 0; i < NPC_MAX_EXTRACTTORS; i++) {
+		lid_cfg = &pst->npc->lid_cfg[intf][i];
+		if (lid_cfg->s.lid != lid)
+			continue;
+		xinfo = &pst->npc->prx_dxcfg_cn20k[intf][i][lt];
+		if (xinfo->is_terminating)
+			pst->terminate = 1;
+
+		if (xinfo->xtract.use_hash)
+			continue;
+		rc = npc_update_extraction_data(pst, info, &xinfo->xtract);
+		if (rc != 0)
+			return rc;
 	}
 
-	if (flow->nix_intf == NIX_INTF_RX) {
-		req->entry_data.kw[0] |= (uint64_t)npc->channel;
-		req->entry_data.kw_mask[0] |= (BIT_ULL(12) - 1);
-	} else {
-		uint16_t pf_func = (flow->npc_action >> 4) & 0xffff;
-
-		pf_func = plt_cpu_to_be_16(pf_func);
-		req->entry_data.kw[0] |= ((uint64_t)pf_func << 32);
-		req->entry_data.kw_mask[0] |= ((uint64_t)0xffff << 32);
-	}
-
-	rc = mbox_process_msg(npc->mbox, (void *)&rsp);
-	if (rc != 0) {
-		plt_err("npc: mcam initialisation write failed");
-		return rc;
-	}
+done:
+	dump = &pst->flow->dump_data[pst->flow->num_patterns++];
+	dump->lid = lid;
+	dump->ltype = lt;
+	pst->pattern++;
 	return 0;
 }
 
-static int
-npc_shift_mcam_entry(struct mbox *mbox, uint16_t old_ent, uint16_t new_ent)
+int
+npc_update_parse_state(struct npc_parse_state *pst, struct npc_parse_item_info *info, int lid,
+		       int lt, uint8_t flags)
+{
+	if (roc_model_is_cn20k())
+		return npc_update_parse_state_o20(pst, info, lid, lt, flags);
+	else
+		return npc_update_parse_state_legacy(pst, info, lid, lt, flags);
+}
+
+int
+npc_mcam_init(struct npc *npc, struct roc_npc_flow *flow, int mcam_id)
+{
+	struct msg_rsp *rsp;
+	struct mbox *mbox = mbox_get(npc->mbox);
+	int rc = 0, idx;
+
+	if (roc_model_is_cn20k()) {
+		struct npc_cn20k_mcam_write_entry_req *req;
+
+		req = mbox_alloc_msg_npc_cn20k_mcam_write_entry(mbox);
+		if (req == NULL) {
+			rc = -ENOSPC;
+			goto exit;
+		}
+		req->cntr = 0;
+		req->entry = mcam_id;
+
+		req->intf = (flow->nix_intf == NIX_INTF_RX) ? NPC_MCAM_RX : NPC_MCAM_TX;
+		req->enable_entry = 1;
+		req->entry_data.action = flow->npc_action;
+		req->entry_data.vtag_action = flow->vtag_action;
+
+		for (idx = 0; idx < ROC_NPC_MAX_MCAM_WIDTH_DWORDS; idx++) {
+			req->entry_data.kw[idx] = 0x0;
+			req->entry_data.kw_mask[idx] = 0x0;
+		}
+
+		if (flow->nix_intf == NIX_INTF_RX) {
+			req->entry_data.kw[0] |= (uint64_t)npc->channel;
+			req->entry_data.kw_mask[0] |= (BIT_ULL(12) - 1);
+		} else {
+			uint16_t pf_func = (flow->npc_action >> 4) & 0xffff;
+
+			pf_func = plt_cpu_to_be_16(pf_func);
+			req->entry_data.kw[0] |= ((uint64_t)pf_func << 32);
+			req->entry_data.kw_mask[0] |= ((uint64_t)0xffff << 32);
+		}
+	} else {
+		struct npc_mcam_write_entry_req *req;
+
+		req = mbox_alloc_msg_npc_mcam_write_entry(mbox);
+		if (req == NULL) {
+			rc = -ENOSPC;
+			goto exit;
+		}
+		req->set_cntr = 0;
+		req->cntr = 0;
+		req->entry = mcam_id;
+
+		req->intf = (flow->nix_intf == NIX_INTF_RX) ? NPC_MCAM_RX : NPC_MCAM_TX;
+		req->enable_entry = 1;
+		req->entry_data.action = flow->npc_action;
+		req->entry_data.vtag_action = flow->vtag_action;
+
+		for (idx = 0; idx < ROC_NPC_MAX_MCAM_WIDTH_DWORDS; idx++) {
+			req->entry_data.kw[idx] = 0x0;
+			req->entry_data.kw_mask[idx] = 0x0;
+		}
+
+		if (flow->nix_intf == NIX_INTF_RX) {
+			req->entry_data.kw[0] |= (uint64_t)npc->channel;
+			req->entry_data.kw_mask[0] |= (BIT_ULL(12) - 1);
+		} else {
+			uint16_t pf_func = (flow->npc_action >> 4) & 0xffff;
+
+			pf_func = plt_cpu_to_be_16(pf_func);
+			req->entry_data.kw[0] |= ((uint64_t)pf_func << 32);
+			req->entry_data.kw_mask[0] |= ((uint64_t)0xffff << 32);
+		}
+	}
+
+	rc = mbox_process_msg(mbox, (void *)&rsp);
+	if (rc != 0) {
+		plt_err("npc: mcam initialisation write failed");
+		goto exit;
+	}
+	rc = 0;
+exit:
+	mbox_put(mbox);
+	return rc;
+}
+
+int
+npc_mcam_move(struct mbox *mbox, uint16_t old_ent, uint16_t new_ent)
 {
 	struct npc_mcam_shift_entry_req *req;
 	struct npc_mcam_shift_entry_rsp *rsp;
@@ -312,18 +731,21 @@ npc_shift_mcam_entry(struct mbox *mbox, uint16_t old_ent, uint16_t new_ent)
 	/* Old entry is disabled & it's contents are moved to new_entry,
 	 * new entry is enabled finally.
 	 */
-	req = mbox_alloc_msg_npc_mcam_shift_entry(mbox);
+	req = mbox_alloc_msg_npc_mcam_shift_entry(mbox_get(mbox));
 	if (req == NULL)
-		return rc;
+		goto exit;
 	req->curr_entry[0] = old_ent;
 	req->new_entry[0] = new_ent;
 	req->shift_count = 1;
 
 	rc = mbox_process_msg(mbox, (void *)&rsp);
 	if (rc)
-		return rc;
+		goto exit;
 
-	return 0;
+	rc = 0;
+exit:
+	mbox_put(mbox);
+	return rc;
 }
 
 enum SHIFT_DIR {
@@ -359,12 +781,10 @@ npc_slide_mcam_entries(struct mbox *mbox, struct npc *npc, int prio,
 			 * Initialise and enable before moving an entry into
 			 * this mcam.
 			 */
-			rc = npc_initialise_mcam_entry(npc, curr->flow,
-						       to_mcam_id);
+			rc = npc_mcam_init(npc, curr->flow, to_mcam_id);
 			if (rc)
 				return rc;
-			rc = npc_shift_mcam_entry(mbox, from_mcam_id,
-						  to_mcam_id);
+			rc = npc_mcam_move(mbox, from_mcam_id, to_mcam_id);
 			if (rc)
 				return rc;
 			curr->flow->mcam_id = to_mcam_id;
@@ -555,33 +975,37 @@ npc_insert_into_flow_list(struct npc *npc, struct npc_prio_flow_entry *entry)
 }
 
 static int
-npc_allocate_mcam_entry(struct mbox *mbox, int prio,
-			struct npc_mcam_alloc_entry_rsp *rsp_local,
-			int ref_entry)
+npc_allocate_mcam_entry(struct mbox *mbox, int prio, struct npc_mcam_alloc_entry_rsp *rsp_local,
+			int ref_entry, uint8_t kw_type)
 {
-	struct npc_mcam_alloc_entry_rsp *rsp_cmd;
-	struct npc_mcam_alloc_entry_req *req;
 	struct npc_mcam_alloc_entry_rsp *rsp;
+	struct npc_mcam_alloc_entry_req *req;
+
 	int rc = -ENOSPC;
 
-	req = mbox_alloc_msg_npc_mcam_alloc_entry(mbox);
+	req = mbox_alloc_msg_npc_mcam_alloc_entry(mbox_get(mbox));
 	if (req == NULL)
-		return rc;
+		goto exit;
 	req->contig = 1;
 	req->count = 1;
-	req->priority = prio;
+	req->ref_priority = prio;
 	req->ref_entry = ref_entry;
+	req->kw_type = kw_type;
 
-	rc = mbox_process_msg(mbox, (void *)&rsp_cmd);
+	rc = mbox_process_msg(mbox, (void *)&rsp);
 	if (rc)
-		return rc;
+		goto exit;
 
-	if (!rsp_cmd->count)
-		return -ENOSPC;
+	if (!rsp->count) {
+		rc = -ENOSPC;
+		goto exit;
+	}
 
-	memcpy(rsp_local, rsp_cmd, sizeof(*rsp));
-
-	return 0;
+	mbox_memcpy(rsp_local, rsp, sizeof(*rsp));
+	rc = 0;
+exit:
+	mbox_put(mbox);
+	return rc;
 }
 
 static void
@@ -621,8 +1045,7 @@ npc_find_mcam_ref_entry(struct roc_npc_flow *flow, struct npc *npc, int *prio,
 }
 
 static int
-npc_alloc_mcam_by_ref_entry(struct mbox *mbox, struct roc_npc_flow *flow,
-			    struct npc *npc,
+npc_alloc_mcam_by_ref_entry(struct mbox *mbox, struct roc_npc_flow *flow, struct npc *npc,
 			    struct npc_mcam_alloc_entry_rsp *rsp_local)
 {
 	int prio, ref_entry = 0, rc = 0, dir = NPC_MCAM_LOWER_PRIO;
@@ -630,9 +1053,9 @@ npc_alloc_mcam_by_ref_entry(struct mbox *mbox, struct roc_npc_flow *flow,
 
 retry:
 	npc_find_mcam_ref_entry(flow, npc, &prio, &ref_entry, dir);
-	rc = npc_allocate_mcam_entry(mbox, prio, rsp_local, ref_entry);
+	rc = npc_allocate_mcam_entry(mbox, prio, rsp_local, ref_entry, flow->key_type);
 	if (rc && !retry_done) {
-		plt_info(
+		plt_npc_dbg(
 			"npc: Failed to allocate lower priority entry. Retrying for higher priority");
 
 		dir = NPC_MCAM_HIGHER_PRIO;
@@ -646,32 +1069,42 @@ retry:
 }
 
 int
-npc_get_free_mcam_entry(struct mbox *mbox, struct roc_npc_flow *flow,
-			struct npc *npc)
+npc_get_free_mcam_entry(struct mbox *mbox, struct roc_npc_flow *flow, struct npc *npc)
 {
 	struct npc_mcam_alloc_entry_rsp rsp_local;
 	struct npc_prio_flow_entry *new_entry;
 	int rc = 0;
 
-	rc = npc_alloc_mcam_by_ref_entry(mbox, flow, npc, &rsp_local);
-
-	if (rc)
-		return rc;
-
 	new_entry = plt_zmalloc(sizeof(*new_entry), 0);
 	if (!new_entry)
 		return -ENOSPC;
 
-	new_entry->flow = flow;
+	if (roc_model_is_cn20k()) {
+		rc = npc_allocate_mcam_entry(mbox, NPC_MCAM_ANY_PRIO, &rsp_local, 0,
+					     flow->key_type);
+		if (rc) {
+			plt_npc_dbg("npc: failed to allocate MCAM entry.");
+			return rc;
+		}
 
-	plt_info("npc: kernel allocated MCAM entry %d", rsp_local.entry);
+		new_entry->flow = flow;
+	} else {
+		rc = npc_alloc_mcam_by_ref_entry(mbox, flow, npc, &rsp_local);
 
-	rc = npc_sort_mcams_by_user_prio_level(mbox, new_entry, npc,
-					       &rsp_local);
-	if (rc)
-		goto err;
+		if (rc)
+			return rc;
+
+		new_entry->flow = flow;
+
+		plt_npc_dbg("kernel allocated MCAM entry %d", rsp_local.entry);
+
+		rc = npc_sort_mcams_by_user_prio_level(mbox, new_entry, npc, &rsp_local);
+		if (rc)
+			goto err;
+
+		plt_npc_dbg("allocated MCAM entry after sorting %d", rsp_local.entry);
+	}
 
-	plt_info("npc: allocated MCAM entry after sorting %d", rsp_local.entry);
 	flow->mcam_id = rsp_local.entry;
 	npc_insert_into_flow_list(npc, new_entry);
 
