@@ -1,13 +1,15 @@
 /* SPDX-License-Identifier: BSD-3-Clause
- * Copyright 2018 NXP
+ * Copyright 2018-2023 NXP
  */
 
+#include <eal_export.h>
 #include <rte_memory.h>
 
 #include "dpaax_iova_table.h"
 #include "dpaax_logs.h"
 
 /* Global table reference */
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_iova_table_p)
 struct dpaax_iova_table *dpaax_iova_table_p;
 
 static int dpaax_handle_memevents(void);
@@ -139,10 +141,12 @@ read_memory_node(unsigned int *count)
 	}
 
 	DPAAX_DEBUG("Device-tree memory node data:");
-	do {
+
+	while (j > 0) {
+		--j;
 		DPAAX_DEBUG("    %08" PRIx64 " %08zu",
 			    nodes[j].addr, nodes[j].len);
-	} while (--j);
+	}
 
 cleanup:
 	close(fd);
@@ -151,6 +155,7 @@ out:
 	return nodes;
 }
 
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_iova_table_populate)
 int
 dpaax_iova_table_populate(void)
 {
@@ -252,18 +257,17 @@ dpaax_iova_table_populate(void)
 	return 0;
 }
 
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_iova_table_depopulate)
 void
 dpaax_iova_table_depopulate(void)
 {
-	if (dpaax_iova_table_p == NULL)
-		return;
-
-	rte_free(dpaax_iova_table_p->entries);
+	rte_free(dpaax_iova_table_p);
 	dpaax_iova_table_p = NULL;
 
-	DPAAX_DEBUG("IOVA Table cleanedup");
+	DPAAX_DEBUG("IOVA Table cleaned");
 }
 
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_iova_table_update)
 int
 dpaax_iova_table_update(phys_addr_t paddr, void *vaddr, size_t length)
 {
@@ -350,6 +354,7 @@ dpaax_iova_table_update(phys_addr_t paddr, void *vaddr, size_t length)
  * Dump the table, with its entries, on screen. Only works in Debug Mode
  * Not for weak hearted - the tables can get quite large
  */
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_iova_table_dump)
 void
 dpaax_iova_table_dump(void)
 {
@@ -462,4 +467,5 @@ dpaax_handle_memevents(void)
 					       dpaax_memevent_cb, NULL);
 }
 
+RTE_EXPORT_INTERNAL_SYMBOL(dpaax_logger)
 RTE_LOG_REGISTER_DEFAULT(dpaax_logger, ERR);
