@@ -12,7 +12,7 @@
 #include <cryptodev_pmd.h>
 #include <rte_crypto.h>
 #include <rte_cryptodev.h>
-#include <rte_bus_vdev.h>
+#include <bus_vdev_driver.h>
 #include <rte_malloc.h>
 #include <rte_security_driver.h>
 #include <rte_hexdump.h>
@@ -53,10 +53,10 @@ static enum sec_driver_state_e g_driver_state = SEC_DRIVER_STATE_IDLE;
 static int g_job_rings_no;
 static int g_job_rings_max;
 
-struct sec_outring_entry {
+struct __rte_packed_begin sec_outring_entry {
 	phys_addr_t desc;	/* Pointer to completed descriptor */
 	uint32_t status;	/* Status for completed descriptor */
-} __rte_packed;
+} __rte_packed_end;
 
 /* virtual address conversin when mempool support is available for ctx */
 static inline phys_addr_t
@@ -123,9 +123,10 @@ void caam_jr_stats_get(struct rte_cryptodev *dev,
 		stats->dequeued_count += qp[i]->rx_pkts;
 		stats->enqueue_err_count += qp[i]->tx_errs;
 		stats->dequeue_err_count += qp[i]->rx_errs;
-		CAAM_JR_INFO("extra stats:\n\tRX Poll ERR = %" PRIu64
-			     "\n\tTX Ring Full = %" PRIu64,
-			     qp[i]->rx_poll_err,
+		CAAM_JR_INFO("extra stats:");
+		CAAM_JR_INFO("\tRX Poll ERR = %" PRIu64,
+			     qp[i]->rx_poll_err);
+		CAAM_JR_INFO("\tTX Ring Full = %" PRIu64,
 			     qp[i]->tx_ring_full);
 	}
 }
@@ -309,7 +310,7 @@ caam_jr_prep_cdb(struct caam_jr_session *ses)
 
 	cdb = caam_jr_dma_mem_alloc(L1_CACHE_BYTES, sizeof(struct sec_cdb));
 	if (!cdb) {
-		CAAM_JR_ERR("failed to allocate memory for cdb\n");
+		CAAM_JR_ERR("failed to allocate memory for cdb");
 		return -1;
 	}
 
@@ -587,8 +588,8 @@ hw_poll_job_ring(struct sec_job_ring_t *job_ring,
 		/* todo check if it is false alarm no desc present */
 		if (!current_desc_addr) {
 			false_alarm++;
-			printf("false alarm %" PRIu64 "real %" PRIu64
-				" sec_err =0x%x cidx Index =0%d\n",
+			CAAM_JR_ERR("false alarm %" PRIu64 "real %" PRIu64
+				" sec_err =0x%x cidx Index =0%d",
 				false_alarm, real_poll,
 				sec_error_code, job_ring->cidx);
 			rte_panic("CAAM JR descriptor NULL");
@@ -606,7 +607,7 @@ hw_poll_job_ring(struct sec_job_ring_t *job_ring,
 		/*TODO for multiple ops, packets*/
 		ctx = container_of(current_desc, struct caam_jr_op_ctx, jobdes);
 		if (unlikely(sec_error_code)) {
-			CAAM_JR_ERR("desc at cidx %d generated error 0x%x\n",
+			CAAM_JR_ERR("desc at cidx %d generated error 0x%x",
 				job_ring->cidx, sec_error_code);
 			hw_handle_job_ring_error(job_ring, sec_error_code);
 			//todo improve with exact errors
@@ -631,15 +632,15 @@ hw_poll_job_ring(struct sec_job_ring_t *job_ring,
 
 			if (ctx->op->sym->m_dst) {
 				/*TODO check for ip header or other*/
-				ip4_hdr = (struct ip *)
-				rte_pktmbuf_mtod(ctx->op->sym->m_dst, char*);
+				ip4_hdr = rte_pktmbuf_mtod(ctx->op->sym->m_dst,
+							   struct ip *);
 				ctx->op->sym->m_dst->pkt_len =
 					rte_be_to_cpu_16(ip4_hdr->ip_len);
 				ctx->op->sym->m_dst->data_len =
 					rte_be_to_cpu_16(ip4_hdr->ip_len);
 			} else {
-				ip4_hdr = (struct ip *)
-				rte_pktmbuf_mtod(ctx->op->sym->m_src, char*);
+				ip4_hdr = rte_pktmbuf_mtod(ctx->op->sym->m_src,
+							   struct ip *);
 				ctx->op->sym->m_src->pkt_len =
 					rte_be_to_cpu_16(ip4_hdr->ip_len);
 				ctx->op->sym->m_src->data_len =
@@ -1356,14 +1357,10 @@ caam_jr_enqueue_op(struct rte_crypto_op *op, struct caam_jr_qp *qp)
 
 	switch (op->sess_type) {
 	case RTE_CRYPTO_OP_WITH_SESSION:
-		ses = (struct caam_jr_session *)
-		get_sym_session_private_data(op->sym->session,
-					cryptodev_driver_id);
+		ses = CRYPTODEV_GET_SYM_SESS_PRIV(op->sym->session);
 		break;
 	case RTE_CRYPTO_OP_SECURITY_SESSION:
-		ses = (struct caam_jr_session *)
-			get_sec_session_private_data(
-					op->sym->sec_session);
+		ses = SECURITY_GET_SESS_PRIV(op->sym->session);
 		break;
 	default:
 		CAAM_JR_DP_ERR("sessionless crypto op not supported");
@@ -1372,7 +1369,7 @@ caam_jr_enqueue_op(struct rte_crypto_op *op, struct caam_jr_qp *qp)
 	}
 
 	if (unlikely(!ses->qp || ses->qp != qp)) {
-		CAAM_JR_DP_DEBUG("Old:sess->qp=%p New qp = %p\n", ses->qp, qp);
+		CAAM_JR_DP_DEBUG("Old:sess->qp=%p New qp = %p", ses->qp, qp);
 		ses->qp = qp;
 		caam_jr_prep_cdb(ses);
 	}
@@ -1414,9 +1411,9 @@ err1:
 			rte_pktmbuf_mtod(op->sym->m_src, void *),
 			rte_pktmbuf_data_len(op->sym->m_src));
 
-	printf("\n JD before conversion\n");
+	fprintf(stdout, "\n JD before conversion\n");
 	for (i = 0; i < 12; i++)
-		printf("\n 0x%08x", ctx->jobdes.desc[i]);
+		fprintf(stdout, "\n 0x%08x", ctx->jobdes.desc[i]);
 #endif
 
 	CAAM_JR_DP_DEBUG("Jr[%p] pi[%d] ci[%d].Before sending desc",
@@ -1558,7 +1555,7 @@ caam_jr_cipher_init(struct rte_cryptodev *dev __rte_unused,
 	session->cipher_key.data = rte_zmalloc(NULL, xform->cipher.key.length,
 					       RTE_CACHE_LINE_SIZE);
 	if (session->cipher_key.data == NULL && xform->cipher.key.length > 0) {
-		CAAM_JR_ERR("No Memory for cipher key\n");
+		CAAM_JR_ERR("No Memory for cipher key");
 		return -ENOMEM;
 	}
 	session->cipher_key.length = xform->cipher.key.length;
@@ -1580,7 +1577,7 @@ caam_jr_auth_init(struct rte_cryptodev *dev __rte_unused,
 	session->auth_key.data = rte_zmalloc(NULL, xform->auth.key.length,
 					     RTE_CACHE_LINE_SIZE);
 	if (session->auth_key.data == NULL && xform->auth.key.length > 0) {
-		CAAM_JR_ERR("No Memory for auth key\n");
+		CAAM_JR_ERR("No Memory for auth key");
 		return -ENOMEM;
 	}
 	session->auth_key.length = xform->auth.key.length;
@@ -1606,7 +1603,7 @@ caam_jr_aead_init(struct rte_cryptodev *dev __rte_unused,
 	session->aead_key.data = rte_zmalloc(NULL, xform->aead.key.length,
 					     RTE_CACHE_LINE_SIZE);
 	if (session->aead_key.data == NULL && xform->aead.key.length > 0) {
-		CAAM_JR_ERR("No Memory for aead key\n");
+		CAAM_JR_ERR("No Memory for aead key");
 		return -ENOMEM;
 	}
 	session->aead_key.length = xform->aead.key.length;
@@ -1692,54 +1689,38 @@ err1:
 }
 
 static int
-caam_jr_sym_session_configure(struct rte_cryptodev *dev,
+caam_jr_sym_session_configure(struct rte_cryptodev *dev __rte_unused,
 			      struct rte_crypto_sym_xform *xform,
-			      struct rte_cryptodev_sym_session *sess,
-			      struct rte_mempool *mempool)
+			      struct rte_cryptodev_sym_session *sess)
 {
 	void *sess_private_data;
 	int ret;
 
 	PMD_INIT_FUNC_TRACE();
-
-	if (rte_mempool_get(mempool, &sess_private_data)) {
-		CAAM_JR_ERR("Couldn't get object from session mempool");
-		return -ENOMEM;
-	}
-
+	sess_private_data = CRYPTODEV_GET_SYM_SESS_PRIV(sess);
 	memset(sess_private_data, 0, sizeof(struct caam_jr_session));
 	ret = caam_jr_set_session_parameters(dev, xform, sess_private_data);
 	if (ret != 0) {
 		CAAM_JR_ERR("failed to configure session parameters");
 		/* Return session to mempool */
-		rte_mempool_put(mempool, sess_private_data);
 		return ret;
 	}
 
-	set_sym_session_private_data(sess, dev->driver_id, sess_private_data);
-
 	return 0;
 }
 
 /* Clear the memory of session so it doesn't leave key material behind */
 static void
-caam_jr_sym_session_clear(struct rte_cryptodev *dev,
+caam_jr_sym_session_clear(struct rte_cryptodev *dev __rte_unused,
 		struct rte_cryptodev_sym_session *sess)
 {
-	uint8_t index = dev->driver_id;
-	void *sess_priv = get_sym_session_private_data(sess, index);
-	struct caam_jr_session *s = (struct caam_jr_session *)sess_priv;
+	struct caam_jr_session *s = CRYPTODEV_GET_SYM_SESS_PRIV(sess);
 
 	PMD_INIT_FUNC_TRACE();
 
-	if (sess_priv) {
-		struct rte_mempool *sess_mp = rte_mempool_from_obj(sess_priv);
-
+	if (s) {
 		rte_free(s->cipher_key.data);
 		rte_free(s->auth_key.data);
-		memset(s, 0, sizeof(struct caam_jr_session));
-		set_sym_session_private_data(sess, index, NULL);
-		rte_mempool_put(sess_mp, sess_priv);
 	}
 }
 
@@ -1775,7 +1756,7 @@ caam_jr_set_ipsec_session(__rte_unused struct rte_cryptodev *dev,
 					       RTE_CACHE_LINE_SIZE);
 	if (session->cipher_key.data == NULL &&
 			cipher_xform->key.length > 0) {
-		CAAM_JR_ERR("No Memory for cipher key\n");
+		CAAM_JR_ERR("No Memory for cipher key");
 		return -ENOMEM;
 	}
 
@@ -1785,7 +1766,7 @@ caam_jr_set_ipsec_session(__rte_unused struct rte_cryptodev *dev,
 					RTE_CACHE_LINE_SIZE);
 	if (session->auth_key.data == NULL &&
 			auth_xform->key.length > 0) {
-		CAAM_JR_ERR("No Memory for auth key\n");
+		CAAM_JR_ERR("No Memory for auth key");
 		rte_free(session->cipher_key.data);
 		return -ENOMEM;
 	}
@@ -1830,11 +1811,11 @@ caam_jr_set_ipsec_session(__rte_unused struct rte_cryptodev *dev,
 	case RTE_CRYPTO_AUTH_KASUMI_F9:
 	case RTE_CRYPTO_AUTH_AES_CBC_MAC:
 	case RTE_CRYPTO_AUTH_ZUC_EIA3:
-		CAAM_JR_ERR("Crypto: Unsupported auth alg %u\n",
+		CAAM_JR_ERR("Crypto: Unsupported auth alg %u",
 			auth_xform->algo);
 		goto out;
 	default:
-		CAAM_JR_ERR("Crypto: Undefined Auth specified %u\n",
+		CAAM_JR_ERR("Crypto: Undefined Auth specified %u",
 			auth_xform->algo);
 		goto out;
 	}
@@ -1854,11 +1835,11 @@ caam_jr_set_ipsec_session(__rte_unused struct rte_cryptodev *dev,
 	case RTE_CRYPTO_CIPHER_3DES_ECB:
 	case RTE_CRYPTO_CIPHER_AES_ECB:
 	case RTE_CRYPTO_CIPHER_KASUMI_F8:
-		CAAM_JR_ERR("Crypto: Unsupported Cipher alg %u\n",
+		CAAM_JR_ERR("Crypto: Unsupported Cipher alg %u",
 			cipher_xform->algo);
 		goto out;
 	default:
-		CAAM_JR_ERR("Crypto: Undefined Cipher specified %u\n",
+		CAAM_JR_ERR("Crypto: Undefined Cipher specified %u",
 			cipher_xform->algo);
 		goto out;
 	}
@@ -1918,18 +1899,12 @@ out:
 static int
 caam_jr_security_session_create(void *dev,
 				struct rte_security_session_conf *conf,
-				struct rte_security_session *sess,
-				struct rte_mempool *mempool)
+				struct rte_security_session *sess)
 {
-	void *sess_private_data;
+	void *sess_private_data = SECURITY_GET_SESS_PRIV(sess);
 	struct rte_cryptodev *cdev = (struct rte_cryptodev *)dev;
 	int ret;
 
-	if (rte_mempool_get(mempool, &sess_private_data)) {
-		CAAM_JR_ERR("Couldn't get object from session mempool");
-		return -ENOMEM;
-	}
-
 	switch (conf->protocol) {
 	case RTE_SECURITY_PROTOCOL_IPSEC:
 		ret = caam_jr_set_ipsec_session(cdev, conf,
@@ -1942,13 +1917,8 @@ caam_jr_security_session_create(void *dev,
 	}
 	if (ret != 0) {
 		CAAM_JR_ERR("failed to configure session parameters");
-		/* Return session to mempool */
-		rte_mempool_put(mempool, sess_private_data);
-		return ret;
 	}
 
-	set_sec_session_private_data(sess, sess_private_data);
-
 	return ret;
 }
 
@@ -1958,22 +1928,21 @@ caam_jr_security_session_destroy(void *dev __rte_unused,
 				 struct rte_security_session *sess)
 {
 	PMD_INIT_FUNC_TRACE();
-	void *sess_priv = get_sec_session_private_data(sess);
-
-	struct caam_jr_session *s = (struct caam_jr_session *)sess_priv;
-
-	if (sess_priv) {
-		struct rte_mempool *sess_mp = rte_mempool_from_obj(sess_priv);
+	struct caam_jr_session *s = SECURITY_GET_SESS_PRIV(sess);
 
+	if (s) {
 		rte_free(s->cipher_key.data);
 		rte_free(s->auth_key.data);
-		memset(sess, 0, sizeof(struct caam_jr_session));
-		set_sec_session_private_data(sess, NULL);
-		rte_mempool_put(sess_mp, sess_priv);
+		memset(s, 0, sizeof(struct caam_jr_session));
 	}
 	return 0;
 }
 
+static unsigned int
+caam_jr_security_session_get_size(void *device __rte_unused)
+{
+	return sizeof(struct caam_jr_session);
+}
 
 static int
 caam_jr_dev_configure(struct rte_cryptodev *dev,
@@ -1994,7 +1963,7 @@ caam_jr_dev_configure(struct rte_cryptodev *dev,
 						NULL, NULL, NULL, NULL,
 						SOCKET_ID_ANY, 0);
 		if (!internals->ctx_pool) {
-			CAAM_JR_ERR("%s create failed\n", str);
+			CAAM_JR_ERR("%s create failed", str);
 			return -ENOMEM;
 		}
 	} else
@@ -2068,6 +2037,7 @@ static struct rte_cryptodev_ops caam_jr_ops = {
 static struct rte_security_ops caam_jr_security_ops = {
 	.session_create = caam_jr_security_session_create,
 	.session_update = NULL,
+	.session_get_size = caam_jr_security_session_get_size,
 	.session_stats_get = NULL,
 	.session_destroy = caam_jr_security_session_destroy,
 	.set_pkt_metadata = NULL,
@@ -2211,7 +2181,7 @@ init_job_ring(void *reg_base_addr, int irq_id)
 		}
 	}
 	if (job_ring == NULL) {
-		CAAM_JR_ERR("No free job ring\n");
+		CAAM_JR_ERR("No free job ring");
 		return NULL;
 	}
 
@@ -2332,7 +2302,7 @@ caam_jr_dev_init(const char *name,
 						job_ring->uio_fd);
 
 	if (!dev->data->dev_private) {
-		CAAM_JR_ERR("Ring memory allocation failed\n");
+		CAAM_JR_ERR("Ring memory allocation failed");
 		goto cleanup2;
 	}
 
@@ -2365,7 +2335,7 @@ caam_jr_dev_init(const char *name,
 	security_instance = rte_malloc("caam_jr",
 				sizeof(struct rte_security_ctx), 0);
 	if (security_instance == NULL) {
-		CAAM_JR_ERR("memory allocation failed\n");
+		CAAM_JR_ERR("memory allocation failed");
 		//todo error handling.
 		goto cleanup2;
 	}
@@ -2377,7 +2347,7 @@ caam_jr_dev_init(const char *name,
 
 	rte_cryptodev_pmd_probing_finish(dev);
 
-	RTE_LOG(INFO, PMD, "%s cryptodev init\n", dev->data->name);
+	CAAM_JR_INFO("%s cryptodev init", dev->data->name);
 
 	return 0;
 
@@ -2417,8 +2387,7 @@ cryptodev_caam_jr_probe(struct rte_vdev_device *vdev)
 
 	ret = of_init();
 	if (ret) {
-		RTE_LOG(ERR, PMD,
-		"of_init failed\n");
+		CAAM_JR_ERR("of_init failed");
 		return -EINVAL;
 	}
 	/* if sec device version is not configured */
@@ -2438,8 +2407,7 @@ cryptodev_caam_jr_probe(struct rte_vdev_device *vdev)
 	}
 #ifdef RTE_LIBRTE_PMD_CAAM_JR_BE
 	if (rta_get_sec_era() > RTA_SEC_ERA_8) {
-		RTE_LOG(ERR, PMD,
-		"CAAM is compiled in BE mode for device with sec era > 8???\n");
+		CAAM_JR_ERR("CAAM is compiled in BE mode for device with sec era > 8");
 		return -EINVAL;
 	}
 #endif
