@@ -12,7 +12,7 @@ struct ioat_dmadev {
 	struct rte_dma_vchan_conf qcfg;
 	struct rte_dma_stats stats;
 
-	volatile uint16_t *doorbell __rte_cache_aligned;
+	volatile alignas(RTE_CACHE_LINE_SIZE) uint16_t *doorbell;
 	phys_addr_t status_addr;
 	phys_addr_t ring_addr;
 
@@ -25,7 +25,7 @@ struct ioat_dmadev {
 	unsigned int failure; /* Used to store chanerr for error handling. */
 
 	/* To report completions, the device will write status back here. */
-	volatile uint64_t status __rte_cache_aligned;
+	volatile alignas(RTE_CACHE_LINE_SIZE) uint64_t status;
 
 	/* Pointer to the register bar. */
 	volatile struct ioat_registers *regs;
@@ -35,13 +35,14 @@ struct ioat_dmadev {
 };
 
 extern int ioat_pmd_logtype;
+#define RTE_LOGTYPE_IOAT_PMD ioat_pmd_logtype
 
-#define IOAT_PMD_LOG(level, fmt, args...) rte_log(RTE_LOG_ ## level, \
-		ioat_pmd_logtype, "IOAT: %s(): " fmt "\n", __func__, ##args)
+#define IOAT_PMD_LOG(level, ...) \
+	RTE_LOG_LINE_PREFIX(level, IOAT_PMD, "%s(): ", __func__, __VA_ARGS__)
 
-#define IOAT_PMD_DEBUG(fmt, args...)  IOAT_PMD_LOG(DEBUG, fmt, ## args)
-#define IOAT_PMD_INFO(fmt, args...)   IOAT_PMD_LOG(INFO, fmt, ## args)
-#define IOAT_PMD_ERR(fmt, args...)    IOAT_PMD_LOG(ERR, fmt, ## args)
-#define IOAT_PMD_WARN(fmt, args...)   IOAT_PMD_LOG(WARNING, fmt, ## args)
+#define IOAT_PMD_DEBUG(fmt, ...)  IOAT_PMD_LOG(DEBUG, fmt, ## __VA_ARGS__)
+#define IOAT_PMD_INFO(fmt, ...)   IOAT_PMD_LOG(INFO, fmt, ## __VA_ARGS__)
+#define IOAT_PMD_ERR(fmt, ...)    IOAT_PMD_LOG(ERR, fmt, ## __VA_ARGS__)
+#define IOAT_PMD_WARN(fmt, ...)   IOAT_PMD_LOG(WARNING, fmt, ## __VA_ARGS__)
 
 #endif /* _IOAT_INTERNAL_H_ */
