@@ -8,11 +8,15 @@
 #error "roc_api.h is included"
 #endif
 
+#if !defined(CNXK_DIS_TMPLT_FUNC)
+
 #define R(name, flags)                                                         \
 	SSO_CMN_DEQ_BURST(cn10k_sso_hws_deq_burst_##name,                      \
 			  cn10k_sso_hws_deq_##name, flags)                     \
 	SSO_CMN_DEQ_BURST(cn10k_sso_hws_reas_deq_burst_##name,                 \
-			  cn10k_sso_hws_reas_deq_##name, flags | NIX_RX_REAS_F_)
+			  cn10k_sso_hws_reas_deq_##name, flags | NIX_RX_REAS_F)
 
-NIX_RX_FASTPATH_MODES_32_47
+NIX_RX_FASTPATH_MODES_48_63
 #undef R
+
+#endif
