@@ -6,9 +6,13 @@
 #include "cnxk_eventdev.h"
 #include "cnxk_worker.h"
 
+#if !defined(CNXK_DIS_TMPLT_FUNC)
+
 #define R(name, flags)                                                         \
 	SSO_CMN_DEQ_SEG_BURST(cn9k_sso_hws_dual_deq_seg_burst_##name,          \
 			      cn9k_sso_hws_dual_deq_seg_##name, flags)
 
-NIX_RX_FASTPATH_MODES_64_79
+NIX_RX_FASTPATH_MODES_32_47
 #undef R
+
+#endif
