@@ -4,8 +4,12 @@
 
 #include "cn9k_worker.h"
 
+#if !defined(CNXK_DIS_TMPLT_FUNC)
+
 #define T(name, sz, flags)                                                     \
 	SSO_DUAL_TX(cn9k_sso_hws_dual_tx_adptr_enq_##name, sz, flags)
 
-NIX_TX_FASTPATH_MODES_32_47
+NIX_TX_FASTPATH_MODES_96_111
 #undef T
+
+#endif
