@@ -359,13 +359,13 @@ atl_rx_init(struct rte_eth_dev *eth_dev)
 		buff_size = RTE_ALIGN_FLOOR(buff_size, 1024);
 		if (buff_size > HW_ATL_B0_RXD_BUF_SIZE_MAX) {
 			PMD_INIT_LOG(WARNING,
-				"Port %d queue %d: mem pool buff size is too big\n",
+				"Port %d queue %d: mem pool buff size is too big",
 				rxq->port_id, rxq->queue_id);
 			buff_size = HW_ATL_B0_RXD_BUF_SIZE_MAX;
 		}
 		if (buff_size < 1024) {
 			PMD_INIT_LOG(ERR,
-				"Port %d queue %d: mem pool buff size is too small\n",
+				"Port %d queue %d: mem pool buff size is too small",
 				rxq->port_id, rxq->queue_id);
 			return -EINVAL;
 		}
@@ -688,7 +688,7 @@ atl_txq_info_get(struct rte_eth_dev *dev, uint16_t queue_id,
 
 /* Return Rx queue avail count */
 
-uint32_t
+int
 atl_rx_queue_count(void *rx_queue)
 {
 	struct atl_rx_queue *rxq;
@@ -1094,7 +1094,7 @@ err_stop:
 	 * register.
 	 * Update the RDT with the value of the last processed RX descriptor
 	 * minus 1, to guarantee that the RDT register is never equal to the
-	 * RDH register, which creates a "full" ring situtation from the
+	 * RDH register, which creates a "full" ring situation from the
 	 * hardware point of view...
 	 */
 	nb_hold = (uint16_t)(nb_hold + rxq->nb_rx_hold);
@@ -1127,10 +1127,9 @@ atl_xmit_cleanup(struct atl_tx_queue *txq)
 	if (txq != NULL) {
 		sw_ring = txq->sw_ring;
 		int head = txq->tx_head;
-		int cnt;
-		int i;
+		int cnt = head;
 
-		for (i = 0, cnt = head; ; i++) {
+		while (true) {
 			txd = &txq->hw_ring[cnt];
 
 			if (txd->dd)
