@@ -16,13 +16,13 @@
 #include <rte_atomic.h>
 #include <rte_branch_prediction.h>
 #include <rte_pci.h>
-#include <rte_bus_pci.h>
+#include <bus_pci_driver.h>
 #include <rte_ether.h>
 #include <rte_common.h>
 #include <rte_cycles.h>
 #include <rte_spinlock.h>
 #include <rte_byteorder.h>
-#include <rte_dev.h>
+#include <dev_driver.h>
 #include <rte_memory.h>
 #include <rte_eal.h>
 #include <rte_io.h>
@@ -79,7 +79,8 @@ static void avp_dev_rx_queue_release(struct rte_eth_dev *dev, uint16_t qid);
 static void avp_dev_tx_queue_release(struct rte_eth_dev *dev, uint16_t qid);
 
 static int avp_dev_stats_get(struct rte_eth_dev *dev,
-			      struct rte_eth_stats *stats);
+			      struct rte_eth_stats *stats,
+			      struct eth_queue_stats *qstats);
 static int avp_dev_stats_reset(struct rte_eth_dev *dev);
 
 
@@ -154,7 +155,7 @@ static const struct eth_dev_ops avp_eth_dev_ops = {
  * Defines the AVP device attributes which are attached to an RTE ethernet
  * device
  */
-struct avp_dev {
+struct __rte_cache_aligned avp_dev {
 	uint32_t magic; /**< Memory validation marker */
 	uint64_t device_id; /**< Unique system identifier */
 	struct rte_ether_addr ethaddr; /**< Host specified MAC address */
@@ -190,12 +191,12 @@ struct avp_dev {
 	void *sync_addr; /**< Req/Resp Mem address */
 	void *host_mbuf_addr; /**< (host) MBUF pool start address */
 	void *mbuf_addr; /**< MBUF pool start address */
-} __rte_cache_aligned;
+};
 
 /* RTE ethernet private data */
-struct avp_adapter {
+struct __rte_cache_aligned avp_adapter {
 	struct avp_dev avp;
-} __rte_cache_aligned;
+};
 
 
 /* 32-bit MMIO register write */
@@ -240,18 +241,18 @@ avp_dev_process_request(struct avp_dev *avp, struct rte_avp_request *request)
 	unsigned int count;
 	int ret;
 
-	PMD_DRV_LOG(DEBUG, "Sending request %u to host\n", request->req_id);
+	PMD_DRV_LOG_LINE(DEBUG, "Sending request %u to host", request->req_id);
 
 	request->result = -ENOTSUP;
 
 	/* Discard any stale responses before starting a new request */
 	while (avp_fifo_get(avp->resp_q, (void **)&resp_addr, 1))
-		PMD_DRV_LOG(DEBUG, "Discarding stale response\n");
+		PMD_DRV_LOG_LINE(DEBUG, "Discarding stale response");
 
 	rte_memcpy(avp->sync_addr, request, sizeof(*request));
 	count = avp_fifo_put(avp->req_q, &avp->host_sync_addr, 1);
 	if (count < 1) {
-		PMD_DRV_LOG(ERR, "Cannot send request %u to host\n",
+		PMD_DRV_LOG_LINE(ERR, "Cannot send request %u to host",
 			    request->req_id);
 		ret = -EBUSY;
 		goto done;
@@ -268,7 +269,7 @@ avp_dev_process_request(struct avp_dev *avp, struct rte_avp_request *request)
 		}
 
 		if (retry == 0) {
-			PMD_DRV_LOG(ERR, "Timeout while waiting for a response for %u\n",
+			PMD_DRV_LOG_LINE(ERR, "Timeout while waiting for a response for %u",
 				    request->req_id);
 			ret = -ETIME;
 			goto done;
@@ -278,7 +279,7 @@ avp_dev_process_request(struct avp_dev *avp, struct rte_avp_request *request)
 	/* retrieve the response */
 	count = avp_fifo_get(avp->resp_q, (void **)&resp_addr, 1);
 	if ((count != 1) || (resp_addr != avp->host_sync_addr)) {
-		PMD_DRV_LOG(ERR, "Invalid response from host, count=%u resp=%p host_sync_addr=%p\n",
+		PMD_DRV_LOG_LINE(ERR, "Invalid response from host, count=%u resp=%p host_sync_addr=%p",
 			    count, resp_addr, avp->host_sync_addr);
 		ret = -ENODATA;
 		goto done;
@@ -288,7 +289,7 @@ avp_dev_process_request(struct avp_dev *avp, struct rte_avp_request *request)
 	rte_memcpy(request, avp->sync_addr, sizeof(*request));
 	ret = 0;
 
-	PMD_DRV_LOG(DEBUG, "Result %d received for request %u\n",
+	PMD_DRV_LOG_LINE(DEBUG, "Result %d received for request %u",
 		    request->result, request->req_id);
 
 done:
@@ -383,7 +384,7 @@ avp_dev_translate_address(struct rte_eth_dev *eth_dev,
 			offset += (host_phys_addr - map->phys_addr);
 			addr = RTE_PTR_ADD(addr, (uintptr_t)offset);
 
-			PMD_DRV_LOG(DEBUG, "Translating host physical 0x%" PRIx64 " to guest virtual 0x%p\n",
+			PMD_DRV_LOG_LINE(DEBUG, "Translating host physical 0x%" PRIx64 " to guest virtual 0x%p",
 				    host_phys_addr, addr);
 
 			return addr;
@@ -425,7 +426,7 @@ avp_dev_check_regions(struct rte_eth_dev *eth_dev)
 		if ((resource->phys_addr == 0) || (resource->len == 0))
 			continue;
 
-		PMD_DRV_LOG(DEBUG, "resource[%u]: phys=0x%" PRIx64 " len=%" PRIu64 " addr=%p\n",
+		PMD_DRV_LOG_LINE(DEBUG, "resource[%u]: phys=0x%" PRIx64 " len=%" PRIu64 " addr=%p",
 			    i, resource->phys_addr,
 			    resource->len, resource->addr);
 
@@ -434,7 +435,7 @@ avp_dev_check_regions(struct rte_eth_dev *eth_dev)
 			memmap = (struct rte_avp_memmap_info *)resource->addr;
 			if ((memmap->magic != RTE_AVP_MEMMAP_MAGIC) ||
 			    (memmap->version != RTE_AVP_MEMMAP_VERSION)) {
-				PMD_DRV_LOG(ERR, "Invalid memmap magic 0x%08x and version %u\n",
+				PMD_DRV_LOG_LINE(ERR, "Invalid memmap magic 0x%08x and version %u",
 					    memmap->magic, memmap->version);
 				return -EINVAL;
 			}
@@ -444,7 +445,7 @@ avp_dev_check_regions(struct rte_eth_dev *eth_dev)
 			info = (struct rte_avp_device_info *)resource->addr;
 			if ((info->magic != RTE_AVP_DEVICE_MAGIC) ||
 			    avp_dev_version_check(info->version)) {
-				PMD_DRV_LOG(ERR, "Invalid device info magic 0x%08x or version 0x%08x > 0x%08x\n",
+				PMD_DRV_LOG_LINE(ERR, "Invalid device info magic 0x%08x or version 0x%08x > 0x%08x",
 					    info->magic, info->version,
 					    AVP_DPDK_DRIVER_VERSION);
 				return -EINVAL;
@@ -454,7 +455,7 @@ avp_dev_check_regions(struct rte_eth_dev *eth_dev)
 		case RTE_AVP_PCI_MEMORY_BAR:
 		case RTE_AVP_PCI_MMIO_BAR:
 			if (resource->addr == NULL) {
-				PMD_DRV_LOG(ERR, "Missing address space for BAR%u\n",
+				PMD_DRV_LOG_LINE(ERR, "Missing address space for BAR%u",
 					    i);
 				return -EINVAL;
 			}
@@ -476,13 +477,13 @@ avp_dev_detach(struct rte_eth_dev *eth_dev)
 	struct avp_dev *avp = AVP_DEV_PRIVATE_TO_HW(eth_dev->data->dev_private);
 	int ret;
 
-	PMD_DRV_LOG(NOTICE, "Detaching port %u from AVP device 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(NOTICE, "Detaching port %u from AVP device 0x%" PRIx64 "",
 		    eth_dev->data->port_id, avp->device_id);
 
 	rte_spinlock_lock(&avp->lock);
 
 	if (avp->flags & AVP_F_DETACHED) {
-		PMD_DRV_LOG(NOTICE, "port %u already detached\n",
+		PMD_DRV_LOG_LINE(NOTICE, "port %u already detached",
 			    eth_dev->data->port_id);
 		ret = 0;
 		goto unlock;
@@ -491,7 +492,7 @@ avp_dev_detach(struct rte_eth_dev *eth_dev)
 	/* shutdown the device first so the host stops sending us packets. */
 	ret = avp_dev_ctrl_shutdown(eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to send/recv shutdown to host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to send/recv shutdown to host, ret=%d",
 			    ret);
 		avp->flags &= ~AVP_F_DETACHED;
 		goto unlock;
@@ -540,7 +541,7 @@ _avp_set_rx_queue_mappings(struct rte_eth_dev *eth_dev, uint16_t rx_queue_id)
 		rxq->queue_limit = rxq->queue_base + queue_count - 1;
 	}
 
-	PMD_DRV_LOG(DEBUG, "rxq %u at %p base %u limit %u\n",
+	PMD_DRV_LOG_LINE(DEBUG, "rxq %u at %p base %u limit %u",
 		    rx_queue_id, rxq, rxq->queue_base, rxq->queue_limit);
 
 	rxq->queue_id = rxq->queue_base;
@@ -574,7 +575,7 @@ _avp_set_queue_counts(struct rte_eth_dev *eth_dev)
 	avp->num_rx_queues = RTE_MAX(host_info->min_rx_queues,
 				     eth_dev->data->nb_rx_queues);
 
-	PMD_DRV_LOG(DEBUG, "Requesting %u Tx and %u Rx queues from host\n",
+	PMD_DRV_LOG_LINE(DEBUG, "Requesting %u Tx and %u Rx queues from host",
 		    avp->num_tx_queues, avp->num_rx_queues);
 }
 
@@ -586,13 +587,13 @@ avp_dev_attach(struct rte_eth_dev *eth_dev)
 	unsigned int i;
 	int ret;
 
-	PMD_DRV_LOG(NOTICE, "Attaching port %u to AVP device 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(NOTICE, "Attaching port %u to AVP device 0x%" PRIx64 "",
 		    eth_dev->data->port_id, avp->device_id);
 
 	rte_spinlock_lock(&avp->lock);
 
 	if (!(avp->flags & AVP_F_DETACHED)) {
-		PMD_DRV_LOG(NOTICE, "port %u already attached\n",
+		PMD_DRV_LOG_LINE(NOTICE, "port %u already attached",
 			    eth_dev->data->port_id);
 		ret = 0;
 		goto unlock;
@@ -611,7 +612,7 @@ avp_dev_attach(struct rte_eth_dev *eth_dev)
 	 */
 	ret = avp_dev_create(RTE_ETH_DEV_TO_PCI(eth_dev), eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to re-create AVP device, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to re-create AVP device, ret=%d",
 			    ret);
 		goto unlock;
 	}
@@ -643,7 +644,7 @@ avp_dev_attach(struct rte_eth_dev *eth_dev)
 
 		ret = avp_dev_ctrl_set_config(eth_dev, &config);
 		if (ret < 0) {
-			PMD_DRV_LOG(ERR, "Config request failed by host, ret=%d\n",
+			PMD_DRV_LOG_LINE(ERR, "Config request failed by host, ret=%d",
 				    ret);
 			goto unlock;
 		}
@@ -692,7 +693,7 @@ avp_dev_interrupt_handler(void *data)
 			ret = avp_dev_attach(eth_dev);
 			break;
 		default:
-			PMD_DRV_LOG(ERR, "unexpected migration status, status=%u\n",
+			PMD_DRV_LOG_LINE(ERR, "unexpected migration status, status=%u",
 				    value);
 			ret = -EINVAL;
 		}
@@ -703,17 +704,17 @@ avp_dev_interrupt_handler(void *data)
 			    RTE_PTR_ADD(registers,
 					RTE_AVP_MIGRATION_ACK_OFFSET));
 
-		PMD_DRV_LOG(NOTICE, "AVP migration interrupt handled\n");
+		PMD_DRV_LOG_LINE(NOTICE, "AVP migration interrupt handled");
 	}
 
 	if (status & ~RTE_AVP_MIGRATION_INTERRUPT_MASK)
-		PMD_DRV_LOG(WARNING, "AVP unexpected interrupt, status=0x%08x\n",
+		PMD_DRV_LOG_LINE(WARNING, "AVP unexpected interrupt, status=0x%08x",
 			    status);
 
 	/* re-enable UIO interrupt handling */
 	ret = rte_intr_ack(pci_dev->intr_handle);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to re-enable UIO interrupts, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to re-enable UIO interrupts, ret=%d",
 			    ret);
 		/* continue */
 	}
@@ -732,7 +733,7 @@ avp_dev_enable_interrupts(struct rte_eth_dev *eth_dev)
 	/* enable UIO interrupt handling */
 	ret = rte_intr_enable(pci_dev->intr_handle);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to enable UIO interrupts, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to enable UIO interrupts, ret=%d",
 			    ret);
 		return ret;
 	}
@@ -761,7 +762,7 @@ avp_dev_disable_interrupts(struct rte_eth_dev *eth_dev)
 	/* enable UIO interrupt handling */
 	ret = rte_intr_disable(pci_dev->intr_handle);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to disable UIO interrupts, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to disable UIO interrupts, ret=%d",
 			    ret);
 		return ret;
 	}
@@ -780,7 +781,7 @@ avp_dev_setup_interrupts(struct rte_eth_dev *eth_dev)
 					 avp_dev_interrupt_handler,
 					 (void *)eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to register UIO interrupt callback, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to register UIO interrupt callback, ret=%d",
 			    ret);
 		return ret;
 	}
@@ -826,7 +827,7 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 
 	resource = &pci_dev->mem_resource[RTE_AVP_PCI_DEVICE_BAR];
 	if (resource->addr == NULL) {
-		PMD_DRV_LOG(ERR, "BAR%u is not mapped\n",
+		PMD_DRV_LOG_LINE(ERR, "BAR%u is not mapped",
 			    RTE_AVP_PCI_DEVICE_BAR);
 		return -EFAULT;
 	}
@@ -834,22 +835,22 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 
 	if ((host_info->magic != RTE_AVP_DEVICE_MAGIC) ||
 		avp_dev_version_check(host_info->version)) {
-		PMD_DRV_LOG(ERR, "Invalid AVP PCI device, magic 0x%08x version 0x%08x > 0x%08x\n",
+		PMD_DRV_LOG_LINE(ERR, "Invalid AVP PCI device, magic 0x%08x version 0x%08x > 0x%08x",
 			    host_info->magic, host_info->version,
 			    AVP_DPDK_DRIVER_VERSION);
 		return -EINVAL;
 	}
 
-	PMD_DRV_LOG(DEBUG, "AVP host device is v%u.%u.%u\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host device is v%u.%u.%u",
 		    RTE_AVP_GET_RELEASE_VERSION(host_info->version),
 		    RTE_AVP_GET_MAJOR_VERSION(host_info->version),
 		    RTE_AVP_GET_MINOR_VERSION(host_info->version));
 
-	PMD_DRV_LOG(DEBUG, "AVP host supports %u to %u TX queue(s)\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host supports %u to %u TX queue(s)",
 		    host_info->min_tx_queues, host_info->max_tx_queues);
-	PMD_DRV_LOG(DEBUG, "AVP host supports %u to %u RX queue(s)\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host supports %u to %u RX queue(s)",
 		    host_info->min_rx_queues, host_info->max_rx_queues);
-	PMD_DRV_LOG(DEBUG, "AVP host supports features 0x%08x\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host supports features 0x%08x",
 		    host_info->features);
 
 	if (avp->magic != AVP_ETHDEV_MAGIC) {
@@ -876,7 +877,7 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 
 		/* TODO... requires validation of host values */
 		if ((host_info->features & avp->features) != avp->features) {
-			PMD_DRV_LOG(ERR, "AVP host features mismatched; 0x%08x, host=0x%08x\n",
+			PMD_DRV_LOG_LINE(ERR, "AVP host features mismatched; 0x%08x, host=0x%08x",
 				    avp->features, host_info->features);
 			/* this should not be possible; continue for now */
 		}
@@ -886,9 +887,9 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 	avp->device_id = host_info->device_id;
 
 	/* translate incoming host addresses to guest address space */
-	PMD_DRV_LOG(DEBUG, "AVP first host tx queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP first host tx queue at 0x%" PRIx64 "",
 		    host_info->tx_phys);
-	PMD_DRV_LOG(DEBUG, "AVP first host alloc queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP first host alloc queue at 0x%" PRIx64 "",
 		    host_info->alloc_phys);
 	for (i = 0; i < avp->max_tx_queues; i++) {
 		avp->tx_q[i] = avp_dev_translate_address(eth_dev,
@@ -898,9 +899,9 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 			host_info->alloc_phys + (i * host_info->alloc_size));
 	}
 
-	PMD_DRV_LOG(DEBUG, "AVP first host rx queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP first host rx queue at 0x%" PRIx64 "",
 		    host_info->rx_phys);
-	PMD_DRV_LOG(DEBUG, "AVP first host free queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP first host free queue at 0x%" PRIx64 "",
 		    host_info->free_phys);
 	for (i = 0; i < avp->max_rx_queues; i++) {
 		avp->rx_q[i] = avp_dev_translate_address(eth_dev,
@@ -909,13 +910,13 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 			host_info->free_phys + (i * host_info->free_size));
 	}
 
-	PMD_DRV_LOG(DEBUG, "AVP host request queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host request queue at 0x%" PRIx64 "",
 		    host_info->req_phys);
-	PMD_DRV_LOG(DEBUG, "AVP host response queue at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host response queue at 0x%" PRIx64 "",
 		    host_info->resp_phys);
-	PMD_DRV_LOG(DEBUG, "AVP host sync address at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host sync address at 0x%" PRIx64 "",
 		    host_info->sync_phys);
-	PMD_DRV_LOG(DEBUG, "AVP host mbuf address at 0x%" PRIx64 "\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host mbuf address at 0x%" PRIx64 "",
 		    host_info->mbuf_phys);
 	avp->req_q = avp_dev_translate_address(eth_dev, host_info->req_phys);
 	avp->resp_q = avp_dev_translate_address(eth_dev, host_info->resp_phys);
@@ -935,7 +936,7 @@ avp_dev_create(struct rte_pci_device *pci_dev,
 	 * store the maximum packet length that is supported by the host.
 	 */
 	avp->max_rx_pkt_len = host_info->max_rx_pkt_len;
-	PMD_DRV_LOG(DEBUG, "AVP host max receive packet length is %u\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP host max receive packet length is %u",
 				host_info->max_rx_pkt_len);
 
 	return 0;
@@ -966,7 +967,7 @@ eth_avp_dev_init(struct rte_eth_dev *eth_dev)
 		 * be valid.
 		 */
 		if (eth_dev->data->scattered_rx) {
-			PMD_DRV_LOG(NOTICE, "AVP device configured for chained mbufs\n");
+			PMD_DRV_LOG_LINE(NOTICE, "AVP device configured for chained mbufs");
 			eth_dev->rx_pkt_burst = avp_recv_scattered_pkts;
 			eth_dev->tx_pkt_burst = avp_xmit_scattered_pkts;
 		}
@@ -978,14 +979,14 @@ eth_avp_dev_init(struct rte_eth_dev *eth_dev)
 
 	/* Check current migration status */
 	if (avp_dev_migration_pending(eth_dev)) {
-		PMD_DRV_LOG(ERR, "VM live migration operation in progress\n");
+		PMD_DRV_LOG_LINE(ERR, "VM live migration operation in progress");
 		return -EBUSY;
 	}
 
 	/* Check BAR resources */
 	ret = avp_dev_check_regions(eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to validate BAR resources, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to validate BAR resources, ret=%d",
 			    ret);
 		return ret;
 	}
@@ -993,14 +994,14 @@ eth_avp_dev_init(struct rte_eth_dev *eth_dev)
 	/* Enable interrupts */
 	ret = avp_dev_setup_interrupts(eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to enable interrupts, ret=%d\n", ret);
+		PMD_DRV_LOG_LINE(ERR, "Failed to enable interrupts, ret=%d", ret);
 		return ret;
 	}
 
 	/* Handle each subtype */
 	ret = avp_dev_create(pci_dev, eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to create device, ret=%d\n", ret);
+		PMD_DRV_LOG_LINE(ERR, "Failed to create device, ret=%d", ret);
 		return ret;
 	}
 
@@ -1008,7 +1009,7 @@ eth_avp_dev_init(struct rte_eth_dev *eth_dev)
 	eth_dev->data->mac_addrs = rte_zmalloc("avp_ethdev",
 					RTE_ETHER_ADDR_LEN, 0);
 	if (eth_dev->data->mac_addrs == NULL) {
-		PMD_DRV_LOG(ERR, "Failed to allocate %d bytes needed to store MAC addresses\n",
+		PMD_DRV_LOG_LINE(ERR, "Failed to allocate %d bytes needed to store MAC addresses",
 			    RTE_ETHER_ADDR_LEN);
 		return -ENOMEM;
 	}
@@ -1101,7 +1102,7 @@ avp_dev_rx_queue_setup(struct rte_eth_dev *eth_dev,
 	struct avp_queue *rxq;
 
 	if (rx_queue_id >= eth_dev->data->nb_rx_queues) {
-		PMD_DRV_LOG(ERR, "RX queue id is out of range: rx_queue_id=%u, nb_rx_queues=%u\n",
+		PMD_DRV_LOG_LINE(ERR, "RX queue id is out of range: rx_queue_id=%u, nb_rx_queues=%u",
 			    rx_queue_id, eth_dev->data->nb_rx_queues);
 		return -EINVAL;
 	}
@@ -1116,14 +1117,14 @@ avp_dev_rx_queue_setup(struct rte_eth_dev *eth_dev,
 
 	if (avp_dev_enable_scattered(eth_dev, avp)) {
 		if (!eth_dev->data->scattered_rx) {
-			PMD_DRV_LOG(NOTICE, "AVP device configured for chained mbufs\n");
+			PMD_DRV_LOG_LINE(NOTICE, "AVP device configured for chained mbufs");
 			eth_dev->data->scattered_rx = 1;
 			eth_dev->rx_pkt_burst = avp_recv_scattered_pkts;
 			eth_dev->tx_pkt_burst = avp_xmit_scattered_pkts;
 		}
 	}
 
-	PMD_DRV_LOG(DEBUG, "AVP max_rx_pkt_len=(%u,%u) mbuf_size=(%u,%u)\n",
+	PMD_DRV_LOG_LINE(DEBUG, "AVP max_rx_pkt_len=(%u,%u) mbuf_size=(%u,%u)",
 		    avp->max_rx_pkt_len,
 		    eth_dev->data->mtu + RTE_ETHER_HDR_LEN + RTE_ETHER_CRC_LEN,
 		    avp->host_mbuf_size,
@@ -1133,7 +1134,7 @@ avp_dev_rx_queue_setup(struct rte_eth_dev *eth_dev,
 	rxq = rte_zmalloc_socket("ethdev RX queue", sizeof(struct avp_queue),
 				 RTE_CACHE_LINE_SIZE, socket_id);
 	if (rxq == NULL) {
-		PMD_DRV_LOG(ERR, "Failed to allocate new Rx queue object\n");
+		PMD_DRV_LOG_LINE(ERR, "Failed to allocate new Rx queue object");
 		return -ENOMEM;
 	}
 
@@ -1145,7 +1146,7 @@ avp_dev_rx_queue_setup(struct rte_eth_dev *eth_dev,
 	/* setup the queue receive mapping for the current queue. */
 	_avp_set_rx_queue_mappings(eth_dev, rx_queue_id);
 
-	PMD_DRV_LOG(DEBUG, "Rx queue %u setup at %p\n", rx_queue_id, rxq);
+	PMD_DRV_LOG_LINE(DEBUG, "Rx queue %u setup at %p", rx_queue_id, rxq);
 
 	(void)nb_rx_desc;
 	(void)rx_conf;
@@ -1163,7 +1164,7 @@ avp_dev_tx_queue_setup(struct rte_eth_dev *eth_dev,
 	struct avp_queue *txq;
 
 	if (tx_queue_id >= eth_dev->data->nb_tx_queues) {
-		PMD_DRV_LOG(ERR, "TX queue id is out of range: tx_queue_id=%u, nb_tx_queues=%u\n",
+		PMD_DRV_LOG_LINE(ERR, "TX queue id is out of range: tx_queue_id=%u, nb_tx_queues=%u",
 			    tx_queue_id, eth_dev->data->nb_tx_queues);
 		return -EINVAL;
 	}
@@ -1172,7 +1173,7 @@ avp_dev_tx_queue_setup(struct rte_eth_dev *eth_dev,
 	txq = rte_zmalloc_socket("ethdev TX queue", sizeof(struct avp_queue),
 				 RTE_CACHE_LINE_SIZE, socket_id);
 	if (txq == NULL) {
-		PMD_DRV_LOG(ERR, "Failed to allocate new Tx queue object\n");
+		PMD_DRV_LOG_LINE(ERR, "Failed to allocate new Tx queue object");
 		return -ENOMEM;
 	}
 
@@ -1186,7 +1187,7 @@ avp_dev_tx_queue_setup(struct rte_eth_dev *eth_dev,
 	txq->dev_data = eth_dev->data;
 	eth_dev->data->tx_queues[tx_queue_id] = (void *)txq;
 
-	PMD_DRV_LOG(DEBUG, "Tx queue %u setup at %p\n", tx_queue_id, txq);
+	PMD_DRV_LOG_LINE(DEBUG, "Tx queue %u setup at %p", tx_queue_id, txq);
 
 	(void)nb_tx_desc;
 	(void)tx_conf;
@@ -1428,7 +1429,7 @@ avp_recv_scattered_pkts(void *rx_queue,
 
 	/* retrieve pending packets */
 	n = avp_fifo_get(rx_q, (void **)&avp_bufs, count);
-	PMD_RX_LOG(DEBUG, "Receiving %u packets from Rx queue at %p\n",
+	PMD_RX_LOG_LINE(DEBUG, "Receiving %u packets from Rx queue at %p",
 		   count, rx_q);
 
 	count = 0;
@@ -1525,7 +1526,7 @@ avp_recv_pkts(void *rx_queue,
 
 	/* retrieve pending packets */
 	n = avp_fifo_get(rx_q, (void **)&avp_bufs, count);
-	PMD_RX_LOG(DEBUG, "Receiving %u packets from Rx queue at %p\n",
+	PMD_RX_LOG_LINE(DEBUG, "Receiving %u packets from Rx queue at %p",
 		   count, rx_q);
 
 	count = 0;
@@ -1761,14 +1762,13 @@ avp_xmit_scattered_pkts(void *tx_queue,
 		return 0;
 	}
 
-	PMD_TX_LOG(DEBUG, "Sending %u packets on Tx queue at %p\n",
+	PMD_TX_LOG_LINE(DEBUG, "Sending %u packets on Tx queue at %p",
 		   nb_pkts, tx_q);
 
 	/* retrieve sufficient send buffers */
 	n = avp_fifo_get(alloc_q, (void **)&avp_bufs, segments);
 	if (unlikely(n != segments)) {
-		PMD_TX_LOG(DEBUG, "Failed to allocate buffers "
-			   "n=%u, segments=%u, orig=%u\n",
+		PMD_TX_LOG_LINE(DEBUG, "Failed to allocate buffers n=%u, segments=%u, orig=%u",
 			   n, segments, orig_nb_pkts);
 		txq->errors += orig_nb_pkts;
 		return 0;
@@ -1856,7 +1856,7 @@ avp_xmit_pkts(void *tx_queue, struct rte_mbuf **tx_pkts, uint16_t nb_pkts)
 		return 0;
 	}
 
-	PMD_TX_LOG(DEBUG, "Sending %u packets on Tx queue at %p\n",
+	PMD_TX_LOG_LINE(DEBUG, "Sending %u packets on Tx queue at %p",
 		   count, tx_q);
 
 	/* retrieve sufficient send buffers */
@@ -1987,7 +1987,7 @@ avp_dev_configure(struct rte_eth_dev *eth_dev)
 
 	rte_spinlock_lock(&avp->lock);
 	if (avp->flags & AVP_F_DETACHED) {
-		PMD_DRV_LOG(ERR, "Operation not supported during VM live migration\n");
+		PMD_DRV_LOG_LINE(ERR, "Operation not supported during VM live migration");
 		ret = -ENOTSUP;
 		goto unlock;
 	}
@@ -2003,7 +2003,7 @@ avp_dev_configure(struct rte_eth_dev *eth_dev)
 		RTE_ETH_VLAN_EXTEND_MASK);
 	ret = avp_vlan_offload_set(eth_dev, mask);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "VLAN offload set failed by host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "VLAN offload set failed by host, ret=%d",
 			    ret);
 		goto unlock;
 	}
@@ -2019,7 +2019,7 @@ avp_dev_configure(struct rte_eth_dev *eth_dev)
 
 	ret = avp_dev_ctrl_set_config(eth_dev, &config);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Config request failed by host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Config request failed by host, ret=%d",
 			    ret);
 		goto unlock;
 	}
@@ -2036,11 +2036,12 @@ static int
 avp_dev_start(struct rte_eth_dev *eth_dev)
 {
 	struct avp_dev *avp = AVP_DEV_PRIVATE_TO_HW(eth_dev->data->dev_private);
+	uint16_t i;
 	int ret;
 
 	rte_spinlock_lock(&avp->lock);
 	if (avp->flags & AVP_F_DETACHED) {
-		PMD_DRV_LOG(ERR, "Operation not supported during VM live migration\n");
+		PMD_DRV_LOG_LINE(ERR, "Operation not supported during VM live migration");
 		ret = -ENOTSUP;
 		goto unlock;
 	}
@@ -2048,7 +2049,7 @@ avp_dev_start(struct rte_eth_dev *eth_dev)
 	/* update link state */
 	ret = avp_dev_ctrl_set_link_state(eth_dev, 1);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Link state change failed by host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Link state change failed by host, ret=%d",
 			    ret);
 		goto unlock;
 	}
@@ -2056,6 +2057,11 @@ avp_dev_start(struct rte_eth_dev *eth_dev)
 	/* remember current link state */
 	avp->flags |= AVP_F_LINKUP;
 
+	for (i = 0; i < avp->num_rx_queues; i++)
+		eth_dev->data->rx_queue_state[i] = RTE_ETH_QUEUE_STATE_STARTED;
+	for (i = 0; i < avp->num_tx_queues; i++)
+		eth_dev->data->tx_queue_state[i] = RTE_ETH_QUEUE_STATE_STARTED;
+
 	ret = 0;
 
 unlock:
@@ -2067,11 +2073,12 @@ static int
 avp_dev_stop(struct rte_eth_dev *eth_dev)
 {
 	struct avp_dev *avp = AVP_DEV_PRIVATE_TO_HW(eth_dev->data->dev_private);
+	uint16_t i;
 	int ret;
 
 	rte_spinlock_lock(&avp->lock);
 	if (avp->flags & AVP_F_DETACHED) {
-		PMD_DRV_LOG(ERR, "Operation not supported during VM live migration\n");
+		PMD_DRV_LOG_LINE(ERR, "Operation not supported during VM live migration");
 		ret = -ENOTSUP;
 		goto unlock;
 	}
@@ -2082,10 +2089,15 @@ avp_dev_stop(struct rte_eth_dev *eth_dev)
 	/* update link state */
 	ret = avp_dev_ctrl_set_link_state(eth_dev, 0);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Link state change failed by host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Link state change failed by host, ret=%d",
 			    ret);
 	}
 
+	for (i = 0; i < avp->num_rx_queues; i++)
+		eth_dev->data->rx_queue_state[i] = RTE_ETH_QUEUE_STATE_STOPPED;
+	for (i = 0; i < avp->num_tx_queues; i++)
+		eth_dev->data->tx_queue_state[i] = RTE_ETH_QUEUE_STATE_STOPPED;
+
 unlock:
 	rte_spinlock_unlock(&avp->lock);
 	return ret;
@@ -2102,7 +2114,7 @@ avp_dev_close(struct rte_eth_dev *eth_dev)
 
 	rte_spinlock_lock(&avp->lock);
 	if (avp->flags & AVP_F_DETACHED) {
-		PMD_DRV_LOG(ERR, "Operation not supported during VM live migration\n");
+		PMD_DRV_LOG_LINE(ERR, "Operation not supported during VM live migration");
 		goto unlock;
 	}
 
@@ -2112,14 +2124,14 @@ avp_dev_close(struct rte_eth_dev *eth_dev)
 
 	ret = avp_dev_disable_interrupts(eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Failed to disable interrupts\n");
+		PMD_DRV_LOG_LINE(ERR, "Failed to disable interrupts");
 		/* continue */
 	}
 
 	/* update device state */
 	ret = avp_dev_ctrl_shutdown(eth_dev);
 	if (ret < 0) {
-		PMD_DRV_LOG(ERR, "Device shutdown failed by host, ret=%d\n",
+		PMD_DRV_LOG_LINE(ERR, "Device shutdown failed by host, ret=%d",
 			    ret);
 		/* continue */
 	}
@@ -2155,7 +2167,7 @@ avp_dev_promiscuous_enable(struct rte_eth_dev *eth_dev)
 	rte_spinlock_lock(&avp->lock);
 	if ((avp->flags & AVP_F_PROMISC) == 0) {
 		avp->flags |= AVP_F_PROMISC;
-		PMD_DRV_LOG(DEBUG, "Promiscuous mode enabled on %u\n",
+		PMD_DRV_LOG_LINE(DEBUG, "Promiscuous mode enabled on %u",
 			    eth_dev->data->port_id);
 	}
 	rte_spinlock_unlock(&avp->lock);
@@ -2171,7 +2183,7 @@ avp_dev_promiscuous_disable(struct rte_eth_dev *eth_dev)
 	rte_spinlock_lock(&avp->lock);
 	if ((avp->flags & AVP_F_PROMISC) != 0) {
 		avp->flags &= ~AVP_F_PROMISC;
-		PMD_DRV_LOG(DEBUG, "Promiscuous mode disabled on %u\n",
+		PMD_DRV_LOG_LINE(DEBUG, "Promiscuous mode disabled on %u",
 			    eth_dev->data->port_id);
 	}
 	rte_spinlock_unlock(&avp->lock);
@@ -2212,25 +2224,26 @@ avp_vlan_offload_set(struct rte_eth_dev *eth_dev, int mask)
 			else
 				avp->features &= ~RTE_AVP_FEATURE_VLAN_OFFLOAD;
 		} else {
-			PMD_DRV_LOG(ERR, "VLAN strip offload not supported\n");
+			PMD_DRV_LOG_LINE(ERR, "VLAN strip offload not supported");
 		}
 	}
 
 	if (mask & RTE_ETH_VLAN_FILTER_MASK) {
 		if (offloads & RTE_ETH_RX_OFFLOAD_VLAN_FILTER)
-			PMD_DRV_LOG(ERR, "VLAN filter offload not supported\n");
+			PMD_DRV_LOG_LINE(ERR, "VLAN filter offload not supported");
 	}
 
 	if (mask & RTE_ETH_VLAN_EXTEND_MASK) {
 		if (offloads & RTE_ETH_RX_OFFLOAD_VLAN_EXTEND)
-			PMD_DRV_LOG(ERR, "VLAN extend offload not supported\n");
+			PMD_DRV_LOG_LINE(ERR, "VLAN extend offload not supported");
 	}
 
 	return 0;
 }
 
 static int
-avp_dev_stats_get(struct rte_eth_dev *eth_dev, struct rte_eth_stats *stats)
+avp_dev_stats_get(struct rte_eth_dev *eth_dev, struct rte_eth_stats *stats,
+		  struct eth_queue_stats *qstats)
 {
 	struct avp_dev *avp = AVP_DEV_PRIVATE_TO_HW(eth_dev->data->dev_private);
 	unsigned int i;
@@ -2243,9 +2256,11 @@ avp_dev_stats_get(struct rte_eth_dev *eth_dev, struct rte_eth_stats *stats)
 			stats->ibytes += rxq->bytes;
 			stats->ierrors += rxq->errors;
 
-			stats->q_ipackets[i] += rxq->packets;
-			stats->q_ibytes[i] += rxq->bytes;
-			stats->q_errors[i] += rxq->errors;
+			if (qstats != NULL && i < RTE_ETHDEV_QUEUE_STAT_CNTRS) {
+				qstats->q_ipackets[i] += rxq->packets;
+				qstats->q_ibytes[i] += rxq->bytes;
+				qstats->q_errors[i] += rxq->errors;
+			}
 		}
 	}
 
@@ -2257,8 +2272,10 @@ avp_dev_stats_get(struct rte_eth_dev *eth_dev, struct rte_eth_stats *stats)
 			stats->obytes += txq->bytes;
 			stats->oerrors += txq->errors;
 
-			stats->q_opackets[i] += txq->packets;
-			stats->q_obytes[i] += txq->bytes;
+			if (qstats != NULL && i < RTE_ETHDEV_QUEUE_STAT_CNTRS) {
+				qstats->q_opackets[i] += txq->packets;
+				qstats->q_obytes[i] += txq->bytes;
+			}
 		}
 	}
 
