@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: BSD-3-Clause
- * Copyright(c) 2017-2021 Broadcom
+ * Copyright(c) 2017-2023 Broadcom
  * All rights reserved.
  */
 
@@ -7,8 +7,9 @@
 #include <stdbool.h>
 #include <unistd.h>
 
-#include <rte_dev.h>
+#include <dev_driver.h>
 #include <ethdev_driver.h>
+#include <eal_export.h>
 #include <rte_malloc.h>
 #include <rte_cycles.h>
 #include <rte_byteorder.h>
@@ -39,6 +40,7 @@ int bnxt_rcv_msg_from_vf(struct bnxt *bp, uint16_t vf_id, void *msg)
 		true : false;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_tx_loopback)
 int rte_pmd_bnxt_set_tx_loopback(uint16_t port, uint8_t on)
 {
 	struct rte_eth_dev *eth_dev;
@@ -57,8 +59,8 @@ int rte_pmd_bnxt_set_tx_loopback(uint16_t port, uint8_t on)
 	bp = eth_dev->data->dev_private;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set Tx loopback on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set Tx loopback on non-PF port %d!",
 			port);
 		return -ENOTSUP;
 	}
@@ -80,6 +82,7 @@ rte_pmd_bnxt_set_all_queues_drop_en_cb(struct bnxt_vnic_info *vnic, void *onptr)
 	vnic->bd_stall = !(*on);
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_all_queues_drop_en)
 int rte_pmd_bnxt_set_all_queues_drop_en(uint16_t port, uint8_t on)
 {
 	struct rte_eth_dev *eth_dev;
@@ -99,8 +102,8 @@ int rte_pmd_bnxt_set_all_queues_drop_en(uint16_t port, uint8_t on)
 	bp = eth_dev->data->dev_private;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set all queues drop on non-PF port!\n");
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set all queues drop on non-PF port!");
 		return -ENOTSUP;
 	}
 
@@ -112,7 +115,7 @@ int rte_pmd_bnxt_set_all_queues_drop_en(uint16_t port, uint8_t on)
 		bp->vnic_info[i].bd_stall = !on;
 		rc = bnxt_hwrm_vnic_cfg(bp, &bp->vnic_info[i]);
 		if (rc) {
-			PMD_DRV_LOG(ERR, "Failed to update PF VNIC %d.\n", i);
+			PMD_DRV_LOG_LINE(ERR, "Failed to update PF VNIC %d.", i);
 			return rc;
 		}
 	}
@@ -123,7 +126,7 @@ int rte_pmd_bnxt_set_all_queues_drop_en(uint16_t port, uint8_t on)
 				rte_pmd_bnxt_set_all_queues_drop_en_cb, &on,
 				bnxt_hwrm_vnic_cfg);
 		if (rc) {
-			PMD_DRV_LOG(ERR, "Failed to update VF VNIC %d.\n", i);
+			PMD_DRV_LOG_LINE(ERR, "Failed to update VF VNIC %d.", i);
 			break;
 		}
 	}
@@ -131,6 +134,7 @@ int rte_pmd_bnxt_set_all_queues_drop_en(uint16_t port, uint8_t on)
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_mac_addr)
 int rte_pmd_bnxt_set_vf_mac_addr(uint16_t port, uint16_t vf,
 				struct rte_ether_addr *mac_addr)
 {
@@ -147,8 +151,8 @@ int rte_pmd_bnxt_set_vf_mac_addr(uint16_t port, uint16_t vf,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -160,8 +164,8 @@ int rte_pmd_bnxt_set_vf_mac_addr(uint16_t port, uint16_t vf,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set VF %d mac address on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set VF %d mac address on non-PF port %d!",
 			vf, port);
 		return -ENOTSUP;
 	}
@@ -171,13 +175,14 @@ int rte_pmd_bnxt_set_vf_mac_addr(uint16_t port, uint16_t vf,
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_rate_limit)
 int rte_pmd_bnxt_set_vf_rate_limit(uint16_t port, uint16_t vf,
-				uint16_t tx_rate, uint64_t q_msk)
+				uint32_t tx_rate, uint64_t q_msk)
 {
 	struct rte_eth_dev *eth_dev;
 	struct rte_eth_dev_info dev_info;
 	struct bnxt *bp;
-	uint16_t tot_rate = 0;
+	uint32_t tot_rate = 0;
 	uint64_t idx;
 	int rc;
 
@@ -189,8 +194,8 @@ int rte_pmd_bnxt_set_vf_rate_limit(uint16_t port, uint16_t vf,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -211,7 +216,7 @@ int rte_pmd_bnxt_set_vf_rate_limit(uint16_t port, uint16_t vf,
 
 	/* Requested BW can't be greater than link speed */
 	if (tot_rate > eth_dev->data->dev_link.link_speed) {
-		PMD_DRV_LOG(ERR, "Rate > Link speed. Set to %d\n", tot_rate);
+		PMD_DRV_LOG_LINE(ERR, "Rate > Link speed. Set to %d", tot_rate);
 		return -EINVAL;
 	}
 
@@ -228,6 +233,7 @@ int rte_pmd_bnxt_set_vf_rate_limit(uint16_t port, uint16_t vf,
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_mac_anti_spoof)
 int rte_pmd_bnxt_set_vf_mac_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 {
 	struct rte_eth_dev_info dev_info;
@@ -247,8 +253,8 @@ int rte_pmd_bnxt_set_vf_mac_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -256,8 +262,8 @@ int rte_pmd_bnxt_set_vf_mac_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 	bp = dev->data->dev_private;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set mac spoof on non-PF port %d!\n", port);
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set mac spoof on non-PF port %d!", port);
 		return -EINVAL;
 	}
 
@@ -288,6 +294,7 @@ int rte_pmd_bnxt_set_vf_mac_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_vlan_anti_spoof)
 int rte_pmd_bnxt_set_vf_vlan_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 {
 	struct rte_eth_dev_info dev_info;
@@ -306,8 +313,8 @@ int rte_pmd_bnxt_set_vf_vlan_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -315,8 +322,8 @@ int rte_pmd_bnxt_set_vf_vlan_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 	bp = dev->data->dev_private;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set VLAN spoof on non-PF port %d!\n", port);
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set VLAN spoof on non-PF port %d!", port);
 		return -EINVAL;
 	}
 
@@ -334,7 +341,7 @@ int rte_pmd_bnxt_set_vf_vlan_anti_spoof(uint16_t port, uint16_t vf, uint8_t on)
 				rc = -1;
 		}
 	} else {
-		PMD_DRV_LOG(ERR, "Failed to update VF VNIC %d.\n", vf);
+		PMD_DRV_LOG_LINE(ERR, "Failed to update VF VNIC %d.", vf);
 	}
 
 	return rc;
@@ -347,6 +354,7 @@ rte_pmd_bnxt_set_vf_vlan_stripq_cb(struct bnxt_vnic_info *vnic, void *onptr)
 	vnic->vlan_strip = *on;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_vlan_stripq)
 int
 rte_pmd_bnxt_set_vf_vlan_stripq(uint16_t port, uint16_t vf, uint8_t on)
 {
@@ -363,8 +371,8 @@ rte_pmd_bnxt_set_vf_vlan_stripq(uint16_t port, uint16_t vf, uint8_t on)
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -375,8 +383,8 @@ rte_pmd_bnxt_set_vf_vlan_stripq(uint16_t port, uint16_t vf, uint8_t on)
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set VF %d stripq on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set VF %d stripq on non-PF port %d!",
 			vf, port);
 		return -ENOTSUP;
 	}
@@ -385,11 +393,12 @@ rte_pmd_bnxt_set_vf_vlan_stripq(uint16_t port, uint16_t vf, uint8_t on)
 				rte_pmd_bnxt_set_vf_vlan_stripq_cb, &on,
 				bnxt_hwrm_vnic_cfg);
 	if (rc)
-		PMD_DRV_LOG(ERR, "Failed to update VF VNIC %d.\n", vf);
+		PMD_DRV_LOG_LINE(ERR, "Failed to update VF VNIC %d.", vf);
 
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_rxmode)
 int rte_pmd_bnxt_set_vf_rxmode(uint16_t port, uint16_t vf,
 				uint16_t rx_mask, uint8_t on)
 {
@@ -407,8 +416,8 @@ int rte_pmd_bnxt_set_vf_rxmode(uint16_t port, uint16_t vf,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -422,7 +431,7 @@ int rte_pmd_bnxt_set_vf_rxmode(uint16_t port, uint16_t vf,
 		return -EINVAL;
 
 	if (rx_mask & RTE_ETH_VMDQ_ACCEPT_UNTAG) {
-		PMD_DRV_LOG(ERR, "Currently cannot toggle this setting\n");
+		PMD_DRV_LOG_LINE(ERR, "Currently cannot toggle this setting");
 		return -ENOTSUP;
 	}
 
@@ -445,7 +454,7 @@ int rte_pmd_bnxt_set_vf_rxmode(uint16_t port, uint16_t vf,
 					&bp->pf->vf_info[vf].l2_rx_mask,
 					bnxt_set_rx_mask_no_vlan);
 	if (rc)
-		PMD_DRV_LOG(ERR, "bnxt_hwrm_func_vf_vnic_set_rxmask failed\n");
+		PMD_DRV_LOG_LINE(ERR, "bnxt_hwrm_func_vf_vnic_set_rxmask failed");
 
 	return rc;
 }
@@ -457,8 +466,8 @@ static int bnxt_set_vf_table(struct bnxt *bp, uint16_t vf)
 	struct bnxt_vnic_info vnic;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set VLAN table on non-PF port!\n");
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set VLAN table on non-PF port!");
 		return -EINVAL;
 	}
 
@@ -470,7 +479,7 @@ static int bnxt_set_vf_table(struct bnxt *bp, uint16_t vf)
 		/* This simply indicates there's no driver loaded.
 		 * This is not an error.
 		 */
-		PMD_DRV_LOG(ERR, "Unable to get default VNIC for VF %d\n", vf);
+		PMD_DRV_LOG_LINE(ERR, "Unable to get default VNIC for VF %d", vf);
 	} else {
 		memset(&vnic, 0, sizeof(vnic));
 		vnic.fw_vnic_id = dflt_vnic;
@@ -488,6 +497,7 @@ static int bnxt_set_vf_table(struct bnxt *bp, uint16_t vf)
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_vlan_filter)
 int rte_pmd_bnxt_set_vf_vlan_filter(uint16_t port, uint16_t vlan,
 				    uint64_t vf_mask, uint8_t vlan_on)
 {
@@ -534,10 +544,10 @@ int rte_pmd_bnxt_set_vf_vlan_filter(uint16_t port, uint16_t vlan,
 				/* Now check that there's space */
 				if (cnt == getpagesize() / sizeof(struct
 				    bnxt_vlan_antispoof_table_entry)) {
-					PMD_DRV_LOG(ERR,
-					     "VLAN anti-spoof table is full\n");
-					PMD_DRV_LOG(ERR,
-						"VF %d cannot add VLAN %u\n",
+					PMD_DRV_LOG_LINE(ERR,
+					     "VLAN anti-spoof table is full");
+					PMD_DRV_LOG_LINE(ERR,
+						"VF %d cannot add VLAN %u",
 						i, vlan);
 					rc = -1;
 					continue;
@@ -583,6 +593,7 @@ int rte_pmd_bnxt_set_vf_vlan_filter(uint16_t port, uint16_t vlan,
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_get_vf_stats)
 int rte_pmd_bnxt_get_vf_stats(uint16_t port,
 			      uint16_t vf_id,
 			      struct rte_eth_stats *stats)
@@ -598,8 +609,8 @@ int rte_pmd_bnxt_get_vf_stats(uint16_t port,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -610,8 +621,8 @@ int rte_pmd_bnxt_get_vf_stats(uint16_t port,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to get VF %d stats on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to get VF %d stats on non-PF port %d!",
 			vf_id, port);
 		return -ENOTSUP;
 	}
@@ -620,6 +631,7 @@ int rte_pmd_bnxt_get_vf_stats(uint16_t port,
 				     NULL);
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_reset_vf_stats)
 int rte_pmd_bnxt_reset_vf_stats(uint16_t port,
 				uint16_t vf_id)
 {
@@ -634,8 +646,8 @@ int rte_pmd_bnxt_reset_vf_stats(uint16_t port,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -646,8 +658,8 @@ int rte_pmd_bnxt_reset_vf_stats(uint16_t port,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to reset VF %d stats on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to reset VF %d stats on non-PF port %d!",
 			vf_id, port);
 		return -ENOTSUP;
 	}
@@ -655,6 +667,7 @@ int rte_pmd_bnxt_reset_vf_stats(uint16_t port,
 	return bnxt_hwrm_func_clr_stats(bp, bp->pf->first_vf_id + vf_id);
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_get_vf_rx_status)
 int rte_pmd_bnxt_get_vf_rx_status(uint16_t port, uint16_t vf_id)
 {
 	struct rte_eth_dev *dev;
@@ -668,8 +681,8 @@ int rte_pmd_bnxt_get_vf_rx_status(uint16_t port, uint16_t vf_id)
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -680,8 +693,8 @@ int rte_pmd_bnxt_get_vf_rx_status(uint16_t port, uint16_t vf_id)
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to query VF %d RX stats on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to query VF %d RX stats on non-PF port %d!",
 			vf_id, port);
 		return -ENOTSUP;
 	}
@@ -689,6 +702,7 @@ int rte_pmd_bnxt_get_vf_rx_status(uint16_t port, uint16_t vf_id)
 	return bnxt_vf_vnic_count(bp, vf_id);
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_get_vf_tx_drop_count)
 int rte_pmd_bnxt_get_vf_tx_drop_count(uint16_t port, uint16_t vf_id,
 				      uint64_t *count)
 {
@@ -703,8 +717,8 @@ int rte_pmd_bnxt_get_vf_tx_drop_count(uint16_t port, uint16_t vf_id,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -715,8 +729,8 @@ int rte_pmd_bnxt_get_vf_tx_drop_count(uint16_t port, uint16_t vf_id,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to query VF %d TX drops on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to query VF %d TX drops on non-PF port %d!",
 			vf_id, port);
 		return -ENOTSUP;
 	}
@@ -725,6 +739,7 @@ int rte_pmd_bnxt_get_vf_tx_drop_count(uint16_t port, uint16_t vf_id,
 					     count);
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_mac_addr_add)
 int rte_pmd_bnxt_mac_addr_add(uint16_t port, struct rte_ether_addr *addr,
 				uint32_t vf_id)
 {
@@ -742,8 +757,8 @@ int rte_pmd_bnxt_mac_addr_add(uint16_t port, struct rte_ether_addr *addr,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -754,8 +769,8 @@ int rte_pmd_bnxt_mac_addr_add(uint16_t port, struct rte_ether_addr *addr,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to config VF %d MAC on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to config VF %d MAC on non-PF port %d!",
 			vf_id, port);
 		return -ENOTSUP;
 	}
@@ -808,6 +823,7 @@ exit:
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_vlan_insert)
 int
 rte_pmd_bnxt_set_vf_vlan_insert(uint16_t port, uint16_t vf,
 		uint16_t vlan_id)
@@ -825,8 +841,8 @@ rte_pmd_bnxt_set_vf_vlan_insert(uint16_t port, uint16_t vf,
 
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -837,8 +853,8 @@ rte_pmd_bnxt_set_vf_vlan_insert(uint16_t port, uint16_t vf,
 		return -EINVAL;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set VF %d vlan insert on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set VF %d vlan insert on non-PF port %d!",
 			vf, port);
 		return -ENOTSUP;
 	}
@@ -853,6 +869,7 @@ rte_pmd_bnxt_set_vf_vlan_insert(uint16_t port, uint16_t vf,
 	return rc;
 }
 
+RTE_EXPORT_SYMBOL(rte_pmd_bnxt_set_vf_persist_stats)
 int rte_pmd_bnxt_set_vf_persist_stats(uint16_t port, uint16_t vf, uint8_t on)
 {
 	struct rte_eth_dev_info dev_info;
@@ -869,8 +886,8 @@ int rte_pmd_bnxt_set_vf_persist_stats(uint16_t port, uint16_t vf, uint8_t on)
 	dev = &rte_eth_devices[port];
 	rc = rte_eth_dev_info_get(port, &dev_info);
 	if (rc != 0) {
-		PMD_DRV_LOG(ERR,
-			"Error during getting device (port %u) info: %s\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Error during getting device (port %u) info: %s",
 			port, strerror(-rc));
 
 		return rc;
@@ -878,8 +895,8 @@ int rte_pmd_bnxt_set_vf_persist_stats(uint16_t port, uint16_t vf, uint8_t on)
 	bp = dev->data->dev_private;
 
 	if (!BNXT_PF(bp)) {
-		PMD_DRV_LOG(ERR,
-			"Attempt to set persist stats on non-PF port %d!\n",
+		PMD_DRV_LOG_LINE(ERR,
+			"Attempt to set persist stats on non-PF port %d!",
 			port);
 		return -EINVAL;
 	}
