@@ -16,6 +16,8 @@
 #include "rte_sctp.h"
 #include "rte_flow.h"
 #include "tf_core.h"
+#include "cfa_resources.h"
+#include "cfa_types.h"
 
 /* Number of fields for each protocol */
 #define BNXT_ULP_PROTO_HDR_SVIF_NUM	2
@@ -27,9 +29,15 @@
 #define BNXT_ULP_PROTO_HDR_UDP_NUM	4
 #define BNXT_ULP_PROTO_HDR_TCP_NUM	9
 #define BNXT_ULP_PROTO_HDR_VXLAN_NUM	4
+#define BNXT_ULP_PROTO_HDR_VXLAN_GPE_NUM 5
+#define BNXT_ULP_PROTO_HDR_GENEVE_NUM 4
 #define BNXT_ULP_PROTO_HDR_GRE_NUM	2
+#define BNXT_ULP_PROTO_HDR_GRE_KEY_NUM	1
 #define BNXT_ULP_PROTO_HDR_ICMP_NUM	5
 #define BNXT_ULP_PROTO_HDR_ECPRI_NUM	2
+#define	BNXT_ULP_PROTO_HDR_IPV6_EXT_NUM	1
+#define BNXT_ULP_PROTO_HDR_SRV6_NUM	7
+#define BNXT_ULP_PROTO_HDR_MPLS_NUM	2
 #define BNXT_ULP_PROTO_HDR_MAX		128
 #define BNXT_ULP_PROTO_HDR_ENCAP_MAX	64
 #define BNXT_ULP_PROTO_HDR_FIELD_SVIF_IDX	1
@@ -93,7 +101,11 @@ struct ulp_rte_parser_params {
 	uint32_t			act_pattern_id;
 	uint8_t				app_id;
 	uint8_t				tun_idx;
-
+	uint16_t			gre_cks_rsvd0_ver; /* GRE C, K, S bits and version */
+	uint16_t			class_info_idx;
+	uint64_t			wc_field_bitmap;
+	uint64_t			cf_bitmap;
+	uint64_t			exclude_field_bitmap;
 };
 
 /* Flow Parser Header Information Structure */
@@ -145,6 +157,12 @@ struct bnxt_ulp_class_match_info {
 	uint32_t		hdr_sig_id;
 	uint64_t		flow_sig_id;
 	uint32_t		flow_pattern_id;
+	uint8_t			app_id;
+	struct ulp_rte_bitmap	hdr_bitmap;
+	uint64_t		field_man_bitmap;
+	uint64_t		field_opt_bitmap;
+	uint64_t		field_exclude_bitmap;
+	uint8_t			field_list[BNXT_ULP_GLB_FIELD_TBL_SIZE + 1];
 };
 
 /* Flow Matcher templates Structure for class entries */
@@ -158,11 +176,8 @@ struct bnxt_ulp_action_match_info {
 };
 
 struct bnxt_ulp_act_match_info {
-	struct ulp_rte_bitmap	act_sig;
-	uint32_t		act_hid;
+	struct ulp_rte_bitmap	act_bitmap;
 	uint32_t		act_tid;
-	uint32_t		act_pattern_id;
-	uint8_t			app_sig;
 };
 
 /* Flow Matcher templates Structure for action entries */
@@ -187,9 +202,10 @@ struct bnxt_ulp_mapper_func_info {
 	enum bnxt_ulp_func_opc		func_opc;
 	enum bnxt_ulp_func_src		func_src1;
 	enum bnxt_ulp_func_src		func_src2;
-	uint16_t			func_opr1;
-	uint16_t			func_opr2;
+	uint64_t			func_opr1;
+	uint64_t			func_opr2;
 	uint16_t			func_dst_opr;
+	uint32_t			func_oper_size;
 };
 
 struct bnxt_ulp_template_device_tbls {
@@ -199,12 +215,17 @@ struct bnxt_ulp_template_device_tbls {
 	uint32_t tbl_list_size;
 	struct bnxt_ulp_mapper_key_info *key_info_list;
 	uint32_t key_info_list_size;
+	struct bnxt_ulp_mapper_field_info *key_ext_list;
+	uint32_t key_ext_list_size;
 	struct bnxt_ulp_mapper_field_info *result_field_list;
 	uint32_t result_field_list_size;
 	struct bnxt_ulp_mapper_ident_info *ident_list;
 	uint32_t ident_list_size;
 	struct bnxt_ulp_mapper_cond_info *cond_list;
 	uint32_t cond_list_size;
+	struct bnxt_ulp_mapper_cond_list_info *cond_oper_list;
+	uint32_t cond_oper_list_size;
+
 };
 
 struct bnxt_ulp_dyn_size_map {
@@ -250,6 +271,9 @@ struct bnxt_ulp_device_params {
 	uint32_t			wc_mode_list[4];
 	uint32_t			wc_mod_list_max_size;
 	uint32_t			wc_ctl_size_bits;
+	uint32_t			dev_features;
+	const struct bnxt_ulp_generic_tbl_params *gen_tbl_params;
+	const struct bnxt_ulp_allocator_tbl_params *allocator_tbl_params;
 	const struct bnxt_ulp_template_device_tbls *dev_tbls;
 };
 
@@ -278,10 +302,16 @@ struct bnxt_ulp_mapper_tbl_info {
 
 	enum bnxt_ulp_critical_resource		critical_resource;
 
-	/* Information for accessing the ulp_key_field_list */
+	/* Information for accessing the key in ulp_key_field_list */
 	uint32_t	key_start_idx;
 	uint16_t	key_bit_size;
 	uint16_t	key_num_fields;
+
+	/* Information for accessing the partial key in ulp_key_field_list */
+	uint32_t	partial_key_start_idx;
+	uint16_t	partial_key_bit_size;
+	uint16_t	partial_key_num_fields;
+
 	/* Size of the blob that holds the key */
 	uint16_t	blob_key_bit_size;
 	uint16_t	record_size;
@@ -312,6 +342,16 @@ struct bnxt_ulp_mapper_tbl_info {
 
 	/* Shared session */
 	enum bnxt_ulp_session_type	session_type;
+	enum cfa_track_type		track_type;
+
+	/* Key recipes for generic templates */
+	enum bnxt_ulp_key_recipe_opc key_recipe_opcode;
+	uint32_t key_recipe_operand;
+
+	/* control table messages */
+	const char			*false_message;
+	const char			*true_message;
+	const char			*description;
 };
 
 struct bnxt_ulp_mapper_field_info {
@@ -363,6 +403,10 @@ struct bnxt_ulp_resource_resv_info {
 
 struct bnxt_ulp_app_capabilities_info {
 	uint8_t				app_id;
+	uint32_t			default_priority;
+	uint32_t			max_def_priority;
+	uint32_t			min_flow_priority;
+	uint32_t			max_flow_priority;
 	uint32_t			vxlan_port;
 	uint32_t			vxlan_ip_port;
 	uint32_t			ecpri_udp_port;
@@ -371,7 +415,21 @@ struct bnxt_ulp_app_capabilities_info {
 	uint8_t				ha_pool_id;
 	uint8_t				ha_reg_state;
 	uint8_t				ha_reg_cnt;
+	uint8_t				tunnel_next_proto;
 	uint32_t			flags;
+	uint32_t			max_pools;
+	uint8_t				em_multiplier;
+	uint32_t			num_rx_flows;
+	uint32_t			num_tx_flows;
+	uint16_t			act_rx_max_sz;
+	uint16_t			act_tx_max_sz;
+	uint16_t			em_rx_key_max_sz;
+	uint16_t			em_tx_key_max_sz;
+	uint32_t			pbl_page_sz_in_bytes;
+	uint16_t			num_key_recipes_per_dir;
+	uint64_t			feature_bits;
+	uint64_t			default_class_bits;
+	uint64_t			default_act_bits;
 };
 
 struct bnxt_ulp_cache_tbl_params {
@@ -380,12 +438,19 @@ struct bnxt_ulp_cache_tbl_params {
 
 struct bnxt_ulp_generic_tbl_params {
 	const char			*name;
+	enum bnxt_ulp_gen_tbl_type	gen_tbl_type;
 	uint16_t			result_num_entries;
 	uint16_t			result_num_bytes;
 	enum bnxt_ulp_byte_order	result_byte_order;
 	uint32_t			hash_tbl_entries;
 	uint16_t			num_buckets;
 	uint16_t			key_num_bytes;
+	uint16_t			partial_key_num_bytes;
+};
+
+struct bnxt_ulp_allocator_tbl_params  {
+	const char			*name;
+	uint16_t			num_entries;
 };
 
 struct bnxt_ulp_shared_act_info {
