@@ -5,7 +5,11 @@
 #include "cn9k_ethdev.h"
 #include "cn9k_rx.h"
 
+#if !defined(CNXK_DIS_TMPLT_FUNC)
+
 #define R(name, flags) NIX_RX_RECV_VEC(cn9k_nix_recv_pkts_vec_##name, flags)
 
-NIX_RX_FASTPATH_MODES_16_31
+NIX_RX_FASTPATH_MODES_48_63
 #undef R
+
+#endif
