@@ -5,8 +5,12 @@
 #include "cn9k_ethdev.h"
 #include "cn9k_rx.h"
 
+#if !defined(CNXK_DIS_TMPLT_FUNC)
+
 #define R(name, flags)                                                         \
 	NIX_RX_RECV_VEC_MSEG(cn9k_nix_recv_pkts_vec_mseg_##name, flags)
 
-NIX_RX_FASTPATH_MODES_32_47
+NIX_RX_FASTPATH_MODES_96_111
 #undef R
+
+#endif
