@@ -11,7 +11,8 @@
 #include <ethdev_vdev.h>
 #include <rte_devargs.h>
 #include <rte_kvargs.h>
-#include <rte_bus_vdev.h>
+#include <bus_driver.h>
+#include <bus_vdev_driver.h>
 
 #include "failsafe_private.h"
 
@@ -146,12 +147,8 @@ fs_mutex_init(struct fs_priv *priv)
 		ERROR("Cannot set mutex type - %s", strerror(ret));
 		return ret;
 	}
-	ret = pthread_mutex_init(&priv->hotplug_mutex, &attr);
-	if (ret) {
-		ERROR("Cannot initiate mutex - %s", strerror(ret));
-		return ret;
-	}
-	return 0;
+
+	return pthread_mutex_init(&priv->hotplug_mutex, &attr);
 }
 
 static int
@@ -308,8 +305,8 @@ fs_rte_eth_free(const char *name)
 	if (dev == NULL)
 		return 0; /* port already released */
 	ret = failsafe_eth_dev_close(dev);
-	rte_eth_dev_release_port(dev);
 	rte_intr_instance_free(PRIV(dev)->intr_handle);
+	rte_eth_dev_release_port(dev);
 	return ret;
 }
 
