@@ -106,7 +106,7 @@ mlx4_init_shared_data(void)
 						 sizeof(*mlx4_shared_data),
 						 SOCKET_ID_ANY, 0);
 			if (mz == NULL) {
-				ERROR("Cannot allocate mlx4 shared data\n");
+				ERROR("Cannot allocate mlx4 shared data");
 				ret = -rte_errno;
 				goto error;
 			}
@@ -117,7 +117,7 @@ mlx4_init_shared_data(void)
 			/* Lookup allocated shared memory. */
 			mz = rte_memzone_lookup(MZ_MLX4_PMD_SHARED_DATA);
 			if (mz == NULL) {
-				ERROR("Cannot attach mlx4 shared data\n");
+				ERROR("Cannot attach mlx4 shared data");
 				ret = -rte_errno;
 				goto error;
 			}
@@ -1058,7 +1058,7 @@ err_secondary:
 		priv->intr_handle =
 			rte_intr_instance_alloc(RTE_INTR_INSTANCE_F_SHARED);
 		if (priv->intr_handle == NULL) {
-			RTE_LOG(ERR, EAL, "Fail to allocate intr_handle\n");
+			ERROR("can not allocate intr_handle");
 			goto port_error;
 		}
 
@@ -1385,7 +1385,7 @@ RTE_INIT(rte_mlx4_pmd_init)
 	rte_pci_register(&mlx4_driver);
 }
 
-RTE_PMD_EXPORT_NAME(net_mlx4, __COUNTER__);
+RTE_PMD_EXPORT_NAME(net_mlx4);
 RTE_PMD_REGISTER_PCI_TABLE(net_mlx4, mlx4_pci_id_map);
 RTE_PMD_REGISTER_KMOD_DEP(net_mlx4,
 	"* ib_uverbs & mlx4_en & mlx4_core & mlx4_ib");
