@@ -46,9 +46,6 @@
 /* Maximum number of DCS created per port. */
 #define MLX5_HWS_CNT_DCS_NUM 4
 
-/* Alarm timeout. */
-#define MLX5_ALARM_TIMEOUT_US 100000
-
 /* Maximum number of extended statistics counters. */
 #define MLX5_MAX_XSTATS 64
 
@@ -170,9 +167,6 @@
 /* Size of the hash table for tag table. */
 #define MLX5_TAGS_HLIST_ARRAY_SIZE	(1 << 15)
 
-/* Size fo the hash table for SFT table. */
-#define MLX5_FLOW_SFT_HLIST_ARRAY_SIZE	4096
-
 /* Hairpin TX/RX queue configuration parameters. */
 #define MLX5_HAIRPIN_QUEUE_STRIDE 6
 #define MLX5_HAIRPIN_JUMBO_LOG_SIZE (14 + 2)
@@ -183,6 +177,9 @@
 /* Maximum number of external Rx queues supported by rte_flow */
 #define MLX5_MAX_EXT_RX_QUEUES (UINT16_MAX - RTE_PMD_MLX5_EXTERNAL_RX_QUEUE_ID_MIN + 1)
 
+/* Maximum number of external Tx queues supported by rte_flow */
+#define MLX5_MAX_EXT_TX_QUEUES (UINT16_MAX - MLX5_EXTERNAL_TX_QUEUE_ID_MIN + 1)
+
 /*
  * Linux definition of static_assert is found in /usr/include/assert.h.
  * Windows does not require a redefinition.
@@ -193,4 +190,6 @@
 
 #define MLX5_CNT_SVC_CYCLE_TIME_DEFAULT 500
 
+#define MLX5_SIZE_MOV16 16
+
 #endif /* RTE_PMD_MLX5_DEFS_H_ */
