@@ -22,12 +22,6 @@
 
 #include "mlx5_defs.h"
 
-/* Convert a bit number to the corresponding 64-bit mask */
-#define MLX5_BITSHIFT(v) (UINT64_C(1) << (v))
-
-/* Save and restore errno around argument evaluation. */
-#define ERRNO_SAFE(x) ((errno = (int []){ errno, ((x), 0) }[0]))
-
 extern int mlx5_logtype;
 
 #define MLX5_NET_LOG_PREFIX "mlx5_net"
@@ -55,7 +49,7 @@ extern int mlx5_logtype;
 
 /*
  * For the case which data is linked with sequence increased index, the
- * array table will be more efficiect than hash table once need to serarch
+ * array table will be more efficient than hash table once need to search
  * one data entry in large numbers of entries. Since the traditional hash
  * tables has fixed table size, when huge numbers of data saved to the hash
  * table, it also comes lots of hash conflict.
@@ -118,44 +112,44 @@ struct mlx5_l3t_level_tbl {
 };
 
 /* L3 word entry table data structure. */
-struct mlx5_l3t_entry_word {
+struct __rte_packed_begin mlx5_l3t_entry_word {
 	uint32_t idx; /* Table index. */
 	uint64_t ref_cnt; /* Table ref_cnt. */
 	struct {
 		uint16_t data;
 		uint32_t ref_cnt;
 	} entry[MLX5_L3T_ET_SIZE]; /* Entry array */
-} __rte_packed;
+} __rte_packed_end;
 
 /* L3 double word entry table data structure. */
-struct mlx5_l3t_entry_dword {
+struct __rte_packed_begin mlx5_l3t_entry_dword {
 	uint32_t idx; /* Table index. */
 	uint64_t ref_cnt; /* Table ref_cnt. */
 	struct {
 		uint32_t data;
 		int32_t ref_cnt;
 	} entry[MLX5_L3T_ET_SIZE]; /* Entry array */
-} __rte_packed;
+} __rte_packed_end;
 
 /* L3 quad word entry table data structure. */
-struct mlx5_l3t_entry_qword {
+struct __rte_packed_begin mlx5_l3t_entry_qword {
 	uint32_t idx; /* Table index. */
 	uint64_t ref_cnt; /* Table ref_cnt. */
 	struct {
 		uint64_t data;
 		uint32_t ref_cnt;
 	} entry[MLX5_L3T_ET_SIZE]; /* Entry array */
-} __rte_packed;
+} __rte_packed_end;
 
 /* L3 pointer entry table data structure. */
-struct mlx5_l3t_entry_ptr {
+struct __rte_packed_begin mlx5_l3t_entry_ptr {
 	uint32_t idx; /* Table index. */
 	uint64_t ref_cnt; /* Table ref_cnt. */
 	struct {
 		void *data;
 		uint32_t ref_cnt;
 	} entry[MLX5_L3T_ET_SIZE]; /* Entry array */
-} __rte_packed;
+} __rte_packed_end;
 
 /* L3 table data structure. */
 struct mlx5_l3t_tbl {
@@ -170,6 +164,14 @@ struct mlx5_l3t_tbl {
 typedef int32_t (*mlx5_l3t_alloc_callback_fn)(void *ctx,
 					   union mlx5_l3t_data *data);
 
+/*
+ * The default ipool threshold value indicates which per_core_cache
+ * value to set.
+ */
+#define MLX5_HW_IPOOL_SIZE_THRESHOLD (1 << 19)
+/* The default min local cache size. */
+#define MLX5_HW_IPOOL_CACHE_MIN (1 << 9)
+
 /*
  * The indexed memory entry index is made up of trunk index and offset of
  * the entry in the trunk. Since the entry index is 32 bits, in case user
@@ -185,6 +187,15 @@ typedef int32_t (*mlx5_l3t_alloc_callback_fn)(void *ctx,
 #define POOL_DEBUG 1
 #endif
 
+extern int mlx5_logtype_ipool;
+#define MLX5_NET_LOG_PREFIX_IPOOL "mlx5_ipool"
+
+/* Generic printf()-like logging macro with automatic line feed. */
+#define DRV_LOG_IPOOL(level, ...) \
+	PMD_DRV_LOG_(level, mlx5_logtype_ipool, MLX5_NET_LOG_PREFIX_IPOOL, \
+		__VA_ARGS__ PMD_DRV_LOG_STRIP PMD_DRV_LOG_OPAREN, \
+		PMD_DRV_LOG_CPAREN)
+
 struct mlx5_indexed_pool_config {
 	uint32_t size; /* Pool entry size. */
 	uint32_t trunk_size:22;
@@ -207,7 +218,7 @@ struct mlx5_indexed_pool_config {
 	 */
 	uint32_t need_lock:1;
 	/* Lock is needed for multiple thread usage. */
-	uint32_t release_mem_en:1; /* Rlease trunk when it is free. */
+	uint32_t release_mem_en:1; /* Release trunk when it is free. */
 	uint32_t max_idx; /* The maximum index can be allocated. */
 	uint32_t per_core_cache;
 	/*
@@ -227,12 +238,12 @@ struct mlx5_indexed_trunk {
 	uint32_t next; /* Next free trunk in free list. */
 	uint32_t free; /* Free entries available */
 	struct rte_bitmap *bmp;
-	uint8_t data[] __rte_cache_aligned; /* Entry data start. */
+	alignas(RTE_CACHE_LINE_SIZE) uint8_t data[]; /* Entry data start. */
 };
 
 struct mlx5_indexed_cache {
 	struct mlx5_indexed_trunk **trunks;
-	volatile uint32_t n_trunk_valid; /* Trunks allocated. */
+	volatile RTE_ATOMIC(uint32_t) n_trunk_valid; /* Trunks allocated. */
 	uint32_t n_trunk; /* Trunk pointer array size. */
 	uint32_t ref_cnt;
 	uint32_t len;
@@ -245,6 +256,15 @@ struct mlx5_ipool_per_lcore {
 	uint32_t idx[]; /**< Cache objects. */
 };
 
+#ifdef POOL_DEBUG
+struct mlx5_ipool_cache_validation {
+	rte_spinlock_t lock;
+	uint32_t bmp_size;
+	struct rte_bitmap *bmp;
+	void *bmp_mem;
+};
+#endif
+
 struct mlx5_indexed_pool {
 	struct mlx5_indexed_pool_config cfg; /* Indexed pool configuration. */
 	rte_spinlock_t rsz_lock; /* Pool lock for multiple thread usage. */
@@ -258,13 +278,16 @@ struct mlx5_indexed_pool {
 			uint32_t free_list; /* Index to first free trunk. */
 		};
 		struct {
-			struct mlx5_indexed_cache *gc;
+			RTE_ATOMIC(struct mlx5_indexed_cache *) gc;
 			/* Global cache. */
 			struct mlx5_ipool_per_lcore *cache[RTE_MAX_LCORE + 1];
 			/* Local cache. */
 			struct rte_bitmap *ibmp;
 			void *bmp_mem;
 			/* Allocate objects bitmap. Use during flush. */
+#ifdef POOL_DEBUG
+			struct mlx5_ipool_cache_validation cache_validator;
+#endif
 		};
 	};
 #ifdef POOL_DEBUG
@@ -419,6 +442,23 @@ void mlx5_ipool_flush_cache(struct mlx5_indexed_pool *pool);
  */
 void *mlx5_ipool_get_next(struct mlx5_indexed_pool *pool, uint32_t *pos);
 
+/**
+ * This function resize the ipool.
+ *
+ * @param pool
+ *   Pointer to the index memory pool handler.
+ * @param num_entries
+ *   Number of entries to be added to the pool.
+ *   This number should be divisible by trunk_size.
+ *
+ * @return
+ *   - non-zero value on error.
+ *   - 0 on success.
+ *
+ */
+int mlx5_ipool_resize(struct mlx5_indexed_pool *pool, uint32_t num_entries,
+		      struct rte_flow_error *error);
+
 /**
  * This function allocates new empty Three-level table.
  *
@@ -459,34 +499,6 @@ void mlx5_l3t_destroy(struct mlx5_l3t_tbl *tbl);
 int32_t mlx5_l3t_get_entry(struct mlx5_l3t_tbl *tbl, uint32_t idx,
 			    union mlx5_l3t_data *data);
 
-/**
- * This function gets the index entry from Three-level table.
- *
- * If the index entry is not available, allocate new one by callback
- * function and fill in the entry.
- *
- * @param tbl
- *   Pointer to the l3t.
- * @param idx
- *   Index to the entry.
- * @param data
- *   Pointer to the memory which saves the entry data.
- *   When function call returns 0, data contains the entry data get from
- *   l3t.
- *   When function call returns -1, data is not modified.
- * @param cb
- *   Callback function to allocate new data.
- * @param ctx
- *   Context for callback function.
- *
- * @return
- *   0 if success, -1 on error.
- */
-
-int32_t mlx5_l3t_prepare_entry(struct mlx5_l3t_tbl *tbl, uint32_t idx,
-			       union mlx5_l3t_data *data,
-			       mlx5_l3t_alloc_callback_fn cb, void *ctx);
-
 /**
  * This function decreases and clear index entry if reference
  * counter is 0 from Three-level table.
