@@ -215,7 +215,7 @@ mrvl_pp2_fixup_init(void)
 		dummy_pool_id[i] = mrvl_reserve_bit(&used_bpools[i],
 					     PP2_BPOOL_NUM_POOLS);
 		if (dummy_pool_id[i] < 0) {
-			MRVL_LOG(ERR, "Can't find free pool\n");
+			MRVL_LOG(ERR, "Can't find free pool");
 			return -1;
 		}
 
@@ -227,7 +227,7 @@ mrvl_pp2_fixup_init(void)
 		bpool_params.dummy_short_pool = 1;
 		err = pp2_bpool_init(&bpool_params, &dummy_pool[i]);
 		if (err != 0 || !dummy_pool[i]) {
-			MRVL_LOG(ERR, "BPool init failed!\n");
+			MRVL_LOG(ERR, "BPool init failed!");
 			used_bpools[i] &= ~(1 << dummy_pool_id[i]);
 			return -1;
 		}
@@ -415,10 +415,10 @@ mrvl_set_tx_function(struct rte_eth_dev *dev)
 
 	/* Use a simple Tx queue (no offloads, no multi segs) if possible */
 	if (priv->multiseg) {
-		RTE_LOG(INFO, PMD, "Using multi-segment tx callback\n");
+		MRVL_LOG(INFO, "Using multi-segment tx callback");
 		dev->tx_pkt_burst = mrvl_tx_sg_pkt_burst;
 	} else {
-		RTE_LOG(INFO, PMD, "Using single-segment tx callback\n");
+		MRVL_LOG(INFO, "Using single-segment tx callback");
 		dev->tx_pkt_burst = mrvl_tx_pkt_burst;
 	}
 }
@@ -491,7 +491,7 @@ mrvl_dev_configure(struct rte_eth_dev *dev)
 	}
 
 	if (dev->data->dev_conf.rxmode.mtu > priv->max_mtu) {
-		MRVL_LOG(ERR, "MTU %u is larger than max_mtu %u\n",
+		MRVL_LOG(ERR, "MTU %u is larger than max_mtu %u",
 			 dev->data->dev_conf.rxmode.mtu,
 			 priv->max_mtu);
 		return -EINVAL;
@@ -770,7 +770,7 @@ static int mrvl_populate_vlan_table(struct rte_eth_dev *dev, int on)
 				vbit >>= 1;
 			ret = mrvl_vlan_filter_set(dev, vlan, on);
 			if (ret) {
-				MRVL_LOG(ERR, "Failed to setup VLAN filter\n");
+				MRVL_LOG(ERR, "Failed to setup VLAN filter");
 				return ret;
 			}
 		}
@@ -1486,7 +1486,8 @@ mrvl_mac_addr_set(struct rte_eth_dev *dev, struct rte_ether_addr *mac_addr)
  *   0 on success, negative error value otherwise.
  */
 static int
-mrvl_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
+mrvl_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats,
+	       struct eth_queue_stats *qstats)
 {
 	struct mrvl_priv *priv = dev->data->dev_private;
 	struct pp2_ppio_statistics ppio_stats;
@@ -1521,12 +1522,14 @@ mrvl_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 			break;
 		}
 
-		stats->q_ibytes[idx] = rxq->bytes_recv;
-		stats->q_ipackets[idx] = rx_stats.enq_desc - rxq->drop_mac;
-		stats->q_errors[idx] = rx_stats.drop_early +
-				       rx_stats.drop_fullq +
-				       rx_stats.drop_bm +
-				       rxq->drop_mac;
+		if (qstats != NULL) {
+			qstats->q_ibytes[idx] = rxq->bytes_recv;
+			qstats->q_ipackets[idx] = rx_stats.enq_desc - rxq->drop_mac;
+			qstats->q_errors[idx] = rx_stats.drop_early +
+					       rx_stats.drop_fullq +
+					       rx_stats.drop_bm +
+					       rxq->drop_mac;
+		}
 		stats->ibytes += rxq->bytes_recv;
 		drop_mac += rxq->drop_mac;
 	}
@@ -1553,8 +1556,10 @@ mrvl_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 			break;
 		}
 
-		stats->q_opackets[idx] = tx_stats.deq_desc;
-		stats->q_obytes[idx] = txq->bytes_sent;
+		if (qstats != NULL) {
+			qstats->q_opackets[idx] = tx_stats.deq_desc;
+			qstats->q_obytes[idx] = txq->bytes_sent;
+		}
 		stats->obytes += txq->bytes_sent;
 	}
 
@@ -1764,7 +1769,8 @@ mrvl_dev_infos_get(struct rte_eth_dev *dev,
  *   Const pointer to the table with supported packet types.
  */
 static const uint32_t *
-mrvl_dev_supported_ptypes_get(struct rte_eth_dev *dev __rte_unused)
+mrvl_dev_supported_ptypes_get(struct rte_eth_dev *dev __rte_unused,
+			      size_t *no_of_elements)
 {
 	static const uint32_t ptypes[] = {
 		RTE_PTYPE_L2_ETHER,
@@ -1777,9 +1783,10 @@ mrvl_dev_supported_ptypes_get(struct rte_eth_dev *dev __rte_unused)
 		RTE_PTYPE_L3_IPV6_EXT,
 		RTE_PTYPE_L2_ETHER_ARP,
 		RTE_PTYPE_L4_TCP,
-		RTE_PTYPE_L4_UDP
+		RTE_PTYPE_L4_UDP,
 	};
 
+	*no_of_elements = RTE_DIM(ptypes);
 	return ptypes;
 }
 
@@ -1873,7 +1880,7 @@ static int mrvl_vlan_offload_set(struct rte_eth_dev *dev, int mask)
 	int ret;
 
 	if (mask & RTE_ETH_VLAN_STRIP_MASK) {
-		MRVL_LOG(ERR, "VLAN stripping is not supported\n");
+		MRVL_LOG(ERR, "VLAN stripping is not supported");
 		return -ENOTSUP;
 	}
 
@@ -1888,7 +1895,7 @@ static int mrvl_vlan_offload_set(struct rte_eth_dev *dev, int mask)
 	}
 
 	if (mask & RTE_ETH_VLAN_EXTEND_MASK) {
-		MRVL_LOG(ERR, "Extend VLAN not supported\n");
+		MRVL_LOG(ERR, "Extend VLAN not supported");
 		return -ENOTSUP;
 	}
 
@@ -1939,7 +1946,7 @@ mrvl_fill_bpool(struct mrvl_rxq *rxq, int num)
 			!= cookie_addr_high) {
 			MRVL_LOG(ERR,
 				"mbuf virtual addr high is out of range "
-				"0x%x instead of 0x%x\n",
+				"0x%x instead of 0x%x",
 				(uint32_t)((uint64_t)mbufs[i] >> 32),
 				(uint32_t)(cookie_addr_high >> 32));
 			goto out;
@@ -2976,8 +2983,7 @@ mrvl_tx_sg_pkt_burst(void *txq, struct rte_mbuf **tx_pkts,
 		 */
 		if (nb_segs > PP2_PPIO_DESC_NUM_FRAGS) {
 			total_descs -= nb_segs;
-			RTE_LOG(ERR, PMD,
-				"Too many segments. Packet won't be sent.\n");
+			MRVL_LOG(ERR, "Too many segments. Packet won't be sent.");
 			break;
 		}
 
