@@ -16,11 +16,27 @@
 #include "../nfp_cpp_bridge.h"
 #include "../nfp_logs.h"
 #include "../nfp_mtr.h"
+#include "../nfp_net_meta.h"
 #include "nfp_flower_ctrl.h"
 #include "nfp_flower_representor.h"
+#include "nfp_flower_service.h"
 
 #define CTRL_VNIC_NB_DESC 512
 
+int
+nfp_flower_pf_stop(struct rte_eth_dev *dev)
+{
+	struct nfp_net_hw_priv *hw_priv;
+	struct nfp_flower_representor *repr;
+
+	repr = dev->data->dev_private;
+	hw_priv = dev->process_private;
+	nfp_flower_cmsg_port_mod(repr->app_fw_flower, repr->port_id, false);
+	(void)nfp_eth_set_configured(hw_priv->pf_dev->cpp, repr->nfp_idx, 0);
+
+	return nfp_net_stop(dev);
+}
+
 int
 nfp_flower_pf_start(struct rte_eth_dev *dev)
 {
@@ -30,6 +46,9 @@ nfp_flower_pf_start(struct rte_eth_dev *dev)
 	uint32_t new_ctrl;
 	uint32_t update = 0;
 	struct nfp_net_hw *net_hw;
+	struct rte_eth_conf *dev_conf;
+	struct rte_eth_rxmode *rxmode;
+	struct nfp_net_hw_priv *hw_priv;
 	struct nfp_flower_representor *repr;
 
 	repr = dev->data->dev_private;
@@ -47,8 +66,14 @@ nfp_flower_pf_start(struct rte_eth_dev *dev)
 	/* Writing configuration parameters in the device */
 	nfp_net_params_setup(net_hw);
 
-	update |= NFP_NET_CFG_UPDATE_RSS;
-	new_ctrl |= nfp_net_cfg_ctrl_rss(hw->cap);
+	dev_conf = &dev->data->dev_conf;
+	rxmode = &dev_conf->rxmode;
+
+	if ((rxmode->mq_mode & RTE_ETH_MQ_RX_RSS_FLAG) != 0) {
+		nfp_net_rss_config_default(dev);
+		update |= NFP_NET_CFG_UPDATE_RSS;
+		new_ctrl |= nfp_net_cfg_ctrl_rss(hw->cap);
+	}
 
 	/* Enable device */
 	new_ctrl |= NFP_NET_CFG_CTRL_ENABLE;
@@ -61,7 +86,7 @@ nfp_flower_pf_start(struct rte_eth_dev *dev)
 	/* If an error when reconfig we avoid to change hw state */
 	ret = nfp_reconfig(hw, new_ctrl, update);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Failed to reconfig PF vnic");
+		PMD_INIT_LOG(ERR, "Failed to reconfig PF vnic.");
 		return -EIO;
 	}
 
@@ -70,10 +95,16 @@ nfp_flower_pf_start(struct rte_eth_dev *dev)
 	/* Setup the freelist ring */
 	ret = nfp_net_rx_freelist_setup(dev);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Error with flower PF vNIC freelist setup");
+		PMD_INIT_LOG(ERR, "Error with flower PF vNIC freelist setup.");
 		return -EIO;
 	}
 
+	hw_priv = dev->process_private;
+	if (hw_priv->pf_dev->multi_pf.enabled) {
+		(void)nfp_eth_set_configured(hw_priv->pf_dev->cpp, repr->nfp_idx, 1);
+		nfp_flower_cmsg_port_mod(repr->app_fw_flower, repr->port_id, true);
+	}
+
 	for (i = 0; i < dev->data->nb_rx_queues; i++)
 		dev->data->rx_queue_state[i] = RTE_ETH_QUEUE_STATE_STARTED;
 	for (i = 0; i < dev->data->nb_tx_queues; i++)
@@ -82,63 +113,6 @@ nfp_flower_pf_start(struct rte_eth_dev *dev)
 	return 0;
 }
 
-/* Reset and stop device. The device can not be restarted. */
-static int
-nfp_flower_pf_close(struct rte_eth_dev *dev)
-{
-	uint16_t i;
-	struct nfp_net_hw *hw;
-	struct nfp_pf_dev *pf_dev;
-	struct nfp_net_txq *this_tx_q;
-	struct nfp_net_rxq *this_rx_q;
-	struct nfp_flower_representor *repr;
-	struct nfp_app_fw_flower *app_fw_flower;
-
-	if (rte_eal_process_type() != RTE_PROC_PRIMARY)
-		return 0;
-
-	repr = dev->data->dev_private;
-	hw = repr->app_fw_flower->pf_hw;
-	pf_dev = hw->pf_dev;
-	app_fw_flower = NFP_PRIV_TO_APP_FW_FLOWER(pf_dev->app_fw_priv);
-
-	nfp_mtr_priv_uninit(pf_dev);
-
-	/*
-	 * We assume that the DPDK application is stopping all the
-	 * threads/queues before calling the device close function.
-	 */
-	nfp_net_disable_queues(dev);
-
-	/* Clear queues */
-	for (i = 0; i < dev->data->nb_tx_queues; i++) {
-		this_tx_q = dev->data->tx_queues[i];
-		nfp_net_reset_tx_queue(this_tx_q);
-	}
-
-	for (i = 0; i < dev->data->nb_rx_queues; i++) {
-		this_rx_q = dev->data->rx_queues[i];
-		nfp_net_reset_rx_queue(this_rx_q);
-	}
-
-	/* Cancel possible impending LSC work here before releasing the port */
-	rte_eal_alarm_cancel(nfp_net_dev_interrupt_delayed_handler, (void *)dev);
-
-	nn_cfg_writeb(&hw->super, NFP_NET_CFG_LSC, 0xff);
-
-	/* Now it is safe to free all PF resources */
-	PMD_DRV_LOG(INFO, "Freeing PF resources");
-	nfp_cpp_area_free(pf_dev->ctrl_area);
-	nfp_cpp_area_free(pf_dev->qc_area);
-	free(pf_dev->hwinfo);
-	free(pf_dev->sym_tbl);
-	nfp_cpp_free(pf_dev->cpp);
-	rte_free(app_fw_flower);
-	rte_free(pf_dev);
-
-	return 0;
-}
-
 static const struct eth_dev_ops nfp_flower_pf_vnic_ops = {
 	.dev_infos_get          = nfp_net_infos_get,
 	.link_update            = nfp_net_link_update,
@@ -146,25 +120,24 @@ static const struct eth_dev_ops nfp_flower_pf_vnic_ops = {
 
 	.dev_start              = nfp_flower_pf_start,
 	.dev_stop               = nfp_net_stop,
-	.dev_close              = nfp_flower_pf_close,
 };
 
 static inline struct nfp_flower_representor *
-nfp_flower_get_repr(struct nfp_net_hw *hw,
+nfp_flower_get_repr(struct nfp_net_hw_priv *hw_priv,
 		uint32_t port_id)
 {
 	uint8_t port;
 	struct nfp_app_fw_flower *app_fw_flower;
 
 	/* Obtain handle to app_fw_flower here */
-	app_fw_flower = NFP_PRIV_TO_APP_FW_FLOWER(hw->pf_dev->app_fw_priv);
+	app_fw_flower = NFP_PRIV_TO_APP_FW_FLOWER(hw_priv->pf_dev->app_fw_priv);
 
 	switch (NFP_FLOWER_CMSG_PORT_TYPE(port_id)) {
 	case NFP_FLOWER_CMSG_PORT_TYPE_PHYS_PORT:
 		port =  NFP_FLOWER_CMSG_PORT_PHYS_PORT_NUM(port_id);
 		return app_fw_flower->phy_reprs[port];
 	case NFP_FLOWER_CMSG_PORT_TYPE_PCIE_PORT:
-		port = NFP_FLOWER_CMSG_PORT_VNIC(port_id);
+		port = NFP_FLOWER_CMSG_PORT_VNIC_OFFSET(port_id, hw_priv->pf_dev->vf_base_id);
 		return app_fw_flower->vf_reprs[port];
 	default:
 		break;
@@ -174,24 +147,26 @@ nfp_flower_get_repr(struct nfp_net_hw *hw,
 }
 
 bool
-nfp_flower_pf_dispatch_pkts(struct nfp_net_hw *hw,
+nfp_flower_pf_dispatch_pkts(struct nfp_net_rxq *rxq,
 		struct rte_mbuf *mbuf,
 		uint32_t port_id)
 {
 	struct nfp_flower_representor *repr;
 
-	repr = nfp_flower_get_repr(hw, port_id);
+	repr = nfp_flower_get_repr(rxq->hw_priv, port_id);
 	if (repr == NULL) {
-		PMD_RX_LOG(ERR, "Can not get repr for port %u", hw->idx);
+		PMD_RX_LOG(ERR, "Can not get repr for port %u.", port_id);
 		return false;
 	}
 
-	if (repr->ring == NULL) {
-		PMD_RX_LOG(ERR, "No ring available for repr_port %s", repr->name);
+	if (repr->ring == NULL || repr->ring[rxq->qidx] == NULL) {
+		PMD_RX_LOG(ERR, "No ring available for repr_port %s.", repr->name);
 		return false;
 	}
 
-	rte_ring_enqueue(repr->ring, (void *)mbuf);
+	if (rte_ring_enqueue(repr->ring[rxq->qidx], (void *)mbuf) != 0)
+		return false;
+
 	return true;
 }
 
@@ -212,15 +187,15 @@ nfp_flower_pf_nfdk_xmit_pkts(void *tx_queue,
 }
 
 static void
-nfp_flower_pf_xmit_pkts_register(struct nfp_app_fw_flower *app_fw_flower)
+nfp_flower_pf_xmit_pkts_register(struct nfp_pf_dev *pf_dev)
 {
-	struct nfp_net_hw *hw;
 	struct nfp_flower_nfd_func *nfd_func;
+	struct nfp_app_fw_flower *app_fw_flower;
 
-	hw = app_fw_flower->pf_hw;
+	app_fw_flower = pf_dev->app_fw_priv;
 	nfd_func = &app_fw_flower->nfd_func;
 
-	if (hw->ver.extend == NFP_NET_CFG_VERSION_DP_NFD3)
+	if (pf_dev->ver.extend == NFP_NET_CFG_VERSION_DP_NFD3)
 		nfd_func->pf_xmit_t = nfp_flower_pf_nfd3_xmit_pkts;
 	else
 		nfd_func->pf_xmit_t = nfp_flower_pf_nfdk_xmit_pkts;
@@ -235,13 +210,82 @@ nfp_flower_pf_xmit_pkts(void *tx_queue,
 	struct nfp_app_fw_flower *app_fw_flower;
 
 	txq = tx_queue;
-	app_fw_flower = txq->hw->pf_dev->app_fw_priv;
+	app_fw_flower = txq->hw_priv->pf_dev->app_fw_priv;
 
 	return app_fw_flower->nfd_func.pf_xmit_t(tx_queue, tx_pkts, nb_pkts);
 }
 
+uint16_t
+nfp_flower_multiple_pf_recv_pkts(void *rx_queue,
+		struct rte_mbuf **rx_pkts,
+		uint16_t nb_pkts)
+{
+	int i;
+	uint16_t recv;
+	uint32_t data_len;
+	struct nfp_net_rxq *rxq;
+	struct rte_eth_dev *repr_dev;
+	struct nfp_flower_representor *repr;
+
+	recv = nfp_net_recv_pkts(rx_queue, rx_pkts, nb_pkts);
+	if (recv != 0) {
+		/* Grab a handle to the representor struct */
+		rxq = rx_queue;
+		repr_dev = &rte_eth_devices[rxq->port_id];
+		repr = repr_dev->data->dev_private;
+
+		data_len = 0;
+		for (i = 0; i < recv; i++)
+			data_len += rx_pkts[i]->data_len;
+
+		repr->repr_qstats.q_ipackets[rxq->qidx] += recv;
+		repr->repr_qstats.q_ibytes[rxq->qidx] += data_len;
+	}
+
+	return recv;
+}
+
+uint16_t
+nfp_flower_multiple_pf_xmit_pkts(void *tx_queue,
+		struct rte_mbuf **tx_pkts,
+		uint16_t nb_pkts)
+{
+	int i;
+	uint16_t sent;
+	uint32_t data_len;
+	struct nfp_net_txq *txq;
+	struct rte_eth_dev *repr_dev;
+	struct nfp_flower_representor *repr;
+
+	txq = tx_queue;
+	if (unlikely(txq == NULL)) {
+		PMD_TX_LOG(ERR, "TX Bad queue.");
+		return 0;
+	}
+
+	/* Grab a handle to the representor struct */
+	repr_dev = &rte_eth_devices[txq->port_id];
+	repr = repr_dev->data->dev_private;
+	for (i = 0; i < nb_pkts; i++)
+		nfp_flower_pkt_add_metadata(repr->app_fw_flower,
+				tx_pkts[i], repr->port_id);
+
+	sent = nfp_flower_pf_xmit_pkts(tx_queue, tx_pkts, nb_pkts);
+	if (sent != 0) {
+		data_len = 0;
+		for (i = 0; i < sent; i++)
+			data_len += tx_pkts[i]->data_len;
+
+		repr->repr_qstats.q_opackets[txq->qidx] += sent;
+		repr->repr_qstats.q_obytes[txq->qidx] += data_len;
+	}
+
+	return sent;
+}
+
 static int
-nfp_flower_init_vnic_common(struct nfp_net_hw *hw,
+nfp_flower_init_vnic_common(struct nfp_net_hw_priv *hw_priv,
+		struct nfp_net_hw *hw,
 		const char *vnic_type)
 {
 	int err;
@@ -249,14 +293,12 @@ nfp_flower_init_vnic_common(struct nfp_net_hw *hw,
 	uint64_t rx_bar_off;
 	uint64_t tx_bar_off;
 	struct nfp_pf_dev *pf_dev;
-	struct rte_pci_device *pci_dev;
 
-	pf_dev = hw->pf_dev;
-	pci_dev = hw->pf_dev->pci_dev;
+	pf_dev = hw_priv->pf_dev;
 
-	PMD_INIT_LOG(DEBUG, "%s vNIC ctrl bar: %p", vnic_type, hw->super.ctrl_bar);
+	PMD_INIT_LOG(DEBUG, "%s vNIC ctrl bar: %p.", vnic_type, hw->super.ctrl_bar);
 
-	err = nfp_net_common_init(pci_dev, hw);
+	err = nfp_net_common_init(pf_dev, hw);
 	if (err != 0)
 		return err;
 
@@ -285,13 +327,15 @@ nfp_flower_init_vnic_common(struct nfp_net_hw *hw,
 }
 
 static int
-nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
+nfp_flower_init_ctrl_vnic(struct nfp_app_fw_flower *app_fw_flower,
+		struct nfp_net_hw_priv *hw_priv)
 {
 	uint16_t i;
 	int ret = 0;
 	uint16_t n_txq;
 	uint16_t n_rxq;
 	const char *pci_name;
+	struct nfp_net_hw *hw;
 	unsigned int numa_node;
 	struct rte_mempool *mp;
 	struct nfp_net_rxq *rxq;
@@ -299,37 +343,36 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 	struct nfp_pf_dev *pf_dev;
 	struct rte_eth_dev *eth_dev;
 	const struct rte_memzone *tz;
-	struct nfp_app_fw_flower *app_fw_flower;
 	char ctrl_rxring_name[RTE_MEMZONE_NAMESIZE];
 	char ctrl_txring_name[RTE_MEMZONE_NAMESIZE];
 	char ctrl_pktmbuf_pool_name[RTE_MEMZONE_NAMESIZE];
 
 	/* Set up some pointers here for ease of use */
-	pf_dev = hw->pf_dev;
-	app_fw_flower = NFP_PRIV_TO_APP_FW_FLOWER(pf_dev->app_fw_priv);
+	pf_dev = hw_priv->pf_dev;
+	hw = app_fw_flower->ctrl_hw;
 
-	ret = nfp_flower_init_vnic_common(hw, "ctrl_vnic");
+	ret = nfp_flower_init_vnic_common(hw_priv, hw, "ctrl_vnic");
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not init pf vnic");
+		PMD_INIT_LOG(ERR, "Could not init pf vnic.");
 		return -EINVAL;
 	}
 
 	/* Allocate memory for the eth_dev of the vNIC */
-	hw->eth_dev = rte_zmalloc("nfp_ctrl_vnic",
+	app_fw_flower->ctrl_ethdev = rte_zmalloc("nfp_ctrl_vnic",
 			sizeof(struct rte_eth_dev), RTE_CACHE_LINE_SIZE);
-	if (hw->eth_dev == NULL) {
-		PMD_INIT_LOG(ERR, "Could not allocate ctrl vnic");
+	if (app_fw_flower->ctrl_ethdev == NULL) {
+		PMD_INIT_LOG(ERR, "Could not allocate ctrl vnic.");
 		return -ENOMEM;
 	}
 
 	/* Grab the pointer to the newly created rte_eth_dev here */
-	eth_dev = hw->eth_dev;
+	eth_dev = app_fw_flower->ctrl_ethdev;
 
 	/* Also allocate memory for the data part of the eth_dev */
 	eth_dev->data = rte_zmalloc("nfp_ctrl_vnic_data",
 			sizeof(struct rte_eth_dev_data), RTE_CACHE_LINE_SIZE);
 	if (eth_dev->data == NULL) {
-		PMD_INIT_LOG(ERR, "Could not allocate ctrl vnic data");
+		PMD_INIT_LOG(ERR, "Could not allocate ctrl vnic data.");
 		ret = -ENOMEM;
 		goto eth_dev_cleanup;
 	}
@@ -344,7 +387,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 			rte_pktmbuf_pool_create(ctrl_pktmbuf_pool_name,
 			4 * CTRL_VNIC_NB_DESC, 64, 0, 9216, numa_node);
 	if (app_fw_flower->ctrl_pktmbuf_pool == NULL) {
-		PMD_INIT_LOG(ERR, "Create mbuf pool for ctrl vnic failed");
+		PMD_INIT_LOG(ERR, "Create mbuf pool for ctrl vnic failed.");
 		ret = -ENOMEM;
 		goto dev_data_cleanup;
 	}
@@ -358,7 +401,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 			sizeof(eth_dev->data->rx_queues[0]) * n_rxq,
 			RTE_CACHE_LINE_SIZE);
 	if (eth_dev->data->rx_queues == NULL) {
-		PMD_INIT_LOG(ERR, "rte_zmalloc failed for ctrl vNIC rx queues");
+		PMD_INIT_LOG(ERR, "The rte_zmalloc failed for ctrl vNIC rx queues.");
 		ret = -ENOMEM;
 		goto mempool_cleanup;
 	}
@@ -367,7 +410,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 			sizeof(eth_dev->data->tx_queues[0]) * n_txq,
 			RTE_CACHE_LINE_SIZE);
 	if (eth_dev->data->tx_queues == NULL) {
-		PMD_INIT_LOG(ERR, "rte_zmalloc failed for ctrl vNIC tx queues");
+		PMD_INIT_LOG(ERR, "The rte_zmalloc failed for ctrl vNIC tx queues.");
 		ret = -ENOMEM;
 		goto rx_queue_free;
 	}
@@ -385,7 +428,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 				sizeof(struct nfp_net_rxq), RTE_CACHE_LINE_SIZE,
 				numa_node);
 		if (rxq == NULL) {
-			PMD_DRV_LOG(ERR, "Error allocating rxq");
+			PMD_DRV_LOG(ERR, "Error allocating rxq.");
 			ret = -ENOMEM;
 			goto rx_queue_setup_cleanup;
 		}
@@ -416,10 +459,10 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 		 */
 		tz = rte_eth_dma_zone_reserve(eth_dev, ctrl_rxring_name, i,
 				sizeof(struct nfp_net_rx_desc) *
-				hw->dev_info->max_qc_size,
+				hw_priv->dev_info->max_qc_size,
 				NFP_MEMZONE_ALIGN, numa_node);
 		if (tz == NULL) {
-			PMD_DRV_LOG(ERR, "Error allocating rx dma");
+			PMD_DRV_LOG(ERR, "Error allocating rx dma.");
 			rte_free(rxq);
 			ret = -ENOMEM;
 			goto rx_queue_setup_cleanup;
@@ -443,6 +486,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 		nfp_net_reset_rx_queue(rxq);
 
 		rxq->hw = hw;
+		rxq->hw_priv = hw_priv;
 
 		/*
 		 * Telling the HW about the physical address of the RX ring and number
@@ -459,7 +503,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 				sizeof(struct nfp_net_txq), RTE_CACHE_LINE_SIZE,
 				numa_node);
 		if (txq == NULL) {
-			PMD_DRV_LOG(ERR, "Error allocating txq");
+			PMD_DRV_LOG(ERR, "Error allocating txq.");
 			ret = -ENOMEM;
 			goto tx_queue_setup_cleanup;
 		}
@@ -473,10 +517,10 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 		 */
 		tz = rte_eth_dma_zone_reserve(eth_dev, ctrl_txring_name, i,
 				sizeof(struct nfp_net_nfd3_tx_desc) *
-				hw->dev_info->max_qc_size,
+				hw_priv->dev_info->max_qc_size,
 				NFP_MEMZONE_ALIGN, numa_node);
 		if (tz == NULL) {
-			PMD_DRV_LOG(ERR, "Error allocating tx dma");
+			PMD_DRV_LOG(ERR, "Error allocating tx dma.");
 			rte_free(txq);
 			ret = -ENOMEM;
 			goto tx_queue_setup_cleanup;
@@ -508,6 +552,7 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 		nfp_net_reset_tx_queue(txq);
 
 		txq->hw = hw;
+		txq->hw_priv = hw_priv;
 
 		/*
 		 * Telling the HW about the physical address of the TX ring and number
@@ -517,6 +562,13 @@ nfp_flower_init_ctrl_vnic(struct nfp_net_hw *hw)
 		nn_cfg_writeb(&hw->super, NFP_NET_CFG_TXR_SZ(i), rte_log2_u32(CTRL_VNIC_NB_DESC));
 	}
 
+	/* Alloc sync memory zone */
+	ret = nfp_flower_service_sync_alloc(hw_priv);
+	if (ret != 0) {
+		PMD_INIT_LOG(ERR, "Alloc sync memory zone failed.");
+		goto tx_queue_setup_cleanup;
+	}
+
 	return 0;
 
 tx_queue_setup_cleanup:
@@ -551,21 +603,24 @@ eth_dev_cleanup:
 }
 
 static void
-nfp_flower_cleanup_ctrl_vnic(struct nfp_net_hw *hw)
+nfp_flower_cleanup_ctrl_vnic(struct nfp_app_fw_flower *app_fw_flower,
+		struct nfp_net_hw_priv *hw_priv)
 {
 	uint32_t i;
 	const char *pci_name;
+	struct nfp_net_hw *hw;
 	struct nfp_net_rxq *rxq;
 	struct nfp_net_txq *txq;
 	struct rte_eth_dev *eth_dev;
-	struct nfp_app_fw_flower *app_fw_flower;
 	char ctrl_txring_name[RTE_MEMZONE_NAMESIZE];
 	char ctrl_rxring_name[RTE_MEMZONE_NAMESIZE];
 
-	eth_dev = hw->eth_dev;
-	app_fw_flower = NFP_PRIV_TO_APP_FW_FLOWER(hw->pf_dev->app_fw_priv);
+	hw = app_fw_flower->ctrl_hw;
+	eth_dev = app_fw_flower->ctrl_ethdev;
 
-	pci_name = strchr(app_fw_flower->pf_hw->pf_dev->pci_dev->name, ':') + 1;
+	pci_name = strchr(hw_priv->pf_dev->pci_dev->name, ':') + 1;
+
+	nfp_net_disable_queues(eth_dev);
 
 	snprintf(ctrl_txring_name, sizeof(ctrl_txring_name), "%s_cttx_ring", pci_name);
 	for (i = 0; i < hw->max_tx_queues; i++) {
@@ -587,6 +642,7 @@ nfp_flower_cleanup_ctrl_vnic(struct nfp_net_hw *hw)
 		}
 	}
 
+	nfp_flower_service_sync_free(hw_priv);
 	rte_free(eth_dev->data->tx_queues);
 	rte_free(eth_dev->data->rx_queues);
 	rte_mempool_free(app_fw_flower->ctrl_pktmbuf_pool);
@@ -595,15 +651,17 @@ nfp_flower_cleanup_ctrl_vnic(struct nfp_net_hw *hw)
 }
 
 static int
-nfp_flower_start_ctrl_vnic(struct nfp_net_hw *net_hw)
+nfp_flower_start_ctrl_vnic(struct nfp_app_fw_flower *app_fw_flower)
 {
 	int ret;
 	uint32_t update;
 	uint32_t new_ctrl;
 	struct nfp_hw *hw;
 	struct rte_eth_dev *dev;
+	struct nfp_net_hw *net_hw;
 
-	dev = net_hw->eth_dev;
+	net_hw = app_fw_flower->ctrl_hw;
+	dev = app_fw_flower->ctrl_ethdev;
 	hw = &net_hw->super;
 
 	/* Disabling queues just in case... */
@@ -624,7 +682,7 @@ nfp_flower_start_ctrl_vnic(struct nfp_net_hw *net_hw)
 	/* If an error when reconfig we avoid to change hw state */
 	ret = nfp_reconfig(hw, new_ctrl, update);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Failed to reconfig ctrl vnic");
+		PMD_INIT_LOG(ERR, "Failed to reconfig ctrl vnic.");
 		return -EIO;
 	}
 
@@ -633,64 +691,23 @@ nfp_flower_start_ctrl_vnic(struct nfp_net_hw *net_hw)
 	/* Setup the freelist ring */
 	ret = nfp_net_rx_freelist_setup(dev);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Error with flower ctrl vNIC freelist setup");
+		PMD_INIT_LOG(ERR, "Error with flower ctrl vNIC freelist setup.");
 		return -EIO;
 	}
 
 	return 0;
 }
 
-static int
-nfp_flower_ctrl_vnic_service(void *arg)
-{
-	struct nfp_app_fw_flower *app_fw_flower = arg;
-
-	nfp_flower_ctrl_vnic_poll(app_fw_flower);
-
-	return 0;
-}
-
-static int
-nfp_flower_enable_services(struct nfp_app_fw_flower *app_fw_flower)
-{
-	int ret;
-	uint32_t service_id;
-	const struct rte_service_spec flower_service = {
-		.name              = "flower_ctrl_vnic_service",
-		.callback          = nfp_flower_ctrl_vnic_service,
-		.callback_userdata = (void *)app_fw_flower,
-	};
-
-	/* Register the flower services */
-	ret = rte_service_component_register(&flower_service, &service_id);
-	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not register %s", flower_service.name);
-		return -EINVAL;
-	}
-
-	app_fw_flower->ctrl_vnic_id = service_id;
-	PMD_INIT_LOG(INFO, "%s registered", flower_service.name);
-
-	/* Map them to available service cores */
-	ret = nfp_map_service(service_id);
-	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not map %s", flower_service.name);
-		return -EINVAL;
-	}
-
-	return 0;
-}
-
 static void
-nfp_flower_pkt_add_metadata_register(struct nfp_app_fw_flower *app_fw_flower)
+nfp_flower_pkt_add_metadata_register(struct nfp_pf_dev *pf_dev)
 {
-	struct nfp_net_hw *hw;
 	struct nfp_flower_nfd_func *nfd_func;
+	struct nfp_app_fw_flower *app_fw_flower;
 
-	hw = app_fw_flower->pf_hw;
+	app_fw_flower = pf_dev->app_fw_priv;
 	nfd_func = &app_fw_flower->nfd_func;
 
-	if (hw->ver.extend == NFP_NET_CFG_VERSION_DP_NFD3)
+	if (pf_dev->ver.extend == NFP_NET_CFG_VERSION_DP_NFD3)
 		nfd_func->pkt_add_metadata_t = nfp_flower_nfd3_pkt_add_metadata;
 	else
 		nfd_func->pkt_add_metadata_t = nfp_flower_nfdk_pkt_add_metadata;
@@ -705,32 +722,36 @@ nfp_flower_pkt_add_metadata(struct nfp_app_fw_flower *app_fw_flower,
 }
 
 static void
-nfp_flower_nfd_func_register(struct nfp_app_fw_flower *app_fw_flower)
+nfp_flower_nfd_func_register(struct nfp_pf_dev *pf_dev)
 {
-	nfp_flower_pkt_add_metadata_register(app_fw_flower);
-	nfp_flower_ctrl_vnic_xmit_register(app_fw_flower);
-	nfp_flower_pf_xmit_pkts_register(app_fw_flower);
+	nfp_flower_pkt_add_metadata_register(pf_dev);
+	nfp_flower_ctrl_vnic_xmit_register(pf_dev);
+	nfp_flower_pf_xmit_pkts_register(pf_dev);
 }
 
 int
-nfp_init_app_fw_flower(struct nfp_pf_dev *pf_dev,
-		const struct nfp_dev_info *dev_info)
+nfp_init_app_fw_flower(struct nfp_net_hw_priv *hw_priv)
 {
 	int ret;
 	int err;
+	uint8_t id;
 	uint64_t ext_features;
 	unsigned int numa_node;
 	struct nfp_net_hw *pf_hw;
 	struct nfp_net_hw *ctrl_hw;
+	char bar_name[RTE_ETH_NAME_MAX_LEN];
+	char ctrl_name[RTE_ETH_NAME_MAX_LEN];
 	struct nfp_app_fw_flower *app_fw_flower;
+	struct nfp_pf_dev *pf_dev = hw_priv->pf_dev;
 
 	numa_node = rte_socket_id();
+	id = nfp_function_id_get(pf_dev, 0);
 
 	/* Allocate memory for the Flower app */
 	app_fw_flower = rte_zmalloc_socket("nfp_app_fw_flower", sizeof(*app_fw_flower),
 			RTE_CACHE_LINE_SIZE, numa_node);
 	if (app_fw_flower == NULL) {
-		PMD_INIT_LOG(ERR, "Could not malloc app fw flower");
+		PMD_INIT_LOG(ERR, "Could not malloc app fw flower.");
 		return -ENOMEM;
 	}
 
@@ -738,13 +759,13 @@ nfp_init_app_fw_flower(struct nfp_pf_dev *pf_dev,
 
 	ret = nfp_flow_priv_init(pf_dev);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "init flow priv failed");
+		PMD_INIT_LOG(ERR, "Init flow priv failed.");
 		goto app_cleanup;
 	}
 
 	ret = nfp_mtr_priv_init(pf_dev);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Error initializing metering private data");
+		PMD_INIT_LOG(ERR, "Error initializing metering private data.");
 		goto flow_priv_cleanup;
 	}
 
@@ -752,16 +773,17 @@ nfp_init_app_fw_flower(struct nfp_pf_dev *pf_dev,
 	pf_hw = rte_zmalloc_socket("nfp_pf_vnic", 2 * sizeof(struct nfp_net_hw),
 			RTE_CACHE_LINE_SIZE, numa_node);
 	if (pf_hw == NULL) {
-		PMD_INIT_LOG(ERR, "Could not malloc nfp pf vnic");
+		PMD_INIT_LOG(ERR, "Could not malloc nfp pf vnic.");
 		ret = -ENOMEM;
 		goto mtr_priv_cleanup;
 	}
 
 	/* Map the PF ctrl bar */
-	pf_dev->ctrl_bar = nfp_rtsym_map(pf_dev->sym_tbl, "_pf0_net_bar0",
-			NFP_NET_CFG_BAR_SZ, &pf_dev->ctrl_area);
+	snprintf(bar_name, sizeof(bar_name), "_pf%u_net_bar0", id);
+	pf_dev->ctrl_bar = nfp_rtsym_map(pf_dev->sym_tbl, bar_name,
+			pf_dev->ctrl_bar_size, &pf_dev->ctrl_area);
 	if (pf_dev->ctrl_bar == NULL) {
-		PMD_INIT_LOG(ERR, "Cloud not map the PF vNIC ctrl bar");
+		PMD_INIT_LOG(ERR, "Could not map the PF vNIC ctrl bar.");
 		ret = -ENODEV;
 		goto vnic_cleanup;
 	}
@@ -770,7 +792,7 @@ nfp_init_app_fw_flower(struct nfp_pf_dev *pf_dev,
 	ext_features = nfp_rtsym_read_le(pf_dev->sym_tbl, "_abi_flower_extra_features",
 			&err);
 	if (err != 0) {
-		PMD_INIT_LOG(ERR, "Couldn't read extra features from fw");
+		PMD_INIT_LOG(ERR, "Could not read extra features from fw.");
 		ret = -EIO;
 		goto pf_cpp_area_cleanup;
 	}
@@ -781,67 +803,69 @@ nfp_init_app_fw_flower(struct nfp_pf_dev *pf_dev,
 	/* Fill in the PF vNIC and populate app struct */
 	app_fw_flower->pf_hw = pf_hw;
 	pf_hw->super.ctrl_bar = pf_dev->ctrl_bar;
-	pf_hw->pf_dev = pf_dev;
-	pf_hw->cpp = pf_dev->cpp;
-	pf_hw->dev_info = dev_info;
+	pf_hw->nfp_idx = pf_dev->nfp_eth_table->ports[id].index;
 
-	ret = nfp_flower_init_vnic_common(app_fw_flower->pf_hw, "pf_vnic");
+	ret = nfp_flower_init_vnic_common(hw_priv, pf_hw, "pf_vnic");
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not initialize flower PF vNIC");
+		PMD_INIT_LOG(ERR, "Could not initialize flower PF vNIC.");
 		goto pf_cpp_area_cleanup;
 	}
 
-	nfp_flower_nfd_func_register(app_fw_flower);
+	ret = nfp_net_vf_config_app_init(pf_hw, pf_dev);
+	if (ret != 0) {
+		PMD_INIT_LOG(ERR, "Failed to init sriov module.");
+		goto pf_cpp_area_cleanup;
+	}
+
+	nfp_flower_nfd_func_register(pf_dev);
 
 	/* The ctrl vNIC struct comes directly after the PF one */
 	app_fw_flower->ctrl_hw = pf_hw + 1;
 	ctrl_hw = app_fw_flower->ctrl_hw;
 
 	/* Map the ctrl vNIC ctrl bar */
-	ctrl_hw->super.ctrl_bar = nfp_rtsym_map(pf_dev->sym_tbl, "_pf0_net_ctrl_bar",
-			NFP_NET_CFG_BAR_SZ, &ctrl_hw->ctrl_area);
+	snprintf(ctrl_name, sizeof(ctrl_name), "_pf%u_net_ctrl_bar", id);
+	ctrl_hw->super.ctrl_bar = nfp_rtsym_map(pf_dev->sym_tbl, ctrl_name,
+			pf_dev->ctrl_bar_size, &ctrl_hw->ctrl_area);
 	if (ctrl_hw->super.ctrl_bar == NULL) {
-		PMD_INIT_LOG(ERR, "Cloud not map the ctrl vNIC ctrl bar");
+		PMD_INIT_LOG(ERR, "Could not map the ctrl vNIC ctrl bar.");
 		ret = -ENODEV;
 		goto pf_cpp_area_cleanup;
 	}
 
-	/* Now populate the ctrl vNIC */
-	ctrl_hw->pf_dev = pf_dev;
-	ctrl_hw->cpp = pf_dev->cpp;
-	ctrl_hw->dev_info = dev_info;
-
-	ret = nfp_flower_init_ctrl_vnic(app_fw_flower->ctrl_hw);
+	ret = nfp_flower_init_ctrl_vnic(app_fw_flower, hw_priv);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not initialize flower ctrl vNIC");
+		PMD_INIT_LOG(ERR, "Could not initialize flower ctrl vNIC.");
 		goto ctrl_cpp_area_cleanup;
 	}
 
 	/* Start the ctrl vNIC */
-	ret = nfp_flower_start_ctrl_vnic(app_fw_flower->ctrl_hw);
+	ret = nfp_flower_start_ctrl_vnic(app_fw_flower);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not start flower ctrl vNIC");
+		PMD_INIT_LOG(ERR, "Could not start flower ctrl vNIC.");
 		goto ctrl_vnic_cleanup;
 	}
 
 	/* Start up flower services */
-	ret = nfp_flower_enable_services(app_fw_flower);
+	ret = nfp_flower_service_start(hw_priv);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not enable flower services");
+		PMD_INIT_LOG(ERR, "Could not enable flower services.");
 		ret = -ESRCH;
 		goto ctrl_vnic_cleanup;
 	}
 
-	ret = nfp_flower_repr_create(app_fw_flower);
+	ret = nfp_flower_repr_create(app_fw_flower, hw_priv);
 	if (ret != 0) {
-		PMD_INIT_LOG(ERR, "Could not create representor ports");
-		goto ctrl_vnic_cleanup;
+		PMD_INIT_LOG(ERR, "Could not create representor ports.");
+		goto ctrl_vnic_service_stop;
 	}
 
 	return 0;
 
+ctrl_vnic_service_stop:
+	nfp_flower_service_stop(hw_priv);
 ctrl_vnic_cleanup:
-	nfp_flower_cleanup_ctrl_vnic(app_fw_flower->ctrl_hw);
+	nfp_flower_cleanup_ctrl_vnic(app_fw_flower, hw_priv);
 ctrl_cpp_area_cleanup:
 	nfp_cpp_area_free(ctrl_hw->ctrl_area);
 pf_cpp_area_cleanup:
@@ -858,25 +882,54 @@ app_cleanup:
 	return ret;
 }
 
-int
-nfp_secondary_init_app_fw_flower(struct nfp_pf_dev *pf_dev)
+void
+nfp_uninit_app_fw_flower(struct nfp_net_hw_priv *hw_priv)
 {
-	struct rte_eth_dev *eth_dev;
-	const char *port_name = "pf_vnic_eth_dev";
+	struct nfp_app_fw_flower *app_fw_flower;
+	struct nfp_pf_dev *pf_dev = hw_priv->pf_dev;
 
-	PMD_INIT_LOG(DEBUG, "Secondary attaching to port %s", port_name);
+	app_fw_flower = pf_dev->app_fw_priv;
+	nfp_flower_cleanup_ctrl_vnic(app_fw_flower, hw_priv);
+	nfp_cpp_area_free(app_fw_flower->ctrl_hw->ctrl_area);
+	nfp_cpp_area_free(pf_dev->ctrl_area);
+	rte_free(app_fw_flower->pf_hw);
+	nfp_mtr_priv_uninit(pf_dev);
+	nfp_flow_priv_uninit(pf_dev);
+	if (rte_eth_switch_domain_free(app_fw_flower->switch_domain_id) != 0)
+		PMD_DRV_LOG(WARNING, "Failed to free switch domain for device.");
+	rte_free(app_fw_flower);
+}
 
-	eth_dev = rte_eth_dev_attach_secondary(port_name);
-	if (eth_dev == NULL) {
-		PMD_INIT_LOG(ERR, "Secondary process attach to port %s failed", port_name);
-		return -ENODEV;
-	}
-
-	eth_dev->process_private = pf_dev->cpp;
+static int
+nfp_secondary_flower_init(struct rte_eth_dev *eth_dev,
+		void *para)
+{
+	eth_dev->process_private = para;
 	eth_dev->dev_ops = &nfp_flower_pf_vnic_ops;
 	eth_dev->rx_pkt_burst = nfp_net_recv_pkts;
 	eth_dev->tx_pkt_burst = nfp_flower_pf_xmit_pkts;
-	rte_eth_dev_probing_finish(eth_dev);
+
+	return 0;
+}
+
+int
+nfp_secondary_init_app_fw_flower(struct nfp_net_hw_priv *hw_priv)
+{
+	int ret;
+	const char *pci_name;
+	char port_name[RTE_ETH_NAME_MAX_LEN];
+
+	pci_name = strchr(hw_priv->pf_dev->pci_dev->name, ':') + 1;
+	snprintf(port_name, RTE_ETH_NAME_MAX_LEN, "%s_repr_pf", pci_name);
+
+	PMD_INIT_LOG(DEBUG, "Secondary attaching to port %s.", port_name);
+
+	ret = rte_eth_dev_create(&hw_priv->pf_dev->pci_dev->device, port_name, 0, NULL,
+			NULL, nfp_secondary_flower_init, hw_priv);
+	if (ret != 0) {
+		PMD_INIT_LOG(ERR, "Secondary process attach to port %s failed.", port_name);
+		return -ENODEV;
+	}
 
 	return 0;
 }
