@@ -8,10 +8,9 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 
-#include <rte_service_component.h>
-
 #include "nfpcore/nfp_cpp.h"
 #include "nfp_logs.h"
+#include "nfp_service.h"
 
 #define NFP_CPP_MEMIO_BOUNDARY    (1 << 20)
 #define NFP_BRIDGE_OP_READ        20
@@ -25,84 +24,31 @@
 static int nfp_cpp_bridge_service_func(void *args);
 
 int
-nfp_map_service(uint32_t service_id)
+nfp_enable_cpp_service(struct nfp_pf_dev *pf_dev)
 {
-	int32_t ret;
-	uint32_t slcore = 0;
-	int32_t slcore_count;
-	uint8_t service_count;
-	const char *service_name;
-	uint32_t slcore_array[RTE_MAX_LCORE];
-	uint8_t min_service_count = UINT8_MAX;
+	int ret;
+	const char *pci_name;
+	struct rte_service_spec cpp_service = {
+		.callback          = nfp_cpp_bridge_service_func,
+		.callback_userdata = (void *)pf_dev,
+	};
 
-	slcore_count = rte_service_lcore_list(slcore_array, RTE_MAX_LCORE);
-	if (slcore_count <= 0) {
-		PMD_INIT_LOG(DEBUG, "No service cores found");
-		return -ENOENT;
-	}
+	pci_name = strchr(pf_dev->pci_dev->name, ':') + 1;
+	snprintf(cpp_service.name, sizeof(cpp_service.name), "%s_cpp_service", pci_name);
 
-	/*
-	 * Find a service core with the least number of services already
-	 * registered to it.
-	 */
-	while (slcore_count--) {
-		service_count = rte_service_lcore_count_services(slcore_array[slcore_count]);
-		if (service_count < min_service_count) {
-			slcore = slcore_array[slcore_count];
-			min_service_count = service_count;
-		}
-	}
-
-	service_name = rte_service_get_name(service_id);
-	PMD_INIT_LOG(INFO, "Mapping service %s to core %u", service_name, slcore);
-
-	ret = rte_service_map_lcore_set(service_id, slcore, 1);
+	ret = nfp_service_enable(&cpp_service, &pf_dev->cpp_service_info);
 	if (ret != 0) {
-		PMD_INIT_LOG(DEBUG, "Could not map flower service");
-		return -ENOENT;
+		PMD_INIT_LOG(DEBUG, "Could not enable service %s.", cpp_service.name);
+		return ret;
 	}
 
-	rte_service_runstate_set(service_id, 1);
-	rte_service_component_runstate_set(service_id, 1);
-	rte_service_lcore_start(slcore);
-	if (rte_service_may_be_active(slcore) != 0)
-		PMD_INIT_LOG(INFO, "The service %s is running", service_name);
-	else
-		PMD_INIT_LOG(ERR, "The service %s is not running", service_name);
-
 	return 0;
 }
 
-int
-nfp_enable_cpp_service(struct nfp_pf_dev *pf_dev)
+void
+nfp_disable_cpp_service(struct nfp_pf_dev *pf_dev)
 {
-	int ret;
-	uint32_t service_id = 0;
-	struct rte_service_spec cpp_service = {
-		.name         = "nfp_cpp_service",
-		.callback     = nfp_cpp_bridge_service_func,
-	};
-
-	cpp_service.callback_userdata = (void *)pf_dev;
-
-	/* Register the cpp service */
-	ret = rte_service_component_register(&cpp_service, &service_id);
-	if (ret != 0) {
-		PMD_INIT_LOG(WARNING, "Could not register nfp cpp service");
-		return -EINVAL;
-	}
-
-	pf_dev->cpp_bridge_id = service_id;
-	PMD_INIT_LOG(INFO, "NFP cpp service registered");
-
-	/* Map it to available service core */
-	ret = nfp_map_service(service_id);
-	if (ret != 0) {
-		PMD_INIT_LOG(DEBUG, "Could not map nfp cpp service");
-		return -EINVAL;
-	}
-
-	return 0;
+	nfp_service_disable(&pf_dev->cpp_service_info);
 }
 
 /*
@@ -125,7 +71,7 @@ nfp_cpp_bridge_serve_write(int sockfd,
 	uint32_t tmpbuf[16];
 	struct nfp_cpp_area *area;
 
-	PMD_CPP_LOG(DEBUG, "%s: offset size %zu, count_size: %zu", __func__,
+	PMD_CPP_LOG(DEBUG, "%s: offset size %zu, count_size: %zu.", __func__,
 			sizeof(off_t), sizeof(size_t));
 
 	/* Reading the count param */
@@ -144,9 +90,9 @@ nfp_cpp_bridge_serve_write(int sockfd,
 	cpp_id = (offset >> 40) << 8;
 	nfp_offset = offset & ((1ull << 40) - 1);
 
-	PMD_CPP_LOG(DEBUG, "%s: count %zu and offset %jd", __func__, count,
+	PMD_CPP_LOG(DEBUG, "%s: count %zu and offset %jd.", __func__, count,
 			offset);
-	PMD_CPP_LOG(DEBUG, "%s: cpp_id %08x and nfp_offset %jd", __func__,
+	PMD_CPP_LOG(DEBUG, "%s: cpp_id %08x and nfp_offset %jd.", __func__,
 			cpp_id, nfp_offset);
 
 	/* Adjust length if not aligned */
@@ -161,14 +107,14 @@ nfp_cpp_bridge_serve_write(int sockfd,
 		area = nfp_cpp_area_alloc_with_name(cpp, cpp_id, "nfp.cdev",
 				nfp_offset, curlen);
 		if (area == NULL) {
-			PMD_CPP_LOG(ERR, "area alloc fail");
+			PMD_CPP_LOG(ERR, "Area alloc fail.");
 			return -EIO;
 		}
 
 		/* Mapping the target */
 		err = nfp_cpp_area_acquire(area);
 		if (err < 0) {
-			PMD_CPP_LOG(ERR, "area acquire failed");
+			PMD_CPP_LOG(ERR, "Area acquire failed.");
 			nfp_cpp_area_free(area);
 			return -EIO;
 		}
@@ -178,11 +124,11 @@ nfp_cpp_bridge_serve_write(int sockfd,
 			if (len > sizeof(tmpbuf))
 				len = sizeof(tmpbuf);
 
-			PMD_CPP_LOG(DEBUG, "%s: Receive %u of %zu", __func__,
+			PMD_CPP_LOG(DEBUG, "%s: Receive %u of %zu.", __func__,
 					len, count);
 			err = recv(sockfd, tmpbuf, len, MSG_WAITALL);
 			if (err != (int)len) {
-				PMD_CPP_LOG(ERR, "error when receiving, %d of %zu",
+				PMD_CPP_LOG(ERR, "Error when receiving, %d of %zu.",
 						err, count);
 				nfp_cpp_area_release(area);
 				nfp_cpp_area_free(area);
@@ -191,7 +137,7 @@ nfp_cpp_bridge_serve_write(int sockfd,
 
 			err = nfp_cpp_area_write(area, pos, tmpbuf, len);
 			if (err < 0) {
-				PMD_CPP_LOG(ERR, "nfp_cpp_area_write error");
+				PMD_CPP_LOG(ERR, "The nfp_cpp_area_write error.");
 				nfp_cpp_area_release(area);
 				nfp_cpp_area_free(area);
 				return -EIO;
@@ -231,7 +177,7 @@ nfp_cpp_bridge_serve_read(int sockfd,
 	uint32_t tmpbuf[16];
 	struct nfp_cpp_area *area;
 
-	PMD_CPP_LOG(DEBUG, "%s: offset size %zu, count_size: %zu", __func__,
+	PMD_CPP_LOG(DEBUG, "%s: offset size %zu, count_size: %zu.", __func__,
 			sizeof(off_t), sizeof(size_t));
 
 	/* Reading the count param */
@@ -250,9 +196,9 @@ nfp_cpp_bridge_serve_read(int sockfd,
 	cpp_id = (offset >> 40) << 8;
 	nfp_offset = offset & ((1ull << 40) - 1);
 
-	PMD_CPP_LOG(DEBUG, "%s: count %zu and offset %jd", __func__, count,
+	PMD_CPP_LOG(DEBUG, "%s: count %zu and offset %jd.", __func__, count,
 			offset);
-	PMD_CPP_LOG(DEBUG, "%s: cpp_id %08x and nfp_offset %jd", __func__,
+	PMD_CPP_LOG(DEBUG, "%s: cpp_id %08x and nfp_offset %jd.", __func__,
 			cpp_id, nfp_offset);
 
 	/* Adjust length if not aligned */
@@ -266,13 +212,13 @@ nfp_cpp_bridge_serve_read(int sockfd,
 		area = nfp_cpp_area_alloc_with_name(cpp, cpp_id, "nfp.cdev",
 				nfp_offset, curlen);
 		if (area == NULL) {
-			PMD_CPP_LOG(ERR, "area alloc failed");
+			PMD_CPP_LOG(ERR, "Area alloc failed.");
 			return -EIO;
 		}
 
 		err = nfp_cpp_area_acquire(area);
 		if (err < 0) {
-			PMD_CPP_LOG(ERR, "area acquire failed");
+			PMD_CPP_LOG(ERR, "Area acquire failed.");
 			nfp_cpp_area_free(area);
 			return -EIO;
 		}
@@ -284,17 +230,17 @@ nfp_cpp_bridge_serve_read(int sockfd,
 
 			err = nfp_cpp_area_read(area, pos, tmpbuf, len);
 			if (err < 0) {
-				PMD_CPP_LOG(ERR, "nfp_cpp_area_read error");
+				PMD_CPP_LOG(ERR, "The nfp_cpp_area_read error.");
 				nfp_cpp_area_release(area);
 				nfp_cpp_area_free(area);
 				return -EIO;
 			}
-			PMD_CPP_LOG(DEBUG, "%s: sending %u of %zu", __func__,
+			PMD_CPP_LOG(DEBUG, "%s: sending %u of %zu.", __func__,
 					len, count);
 
 			err = send(sockfd, tmpbuf, len, 0);
 			if (err != (int)len) {
-				PMD_CPP_LOG(ERR, "error when sending: %d of %zu",
+				PMD_CPP_LOG(ERR, "Error when sending: %d of %zu.",
 						err, count);
 				nfp_cpp_area_release(area);
 				nfp_cpp_area_free(area);
@@ -332,39 +278,39 @@ nfp_cpp_bridge_serve_ioctl(int sockfd,
 	/* Reading now the IOCTL command */
 	err = recv(sockfd, &cmd, 4, 0);
 	if (err != 4) {
-		PMD_CPP_LOG(ERR, "read error from socket");
+		PMD_CPP_LOG(ERR, "Read error from socket.");
 		return -EIO;
 	}
 
 	/* Only supporting NFP_IOCTL_CPP_IDENTIFICATION */
 	if (cmd != NFP_IOCTL_CPP_IDENTIFICATION) {
-		PMD_CPP_LOG(ERR, "unknown cmd %d", cmd);
+		PMD_CPP_LOG(ERR, "Unknown cmd %d.", cmd);
 		return -EINVAL;
 	}
 
 	err = recv(sockfd, &ident_size, 4, 0);
 	if (err != 4) {
-		PMD_CPP_LOG(ERR, "read error from socket");
+		PMD_CPP_LOG(ERR, "Read error from socket.");
 		return -EIO;
 	}
 
 	tmp = nfp_cpp_model(cpp);
 
-	PMD_CPP_LOG(DEBUG, "%s: sending NFP model %08x", __func__, tmp);
+	PMD_CPP_LOG(DEBUG, "%s: sending NFP model %08x.", __func__, tmp);
 
 	err = send(sockfd, &tmp, 4, 0);
 	if (err != 4) {
-		PMD_CPP_LOG(ERR, "error writing to socket");
+		PMD_CPP_LOG(ERR, "Error writing to socket.");
 		return -EIO;
 	}
 
 	tmp = nfp_cpp_interface(cpp);
 
-	PMD_CPP_LOG(DEBUG, "%s: sending NFP interface %08x", __func__, tmp);
+	PMD_CPP_LOG(DEBUG, "%s: sending NFP interface %08x.", __func__, tmp);
 
 	err = send(sockfd, &tmp, 4, 0);
 	if (err != 4) {
-		PMD_CPP_LOG(ERR, "error writing to socket");
+		PMD_CPP_LOG(ERR, "Error writing to socket.");
 		return -EIO;
 	}
 
@@ -387,15 +333,21 @@ nfp_cpp_bridge_service_func(void *args)
 	int sockfd;
 	int datafd;
 	struct nfp_cpp *cpp;
+	const char *pci_name;
+	char socket_handle[14];
 	struct sockaddr address;
 	struct nfp_pf_dev *pf_dev;
 	struct timeval timeout = {1, 0};
 
-	unlink("/tmp/nfp_cpp");
+	pf_dev = args;
 
+	pci_name = strchr(pf_dev->pci_dev->name, ':') + 1;
+	snprintf(socket_handle, sizeof(socket_handle), "/tmp/%s", pci_name);
+
+	unlink(socket_handle);
 	sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (sockfd < 0) {
-		PMD_CPP_LOG(ERR, "socket creation error. Service failed");
+		PMD_CPP_LOG(ERR, "Socket creation error. Service failed.");
 		return -EIO;
 	}
 
@@ -404,33 +356,32 @@ nfp_cpp_bridge_service_func(void *args)
 	memset(&address, 0, sizeof(struct sockaddr));
 
 	address.sa_family = AF_UNIX;
-	strcpy(address.sa_data, "/tmp/nfp_cpp");
+	strcpy(address.sa_data, socket_handle);
 
 	ret = bind(sockfd, (const struct sockaddr *)&address,
 			sizeof(struct sockaddr));
 	if (ret < 0) {
-		PMD_CPP_LOG(ERR, "bind error (%d). Service failed", errno);
+		PMD_CPP_LOG(ERR, "Bind error (%d). Service failed.", errno);
 		close(sockfd);
 		return ret;
 	}
 
 	ret = listen(sockfd, 20);
 	if (ret < 0) {
-		PMD_CPP_LOG(ERR, "listen error(%d). Service failed", errno);
+		PMD_CPP_LOG(ERR, "Listen error(%d). Service failed.", errno);
 		close(sockfd);
 		return ret;
 	}
 
-	pf_dev = args;
 	cpp = pf_dev->cpp;
-	while (rte_service_runstate_get(pf_dev->cpp_bridge_id) != 0) {
+	while (rte_service_runstate_get(pf_dev->cpp_service_info.id) != 0) {
 		datafd = accept(sockfd, NULL, NULL);
 		if (datafd < 0) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK)
 				continue;
 
-			PMD_CPP_LOG(ERR, "accept call error (%d)", errno);
-			PMD_CPP_LOG(ERR, "service failed");
+			PMD_CPP_LOG(ERR, "Accept call error (%d).", errno);
+			PMD_CPP_LOG(ERR, "Service failed.");
 			close(sockfd);
 			return -EIO;
 		}
@@ -438,11 +389,11 @@ nfp_cpp_bridge_service_func(void *args)
 		for (;;) {
 			ret = recv(datafd, &op, 4, 0);
 			if (ret <= 0) {
-				PMD_CPP_LOG(DEBUG, "%s: socket close", __func__);
+				PMD_CPP_LOG(DEBUG, "%s: socket close.", __func__);
 				break;
 			}
 
-			PMD_CPP_LOG(DEBUG, "%s: getting op %u", __func__, op);
+			PMD_CPP_LOG(DEBUG, "%s: getting op %u.", __func__, op);
 
 			if (op == NFP_BRIDGE_OP_READ)
 				nfp_cpp_bridge_serve_read(datafd, cpp);
