@@ -263,19 +263,6 @@ nfp_bitsize_calc(uint64_t mask)
 	return bit_size;
 }
 
-static int
-nfp_cmp_bars(const void *ptr_a,
-		const void *ptr_b)
-{
-	const struct nfp_bar *a = ptr_a;
-	const struct nfp_bar *b = ptr_b;
-
-	if (a->bitsize == b->bitsize)
-		return a->index - b->index;
-	else
-		return a->bitsize - b->bitsize;
-}
-
 static bool
 nfp_bars_for_secondary(uint32_t index)
 {
@@ -383,8 +370,15 @@ nfp_enable_bars(struct nfp_pcie_user *nfp)
 	if (nfp_bar_write(nfp, bar, barcfg_msix_general) < 0)
 		return -EIO;
 
-	/* Sort bars by bit size - use the smallest possible first. */
-	qsort(&nfp->bar[0], nfp->bars, sizeof(nfp->bar[0]), nfp_cmp_bars);
+
+	/* Reserve BAR2.0 for expansion rom mapping */
+	if (type == RTE_PROC_PRIMARY) {
+		if (nfp->pci_dev->id.device_id == PCI_DEVICE_ID_NFP3800_PF_NIC) {
+			bar = &nfp->bar[16];
+			if (bar != NULL)
+				bar->lock = true;
+		}
+	}
 
 	return 0;
 }
@@ -466,16 +460,18 @@ find_matching_bar(struct nfp_pcie_user *nfp,
 		int width)
 {
 	uint32_t n;
+	uint32_t index;
 
-	for (n = 0; n < nfp->bars; n++) {
-		struct nfp_bar *bar = &nfp->bar[n];
+	for (n = RTE_DIM(nfp->bar) ; n > 0; n--) {
+		index = n - 1;
+		struct nfp_bar *bar = &nfp->bar[index];
 
 		if (bar->lock)
 			continue;
 
 		if (matching_bar_exist(bar, target, action, token,
 				offset, size, width))
-			return n;
+			return index;
 	}
 
 	return -1;
@@ -493,10 +489,12 @@ find_unused_bar_noblock(struct nfp_pcie_user *nfp,
 {
 	int ret;
 	uint32_t n;
+	uint32_t index;
 	const struct nfp_bar *bar;
 
-	for (n = 0; n < nfp->bars; n++) {
-		bar = &nfp->bar[n];
+	for (n = RTE_DIM(nfp->bar); n > 0; n--) {
+		index = n - 1;
+		bar = &nfp->bar[index];
 
 		if (bar->bitsize == 0)
 			continue;
@@ -508,7 +506,7 @@ find_unused_bar_noblock(struct nfp_pcie_user *nfp,
 			continue;
 
 		if (!bar->lock)
-			return n;
+			return index;
 	}
 
 	return -EAGAIN;
@@ -561,7 +559,7 @@ nfp_disable_bars(struct nfp_pcie_user *nfp)
 	uint32_t i;
 	struct nfp_bar *bar;
 
-	for (i = 0; i < nfp->bars; i++) {
+	for (i = 0; i < RTE_DIM(nfp->bar); i++) {
 		bar = &nfp->bar[i];
 		if (bar->iomem != NULL) {
 			bar->iomem = NULL;
@@ -624,7 +622,7 @@ nfp6000_area_acquire(struct nfp_cpp_area *area)
 
 	bar_num = nfp_alloc_bar(nfp, priv);
 	if (bar_num < 0) {
-		PMD_DRV_LOG(ERR, "Failed to allocate bar %d:%d:%d:%#lx: %d",
+		PMD_DRV_LOG(ERR, "Failed to allocate bar %d:%d:%d:%#lx: %d.",
 				priv->target, priv->action, priv->token,
 				priv->offset, bar_num);
 		return bar_num;
@@ -706,7 +704,7 @@ nfp6000_area_read(struct nfp_cpp_area *area,
 
 	/* Unaligned? Translate to an explicit access */
 	if (((priv->offset + offset) & (width - 1)) != 0) {
-		PMD_DRV_LOG(ERR, "aread_read unaligned!!!");
+		PMD_DRV_LOG(ERR, "The aread_read unaligned!!!");
 		return -EINVAL;
 	}
 
@@ -862,7 +860,7 @@ nfp6000_get_dsn(struct rte_pci_device *pci_dev,
 
 	pos = rte_pci_find_ext_capability(pci_dev, RTE_PCI_EXT_CAP_ID_DSN);
 	if (pos <= 0) {
-		PMD_DRV_LOG(ERR, "PCI_EXT_CAP_ID_DSN not found");
+		PMD_DRV_LOG(ERR, "PCI_EXT_CAP_ID_DSN not found.");
 		return -ENODEV;
 	}
 
@@ -870,7 +868,7 @@ nfp6000_get_dsn(struct rte_pci_device *pci_dev,
 	len = sizeof(tmp);
 
 	if (rte_pci_read_config(pci_dev, &tmp, len, pos) < 0) {
-		PMD_DRV_LOG(ERR, "nfp get device serial number failed");
+		PMD_DRV_LOG(ERR, "NFP get device serial number failed.");
 		return -ENOENT;
 	}
 
@@ -935,7 +933,7 @@ nfp6000_init(struct nfp_cpp *cpp)
 
 	ret = nfp_enable_bars(desc);
 	if (ret != 0) {
-		PMD_DRV_LOG(ERR, "Enable bars failed");
+		PMD_DRV_LOG(ERR, "Enable bars failed.");
 		return -1;
 	}
 
@@ -1020,7 +1018,7 @@ nfp_cpp_from_nfp6000_pcie(struct rte_pci_device *pci_dev,
 
 	if (NFP_CPP_INTERFACE_CHANNEL_of(interface) !=
 			NFP_CPP_INTERFACE_CHANNEL_PEROPENER) {
-		PMD_DRV_LOG(ERR, "Interface channel is not right");
+		PMD_DRV_LOG(ERR, "Interface channel is not right.");
 		free(nfp);
 		return NULL;
 	}
@@ -1028,10 +1026,18 @@ nfp_cpp_from_nfp6000_pcie(struct rte_pci_device *pci_dev,
 	/* Probe for all the common NFP devices */
 	cpp = nfp_cpp_from_device_name(pci_dev, nfp, driver_lock_needed);
 	if (cpp == NULL) {
-		PMD_DRV_LOG(ERR, "Get cpp from operation failed");
+		PMD_DRV_LOG(ERR, "Get cpp from operation failed.");
 		free(nfp);
 		return NULL;
 	}
 
 	return cpp;
 }
+
+uint8_t
+nfp_get_pf_id_from_device(void *priv)
+{
+	struct nfp_pcie_user *nfp = priv;
+
+	return nfp->pci_dev->addr.function;
+}
