@@ -6,7 +6,7 @@
 #include <sys/epoll.h>
 #include <rte_kvargs.h>
 #include <ethdev_vdev.h>
-#include <rte_bus_vdev.h>
+#include <bus_vdev_driver.h>
 #include <rte_ether.h>
 #include <dpaa_of.h>
 
@@ -161,7 +161,7 @@ pfe_recv_pkts_on_intr(void *rxq, struct rte_mbuf **rx_pkts, uint16_t nb_pkts)
 		writel(readl(HIF_INT_ENABLE) | HIF_RXPKT_INT, HIF_INT_ENABLE);
 		ret = epoll_wait(priv->pfe->hif.epoll_fd, &epoll_ev, 1, ticks);
 		if (ret < 0 && errno != EINTR)
-			PFE_PMD_ERR("epoll_wait fails with %d\n", errno);
+			PFE_PMD_ERR("epoll_wait fails with %d", errno);
 	}
 
 	return work_done;
@@ -235,28 +235,13 @@ pfe_xmit_pkts(void *tx_queue, struct rte_mbuf **tx_pkts, uint16_t nb_pkts)
 	return nb_pkts;
 }
 
-static uint16_t
-pfe_dummy_xmit_pkts(__rte_unused void *tx_queue,
-		__rte_unused struct rte_mbuf **tx_pkts,
-		__rte_unused uint16_t nb_pkts)
-{
-	return 0;
-}
-
-static uint16_t
-pfe_dummy_recv_pkts(__rte_unused void *rxq,
-		__rte_unused struct rte_mbuf **rx_pkts,
-		__rte_unused uint16_t nb_pkts)
-{
-	return 0;
-}
-
 static int
 pfe_eth_open(struct rte_eth_dev *dev)
 {
 	struct pfe_eth_priv_s *priv = dev->data->dev_private;
 	struct hif_client_s *client;
 	struct hif_shm *hif_shm;
+	uint16_t i;
 	int rc;
 
 	/* Register client driver with HIF */
@@ -334,6 +319,10 @@ pfe_eth_open(struct rte_eth_dev *dev)
 		PFE_PMD_INFO("PFE INTERRUPT Mode enabled");
 	}
 
+	for (i = 0; i < dev->data->nb_rx_queues; i++)
+		dev->data->rx_queue_state[i] = RTE_ETH_QUEUE_STATE_STARTED;
+	for (i = 0; i < dev->data->nb_tx_queues; i++)
+		dev->data->tx_queue_state[i] = RTE_ETH_QUEUE_STATE_STARTED;
 
 err0:
 	return rc;
@@ -349,9 +338,9 @@ pfe_eth_open_cdev(struct pfe_eth_priv_s *priv)
 
 	pfe_cdev_fd = open(PFE_CDEV_PATH, O_RDONLY);
 	if (pfe_cdev_fd < 0) {
-		PFE_PMD_WARN("Unable to open PFE device file (%s).\n",
+		PFE_PMD_WARN("Unable to open PFE device file (%s).",
 			     PFE_CDEV_PATH);
-		PFE_PMD_WARN("Link status update will not be available.\n");
+		PFE_PMD_WARN("Link status update will not be available.");
 		priv->link_fd = PFE_CDEV_INVALID_FD;
 		return -1;
 	}
@@ -377,14 +366,20 @@ static int
 pfe_eth_stop(struct rte_eth_dev *dev/*, int wake*/)
 {
 	struct pfe_eth_priv_s *priv = dev->data->dev_private;
+	uint16_t i;
 
 	dev->data->dev_started = 0;
 
 	gemac_disable(priv->EMAC_baseaddr);
 	gpi_disable(priv->GPI_baseaddr);
 
-	dev->rx_pkt_burst = &pfe_dummy_recv_pkts;
-	dev->tx_pkt_burst = &pfe_dummy_xmit_pkts;
+	dev->rx_pkt_burst = rte_eth_pkt_burst_dummy;
+	dev->tx_pkt_burst = rte_eth_pkt_burst_dummy;
+
+	for (i = 0; i < dev->data->nb_rx_queues; i++)
+		dev->data->rx_queue_state[i] = RTE_ETH_QUEUE_STATE_STOPPED;
+	for (i = 0; i < dev->data->nb_tx_queues; i++)
+		dev->data->tx_queue_state[i] = RTE_ETH_QUEUE_STATE_STOPPED;
 
 	return 0;
 }
@@ -514,7 +509,7 @@ pfe_tx_queue_setup(struct rte_eth_dev *dev,
 }
 
 static const uint32_t *
-pfe_supported_ptypes_get(struct rte_eth_dev *dev)
+pfe_supported_ptypes_get(struct rte_eth_dev *dev, size_t *no_of_elements)
 {
 	static const uint32_t ptypes[] = {
 		/*todo -= add more types */
@@ -525,12 +520,14 @@ pfe_supported_ptypes_get(struct rte_eth_dev *dev)
 		RTE_PTYPE_L3_IPV6_EXT,
 		RTE_PTYPE_L4_TCP,
 		RTE_PTYPE_L4_UDP,
-		RTE_PTYPE_L4_SCTP
+		RTE_PTYPE_L4_SCTP,
 	};
 
 	if (dev->rx_pkt_burst == pfe_recv_pkts ||
-			dev->rx_pkt_burst == pfe_recv_pkts_on_intr)
+			dev->rx_pkt_burst == pfe_recv_pkts_on_intr) {
+		*no_of_elements = RTE_DIM(ptypes);
 		return ptypes;
+	}
 	return NULL;
 }
 
@@ -586,17 +583,16 @@ pfe_eth_link_update(struct rte_eth_dev *dev, int wait_to_complete __rte_unused)
 
 		ret = ioctl(priv->link_fd, ioctl_cmd, &lstatus);
 		if (ret != 0) {
-			PFE_PMD_ERR("Unable to fetch link status (ioctl)\n");
-			/* use dummy link value */
-			link.link_status = 1;
+			PFE_PMD_ERR("Unable to fetch link status (ioctl)");
+			return -1;
 		}
-		PFE_PMD_DEBUG("Fetched link state (%d) for dev %d.\n",
+		PFE_PMD_DEBUG("Fetched link state (%d) for dev %d.",
 			      lstatus, priv->id);
 	}
 
 	if (old.link_status == lstatus) {
 		/* no change in status */
-		PFE_PMD_DEBUG("No change in link status; Not updating.\n");
+		PFE_PMD_DEBUG("No change in link status; Not updating.");
 		return -1;
 	}
 
@@ -607,7 +603,7 @@ pfe_eth_link_update(struct rte_eth_dev *dev, int wait_to_complete __rte_unused)
 
 	pfe_eth_atomic_write_link_status(dev, &link);
 
-	PFE_PMD_INFO("Port (%d) link is %s\n", dev->data->port_id,
+	PFE_PMD_INFO("Port (%d) link is %s", dev->data->port_id,
 		     link.link_status ? "up" : "down");
 
 	return 0;
@@ -717,7 +713,8 @@ pfe_dev_set_mac_addr(struct rte_eth_dev *dev,
 
 static int
 pfe_stats_get(struct rte_eth_dev *dev,
-	      struct rte_eth_stats *stats)
+	      struct rte_eth_stats *stats,
+	      struct eth_queue_stats *qstats __rte_unused)
 {
 	struct pfe_eth_priv_s *priv = dev->data->dev_private;
 	struct rte_eth_stats *eth_stats = &priv->stats;
@@ -769,7 +766,7 @@ pfe_eth_init(struct rte_vdev_device *vdev, struct pfe *pfe, int id)
 	if (eth_dev == NULL)
 		return -ENOMEM;
 
-	/* Extract pltform data */
+	/* Extract platform data */
 	pfe_info = (struct ls1012a_pfe_platform_data *)&pfe->platform_data;
 	if (!pfe_info) {
 		PFE_PMD_ERR("pfe missing additional platform data");
@@ -997,24 +994,24 @@ pmd_pfe_probe(struct rte_vdev_device *vdev)
 
 	addr = of_get_address(np, 0, &cbus_size, NULL);
 	if (!addr) {
-		PFE_PMD_ERR("of_get_address cannot return qman address\n");
+		PFE_PMD_ERR("of_get_address cannot return qman address");
 		goto err;
 	}
 	cbus_addr = of_translate_address(np, addr);
 	if (!cbus_addr) {
-		PFE_PMD_ERR("of_translate_address failed\n");
+		PFE_PMD_ERR("of_translate_address failed");
 		goto err;
 	}
 
 	addr = of_get_address(np, 1, &ddr_size, NULL);
 	if (!addr) {
-		PFE_PMD_ERR("of_get_address cannot return qman address\n");
+		PFE_PMD_ERR("of_get_address cannot return qman address");
 		goto err;
 	}
 
 	g_pfe->ddr_phys_baseaddr = of_translate_address(np, addr);
 	if (!g_pfe->ddr_phys_baseaddr) {
-		PFE_PMD_ERR("of_translate_address failed\n");
+		PFE_PMD_ERR("of_translate_address failed");
 		goto err;
 	}
 
