@@ -1,5 +1,5 @@
 /* SPDX-License-Identifier: BSD-3-Clause
- * Copyright 2018-2019 NXP
+ * Copyright 2018-2021 NXP
  */
 
 #include "pfe_logs.h"
@@ -114,9 +114,9 @@ pfe_hif_init_buffers(struct pfe_hif *hif)
 		 * results, eth id, queue id from PFE block along with data.
 		 * so we have to provide additional memory for each packet to
 		 * HIF rx rings so that PFE block can write its headers.
-		 * so, we are giving the data pointor to HIF rings whose
+		 * so, we are giving the data pointer to HIF rings whose
 		 * calculation is as below:
-		 * mbuf->data_pointor - Required_header_size
+		 * mbuf->data_pointer - Required_header_size
 		 *
 		 * We are utilizing the HEADROOM area to receive the PFE
 		 * block headers. On packet reception, HIF driver will use
@@ -309,7 +309,7 @@ client_put_rxpacket(struct hif_rx_queue *queue,
 	if (readl(&desc->ctrl) & CL_DESC_OWN) {
 		mbuf = rte_cpu_to_le_64(rte_pktmbuf_alloc(pool));
 		if (unlikely(!mbuf)) {
-			PFE_PMD_WARN("Buffer allocation failure\n");
+			PFE_PMD_WARN("Buffer allocation failure");
 			return NULL;
 		}
 
@@ -766,13 +766,13 @@ pfe_hif_rx_idle(struct pfe_hif *hif)
 		if (rx_status & BDP_CSR_RX_DMA_ACTV)
 			send_dummy_pkt_to_hif();
 
-		sleep(1);
+		rte_delay_ms(1);
 	} while (--hif_stop_loop);
 
 	if (readl(HIF_RX_STATUS) & BDP_CSR_RX_DMA_ACTV)
-		PFE_PMD_ERR("Failed\n");
+		PFE_PMD_ERR("Failed");
 	else
-		PFE_PMD_INFO("Done\n");
+		PFE_PMD_INFO("Done");
 }
 #endif
 
@@ -806,7 +806,7 @@ pfe_hif_init(struct pfe *pfe)
 
 		pfe_cdev_fd = open(PFE_CDEV_PATH, O_RDWR);
 		if (pfe_cdev_fd < 0) {
-			PFE_PMD_WARN("Unable to open PFE device file (%s).\n",
+			PFE_PMD_WARN("Unable to open PFE device file (%s).",
 				     PFE_CDEV_PATH);
 			pfe->cdev_fd = PFE_CDEV_INVALID_FD;
 			return -1;
@@ -817,7 +817,7 @@ pfe_hif_init(struct pfe *pfe)
 		/* hif interrupt enable */
 		err = ioctl(pfe->cdev_fd, PFE_CDEV_HIF_INTR_EN, &event_fd);
 		if (err) {
-			PFE_PMD_ERR("\nioctl failed for intr enable err: %d\n",
+			PFE_PMD_ERR("ioctl failed for intr enable err: %d",
 					errno);
 			goto err0;
 		}
@@ -826,7 +826,7 @@ pfe_hif_init(struct pfe *pfe)
 		epoll_ev.data.fd = event_fd;
 		err = epoll_ctl(epoll_fd, EPOLL_CTL_ADD, event_fd, &epoll_ev);
 		if (err < 0) {
-			PFE_PMD_ERR("epoll_ctl failed with err = %d\n", errno);
+			PFE_PMD_ERR("epoll_ctl failed with err = %d", errno);
 			goto err0;
 		}
 		pfe->hif.epoll_fd = epoll_fd;
