@@ -58,6 +58,10 @@ RTE_LOG_REGISTER_SUFFIX(nicvf_logtype_driver, driver, NOTICE);
 #define NICVF_QLM_MODE_SGMII  7
 #define NICVF_QLM_MODE_XFI   12
 
+#define BCAST_ACCEPT      0x01
+#define CAM_ACCEPT        (1 << 3)
+#define BGX_MCAST_MODE(x) ((x) << 1)
+
 enum nicvf_link_speed {
 	NICVF_LINK_SPEED_SGMII,
 	NICVF_LINK_SPEED_XAUI,
@@ -285,7 +289,8 @@ nicvf_dev_get_regs(struct rte_eth_dev *dev, struct rte_dev_reg_info *regs)
 }
 
 static int
-nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
+nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats,
+		    struct eth_queue_stats *qstats)
 {
 	uint16_t qidx;
 	struct nicvf_hw_rx_qstats rx_qstats;
@@ -305,8 +310,10 @@ nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 			break;
 
 		nicvf_hw_get_rx_qstats(nic, &rx_qstats, qidx);
-		stats->q_ibytes[qidx] = rx_qstats.q_rx_bytes;
-		stats->q_ipackets[qidx] = rx_qstats.q_rx_packets;
+		if (qstats != NULL) {
+			qstats->q_ibytes[qidx] = rx_qstats.q_rx_bytes;
+			qstats->q_ipackets[qidx] = rx_qstats.q_rx_packets;
+		}
 	}
 
 	/* TX queue indices for the first VF */
@@ -318,8 +325,10 @@ nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 			break;
 
 		nicvf_hw_get_tx_qstats(nic, &tx_qstats, qidx);
-		stats->q_obytes[qidx] = tx_qstats.q_tx_bytes;
-		stats->q_opackets[qidx] = tx_qstats.q_tx_packets;
+		if (qstats != NULL) {
+			qstats->q_obytes[qidx] = tx_qstats.q_tx_bytes;
+			qstats->q_opackets[qidx] = tx_qstats.q_tx_packets;
+		}
 	}
 
 	for (i = 0; i < nic->sqs_count; i++) {
@@ -338,8 +347,10 @@ nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 
 			nicvf_hw_get_rx_qstats(snic, &rx_qstats,
 					       qidx % MAX_RCV_QUEUES_PER_QS);
-			stats->q_ibytes[qidx] = rx_qstats.q_rx_bytes;
-			stats->q_ipackets[qidx] = rx_qstats.q_rx_packets;
+			if (qstats != NULL) {
+				qstats->q_ibytes[qidx] = rx_qstats.q_rx_bytes;
+				qstats->q_ipackets[qidx] = rx_qstats.q_rx_packets;
+			}
 		}
 
 		/* TX queue indices for a secondary VF */
@@ -351,8 +362,10 @@ nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 
 			nicvf_hw_get_tx_qstats(snic, &tx_qstats,
 					       qidx % MAX_SND_QUEUES_PER_QS);
-			stats->q_obytes[qidx] = tx_qstats.q_tx_bytes;
-			stats->q_opackets[qidx] = tx_qstats.q_tx_packets;
+			if (qstats != NULL) {
+				qstats->q_obytes[qidx] = tx_qstats.q_tx_bytes;
+				qstats->q_opackets[qidx] = tx_qstats.q_tx_packets;
+			}
 		}
 	}
 
@@ -379,7 +392,7 @@ nicvf_dev_stats_get(struct rte_eth_dev *dev, struct rte_eth_stats *stats)
 }
 
 static const uint32_t *
-nicvf_dev_supported_ptypes_get(struct rte_eth_dev *dev)
+nicvf_dev_supported_ptypes_get(struct rte_eth_dev *dev, size_t *no_of_elements)
 {
 	size_t copied;
 	static uint32_t ptypes[32];
@@ -399,7 +412,6 @@ nicvf_dev_supported_ptypes_get(struct rte_eth_dev *dev)
 		RTE_PTYPE_TUNNEL_VXLAN,
 		RTE_PTYPE_TUNNEL_NVGRE,
 	};
-	static const uint32_t ptypes_end = RTE_PTYPE_UNKNOWN;
 
 	copied = sizeof(ptypes_common);
 	memcpy(ptypes, ptypes_common, copied);
@@ -409,9 +421,9 @@ nicvf_dev_supported_ptypes_get(struct rte_eth_dev *dev)
 		copied += sizeof(ptypes_tunnel);
 	}
 
-	memcpy((char *)ptypes + copied, &ptypes_end, sizeof(ptypes_end));
 
 	/* All Ptypes are supported in all Rx functions. */
+	*no_of_elements = copied / sizeof(ptypes[0]);
 	return ptypes;
 }
 
@@ -1811,7 +1823,7 @@ nicvf_dev_start(struct rte_eth_dev *dev)
 	/* Apply new link configurations if changed */
 	ret = nicvf_apply_link_speed(dev);
 	if (ret) {
-		PMD_INIT_LOG(ERR, "Failed to set link configuration\n");
+		PMD_INIT_LOG(ERR, "Failed to set link configuration");
 		return ret;
 	}
 
@@ -2183,9 +2195,22 @@ nicvf_eth_dev_uninit(struct rte_eth_dev *dev)
 	nicvf_dev_close(dev);
 	return 0;
 }
+
+static inline uint64_t ether_addr_to_u64(uint8_t *addr)
+{
+	uint64_t u = 0;
+	int i;
+
+	for (i = 0; i < RTE_ETHER_ADDR_LEN; i++)
+		u = u << 8 | addr[i];
+
+	return u;
+}
+
 static int
 nicvf_eth_dev_init(struct rte_eth_dev *eth_dev)
 {
+	uint8_t dmac_ctrl_reg = 0;
 	int ret;
 	struct rte_pci_device *pci_dev;
 	struct nicvf *nic = nicvf_pmd_priv(eth_dev);
@@ -2309,6 +2334,15 @@ nicvf_eth_dev_init(struct rte_eth_dev *eth_dev)
 		goto malloc_fail;
 	}
 
+	/* set DMAC CTRL reg to allow MAC */
+	dmac_ctrl_reg = BCAST_ACCEPT | BGX_MCAST_MODE(2) | CAM_ACCEPT;
+	ret = nicvf_mbox_set_xcast(nic, dmac_ctrl_reg,
+			ether_addr_to_u64(nic->mac_addr));
+	if (ret) {
+		PMD_INIT_LOG(ERR, "Failed to set mac addr");
+		goto malloc_fail;
+	}
+
 	ret = nicvf_set_first_skip(eth_dev);
 	if (ret) {
 		PMD_INIT_LOG(ERR, "Failed to configure first skip");
