@@ -12,11 +12,16 @@
 
 #define TXGBE_FW_VER_LEN	32
 #define TXGBE_FW_N_TXEQ		0x0002000A
+#define TXGBE_FW_SUPPORT_LLDP	0x0F
+#define TXGBE_FW_GET_LLDP	0x11
+#define TXGBE_FW_MASK		0xFF
 
 #define TXGBE_FW_PTR			0x0F
 #define TXGBE_PBANUM0_PTR		0x05
 #define TXGBE_PBANUM1_PTR		0x06
 #define TXGBE_SW_REGION_PTR             0x1C
+#define TXGBE_SHOWROM_I2C_PTR		0xB00
+#define TXGBE_SHOWROM_I2C_END		0xF00
 
 #define TXGBE_EE_CSUM_MAX		0x800
 #define TXGBE_EEPROM_CHECKSUM		0x2F
