@@ -35,7 +35,7 @@ static bool txgbe_identify_extphy(struct txgbe_hw *hw)
 	u16 phy_addr = 0;
 
 	if (!txgbe_validate_phy_addr(hw, phy_addr)) {
-		DEBUGOUT("Unable to validate PHY address 0x%04X\n",
+		DEBUGOUT("Unable to validate PHY address 0x%04X",
 			phy_addr);
 		return false;
 	}
@@ -80,12 +80,8 @@ static s32 txgbe_read_phy_if(struct txgbe_hw *hw)
 	if (hw->phy.nw_mng_if_sel & TXGBE_ETHPHYIF_MDIO_ACT)
 		hw->phy.addr = TXGBE_ETHPHYIF_MDIO_BASE(hw->phy.nw_mng_if_sel);
 
-	if (!hw->phy.phy_semaphore_mask) {
-		if (hw->bus.lan_id)
-			hw->phy.phy_semaphore_mask = TXGBE_MNGSEM_SWPHY;
-		else
-			hw->phy.phy_semaphore_mask = TXGBE_MNGSEM_SWPHY;
-	}
+	if (!hw->phy.phy_semaphore_mask)
+		hw->phy.phy_semaphore_mask = TXGBE_MNGSEM_SWPHY;
 
 	return 0;
 }
@@ -100,8 +96,6 @@ s32 txgbe_identify_phy(struct txgbe_hw *hw)
 {
 	s32 err = TXGBE_ERR_PHY_ADDR_INVALID;
 
-	DEBUGFUNC("txgbe_identify_phy");
-
 	txgbe_read_phy_if(hw);
 
 	if (hw->phy.type != txgbe_phy_unknown)
@@ -137,11 +131,9 @@ s32 txgbe_check_reset_blocked(struct txgbe_hw *hw)
 {
 	u32 mmngc;
 
-	DEBUGFUNC("txgbe_check_reset_blocked");
-
 	mmngc = rd32(hw, TXGBE_STAT);
 	if (mmngc & TXGBE_STAT_MNGVETO) {
-		DEBUGOUT("MNG_VETO bit detected.\n");
+		DEBUGOUT("MNG_VETO bit detected.");
 		return true;
 	}
 
@@ -159,8 +151,6 @@ bool txgbe_validate_phy_addr(struct txgbe_hw *hw, u32 phy_addr)
 	u16 phy_id = 0;
 	bool valid = false;
 
-	DEBUGFUNC("txgbe_validate_phy_addr");
-
 	hw->phy.addr = phy_addr;
 	hw->phy.read_reg(hw, TXGBE_MD_PHY_ID_HIGH,
 			     TXGBE_MD_DEV_PMA_PMD, &phy_id);
@@ -168,7 +158,7 @@ bool txgbe_validate_phy_addr(struct txgbe_hw *hw, u32 phy_addr)
 	if (phy_id != 0xFFFF && phy_id != 0x0)
 		valid = true;
 
-	DEBUGOUT("PHY ID HIGH is 0x%04X\n", phy_id);
+	DEBUGOUT("PHY ID HIGH is 0x%04X", phy_id);
 
 	return valid;
 }
@@ -183,8 +173,41 @@ s32 txgbe_get_phy_id(struct txgbe_hw *hw)
 	u32 err;
 	u16 phy_id_high = 0;
 	u16 phy_id_low = 0;
+	u32 i = 0;
+	u32 status;
 
-	DEBUGFUNC("txgbe_get_phy_id");
+	if (hw->mac.type == txgbe_mac_aml) {
+		hw->phy.addr = 0;
+
+		for (i = 0; i < 32; i++) {
+			hw->phy.addr = i;
+			status = txgbe_read_phy_reg_mdi(hw, TXGBE_MD_PHY_ID_HIGH, 0, &phy_id_high);
+			if (status) {
+				DEBUGOUT("txgbe_read_phy_reg_mdi failed 1");
+				return status;
+			}
+			DEBUGOUT("%d: phy_id_high 0x%x", i, phy_id_high);
+			if ((phy_id_high & 0xFFFF) == 0x0141)
+				break;
+		}
+
+		if (i == 32) {
+			DEBUGOUT("txgbe_read_phy_reg_mdi failed");
+			return TXGBE_ERR_PHY;
+		}
+
+		status = txgbe_read_phy_reg_mdi(hw, TXGBE_MD_PHY_ID_LOW, 0, &phy_id_low);
+		if (status) {
+			DEBUGOUT("txgbe_read_phy_reg_mdi failed 2");
+			return status;
+		}
+		hw->phy.id = (u32)(phy_id_high & 0xFFFF) << 6;
+		hw->phy.id |= (u32)((phy_id_low & 0xFC00) >> 10);
+
+		DEBUGOUT("phy_id 0x%x", hw->phy.id);
+
+		return status;
+	}
 
 	err = hw->phy.read_reg(hw, TXGBE_MD_PHY_ID_HIGH,
 				      TXGBE_MD_DEV_PMA_PMD,
@@ -198,7 +221,7 @@ s32 txgbe_get_phy_id(struct txgbe_hw *hw)
 		hw->phy.id |= (u32)(phy_id_low & TXGBE_PHY_REVISION_MASK);
 		hw->phy.revision = (u32)(phy_id_low & ~TXGBE_PHY_REVISION_MASK);
 	}
-	DEBUGOUT("PHY_ID_HIGH 0x%04X, PHY_ID_LOW 0x%04X\n",
+	DEBUGOUT("PHY_ID_HIGH 0x%04X, PHY_ID_LOW 0x%04X",
 		  phy_id_high, phy_id_low);
 
 	return err;
@@ -213,8 +236,6 @@ enum txgbe_phy_type txgbe_get_phy_type_from_id(u32 phy_id)
 {
 	enum txgbe_phy_type phy_type;
 
-	DEBUGFUNC("txgbe_get_phy_type_from_id");
-
 	switch (phy_id) {
 	case TXGBE_PHYID_TN1010:
 		phy_type = txgbe_phy_tn;
@@ -272,7 +293,7 @@ txgbe_reset_extphy(struct txgbe_hw *hw)
 
 	if (ctrl & TXGBE_MD_PORT_CTRL_RESET) {
 		err = TXGBE_ERR_RESET_FAILED;
-		DEBUGOUT("PHY reset polling failed to complete.\n");
+		DEBUGOUT("PHY reset polling failed to complete.");
 	}
 
 	return err;
@@ -286,8 +307,6 @@ s32 txgbe_reset_phy(struct txgbe_hw *hw)
 {
 	s32 err = 0;
 
-	DEBUGFUNC("txgbe_reset_phy");
-
 	if (hw->phy.type == txgbe_phy_unknown)
 		err = txgbe_identify_phy(hw);
 
@@ -343,7 +362,7 @@ s32 txgbe_read_phy_reg_mdi(struct txgbe_hw *hw, u32 reg_addr, u32 device_type,
 	 */
 	if (!po32m(hw, TXGBE_MDIOSCD, TXGBE_MDIOSCD_BUSY,
 		0, NULL, 100, 100)) {
-		DEBUGOUT("PHY address command did not complete\n");
+		DEBUGOUT("PHY address command did not complete");
 		return TXGBE_ERR_PHY;
 	}
 
@@ -367,8 +386,6 @@ s32 txgbe_read_phy_reg(struct txgbe_hw *hw, u32 reg_addr,
 	s32 err;
 	u32 gssr = hw->phy.phy_semaphore_mask;
 
-	DEBUGFUNC("txgbe_read_phy_reg");
-
 	if (hw->mac.acquire_swfw_sync(hw, gssr))
 		return TXGBE_ERR_SWFW_SYNC;
 
@@ -406,7 +423,7 @@ s32 txgbe_write_phy_reg_mdi(struct txgbe_hw *hw, u32 reg_addr,
 	/* wait for completion */
 	if (!po32m(hw, TXGBE_MDIOSCD, TXGBE_MDIOSCD_BUSY,
 		0, NULL, 100, 100)) {
-		TLOG_DEBUG("PHY write cmd didn't complete\n");
+		DEBUGOUT("PHY write cmd didn't complete");
 		return -TERR_PHY;
 	}
 
@@ -427,8 +444,6 @@ s32 txgbe_write_phy_reg(struct txgbe_hw *hw, u32 reg_addr,
 	s32 err;
 	u32 gssr = hw->phy.phy_semaphore_mask;
 
-	DEBUGFUNC("txgbe_write_phy_reg");
-
 	if (hw->mac.acquire_swfw_sync(hw, gssr))
 		err = TXGBE_ERR_SWFW_SYNC;
 
@@ -452,8 +467,6 @@ s32 txgbe_setup_phy_link(struct txgbe_hw *hw)
 	bool autoneg = false;
 	u32 speed;
 
-	DEBUGFUNC("txgbe_setup_phy_link");
-
 	txgbe_get_copper_link_capabilities(hw, &speed, &autoneg);
 
 	/* Set or unset auto-negotiation 10G advertisement */
@@ -539,8 +552,6 @@ s32 txgbe_setup_phy_link_speed(struct txgbe_hw *hw,
 {
 	UNREFERENCED_PARAMETER(autoneg_wait_to_complete);
 
-	DEBUGFUNC("txgbe_setup_phy_link_speed");
-
 	/*
 	 * Clear autoneg_advertised and set new values based on input link
 	 * speed.
@@ -623,8 +634,6 @@ s32 txgbe_get_copper_link_capabilities(struct txgbe_hw *hw,
 {
 	s32 err = 0;
 
-	DEBUGFUNC("txgbe_get_copper_link_capabilities");
-
 	*autoneg = true;
 	if (!hw->phy.speeds_supported)
 		err = txgbe_get_copper_speeds_supported(hw);
@@ -652,8 +661,6 @@ s32 txgbe_check_phy_link_tnx(struct txgbe_hw *hw, u32 *speed,
 	u16 phy_speed = 0;
 	u16 phy_data = 0;
 
-	DEBUGFUNC("txgbe_check_phy_link_tnx");
-
 	/* Initialize speed and link to default case */
 	*link_up = false;
 	*speed = TXGBE_LINK_SPEED_10GB_FULL;
@@ -697,8 +704,6 @@ s32 txgbe_setup_phy_link_tnx(struct txgbe_hw *hw)
 	bool autoneg = false;
 	u32 speed;
 
-	DEBUGFUNC("txgbe_setup_phy_link_tnx");
-
 	txgbe_get_copper_link_capabilities(hw, &speed, &autoneg);
 
 	if (speed & TXGBE_LINK_SPEED_10GB_FULL) {
@@ -772,8 +777,6 @@ s32 txgbe_identify_module(struct txgbe_hw *hw)
 {
 	s32 err = TXGBE_ERR_SFP_NOT_PRESENT;
 
-	DEBUGFUNC("txgbe_identify_module");
-
 	switch (hw->phy.media_type) {
 	case txgbe_media_type_fiber:
 		err = txgbe_identify_sfp_module(hw);
@@ -806,12 +809,21 @@ s32 txgbe_identify_sfp_module(struct txgbe_hw *hw)
 	u8 identifier = 0;
 	u8 comp_codes_1g = 0;
 	u8 comp_codes_10g = 0;
+	u8 comp_codes_25g = 0;
+	u8 comp_copper_len = 0;
 	u8 oui_bytes[3] = {0, 0, 0};
 	u8 cable_tech = 0;
 	u8 cable_spec = 0;
 	u16 enforce_sfp = 0;
+	u32 value;
 
-	DEBUGFUNC("txgbe_identify_sfp_module");
+	if (hw->mac.type == txgbe_mac_aml) {
+		value = rd32(hw, TXGBE_GPIOEXT);
+		if (value & TXGBE_SFP1_MOD_ABS_LS) {
+			hw->phy.sfp_type = txgbe_sfp_type_not_present;
+			return TXGBE_ERR_SFP_NOT_PRESENT;
+		}
+	}
 
 	if (hw->phy.media_type != txgbe_media_type_fiber) {
 		hw->phy.sfp_type = txgbe_sfp_type_not_present;
@@ -845,6 +857,16 @@ ERR_I2C:
 	if (err != 0)
 		goto ERR_I2C;
 
+	err = hw->phy.read_i2c_eeprom(hw, TXGBE_SFF_25GBE_COMP_CODES,
+					      &comp_codes_25g);
+	if (err != 0)
+		goto ERR_I2C;
+
+	err = hw->phy.read_i2c_eeprom(hw, TXGBE_SFF_COPPER_LENGTH,
+					      &comp_copper_len);
+	if (err != 0)
+		goto ERR_I2C;
+
 	err = hw->phy.read_i2c_eeprom(hw, TXGBE_SFF_CABLE_TECHNOLOGY,
 					     &cable_tech);
 	if (err != 0)
@@ -866,12 +888,7 @@ ERR_I2C:
 	  * 11  SFP_1g_sx_CORE0 - chip-specific
 	  * 12  SFP_1g_sx_CORE1 - chip-specific
 	  */
-	if (cable_tech & TXGBE_SFF_CABLE_DA_PASSIVE) {
-		if (hw->bus.lan_id == 0)
-			hw->phy.sfp_type = txgbe_sfp_type_da_cu_core0;
-		else
-			hw->phy.sfp_type = txgbe_sfp_type_da_cu_core1;
-	} else if (cable_tech & TXGBE_SFF_CABLE_DA_ACTIVE) {
+	if (cable_tech & TXGBE_SFF_CABLE_DA_ACTIVE) {
 		err = hw->phy.read_i2c_eeprom(hw,
 			TXGBE_SFF_CABLE_SPEC_COMP, &cable_spec);
 		if (err != 0)
@@ -883,6 +900,17 @@ ERR_I2C:
 		} else {
 			hw->phy.sfp_type = txgbe_sfp_type_unknown;
 		}
+	} else if (comp_codes_25g == TXGBE_SFF_25GBASESR_CAPABLE ||
+		   comp_codes_25g == TXGBE_SFF_25GBASEER_CAPABLE) {
+		if (hw->bus.lan_id == 0)
+			hw->phy.sfp_type = txgbe_sfp_type_25g_sr_core0;
+		else
+			hw->phy.sfp_type = txgbe_sfp_type_25g_sr_core1;
+	} else if (comp_codes_25g == TXGBE_SFF_25GBASELR_CAPABLE) {
+		if (hw->bus.lan_id == 0)
+			hw->phy.sfp_type = txgbe_sfp_type_25g_lr_core0;
+		else
+			hw->phy.sfp_type = txgbe_sfp_type_25g_lr_core1;
 	} else if (comp_codes_10g &
 		   (TXGBE_SFF_10GBASESR_CAPABLE |
 		    TXGBE_SFF_10GBASELR_CAPABLE)) {
@@ -910,11 +938,20 @@ ERR_I2C:
 
 	/* Determine if the SFP+ PHY is dual speed or not. */
 	hw->phy.multispeed_fiber = false;
-	if (((comp_codes_1g & TXGBE_SFF_1GBASESX_CAPABLE) &&
-	     (comp_codes_10g & TXGBE_SFF_10GBASESR_CAPABLE)) ||
-	    ((comp_codes_1g & TXGBE_SFF_1GBASELX_CAPABLE) &&
-	     (comp_codes_10g & TXGBE_SFF_10GBASELR_CAPABLE)))
-		hw->phy.multispeed_fiber = true;
+	if (hw->mac.type == txgbe_mac_aml) {
+		if ((comp_codes_25g == TXGBE_SFF_25GBASESR_CAPABLE ||
+		     comp_codes_25g == TXGBE_SFF_25GBASELR_CAPABLE ||
+		     comp_codes_25g == TXGBE_SFF_25GBASEER_CAPABLE) &&
+		   ((comp_codes_10g & TXGBE_SFF_10GBASESR_CAPABLE) ||
+		    (comp_codes_10g & TXGBE_SFF_10GBASELR_CAPABLE)))
+			hw->phy.multispeed_fiber = true;
+	} else {
+		if (((comp_codes_1g & TXGBE_SFF_1GBASESX_CAPABLE) &&
+		     (comp_codes_10g & TXGBE_SFF_10GBASESR_CAPABLE)) ||
+		    ((comp_codes_1g & TXGBE_SFF_1GBASELX_CAPABLE) &&
+		     (comp_codes_10g & TXGBE_SFF_10GBASELR_CAPABLE)))
+			hw->phy.multispeed_fiber = true;
+	}
 
 	/* Determine PHY vendor */
 	if (hw->phy.type != txgbe_phy_nl) {
@@ -972,7 +1009,7 @@ ERR_I2C:
 	}
 
 	/* Verify supported 1G SFP modules */
-	if (comp_codes_10g == 0 &&
+	if (comp_codes_10g == 0 && comp_codes_25g == 0 &&
 	    !(hw->phy.sfp_type == txgbe_sfp_type_1g_cu_core1 ||
 	      hw->phy.sfp_type == txgbe_sfp_type_1g_cu_core0 ||
 	      hw->phy.sfp_type == txgbe_sfp_type_1g_lx_core0 ||
@@ -992,7 +1029,7 @@ ERR_I2C:
 	      hw->phy.sfp_type == txgbe_sfp_type_1g_lx_core1 ||
 	      hw->phy.sfp_type == txgbe_sfp_type_1g_sx_core0 ||
 	      hw->phy.sfp_type == txgbe_sfp_type_1g_sx_core1)) {
-		DEBUGOUT("SFP+ module not supported\n");
+		DEBUGOUT("SFP+ module not supported");
 		hw->phy.type = txgbe_phy_sfp_unsupported;
 		return TXGBE_ERR_SFP_NOT_SUPPORTED;
 	}
@@ -1020,8 +1057,7 @@ s32 txgbe_identify_qsfp_module(struct txgbe_hw *hw)
 	u8 cable_length = 0;
 	u8 device_tech = 0;
 	bool active_cable = false;
-
-	DEBUGFUNC("txgbe_identify_qsfp_module");
+	u32 value;
 
 	if (hw->phy.media_type != txgbe_media_type_fiber_qsfp) {
 		hw->phy.sfp_type = txgbe_sfp_type_not_present;
@@ -1029,6 +1065,16 @@ s32 txgbe_identify_qsfp_module(struct txgbe_hw *hw)
 		goto out;
 	}
 
+	if (hw->mac.type == txgbe_mac_aml40) {
+		/* config GPIO before read i2c */
+		wr32(hw, TXGBE_GPIODATA, TXGBE_GPIOBIT_1);
+		value = rd32(hw, TXGBE_GPIOEXT);
+		if (value & TXGBE_SFP1_MOD_PRST_LS) {
+			hw->phy.sfp_type = txgbe_sfp_type_not_present;
+			return TXGBE_ERR_SFP_NOT_PRESENT;
+		}
+	}
+
 	err = hw->phy.read_i2c_eeprom(hw, TXGBE_SFF_IDENTIFIER,
 					     &identifier);
 ERR_I2C:
@@ -1060,10 +1106,27 @@ ERR_I2C:
 
 	if (comp_codes_10g & TXGBE_SFF_QSFP_DA_PASSIVE_CABLE) {
 		hw->phy.type = txgbe_phy_qsfp_unknown_passive;
+		if (hw->mac.type == txgbe_mac_aml40) {
+			if (hw->bus.lan_id == 0)
+				hw->phy.sfp_type = txgbe_qsfp_type_40g_cu_core0;
+			else
+				hw->phy.sfp_type = txgbe_qsfp_type_40g_cu_core1;
+		} else {
+			if (hw->bus.lan_id == 0)
+				hw->phy.sfp_type = txgbe_sfp_type_da_cu_core0;
+			else
+				hw->phy.sfp_type = txgbe_sfp_type_da_cu_core1;
+		}
+	} else if (comp_codes_10g & TXGBE_SFF_40GBASE_SR4) {
 		if (hw->bus.lan_id == 0)
-			hw->phy.sfp_type = txgbe_sfp_type_da_cu_core0;
+			hw->phy.sfp_type = txgbe_qsfp_type_40g_sr_core0;
 		else
-			hw->phy.sfp_type = txgbe_sfp_type_da_cu_core1;
+			hw->phy.sfp_type = txgbe_qsfp_type_40g_sr_core1;
+	} else if (comp_codes_10g & TXGBE_SFF_40GBASE_LR4) {
+		if (hw->bus.lan_id == 0)
+			hw->phy.sfp_type = txgbe_qsfp_type_40g_lr_core0;
+		else
+			hw->phy.sfp_type = txgbe_qsfp_type_40g_lr_core1;
 	} else if (comp_codes_10g & (TXGBE_SFF_10GBASESR_CAPABLE |
 				     TXGBE_SFF_10GBASELR_CAPABLE)) {
 		if (hw->bus.lan_id == 0)
@@ -1165,10 +1228,10 @@ ERR_I2C:
 				if (hw->allow_unsupported_sfp) {
 					DEBUGOUT("WARNING: Wangxun (R) Network Connections are quality tested using Wangxun (R) Ethernet Optics. "
 						"Using untested modules is not supported and may cause unstable operation or damage to the module or the adapter. "
-						"Wangxun Corporation is not responsible for any harm caused by using untested modules.\n");
+						"Wangxun Corporation is not responsible for any harm caused by using untested modules.");
 					err = 0;
 				} else {
-					DEBUGOUT("QSFP module not supported\n");
+					DEBUGOUT("QSFP module not supported");
 					hw->phy.type =
 						txgbe_phy_sfp_unsupported;
 					err = TXGBE_ERR_SFP_NOT_SUPPORTED;
@@ -1194,8 +1257,6 @@ out:
 s32 txgbe_read_i2c_eeprom(struct txgbe_hw *hw, u8 byte_offset,
 				  u8 *eeprom_data)
 {
-	DEBUGFUNC("txgbe_read_i2c_eeprom");
-
 	return hw->phy.read_i2c_byte(hw, byte_offset,
 					 TXGBE_I2C_EEPROM_DEV_ADDR,
 					 eeprom_data);
@@ -1228,8 +1289,6 @@ s32 txgbe_read_i2c_sff8472(struct txgbe_hw *hw, u8 byte_offset,
 s32 txgbe_write_i2c_eeprom(struct txgbe_hw *hw, u8 byte_offset,
 				   u8 eeprom_data)
 {
-	DEBUGFUNC("txgbe_write_i2c_eeprom");
-
 	return hw->phy.write_i2c_byte(hw, byte_offset,
 					  TXGBE_I2C_EEPROM_DEV_ADDR,
 					  eeprom_data);
@@ -1248,8 +1307,6 @@ s32 txgbe_write_i2c_eeprom(struct txgbe_hw *hw, u8 byte_offset,
 s32 txgbe_read_i2c_byte_unlocked(struct txgbe_hw *hw, u8 byte_offset,
 					   u8 dev_addr, u8 *data)
 {
-	DEBUGFUNC("txgbe_read_i2c_byte");
-
 	txgbe_i2c_start(hw, dev_addr);
 
 	/* wait tx empty */
@@ -1312,8 +1369,6 @@ s32 txgbe_read_i2c_byte(struct txgbe_hw *hw, u8 byte_offset,
 s32 txgbe_write_i2c_byte_unlocked(struct txgbe_hw *hw, u8 byte_offset,
 					    u8 dev_addr, u8 data)
 {
-	DEBUGFUNC("txgbe_write_i2c_byte");
-
 	txgbe_i2c_start(hw, dev_addr);
 
 	/* wait tx empty */
@@ -1367,8 +1422,6 @@ s32 txgbe_write_i2c_byte(struct txgbe_hw *hw, u8 byte_offset,
  **/
 static void txgbe_i2c_start(struct txgbe_hw *hw, u8 dev_addr)
 {
-	DEBUGFUNC("txgbe_i2c_start");
-
 	wr32(hw, TXGBE_I2CENA, 0);
 
 	wr32(hw, TXGBE_I2CCON,
@@ -1396,27 +1449,57 @@ static void txgbe_i2c_start(struct txgbe_hw *hw, u8 dev_addr)
  **/
 static void txgbe_i2c_stop(struct txgbe_hw *hw)
 {
-	DEBUGFUNC("txgbe_i2c_stop");
-
 	/* wait for completion */
 	if (!po32m(hw, TXGBE_I2CSTAT, TXGBE_I2CSTAT_MST,
 		0, NULL, 100, 100)) {
-		DEBUGFUNC("i2c stop timeout.");
+		DEBUGOUT("i2c stop timeout.");
 	}
 
 	wr32(hw, TXGBE_I2CENA, 0);
 }
 
+/**
+ *  txgbe_check_overtemp - Checks if an overtemp occurred.
+ *  @hw: pointer to hardware structure
+ *
+ *  Checks if the temp alarm status was triggered due to overtemp
+ **/
+s32 txgbe_check_overtemp(struct txgbe_hw *hw)
+{
+	s32 status = 0;
+	u32 ts_state;
+
+	/* Check that the temp alarm status was triggered */
+	ts_state = rd32(hw, TXGBE_TS_ALARM_ST);
+
+	if (ts_state & TXGBE_TS_ALARM_ST_DALARM)
+		status = TXGBE_ERR_UNDERTEMP;
+	else if (ts_state & TXGBE_TS_ALARM_ST_ALARM)
+		status = TXGBE_ERR_OVERTEMP;
+
+	return status;
+}
+
 static void
 txgbe_set_sgmii_an37_ability(struct txgbe_hw *hw)
 {
 	u32 value;
+	u8 device_type = hw->subsystem_device_id & 0xF0;
 
 	wr32_epcs(hw, VR_XS_OR_PCS_MMD_DIGI_CTL1, 0x3002);
-	wr32_epcs(hw, SR_MII_MMD_AN_CTL, 0x0105);
+	/* for sgmii + external phy, set to 0x0105 (phy sgmii mode) */
+	/* for sgmii direct link, set to 0x010c (mac sgmii mode) */
+	if (device_type == TXGBE_DEV_ID_MAC_SGMII ||
+			hw->phy.media_type == txgbe_media_type_fiber)
+		wr32_epcs(hw, SR_MII_MMD_AN_CTL, 0x010C);
+	else if (device_type == TXGBE_DEV_ID_SGMII ||
+			device_type == TXGBE_DEV_ID_XAUI)
+		wr32_epcs(hw, SR_MII_MMD_AN_CTL, 0x0105);
 	wr32_epcs(hw, SR_MII_MMD_DIGI_CTL, 0x0200);
 	value = rd32_epcs(hw, SR_MII_MMD_CTL);
-	value = (value & ~0x1200) | (0x1 << 12) | (0x1 << 9);
+	value = (value & ~0x1200) | (0x1 << 9);
+	if (hw->autoneg)
+		value |= SR_MII_MMD_CTL_AN_EN;
 	wr32_epcs(hw, SR_MII_MMD_CTL, value);
 }
 
@@ -1455,6 +1538,10 @@ txgbe_set_link_to_kr(struct txgbe_hw *hw, bool autoneg)
 		if (!(hw->devarg.auto_neg == 1)) {
 			wr32_epcs(hw, SR_AN_CTRL, 0);
 			wr32_epcs(hw, VR_AN_KR_MODE_CL, 0);
+		} else {
+			value = rd32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1);
+			value &= ~(1 << 6);
+			wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1, value);
 		}
 		if (hw->devarg.present == 1) {
 			value = rd32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1);
@@ -1551,8 +1638,9 @@ txgbe_set_link_to_kx4(struct txgbe_hw *hw, bool autoneg)
 		goto out;
 	}
 
-	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE,
-			~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE, ~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACRXCFG, TXGBE_MACRXCFG_ENA, ~TXGBE_MACRXCFG_ENA);
+	hw->mac.disable_sec_tx_path(hw);
 
 	/* 2. Disable xpcs AN-73 */
 	if (!autoneg)
@@ -1725,9 +1813,10 @@ txgbe_set_link_to_kx4(struct txgbe_hw *hw, bool autoneg)
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1, value);
 	} else if (hw->fw_version <= TXGBE_FW_N_TXEQ) {
 		value = (0x1804 & ~0x3F3F);
+		value |= 40 << 8;
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL0, value);
 
-		value = (0x50 & ~0x7F) | 40 | (1 << 6);
+		value = (0x50 & ~0x7F) | (1 << 6);
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1, value);
 	}
 out:
@@ -1765,8 +1854,9 @@ txgbe_set_link_to_kx(struct txgbe_hw *hw,
 		goto out;
 	}
 
-	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE,
-				~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE, ~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACRXCFG, TXGBE_MACRXCFG_ENA, ~TXGBE_MACRXCFG_ENA);
+	hw->mac.disable_sec_tx_path(hw);
 
 	/* 2. Disable xpcs AN-73 */
 	if (!autoneg)
@@ -1939,10 +2029,10 @@ txgbe_set_link_to_kx(struct txgbe_hw *hw,
 		value |= hw->phy.ffe_post | (1 << 6);
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1, value);
 	} else if (hw->fw_version <= TXGBE_FW_N_TXEQ) {
-		value = (0x1804 & ~0x3F3F) | (24 << 8) | 4;
+		value = (0x1804 & ~0x3F3F) | (40 << 8);
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL0, value);
 
-		value = (0x50 & ~0x7F) | 16 | (1 << 6);
+		value = (0x50 & ~0x7F) | (1 << 6);
 		wr32_epcs(hw, TXGBE_PHY_TX_EQ_CTL1, value);
 	}
 out:
@@ -1972,8 +2062,9 @@ txgbe_set_link_to_sfi(struct txgbe_hw *hw,
 		goto out;
 	}
 
-	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE,
-			~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACTXCFG, TXGBE_MACTXCFG_TXE, ~TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACRXCFG, TXGBE_MACRXCFG_ENA, ~TXGBE_MACRXCFG_ENA);
+	hw->mac.disable_sec_tx_path(hw);
 
 	/* 2. Disable xpcs AN-73 */
 	wr32_epcs(hw, SR_AN_CTRL, 0x0);
@@ -2320,8 +2411,12 @@ void txgbe_autoc_write(struct txgbe_hw *hw, u64 autoc)
 		}
 	} else if (hw->phy.media_type == txgbe_media_type_fiber) {
 		txgbe_set_link_to_sfi(hw, speed);
+		if (speed == TXGBE_LINK_SPEED_1GB_FULL)
+			txgbe_set_sgmii_an37_ability(hw);
 	}
 
+	hw->mac.enable_sec_tx_path(hw);
+
 	if (speed == TXGBE_LINK_SPEED_10GB_FULL)
 		mactxcfg = TXGBE_MACTXCFG_SPEED_10G;
 	else if (speed == TXGBE_LINK_SPEED_1GB_FULL)
@@ -2331,6 +2426,7 @@ void txgbe_autoc_write(struct txgbe_hw *hw, u64 autoc)
 	wr32m(hw, TXGBE_MACTXCFG,
 		TXGBE_MACTXCFG_SPEED_MASK | TXGBE_MACTXCFG_TXE,
 		mactxcfg | TXGBE_MACTXCFG_TXE);
+	wr32m(hw, TXGBE_MACRXCFG, TXGBE_MACRXCFG_ENA, TXGBE_MACRXCFG_ENA);
 }
 
 void txgbe_bp_down_event(struct txgbe_hw *hw)
@@ -2416,8 +2512,6 @@ s32 txgbe_kr_handle(struct txgbe_hw *hw)
 	u32 value;
 	s32 status = 0;
 
-	DEBUGFUNC("txgbe_kr_handle");
-
 	value = rd32_epcs(hw, VR_AN_INTR);
 	BP_LOG("AN INTERRUPT!! value: 0x%x\n", value);
 	if (!(value & VR_AN_INTR_PG_RCV)) {
@@ -2441,8 +2535,6 @@ static s32 txgbe_handle_bp_flow(u32 link_mode, struct txgbe_hw *hw)
 	s32 status = 0;
 	struct txgbe_backplane_ability local_ability, lp_ability;
 
-	DEBUGFUNC("txgbe_handle_bp_flow");
-
 	local_ability.current_link_mode = link_mode;
 
 	/* 1. Get the local AN73 Base Page Ability */
@@ -2544,8 +2636,6 @@ static void txgbe_get_bp_ability(struct txgbe_backplane_ability *ability,
 {
 	u32 value = 0;
 
-	DEBUGFUNC("txgbe_get_bp_ability");
-
 	/* Link Partner Base Page */
 	if (link_partner == 1) {
 		/* Read the link partner AN73 Base Page Ability Registers */
@@ -2617,8 +2707,6 @@ static s32 txgbe_check_bp_ability(struct txgbe_backplane_ability *local_ability,
 	u32 com_link_abi;
 	s32 ret = 0;
 
-	DEBUGFUNC("txgbe_check_bp_ability");
-
 	com_link_abi = local_ability->link_ability & lp_ability->link_ability;
 	BP_LOG("com_link_abi = 0x%x, local_ability = 0x%x, lp_ability = 0x%x\n",
 		com_link_abi, local_ability->link_ability,
@@ -2674,8 +2762,6 @@ static void txgbe_clear_bp_intr(u32 bit, u32 bit_high, struct txgbe_hw *hw)
 {
 	u32 rdata = 0, wdata, i;
 
-	DEBUGFUNC("txgbe_clear_bp_intr");
-
 	rdata = rd32_epcs(hw, VR_AN_INTR);
 	BP_LOG("[Before clear]Read VR AN MMD Interrupt Register: 0x%x\n",
 			rdata);
@@ -2700,8 +2786,6 @@ static s32 txgbe_enable_kr_training(struct txgbe_hw *hw)
 	s32 status = 0;
 	u32 value = 0;
 
-	DEBUGFUNC("txgbe_enable_kr_training");
-
 	BP_LOG("Enable Clause 72 KR Training ...\n");
 
 	if (CL72_KRTR_PRBS_MODE_EN != 0xFFFF) {
@@ -2745,8 +2829,6 @@ static s32 txgbe_disable_kr_training(struct txgbe_hw *hw, s32 post, s32 mode)
 {
 	s32 status = 0;
 
-	DEBUGFUNC("txgbe_disable_kr_training");
-
 	BP_LOG("Disable Clause 72 KR Training ...\n");
 	/* Read PHY Lane0 TX EQ before Clause 72 KR Training. */
 	txgbe_read_phy_lane_tx_eq(0, hw, post, mode);
@@ -2763,8 +2845,6 @@ static s32 txgbe_check_kr_training(struct txgbe_hw *hw)
 	int i;
 	int times = hw->devarg.poll ? 35 : 20;
 
-	DEBUGFUNC("txgbe_check_kr_training");
-
 	for (i = 0; i < times; i++) {
 		value = rd32_epcs(hw, SR_PMA_KR_LP_CEU);
 		BP_LOG("SR PMA MMD 10GBASE-KR LP Coefficient Update Register: 0x%x\n",
@@ -2822,8 +2902,6 @@ static void txgbe_read_phy_lane_tx_eq(u16 lane, struct txgbe_hw *hw,
 	u32 addr;
 	u32 tx_main_cursor, tx_pre_cursor, tx_post_cursor, lmain;
 
-	DEBUGFUNC("txgbe_read_phy_lane_tx_eq");
-
 	addr = TXGBE_PHY_LANE0_TX_EQ_CTL1 | (lane << 8);
 	value = rd32_ephy(hw, addr);
 	BP_LOG("PHY LANE TX EQ Read Value: %x\n", lane);
