@@ -165,12 +165,21 @@ configure_fdir_flags(const struct rte_eth_fdir_conf *conf,
 	return 0;
 }
 
+static inline uint16_t
+txgbe_reverse_fdir_bitmasks(uint16_t mask)
+{
+	mask = ((mask & 0x5555) << 1) | ((mask & 0xAAAA) >> 1);
+	mask = ((mask & 0x3333) << 2) | ((mask & 0xCCCC) >> 2);
+	mask = ((mask & 0x0F0F) << 4) | ((mask & 0xF0F0) >> 4);
+	return ((mask & 0x00FF) << 8) | ((mask & 0xFF00) >> 8);
+}
+
 int
 txgbe_fdir_set_input_mask(struct rte_eth_dev *dev)
 {
 	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
 	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
-	enum rte_fdir_mode mode = dev->data->dev_conf.fdir_conf.mode;
+	enum rte_fdir_mode mode = TXGBE_DEV_FDIR_CONF(dev)->mode;
 	/*
 	 * mask VM pool and DIPv6 since there are currently not supported
 	 * mask FLEX byte, it will be set in flex_conf
@@ -187,18 +196,12 @@ txgbe_fdir_set_input_mask(struct rte_eth_dev *dev)
 		return -ENOTSUP;
 	}
 
-	/*
-	 * Program the relevant mask registers.  If src/dst_port or src/dst_addr
-	 * are zero, then assume a full mask for that field. Also assume that
-	 * a VLAN of 0 is unspecified, so mask that out as well.  L4type
-	 * cannot be masked out in this implementation.
-	 */
-	if (info->mask.dst_port_mask == 0 && info->mask.src_port_mask == 0) {
-		/* use the L4 protocol mask for raw IPv4/IPv6 traffic */
-		fdirm |= TXGBE_FDIRMSK_L4P;
-	}
+	/* use the L4 protocol mask for raw IPv4/IPv6 traffic */
+	if (info->mask.pkt_type_mask == 0 && info->mask.dst_port_mask == 0 &&
+	    info->mask.src_port_mask == 0)
+		info->mask.pkt_type_mask |= TXGBE_FDIRMSK_L4P;
 
-	/* TBD: don't support encapsulation yet */
+	fdirm |= info->mask.pkt_type_mask;
 	wr32(hw, TXGBE_FDIRMSK, fdirm);
 
 	/* store the TCP/UDP port masks */
@@ -212,19 +215,16 @@ txgbe_fdir_set_input_mask(struct rte_eth_dev *dev)
 	wr32(hw, TXGBE_FDIRUDPMSK, ~fdirtcpm);
 	wr32(hw, TXGBE_FDIRSCTPMSK, ~fdirtcpm);
 
-	/* Store source and destination IPv4 masks (big-endian) */
-	wr32(hw, TXGBE_FDIRSIP4MSK, ~info->mask.src_ipv4_mask);
-	wr32(hw, TXGBE_FDIRDIP4MSK, ~info->mask.dst_ipv4_mask);
+	/* Store source and destination IPv4 masks (little-endian) */
+	wr32(hw, TXGBE_FDIRSIP4MSK, rte_be_to_cpu_32(~info->mask.src_ipv4_mask));
+	wr32(hw, TXGBE_FDIRDIP4MSK, rte_be_to_cpu_32(~info->mask.dst_ipv4_mask));
 
-	if (mode == RTE_FDIR_MODE_SIGNATURE) {
-		/*
-		 * Store source and destination IPv6 masks (bit reversed)
-		 */
-		fdiripv6m = TXGBE_FDIRIP6MSK_DST(info->mask.dst_ipv6_mask) |
-			    TXGBE_FDIRIP6MSK_SRC(info->mask.src_ipv6_mask);
-
-		wr32(hw, TXGBE_FDIRIP6MSK, ~fdiripv6m);
-	}
+	/*
+	 * Store source and destination IPv6 masks (bit reversed)
+	 */
+	fdiripv6m = txgbe_reverse_fdir_bitmasks(info->mask.dst_ipv6_mask) << 16;
+	fdiripv6m |= txgbe_reverse_fdir_bitmasks(info->mask.src_ipv6_mask);
+	wr32(hw, TXGBE_FDIRIP6MSK, ~fdiripv6m);
 
 	return 0;
 }
@@ -232,9 +232,8 @@ txgbe_fdir_set_input_mask(struct rte_eth_dev *dev)
 static int
 txgbe_fdir_store_input_mask(struct rte_eth_dev *dev)
 {
-	struct rte_eth_fdir_masks *input_mask =
-				&dev->data->dev_conf.fdir_conf.mask;
-	enum rte_fdir_mode mode = dev->data->dev_conf.fdir_conf.mode;
+	struct rte_eth_fdir_masks *input_mask = &TXGBE_DEV_FDIR_CONF(dev)->mask;
+	enum rte_fdir_mode mode = TXGBE_DEV_FDIR_CONF(dev)->mode;
 	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
 	uint16_t dst_ipv6m = 0;
 	uint16_t src_ipv6m = 0;
@@ -259,9 +258,21 @@ txgbe_fdir_store_input_mask(struct rte_eth_dev *dev)
 	return 0;
 }
 
+uint16_t
+txgbe_fdir_get_flex_base(struct txgbe_fdir_rule *rule)
+{
+	if (!rule->flex_relative)
+		return TXGBE_FDIRFLEXCFG_BASE_MAC;
+
+	if (rule->input.flow_type & TXGBE_ATR_L4TYPE_MASK)
+		return TXGBE_FDIRFLEXCFG_BASE_PAY;
+
+	return TXGBE_FDIRFLEXCFG_BASE_L3;
+}
+
 int
 txgbe_fdir_set_flexbytes_offset(struct rte_eth_dev *dev,
-				uint16_t offset)
+				uint16_t offset, uint16_t flex_base)
 {
 	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
 	int i;
@@ -269,7 +280,7 @@ txgbe_fdir_set_flexbytes_offset(struct rte_eth_dev *dev,
 	for (i = 0; i < 64; i++) {
 		uint32_t flexreg, flex;
 		flexreg = rd32(hw, TXGBE_FDIRFLEXCFG(i / 4));
-		flex = TXGBE_FDIRFLEXCFG_BASE_MAC;
+		flex = flex_base;
 		flex |= TXGBE_FDIRFLEXCFG_OFST(offset / 2);
 		flexreg &= ~(TXGBE_FDIRFLEXCFG_ALL(~0UL, i % 4));
 		flexreg |= TXGBE_FDIRFLEXCFG_ALL(flex, i % 4);
@@ -294,7 +305,7 @@ static int
 txgbe_set_fdir_flex_conf(struct rte_eth_dev *dev, uint32_t flex)
 {
 	const struct rte_eth_fdir_flex_conf *conf =
-				&dev->data->dev_conf.fdir_conf.flex_conf;
+				&TXGBE_DEV_FDIR_CONF(dev)->flex_conf;
 	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
 	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
 	const struct rte_eth_flex_payload_cfg *flex_cfg;
@@ -364,7 +375,7 @@ txgbe_fdir_configure(struct rte_eth_dev *dev)
 	int err;
 	uint32_t fdirctrl, flex, pbsize;
 	int i;
-	enum rte_fdir_mode mode = dev->data->dev_conf.fdir_conf.mode;
+	enum rte_fdir_mode mode = TXGBE_DEV_FDIR_CONF(dev)->mode;
 
 	PMD_INIT_FUNC_TRACE();
 
@@ -373,8 +384,7 @@ txgbe_fdir_configure(struct rte_eth_dev *dev)
 	    mode != RTE_FDIR_MODE_PERFECT)
 		return -ENOSYS;
 
-	err = configure_fdir_flags(&dev->data->dev_conf.fdir_conf,
-				   &fdirctrl, &flex);
+	err = configure_fdir_flags(TXGBE_DEV_FDIR_CONF(dev), &fdirctrl, &flex);
 	if (err)
 		return err;
 
@@ -635,6 +645,14 @@ fdir_write_perfect_filter(struct txgbe_hw *hw,
 	fdircmd |= TXGBE_FDIRPICMD_QP(queue);
 	fdircmd |= TXGBE_FDIRPICMD_POOL(input->vm_pool);
 
+	if (input->flow_type & TXGBE_ATR_L3TYPE_IPV6) {
+		/* use SIP4 to store LS Dword of the Source iPv6 address */
+		wr32(hw, TXGBE_FDIRPISIP4, be_to_le32(input->src_ip[3]));
+		wr32(hw, TXGBE_FDIRPISIP6(0), be_to_le32(input->src_ip[2]));
+		wr32(hw, TXGBE_FDIRPISIP6(1), be_to_le32(input->src_ip[1]));
+		wr32(hw, TXGBE_FDIRPISIP6(2), be_to_le32(input->src_ip[0]));
+		fdircmd |= TXGBE_FDIRPICMD_IP6;
+	}
 	wr32(hw, TXGBE_FDIRPICMD, fdircmd);
 
 	PMD_DRV_LOG(DEBUG, "Rx Queue=%x hash=%x", queue, fdirhash);
@@ -757,7 +775,7 @@ txgbe_insert_fdir_filter(struct txgbe_hw_fdir_info *fdir_info,
 
 	TAILQ_INSERT_TAIL(&fdir_info->fdir_list, fdir_filter, entries);
 
-	return 0;
+	return ret;
 }
 
 static inline int
@@ -777,7 +795,27 @@ txgbe_remove_fdir_filter(struct txgbe_hw_fdir_info *fdir_info,
 	TAILQ_REMOVE(&fdir_info->fdir_list, fdir_filter, entries);
 	rte_free(fdir_filter);
 
-	return 0;
+	return ret;
+}
+
+static void
+txgbe_fdir_mask_input(struct txgbe_hw_fdir_mask *mask,
+		      struct txgbe_atr_input *input)
+{
+	int i;
+
+	if (input->flow_type & TXGBE_ATR_L3TYPE_IPV6) {
+		for (i = 0; i < 16; i++) {
+			if (!(mask->src_ipv6_mask & (1 << i)))
+				input->src_ip[i / 4] &= ~(0xFF << ((i % 4) * 8));
+		}
+	} else {
+		input->src_ip[0] &= mask->src_ipv4_mask;
+		input->dst_ip[0] &= mask->dst_ipv4_mask;
+	}
+
+	input->src_port &= mask->src_port_mask;
+	input->dst_port &= mask->dst_port_mask;
 }
 
 int
@@ -792,7 +830,7 @@ txgbe_fdir_filter_program(struct rte_eth_dev *dev,
 	bool is_perfect = FALSE;
 	int err;
 	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
-	enum rte_fdir_mode fdir_mode = dev->data->dev_conf.fdir_conf.mode;
+	enum rte_fdir_mode fdir_mode = TXGBE_DEV_FDIR_CONF(dev)->mode;
 	struct txgbe_fdir_filter *node;
 
 	if (fdir_mode == RTE_FDIR_MODE_NONE ||
@@ -802,18 +840,15 @@ txgbe_fdir_filter_program(struct rte_eth_dev *dev,
 	if (fdir_mode >= RTE_FDIR_MODE_PERFECT)
 		is_perfect = TRUE;
 
+	txgbe_fdir_mask_input(&info->mask, &rule->input);
+
 	if (is_perfect) {
-		if (rule->input.flow_type & TXGBE_ATR_L3TYPE_IPV6) {
-			PMD_DRV_LOG(ERR, "IPv6 is not supported in"
-				    " perfect mode!");
-			return -ENOTSUP;
-		}
 		fdirhash = atr_compute_perfect_hash(&rule->input,
-				dev->data->dev_conf.fdir_conf.pballoc);
+				TXGBE_DEV_FDIR_CONF(dev)->pballoc);
 		fdirhash |= TXGBE_FDIRPIHASH_IDX(rule->soft_id);
 	} else {
 		fdirhash = atr_compute_signature_hash(&rule->input,
-				dev->data->dev_conf.fdir_conf.pballoc);
+				TXGBE_DEV_FDIR_CONF(dev)->pballoc);
 	}
 
 	if (del) {
@@ -839,13 +874,16 @@ txgbe_fdir_filter_program(struct rte_eth_dev *dev,
 				    " signature mode.");
 			return -EINVAL;
 		}
-		queue = dev->data->dev_conf.fdir_conf.drop_queue;
+		queue = TXGBE_DEV_FDIR_CONF(dev)->drop_queue;
 	} else if (rule->queue < TXGBE_MAX_RX_QUEUE_NUM) {
 		queue = rule->queue;
 	} else {
 		return -EINVAL;
 	}
 
+	if (RTE_ETH_DEV_SRIOV(dev).active)
+		queue = RTE_ETH_DEV_SRIOV(dev).def_pool_q_idx + queue;
+
 	node = txgbe_fdir_filter_lookup(info, &rule->input);
 	if (node) {
 		if (!update) {
@@ -891,6 +929,147 @@ txgbe_fdir_filter_program(struct rte_eth_dev *dev,
 	return err;
 }
 
+static void
+txgbevf_flush_fdir_filter(struct rte_eth_dev *dev)
+{
+	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
+	uint32_t msg[2] = {0, 0};
+
+	/* flush bit */
+	msg[TXGBEVF_FDIR_CMD] = 1 << 16;
+
+	txgbevf_set_fdir(hw, msg, FALSE);
+}
+
+static int
+txgbevf_del_fdir_filter(struct rte_eth_dev *dev,
+			struct txgbe_fdir_rule *rule,
+			int id)
+{
+	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
+	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
+	uint32_t msg[2] = {0, 0};
+	int ret = 0;
+
+	/* node id [15:0] */
+	msg[TXGBEVF_FDIR_CMD] = id;
+
+	ret = txgbevf_set_fdir(hw, msg, FALSE);
+	if (ret) {
+		PMD_DRV_LOG(ERR, "VF request PF to delete FDIR filters failed.");
+		return ret;
+	}
+
+	ret = txgbe_remove_fdir_filter(info, &rule->input);
+	if (ret < 0)
+		PMD_DRV_LOG(ERR, "Fail to delete FDIR filter!");
+
+	return 0;
+}
+
+static int
+txgbevf_add_fdir_filter(struct rte_eth_dev *dev,
+			struct txgbe_fdir_rule *rule,
+			int id)
+{
+	struct txgbe_hw *hw = TXGBE_DEV_HW(dev);
+	uint32_t msg[TXGBEVF_FDIR_MAX];
+	int ret = 0;
+
+	memset(msg, 0, sizeof(msg));
+
+	/* node id [15:0] */
+	msg[TXGBEVF_FDIR_CMD] = id;
+	/* ring_idx [23:16] */
+	msg[TXGBEVF_FDIR_CMD] |= rule->queue << 16;
+	/* flow_type [30:24] */
+	msg[TXGBEVF_FDIR_CMD] |= rule->input.flow_type << 24;
+
+	msg[TXGBEVF_FDIR_IP4SA] = rule->input.src_ip[0];
+	msg[TXGBEVF_FDIR_IP4DA] = rule->input.dst_ip[0];
+	msg[TXGBEVF_FDIR_PORT] = (rule->input.dst_port << 16) |
+				rule->input.src_port;
+	if (rule->mask.flex_bytes_mask) {
+		/* base [1:0] */
+		msg[TXGBEVF_FDIR_FLEX] = txgbe_fdir_get_flex_base(rule);
+		/* offset [7:3] */
+		msg[TXGBEVF_FDIR_FLEX] |=
+			TXGBE_FDIRFLEXCFG_OFST(rule->flex_bytes_offset / 2);
+		/* flex bytes [31:16]*/
+		msg[TXGBEVF_FDIR_FLEX] |= rule->input.flex_bytes << 16;
+	}
+	msg[TXGBEVF_FDIR_IP4SM] = rule->mask.src_ipv4_mask;
+	msg[TXGBEVF_FDIR_IP4DM] = rule->mask.dst_ipv4_mask;
+	msg[TXGBEVF_FDIR_PORTM] = (rule->mask.dst_port_mask << 16) |
+				rule->mask.src_port_mask;
+
+	ret = txgbevf_set_fdir(hw, msg, TRUE);
+	if (ret)
+		PMD_DRV_LOG(ERR, "VF request PF to add FDIR filters failed.");
+
+	return ret;
+}
+
+int
+txgbevf_fdir_filter_program(struct rte_eth_dev *dev,
+			    struct txgbe_fdir_rule *rule,
+			    bool del)
+{
+	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
+	struct txgbe_atr_input *input = &rule->input;
+	struct txgbe_fdir_filter *node;
+	uint32_t fdirhash;
+	int ret;
+
+	if (rule->mode != RTE_FDIR_MODE_PERFECT ||
+	    rule->fdirflags == TXGBE_FDIRPICMD_DROP)
+		return -ENOTSUP;
+
+	if (input->flow_type & TXGBE_ATR_FLOW_TYPE_IPV6)
+		return -ENOTSUP;
+
+	fdirhash = atr_compute_perfect_hash(input,
+			TXGBE_DEV_FDIR_CONF(dev)->pballoc);
+
+	ret = rte_hash_lookup(info->hash_handle, (const void *)input);
+	if (ret < 0) {
+		if (del) {
+			PMD_DRV_LOG(ERR, "No such fdir filter to delete!");
+			return ret;
+		}
+	} else {
+		if (!del) {
+			PMD_DRV_LOG(ERR, "Conflict with existing fdir filter!");
+			return -EINVAL;
+		}
+	}
+
+	if (del)
+		return txgbevf_del_fdir_filter(dev, rule, ret);
+
+	node = rte_zmalloc("txgbe_fdir",
+			   sizeof(struct txgbe_fdir_filter), 0);
+	if (!node)
+		return -ENOMEM;
+	rte_memcpy(&node->input, input,
+		   sizeof(struct txgbe_atr_input));
+	node->fdirflags = rule->fdirflags;
+	node->fdirhash = fdirhash;
+	node->queue = rule->queue;
+
+	ret = txgbe_insert_fdir_filter(info, node);
+	if (ret < 0) {
+		rte_free(node);
+		return ret;
+	}
+
+	ret = txgbevf_add_fdir_filter(dev, rule, ret);
+	if (ret)
+		txgbe_remove_fdir_filter(info, input);
+
+	return ret;
+}
+
 static int
 txgbe_fdir_flush(struct rte_eth_dev *dev)
 {
@@ -898,6 +1077,11 @@ txgbe_fdir_flush(struct rte_eth_dev *dev)
 	struct txgbe_hw_fdir_info *info = TXGBE_DEV_FDIR(dev);
 	int ret;
 
+	if (!txgbe_is_pf(hw)) {
+		txgbevf_flush_fdir_filter(dev);
+		return 0;
+	}
+
 	ret = txgbe_reinit_fdir_tables(hw);
 	if (ret < 0) {
 		PMD_INIT_LOG(ERR, "Failed to re-initialize FD table.");
@@ -909,6 +1093,11 @@ txgbe_fdir_flush(struct rte_eth_dev *dev)
 	info->add = 0;
 	info->remove = 0;
 
+	memset(&info->mask, 0, sizeof(struct txgbe_hw_fdir_mask));
+	info->mask_added = false;
+	info->flex_relative = false;
+	info->flex_bytes_offset = 0;
+
 	return ret;
 }
 
@@ -920,7 +1109,7 @@ txgbe_fdir_filter_restore(struct rte_eth_dev *dev)
 	struct txgbe_hw_fdir_info *fdir_info = TXGBE_DEV_FDIR(dev);
 	struct txgbe_fdir_filter *node;
 	bool is_perfect = FALSE;
-	enum rte_fdir_mode fdir_mode = dev->data->dev_conf.fdir_conf.mode;
+	enum rte_fdir_mode fdir_mode = TXGBE_DEV_FDIR_CONF(dev)->mode;
 
 	if (fdir_mode >= RTE_FDIR_MODE_PERFECT &&
 	    fdir_mode <= RTE_FDIR_MODE_PERFECT_TUNNEL)
@@ -950,6 +1139,7 @@ txgbe_fdir_filter_restore(struct rte_eth_dev *dev)
 int
 txgbe_clear_all_fdir_filter(struct rte_eth_dev *dev)
 {
+	struct rte_eth_fdir_conf *fdir_conf = TXGBE_DEV_FDIR_CONF(dev);
 	struct txgbe_hw_fdir_info *fdir_info = TXGBE_DEV_FDIR(dev);
 	struct txgbe_fdir_filter *fdir_filter;
 	struct txgbe_fdir_filter *filter_flag;
@@ -958,7 +1148,9 @@ txgbe_clear_all_fdir_filter(struct rte_eth_dev *dev)
 	/* flush flow director */
 	rte_hash_reset(fdir_info->hash_handle);
 	memset(fdir_info->hash_map, 0,
-	       sizeof(struct txgbe_fdir_filter *) * TXGBE_MAX_FDIR_FILTER_NUM);
+	       sizeof(struct txgbe_fdir_filter *) *
+	       ((1024 << (fdir_conf->pballoc + 1)) - 2));
+	fdir_conf->mode = RTE_FDIR_MODE_NONE;
 	filter_flag = TAILQ_FIRST(&fdir_info->fdir_list);
 	while ((fdir_filter = TAILQ_FIRST(&fdir_info->fdir_list))) {
 		TAILQ_REMOVE(&fdir_info->fdir_list,
