@@ -5,17 +5,16 @@
  */
 
 #include <stdio.h>
-#include <string.h>
 #include <unistd.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <inttypes.h>
 #include <fcntl.h>
 
 #include "cmdline.h"
 #include "cmdline_private.h"
 #include "cmdline_socket.h"
 
+#include <eal_export.h>
+
+RTE_EXPORT_SYMBOL(cmdline_file_new)
 struct cmdline *
 cmdline_file_new(cmdline_parse_ctx_t *ctx, const char *prompt, const char *path)
 {
@@ -33,6 +32,7 @@ cmdline_file_new(cmdline_parse_ctx_t *ctx, const char *prompt, const char *path)
 	return cmdline_new(ctx, prompt, fd, -1);
 }
 
+RTE_EXPORT_SYMBOL(cmdline_stdin_new)
 struct cmdline *
 cmdline_stdin_new(cmdline_parse_ctx_t *ctx, const char *prompt)
 {
@@ -46,6 +46,7 @@ cmdline_stdin_new(cmdline_parse_ctx_t *ctx, const char *prompt)
 	return cl;
 }
 
+RTE_EXPORT_SYMBOL(cmdline_stdin_exit)
 void
 cmdline_stdin_exit(struct cmdline *cl)
 {
