@@ -2,10 +2,13 @@
  * Copyright 2020 Mellanox Technologies, Ltd
  */
 
+#include <errno.h>
 #include <time.h>
 
+#include <eal_export.h>
 #include <rte_cycles.h>
 
+RTE_EXPORT_SYMBOL(rte_delay_us_sleep)
 void
 rte_delay_us_sleep(unsigned int us)
 {
