@@ -1,6 +1,7 @@
 /* SPDX-License-Identifier: BSD-3-Clause
  * Copyright(c) 2018 Intel Corporation.
  */
+#include <eal_export.h>
 #include <rte_spinlock.h>
 #include <rte_service_component.h>
 #include <ethdev_driver.h>
@@ -18,6 +19,9 @@
 #define TXA_INVALID_DEV_ID	INT32_C(-1)
 #define TXA_INVALID_SERVICE_ID	INT64_C(-1)
 
+#define TXA_ADAPTER_ARRAY "txa_adapter_array"
+#define TXA_SERVICE_DATA_ARRAY "txa_service_data_array"
+
 #define txa_evdev(id) (&rte_eventdevs[txa_dev_id_array[(id)]])
 
 #define txa_dev_caps_get(id) txa_evdev((id))->dev_ops->eth_tx_adapter_caps_get
@@ -41,10 +45,19 @@
 
 #define txa_dev_stats_get(t) txa_evdev(t)->dev_ops->eth_tx_adapter_stats_get
 
+#define txa_dev_instance_get(id) \
+			txa_evdev(id)->dev_ops->eth_tx_adapter_instance_get
+
+#define txa_dev_queue_start(id) \
+			txa_evdev(id)->dev_ops->eth_tx_adapter_queue_start
+
+#define txa_dev_queue_stop(id) \
+			txa_evdev(id)->dev_ops->eth_tx_adapter_queue_stop
+
 #define RTE_EVENT_ETH_TX_ADAPTER_ID_VALID_OR_ERR_RET(id, retval) \
 do { \
 	if (!txa_valid_id(id)) { \
-		RTE_EDEV_LOG_ERR("Invalid eth Rx adapter id = %d", id); \
+		RTE_EDEV_LOG_ERR("Invalid eth Tx adapter id = %d", id); \
 		return retval; \
 	} \
 } while (0)
@@ -88,6 +101,8 @@ struct txa_retry {
 struct txa_service_queue_info {
 	/* Queue has been added */
 	uint8_t added;
+	/* Queue is stopped */
+	bool stopped;
 	/* Retry callback argument */
 	struct txa_retry txa_retry;
 	/* Tx buffer */
@@ -95,7 +110,7 @@ struct txa_service_queue_info {
 };
 
 /* PMD private structure */
-struct txa_service_data {
+struct __rte_cache_aligned txa_service_data {
 	/* Max mbufs processed in any service function invocation */
 	uint32_t max_nb_tx;
 	/* Number of Tx queues in adapter */
@@ -110,6 +125,8 @@ struct txa_service_data {
 	uint16_t dev_count;
 	/* Loop count to flush Tx buffers */
 	int loop_cnt;
+	/* Loop count threshold to flush Tx buffers */
+	uint16_t flush_threshold;
 	/* Per ethernet device structure */
 	struct txa_service_ethdev *txa_ethdev;
 	/* Statistics */
@@ -128,7 +145,7 @@ struct txa_service_data {
 	int64_t service_id;
 	/* Memory allocation name */
 	char mem_name[TXA_MEM_NAME_LEN];
-} __rte_cache_aligned;
+};
 
 /* Per eth device structure */
 struct txa_service_ethdev {
@@ -193,13 +210,35 @@ txa_memzone_array_get(const char *name, unsigned int elt_size, int nb_elems)
 	return  mz->addr;
 }
 
+static int
+txa_lookup(void)
+{
+	const struct rte_memzone *mz;
+
+	if (txa_dev_id_array == NULL) {
+		mz = rte_memzone_lookup(TXA_ADAPTER_ARRAY);
+		if (mz == NULL)
+			return -ENOMEM;
+		txa_dev_id_array = mz->addr;
+	}
+
+	if (txa_service_data_array == NULL) {
+		mz = rte_memzone_lookup(TXA_SERVICE_DATA_ARRAY);
+		if (mz == NULL)
+			return -ENOMEM;
+		txa_service_data_array = mz->addr;
+	}
+
+	return 0;
+}
+
 static int
 txa_dev_id_array_init(void)
 {
 	if (txa_dev_id_array == NULL) {
 		int i;
 
-		txa_dev_id_array = txa_memzone_array_get("txa_adapter_array",
+		txa_dev_id_array = txa_memzone_array_get(TXA_ADAPTER_ARRAY,
 					sizeof(int),
 					RTE_EVENT_ETH_TX_ADAPTER_MAX_INSTANCE);
 		if (txa_dev_id_array == NULL)
@@ -222,12 +261,18 @@ static int
 txa_service_data_init(void)
 {
 	if (txa_service_data_array == NULL) {
+		int i;
+
 		txa_service_data_array =
-				txa_memzone_array_get("txa_service_data_array",
-					sizeof(int),
+				txa_memzone_array_get(TXA_SERVICE_DATA_ARRAY,
+					sizeof(*txa_service_data_array),
 					RTE_EVENT_ETH_TX_ADAPTER_MAX_INSTANCE);
 		if (txa_service_data_array == NULL)
 			return -ENOMEM;
+
+		/* Reset the txa service pointers */
+		for (i = 0; i < RTE_EVENT_ETH_TX_ADAPTER_MAX_INSTANCE; i++)
+			txa_service_data_array[i] = NULL;
 	}
 
 	return 0;
@@ -274,6 +319,8 @@ txa_service_conf_cb(uint8_t __rte_unused id, uint8_t dev_id,
 
 	port_id = dev_conf.nb_event_ports;
 	dev_conf.nb_event_ports += 1;
+	if (pc->event_port_cfg & RTE_EVENT_PORT_CFG_SINGLE_LINK)
+		dev_conf.nb_single_link_event_port_queues += 1;
 
 	ret = rte_event_dev_configure(dev_id, &dev_conf);
 	if (ret) {
@@ -288,7 +335,7 @@ txa_service_conf_cb(uint8_t __rte_unused id, uint8_t dev_id,
 
 	ret = rte_event_port_setup(dev_id, port_id, pc);
 	if (ret) {
-		RTE_EDEV_LOG_ERR("failed to setup event port %u\n",
+		RTE_EDEV_LOG_ERR("failed to setup event port %u",
 					port_id);
 		if (started) {
 			if (rte_event_dev_start(dev_id))
@@ -468,14 +515,13 @@ txa_service_ctrl(uint8_t id, int start)
 	struct txa_service_data *txa;
 
 	txa = txa_service_id_to_data(id);
-	if (txa->service_id == TXA_INVALID_SERVICE_ID)
+	if (txa == NULL || txa->service_id == TXA_INVALID_SERVICE_ID)
 		return 0;
 
+	rte_spinlock_lock(&txa->tx_lock);
 	ret = rte_service_runstate_set(txa->service_id, start);
-	if (ret == 0 && !start) {
-		while (rte_service_may_be_active(txa->service_id))
-			rte_pause();
-	}
+	rte_spinlock_unlock(&txa->tx_lock);
+
 	return ret;
 }
 
@@ -523,21 +569,24 @@ txa_process_event_vector(struct txa_service_data *txa,
 		port = vec->port;
 		queue = vec->queue;
 		tqi = txa_service_queue(txa, port, queue);
-		if (unlikely(tqi == NULL || !tqi->added)) {
-			rte_pktmbuf_free_bulk(mbufs, vec->nb_elem);
+		if (unlikely(tqi == NULL || !tqi->added || tqi->stopped)) {
+			rte_pktmbuf_free_bulk(&mbufs[vec->elem_offset],
+					      vec->nb_elem);
 			rte_mempool_put(rte_mempool_from_obj(vec), vec);
 			return 0;
 		}
 		for (i = 0; i < vec->nb_elem; i++) {
 			nb_tx += rte_eth_tx_buffer(port, queue, tqi->tx_buf,
-						   mbufs[i]);
+						   mbufs[i + vec->elem_offset]);
 		}
 	} else {
-		for (i = 0; i < vec->nb_elem; i++) {
+		for (i = vec->elem_offset; i < vec->elem_offset + vec->nb_elem;
+		     i++) {
 			port = mbufs[i]->port;
 			queue = rte_event_eth_tx_adapter_txq_get(mbufs[i]);
 			tqi = txa_service_queue(txa, port, queue);
-			if (unlikely(tqi == NULL || !tqi->added)) {
+			if (unlikely(tqi == NULL || !tqi->added ||
+				     tqi->stopped)) {
 				rte_pktmbuf_free(mbufs[i]);
 				continue;
 			}
@@ -574,7 +623,8 @@ txa_service_tx(struct txa_service_data *txa, struct rte_event *ev,
 			queue = rte_event_eth_tx_adapter_txq_get(m);
 
 			tqi = txa_service_queue(txa, port, queue);
-			if (unlikely(tqi == NULL || !tqi->added)) {
+			if (unlikely(tqi == NULL || !tqi->added ||
+				     tqi->stopped)) {
 				rte_pktmbuf_free(m);
 				continue;
 			}
@@ -594,6 +644,7 @@ txa_service_func(void *args)
 	struct txa_service_data *txa = args;
 	uint8_t dev_id;
 	uint8_t port;
+	int ret = -EAGAIN;
 	uint16_t n;
 	uint32_t nb_tx, max_nb_tx;
 	struct rte_event ev[TXA_BATCH_SIZE];
@@ -603,10 +654,10 @@ txa_service_func(void *args)
 	port = txa->port_id;
 
 	if (txa->nb_queues == 0)
-		return 0;
+		return ret;
 
 	if (!rte_spinlock_trylock(&txa->tx_lock))
-		return 0;
+		return ret;
 
 	for (nb_tx = 0; nb_tx < max_nb_tx; nb_tx += n) {
 
@@ -614,22 +665,24 @@ txa_service_func(void *args)
 		if (!n)
 			break;
 		txa_service_tx(txa, ev, n);
+		ret = 0;
 	}
 
-	if ((txa->loop_cnt++ & (TXA_FLUSH_THRESHOLD - 1)) == 0) {
+	if (txa->loop_cnt++ == txa->flush_threshold) {
 
 		struct txa_service_ethdev *tdi;
 		struct txa_service_queue_info *tqi;
 		struct rte_eth_dev *dev;
 		uint16_t i;
 
+		txa->loop_cnt = 0;
 		tdi = txa->txa_ethdev;
 		nb_tx = 0;
 
 		RTE_ETH_FOREACH_DEV(i) {
 			uint16_t q;
 
-			if (i == txa->dev_count)
+			if (i >= txa->dev_count)
 				break;
 
 			dev = tdi[i].dev;
@@ -638,7 +691,8 @@ txa_service_func(void *args)
 			for (q = 0; q < dev->data->nb_tx_queues; q++) {
 
 				tqi = txa_service_queue(txa, i, q);
-				if (unlikely(tqi == NULL || !tqi->added))
+				if (unlikely(tqi == NULL || !tqi->added ||
+					     tqi->stopped))
 					continue;
 
 				nb_tx += rte_eth_tx_buffer_flush(i, q,
@@ -646,10 +700,13 @@ txa_service_func(void *args)
 			}
 		}
 
-		txa->stats.tx_packets += nb_tx;
+		if (likely(nb_tx > 0)) {
+			txa->stats.tx_packets += nb_tx;
+			ret = 0;
+		}
 	}
 	rte_spinlock_unlock(&txa->tx_lock);
-	return 0;
+	return ret;
 }
 
 static int
@@ -716,6 +773,7 @@ txa_service_adapter_create_ext(uint8_t id, struct rte_eventdev *dev,
 	txa->service_id = TXA_INVALID_SERVICE_ID;
 	rte_spinlock_init(&txa->tx_lock);
 	txa_service_data_array[id] = txa;
+	txa->flush_threshold = TXA_FLUSH_THRESHOLD;
 
 	return 0;
 }
@@ -806,10 +864,8 @@ txa_service_queue_add(uint8_t id,
 
 	rte_spinlock_lock(&txa->tx_lock);
 
-	if (txa_service_is_queue_added(txa, eth_dev, tx_queue_id)) {
-		rte_spinlock_unlock(&txa->tx_lock);
-		return 0;
-	}
+	if (txa_service_is_queue_added(txa, eth_dev, tx_queue_id))
+		goto ret_unlock;
 
 	ret = txa_service_queue_array_alloc(txa, eth_dev->data->port_id);
 	if (ret)
@@ -821,6 +877,8 @@ txa_service_queue_add(uint8_t id,
 
 	tdi = &txa->txa_ethdev[eth_dev->data->port_id];
 	tqi = txa_service_queue(txa, eth_dev->data->port_id, tx_queue_id);
+	if (tqi == NULL)
+		goto err_unlock;
 
 	txa_retry = &tqi->txa_retry;
 	txa_retry->id = txa->id;
@@ -833,9 +891,14 @@ txa_service_queue_add(uint8_t id,
 
 	tqi->tx_buf = tb;
 	tqi->added = 1;
+	tqi->stopped = false;
 	tdi->nb_queues++;
 	txa->nb_queues++;
 
+ret_unlock:
+	rte_spinlock_unlock(&txa->tx_lock);
+	return 0;
+
 err_unlock:
 	if (txa->nb_queues == 0) {
 		txa_service_queue_array_free(txa,
@@ -844,7 +907,21 @@ err_unlock:
 	}
 
 	rte_spinlock_unlock(&txa->tx_lock);
-	return 0;
+	return -1;
+}
+
+static inline void
+txa_txq_buffer_drain(struct txa_service_queue_info *tqi)
+{
+	struct rte_eth_dev_tx_buffer *b;
+	uint16_t i;
+
+	b = tqi->tx_buf;
+
+	for (i = 0; i < b->length; i++)
+		rte_pktmbuf_free(b->pkts[i]);
+
+	b->length = 0;
 }
 
 static int
@@ -864,6 +941,8 @@ txa_service_queue_del(uint8_t id,
 		uint16_t i, q, nb_queues;
 		int ret = 0;
 
+		if (txa->txa_ethdev == NULL)
+			return 0;
 		nb_queues = txa->txa_ethdev[port_id].nb_queues;
 		if (nb_queues == 0)
 			return 0;
@@ -876,10 +955,10 @@ txa_service_queue_del(uint8_t id,
 
 			if (tqi[q].added) {
 				ret = txa_service_queue_del(id, dev, q);
+				i++;
 				if (ret != 0)
 					break;
 			}
-			i++;
 			q++;
 		}
 		return ret;
@@ -887,10 +966,13 @@ txa_service_queue_del(uint8_t id,
 
 	txa = txa_service_id_to_data(id);
 
+	rte_spinlock_lock(&txa->tx_lock);
 	tqi = txa_service_queue(txa, port_id, tx_queue_id);
 	if (tqi == NULL || !tqi->added)
-		return 0;
+		goto ret_unlock;
 
+	/* Drain the buffered mbufs */
+	txa_txq_buffer_drain(tqi);
 	tb = tqi->tx_buf;
 	tqi->added = 0;
 	tqi->tx_buf = NULL;
@@ -899,6 +981,9 @@ txa_service_queue_del(uint8_t id,
 	txa->txa_ethdev[port_id].nb_queues--;
 
 	txa_service_queue_array_free(txa, port_id);
+
+ret_unlock:
+	rte_spinlock_unlock(&txa->tx_lock);
 	return 0;
 }
 
@@ -915,6 +1000,8 @@ txa_service_id_get(uint8_t id, uint32_t *service_id)
 		return -EINVAL;
 
 	*service_id = txa->service_id;
+
+	rte_eventdev_trace_eth_tx_adapter_service_id_get(id, *service_id);
 	return 0;
 }
 
@@ -952,6 +1039,7 @@ txa_service_stop(uint8_t id)
 }
 
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_create)
 int
 rte_event_eth_tx_adapter_create(uint8_t id, uint8_t dev_id,
 				struct rte_event_port_conf *port_conf)
@@ -996,6 +1084,7 @@ rte_event_eth_tx_adapter_create(uint8_t id, uint8_t dev_id,
 	return 0;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_create_ext)
 int
 rte_event_eth_tx_adapter_create_ext(uint8_t id, uint8_t dev_id,
 				rte_event_eth_tx_adapter_conf_cb conf_cb,
@@ -1040,14 +1129,18 @@ rte_event_eth_tx_adapter_create_ext(uint8_t id, uint8_t dev_id,
 }
 
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_event_port_get)
 int
 rte_event_eth_tx_adapter_event_port_get(uint8_t id, uint8_t *event_port_id)
 {
+	rte_eventdev_trace_eth_tx_adapter_event_port_get(id);
+
 	TXA_CHECK_OR_ERR_RET(id);
 
 	return txa_service_event_port_get(id, event_port_id);
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_free)
 int
 rte_event_eth_tx_adapter_free(uint8_t id)
 {
@@ -1067,6 +1160,7 @@ rte_event_eth_tx_adapter_free(uint8_t id)
 	return ret;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_queue_add)
 int
 rte_event_eth_tx_adapter_queue_add(uint8_t id,
 				uint16_t eth_dev_id,
@@ -1100,6 +1194,7 @@ rte_event_eth_tx_adapter_queue_add(uint8_t id,
 	return ret;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_queue_del)
 int
 rte_event_eth_tx_adapter_queue_del(uint8_t id,
 				uint16_t eth_dev_id,
@@ -1132,6 +1227,7 @@ rte_event_eth_tx_adapter_queue_del(uint8_t id,
 	return ret;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_service_id_get)
 int
 rte_event_eth_tx_adapter_service_id_get(uint8_t id, uint32_t *service_id)
 {
@@ -1140,6 +1236,7 @@ rte_event_eth_tx_adapter_service_id_get(uint8_t id, uint32_t *service_id)
 	return txa_service_id_get(id, service_id);
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_start)
 int
 rte_event_eth_tx_adapter_start(uint8_t id)
 {
@@ -1154,6 +1251,7 @@ rte_event_eth_tx_adapter_start(uint8_t id)
 	return ret;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_stats_get)
 int
 rte_event_eth_tx_adapter_stats_get(uint8_t id,
 				struct rte_event_eth_tx_adapter_stats *stats)
@@ -1184,9 +1282,13 @@ rte_event_eth_tx_adapter_stats_get(uint8_t id,
 			ret = txa_service_stats_get(id, stats);
 	}
 
+	rte_eventdev_trace_eth_tx_adapter_stats_get(id, stats->tx_retry, stats->tx_packets,
+						    stats->tx_dropped, ret);
+
 	return ret;
 }
 
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_stats_reset)
 int
 rte_event_eth_tx_adapter_stats_reset(uint8_t id)
 {
@@ -1198,9 +1300,104 @@ rte_event_eth_tx_adapter_stats_reset(uint8_t id)
 		txa_dev_stats_reset(id)(id, txa_evdev(id)) : 0;
 	if (ret == 0)
 		ret = txa_service_stats_reset(id);
+
+	rte_eventdev_trace_eth_tx_adapter_stats_reset(id, ret);
+
 	return ret;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_event_eth_tx_adapter_runtime_params_init, 23.03)
+int
+rte_event_eth_tx_adapter_runtime_params_init(
+		struct rte_event_eth_tx_adapter_runtime_params *txa_params)
+{
+	if (txa_params == NULL)
+		return -EINVAL;
+
+	memset(txa_params, 0, sizeof(*txa_params));
+	txa_params->max_nb_tx = TXA_MAX_NB_TX;
+	txa_params->flush_threshold = TXA_FLUSH_THRESHOLD;
+
+	return 0;
+}
+
+static int
+txa_caps_check(struct txa_service_data *txa)
+{
+	if (!txa->dev_count)
+		return -EINVAL;
+
+	if (txa->service_id != TXA_INVALID_SERVICE_ID)
+		return 0;
+
+	return -ENOTSUP;
+}
+
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_event_eth_tx_adapter_runtime_params_set, 23.03)
+int
+rte_event_eth_tx_adapter_runtime_params_set(uint8_t id,
+		struct rte_event_eth_tx_adapter_runtime_params *txa_params)
+{
+	struct txa_service_data *txa;
+	int ret;
+
+	if (txa_lookup())
+		return -ENOMEM;
+
+	TXA_CHECK_OR_ERR_RET(id);
+
+	if (txa_params == NULL)
+		return -EINVAL;
+
+	txa = txa_service_id_to_data(id);
+	if (txa == NULL)
+		return -EINVAL;
+
+	ret = txa_caps_check(txa);
+	if (ret)
+		return ret;
+
+	rte_spinlock_lock(&txa->tx_lock);
+	txa->flush_threshold = txa_params->flush_threshold;
+	txa->max_nb_tx = txa_params->max_nb_tx;
+	rte_spinlock_unlock(&txa->tx_lock);
+
+	return 0;
+}
+
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_event_eth_tx_adapter_runtime_params_get, 23.03)
+int
+rte_event_eth_tx_adapter_runtime_params_get(uint8_t id,
+		struct rte_event_eth_tx_adapter_runtime_params *txa_params)
+{
+	struct txa_service_data *txa;
+	int ret;
+
+	if (txa_lookup())
+		return -ENOMEM;
+
+	TXA_CHECK_OR_ERR_RET(id);
+
+	if (txa_params == NULL)
+		return -EINVAL;
+
+	txa = txa_service_id_to_data(id);
+	if (txa == NULL)
+		return -EINVAL;
+
+	ret = txa_caps_check(txa);
+	if (ret)
+		return ret;
+
+	rte_spinlock_lock(&txa->tx_lock);
+	txa_params->flush_threshold = txa->flush_threshold;
+	txa_params->max_nb_tx = txa->max_nb_tx;
+	rte_spinlock_unlock(&txa->tx_lock);
+
+	return 0;
+}
+
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_stop)
 int
 rte_event_eth_tx_adapter_stop(uint8_t id)
 {
@@ -1214,3 +1411,155 @@ rte_event_eth_tx_adapter_stop(uint8_t id)
 	rte_eventdev_trace_eth_tx_adapter_stop(id, ret);
 	return ret;
 }
+
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_instance_get)
+int
+rte_event_eth_tx_adapter_instance_get(uint16_t eth_dev_id,
+				      uint16_t tx_queue_id,
+				      uint8_t *txa_inst_id)
+{
+	uint8_t id;
+	int ret = -EINVAL;
+	uint32_t caps;
+	struct txa_service_data *txa;
+
+	if (txa_lookup())
+		return -ENOMEM;
+
+	if (eth_dev_id >= rte_eth_dev_count_avail()) {
+		RTE_EDEV_LOG_ERR("Invalid ethernet port id %u", eth_dev_id);
+		return -EINVAL;
+	}
+
+	if (tx_queue_id >= rte_eth_devices[eth_dev_id].data->nb_tx_queues) {
+		RTE_EDEV_LOG_ERR("Invalid tx queue id %u", tx_queue_id);
+		return -EINVAL;
+	}
+
+	if (txa_inst_id == NULL) {
+		RTE_EDEV_LOG_ERR("txa_instance_id cannot be NULL");
+		return -EINVAL;
+	}
+
+	/* Iterate through all Tx adapter instances */
+	for (id = 0; id < RTE_EVENT_ETH_TX_ADAPTER_MAX_INSTANCE; id++) {
+		txa = txa_service_id_to_data(id);
+		if (!txa)
+			continue;
+
+		caps = 0;
+		if (rte_event_eth_tx_adapter_caps_get(txa->eventdev_id,
+						      eth_dev_id,
+						      &caps))
+			continue;
+
+		if (caps & RTE_EVENT_ETH_TX_ADAPTER_CAP_INTERNAL_PORT) {
+			ret = txa_dev_instance_get(id) ?
+					txa_dev_instance_get(id)(eth_dev_id,
+								 tx_queue_id,
+								 txa_inst_id)
+							: -EINVAL;
+			if (ret == 0) {
+				rte_eventdev_trace_eth_tx_adapter_instance_get(eth_dev_id,
+								tx_queue_id, *txa_inst_id);
+				return ret;
+			}
+		} else {
+			struct rte_eth_dev *eth_dev;
+
+			eth_dev = &rte_eth_devices[eth_dev_id];
+
+			if (txa_service_is_queue_added(txa, eth_dev,
+						       tx_queue_id)) {
+				*txa_inst_id = txa->id;
+				rte_eventdev_trace_eth_tx_adapter_instance_get(eth_dev_id,
+								tx_queue_id, *txa_inst_id);
+				return 0;
+			}
+		}
+	}
+
+	return -EINVAL;
+}
+
+static inline int
+txa_sw_queue_start_state_set(uint16_t eth_dev_id, uint16_t tx_queue_id,
+			     bool start_state, struct txa_service_data *txa)
+{
+	struct txa_service_queue_info *tqi = NULL;
+
+	rte_spinlock_lock(&txa->tx_lock);
+	tqi = txa_service_queue(txa, eth_dev_id, tx_queue_id);
+	if (unlikely(tqi == NULL || !tqi->added)) {
+		rte_spinlock_unlock(&txa->tx_lock);
+		return -EINVAL;
+	}
+	if (start_state == false)
+		txa_txq_buffer_drain(tqi);
+
+	tqi->stopped = !start_state;
+	rte_spinlock_unlock(&txa->tx_lock);
+	return 0;
+}
+
+static int
+txa_queue_start_state_set(uint16_t eth_dev_id, uint16_t tx_queue_id,
+			  bool start_state)
+{
+	struct txa_service_data *txa;
+	uint8_t txa_inst_id;
+	int ret;
+	uint32_t caps = 0;
+
+	/* Below API already does validation of input parameters.
+	 * Hence skipping the validation here.
+	 */
+	ret = rte_event_eth_tx_adapter_instance_get(eth_dev_id,
+						    tx_queue_id,
+						    &txa_inst_id);
+	if (ret < 0)
+		return -EINVAL;
+
+	txa = txa_service_id_to_data(txa_inst_id);
+	ret = rte_event_eth_tx_adapter_caps_get(txa->eventdev_id,
+						eth_dev_id,
+						&caps);
+	if (ret < 0)
+		return -EINVAL;
+
+	if (caps & RTE_EVENT_ETH_TX_ADAPTER_CAP_INTERNAL_PORT) {
+		if (start_state == true) {
+			ret = txa_dev_queue_start(txa_inst_id) ?
+			      txa_dev_queue_start(txa_inst_id)(txa_inst_id,
+							       eth_dev_id,
+							       tx_queue_id) : 0;
+		} else {
+			ret = txa_dev_queue_stop(txa_inst_id) ?
+			      txa_dev_queue_stop(txa_inst_id)(txa_inst_id,
+							      eth_dev_id,
+							      tx_queue_id) : 0;
+		}
+		return ret;
+	}
+
+	return txa_sw_queue_start_state_set(eth_dev_id, tx_queue_id,
+					    start_state, txa);
+}
+
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_queue_start)
+int
+rte_event_eth_tx_adapter_queue_start(uint16_t eth_dev_id, uint16_t tx_queue_id)
+{
+	rte_eventdev_trace_eth_tx_adapter_queue_start(eth_dev_id, tx_queue_id);
+
+	return txa_queue_start_state_set(eth_dev_id, tx_queue_id, true);
+}
+
+RTE_EXPORT_SYMBOL(rte_event_eth_tx_adapter_queue_stop)
+int
+rte_event_eth_tx_adapter_queue_stop(uint16_t eth_dev_id, uint16_t tx_queue_id)
+{
+	rte_eventdev_trace_eth_tx_adapter_queue_stop(eth_dev_id, tx_queue_id);
+
+	return txa_queue_start_state_set(eth_dev_id, tx_queue_id, false);
+}
