@@ -1,4 +1,3 @@
-
 /* SPDX-License-Identifier: BSD-3-Clause
  * Copyright(c) 2010-2014 Intel Corporation
  */
@@ -6,10 +5,6 @@
 #ifndef __INCLUDE_RTE_METER_H__
 #define __INCLUDE_RTE_METER_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /**
  * @file
  * RTE Traffic Metering
@@ -18,17 +13,17 @@ extern "C" {
  *    1. Single Rate Three Color Marker (srTCM): defined by IETF RFC 2697
  *    2. Two Rate Three Color Marker (trTCM): defined by IETF RFC 2698
  *    3. Two Rate Three Color Marker (trTCM): defined by IETF RFC 4115
- *
- ***/
+ */
 
 #include <stdint.h>
 
-#include "rte_compat.h"
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /*
  * Application Programmer's Interface (API)
- *
- ***/
+ */
 
 /**
  * Color
@@ -128,9 +123,6 @@ int
 rte_meter_trtcm_profile_config(struct rte_meter_trtcm_profile *p,
 	struct rte_meter_trtcm_params *params);
 /**
- * @warning
- * @b EXPERIMENTAL: this API may change without prior notice
- *
  * trTCM RFC 4115 profile configuration
  *
  * @param p
@@ -174,9 +166,6 @@ rte_meter_trtcm_config(struct rte_meter_trtcm *m,
 	struct rte_meter_trtcm_profile *p);
 
 /**
- * @warning
- * @b EXPERIMENTAL: this API may change without prior notice
- *
  * trTCM RFC 4115 configuration per metered traffic flow
  *
  * @param m
@@ -277,9 +266,6 @@ rte_meter_trtcm_color_aware_check(struct rte_meter_trtcm *m,
 	enum rte_color pkt_color);
 
 /**
- * @warning
- * @b EXPERIMENTAL: this API may change without prior notice
- *
  * trTCM RFC4115 color blind traffic metering
  *
  * @param m
@@ -301,9 +287,6 @@ rte_meter_trtcm_rfc4115_color_blind_check(
 	uint32_t pkt_len);
 
 /**
- * @warning
- * @b EXPERIMENTAL: this API may change without prior notice
- *
  * trTCM RFC4115 color aware traffic metering
  *
  * @param m
@@ -329,8 +312,7 @@ rte_meter_trtcm_rfc4115_color_aware_check(
 
 /*
  * Inline implementation of run-time methods
- *
- ***/
+ */
 
 struct rte_meter_srtcm_profile {
 	uint64_t cbs;
