@@ -2,10 +2,11 @@
  * Copyright(c) 2018 Intel Corporation
  */
 
+#include <eal_export.h>
 #include <rte_arp.h>
-#include <rte_byteorder.h>
 
 #define RARP_PKT_SIZE	64
+RTE_EXPORT_SYMBOL(rte_net_make_rarp_packet)
 struct rte_mbuf *
 rte_net_make_rarp_packet(struct rte_mempool *mpool,
 		const struct rte_ether_addr *mac)
