@@ -13,10 +13,6 @@
 
 #include <rte_byteorder.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #define RTE_TLS_TYPE_INVALID              0 /**< Invalid TLS message type. */
 #define RTE_TLS_TYPE_CHANGE_CIPHER_SPEC  20 /**< Change cipher spec message. */
 #define RTE_TLS_TYPE_ALERT               21 /**< Alert message. */
@@ -32,17 +28,13 @@ extern "C" {
  * TLS Header
  */
 __extension__
-struct rte_tls_hdr {
+struct __rte_packed_begin rte_tls_hdr {
 	/** Content type of TLS packet. Defined as RTE_TLS_TYPE_*. */
 	uint8_t type;
 	/** TLS Version defined as RTE_TLS_VERSION*. */
 	rte_be16_t version;
 	/** The length (in bytes) of the following TLS packet. */
 	rte_be16_t length;
-} __rte_packed;
-
-#ifdef __cplusplus
-}
-#endif
+} __rte_packed_end;
 
 #endif /* RTE_TLS_H */
