@@ -1,14 +1,18 @@
 /* SPDX-License-Identifier: BSD-3-Clause
  * Copyright(c) 2020 Intel Corporation
  */
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <sys/queue.h>
 #include <unistd.h>
 
+#include <eal_export.h>
 #include <rte_common.h>
 #include <rte_byteorder.h>
+#include <rte_tailq.h>
+#include <rte_eal_memconfig.h>
 
 #include <rte_swx_table_selector.h>
 
@@ -270,6 +274,7 @@ table_params_get(struct rte_swx_ctl_pipeline *ctl, uint32_t table_id)
 	table->params.key_offset = key_offset;
 	table->params.key_mask0 = key_mask;
 	table->params.action_data_size = action_data_size;
+	table->params.hash_func = table->info.hash_func;
 	table->params.n_keys_max = table->info.size;
 
 	table->mf_first = first;
@@ -372,18 +377,34 @@ table_entry_check(struct rte_swx_ctl_pipeline *ctl,
 
 	if (data_check) {
 		struct action *a;
+		struct rte_swx_ctl_table_action_info *tai;
 		uint32_t i;
 
 		/* action_id. */
-		for (i = 0; i < table->info.n_actions; i++)
-			if (entry->action_id == table->actions[i].action_id)
+		for (i = 0; i < table->info.n_actions; i++) {
+			tai = &table->actions[i];
+
+			if (entry->action_id == tai->action_id)
 				break;
+		}
 
 		CHECK(i < table->info.n_actions, EINVAL);
 
 		/* action_data. */
 		a = &ctl->actions[entry->action_id];
 		CHECK(!(a->data_size && !entry->action_data), EINVAL);
+
+		/* When both key_check and data_check are true, we are interested in both the entry
+		 * key and data, which means the operation is _regular_ table entry add.
+		 */
+		if (key_check && !tai->action_is_for_table_entries)
+			return -EINVAL;
+
+		/* When key_check is false while data_check is true, we are only interested in the
+		 * entry data, which means the operation is _default_ table entry add.
+		 */
+		if (!key_check && !tai->action_is_for_default_entry)
+			return -EINVAL;
 	}
 
 	return 0;
@@ -812,7 +833,7 @@ selector_group_duplicate_to_pending(struct selector *s, uint32_t group_id)
 		if (!mp)
 			goto error;
 
-		memcpy(mp, m, sizeof(struct rte_swx_table_selector_member));
+		*mp = *m;
 
 		TAILQ_INSERT_TAIL(&gp->members, mp, node);
 	}
@@ -1005,15 +1026,16 @@ learner_action_data_size_get(struct rte_swx_ctl_pipeline *ctl, struct learner *l
 static void
 table_state_free(struct rte_swx_ctl_pipeline *ctl)
 {
-	uint32_t i;
+	uint32_t table_base_index, selector_base_index, learner_base_index, i;
 
 	if (!ctl->ts_next)
 		return;
 
 	/* For each table, free its table state. */
+	table_base_index = 0;
 	for (i = 0; i < ctl->info.n_tables; i++) {
 		struct table *table = &ctl->tables[i];
-		struct rte_swx_table_state *ts = &ctl->ts_next[i];
+		struct rte_swx_table_state *ts = &ctl->ts_next[table_base_index + i];
 
 		/* Default action data. */
 		free(ts->default_action_data);
@@ -1024,17 +1046,18 @@ table_state_free(struct rte_swx_ctl_pipeline *ctl)
 	}
 
 	/* For each selector table, free its table state. */
+	selector_base_index = ctl->info.n_tables;
 	for (i = 0; i < ctl->info.n_selectors; i++) {
-		struct rte_swx_table_state *ts = &ctl->ts_next[i];
+		struct rte_swx_table_state *ts = &ctl->ts_next[selector_base_index + i];
 
 		/* Table object. */
-		if (ts->obj)
-			rte_swx_table_selector_free(ts->obj);
+		rte_swx_table_selector_free(ts->obj);
 	}
 
 	/* For each learner table, free its table state. */
+	learner_base_index = ctl->info.n_tables + ctl->info.n_selectors;
 	for (i = 0; i < ctl->info.n_learners; i++) {
-		struct rte_swx_table_state *ts = &ctl->ts_next[i];
+		struct rte_swx_table_state *ts = &ctl->ts_next[learner_base_index + i];
 
 		/* Default action data. */
 		free(ts->default_action_data);
@@ -1047,10 +1070,10 @@ table_state_free(struct rte_swx_ctl_pipeline *ctl)
 static int
 table_state_create(struct rte_swx_ctl_pipeline *ctl)
 {
+	uint32_t table_base_index, selector_base_index, learner_base_index, i;
 	int status = 0;
-	uint32_t i;
 
-	ctl->ts_next = calloc(ctl->info.n_tables + ctl->info.n_selectors,
+	ctl->ts_next = calloc(ctl->info.n_tables + ctl->info.n_selectors + ctl->info.n_learners,
 			      sizeof(struct rte_swx_table_state));
 	if (!ctl->ts_next) {
 		status = -ENOMEM;
@@ -1058,10 +1081,11 @@ table_state_create(struct rte_swx_ctl_pipeline *ctl)
 	}
 
 	/* Tables. */
+	table_base_index = 0;
 	for (i = 0; i < ctl->info.n_tables; i++) {
 		struct table *table = &ctl->tables[i];
-		struct rte_swx_table_state *ts = &ctl->ts[i];
-		struct rte_swx_table_state *ts_next = &ctl->ts_next[i];
+		struct rte_swx_table_state *ts = &ctl->ts[table_base_index + i];
+		struct rte_swx_table_state *ts_next = &ctl->ts_next[table_base_index + i];
 
 		/* Table object. */
 		if (!table->is_stub && table->ops.add) {
@@ -1094,9 +1118,10 @@ table_state_create(struct rte_swx_ctl_pipeline *ctl)
 	}
 
 	/* Selector tables. */
+	selector_base_index = ctl->info.n_tables;
 	for (i = 0; i < ctl->info.n_selectors; i++) {
 		struct selector *s = &ctl->selectors[i];
-		struct rte_swx_table_state *ts_next = &ctl->ts_next[ctl->info.n_tables + i];
+		struct rte_swx_table_state *ts_next = &ctl->ts_next[selector_base_index + i];
 
 		/* Table object. */
 		ts_next->obj = rte_swx_table_selector_create(&s->params, NULL, ctl->numa_node);
@@ -1107,10 +1132,11 @@ table_state_create(struct rte_swx_ctl_pipeline *ctl)
 	}
 
 	/* Learner tables. */
+	learner_base_index = ctl->info.n_tables + ctl->info.n_selectors;
 	for (i = 0; i < ctl->info.n_learners; i++) {
 		struct learner *l = &ctl->learners[i];
-		struct rte_swx_table_state *ts = &ctl->ts[i];
-		struct rte_swx_table_state *ts_next = &ctl->ts_next[i];
+		struct rte_swx_table_state *ts = &ctl->ts[learner_base_index + i];
+		struct rte_swx_table_state *ts_next = &ctl->ts_next[learner_base_index + i];
 
 		/* Table object: duplicate from the current table state. */
 		ts_next->obj = ts->obj;
@@ -1136,12 +1162,105 @@ error:
 	return status;
 }
 
+/* Global list of pipeline instances. */
+TAILQ_HEAD(rte_swx_ctl_pipeline_list, rte_tailq_entry);
+
+static struct rte_tailq_elem rte_swx_ctl_pipeline_tailq = {
+	.name = "RTE_SWX_CTL_PIPELINE",
+};
+
+EAL_REGISTER_TAILQ(rte_swx_ctl_pipeline_tailq)
+
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_find, 22.11)
+struct rte_swx_ctl_pipeline *
+rte_swx_ctl_pipeline_find(const char *name)
+{
+	struct rte_swx_ctl_pipeline_list *ctl_list;
+	struct rte_tailq_entry *te = NULL;
+
+	if (!name || !name[0] || (strnlen(name, RTE_SWX_CTL_NAME_SIZE) >= RTE_SWX_CTL_NAME_SIZE))
+		return NULL;
+
+	ctl_list = RTE_TAILQ_CAST(rte_swx_ctl_pipeline_tailq.head, rte_swx_ctl_pipeline_list);
+
+	rte_mcfg_tailq_read_lock();
+
+	TAILQ_FOREACH(te, ctl_list, next) {
+		struct rte_swx_ctl_pipeline *ctl = (struct rte_swx_ctl_pipeline *)te->data;
+
+		if (!strncmp(name, ctl->info.name, sizeof(ctl->info.name))) {
+			rte_mcfg_tailq_read_unlock();
+			return ctl;
+		}
+	}
+
+	rte_mcfg_tailq_read_unlock();
+	return NULL;
+}
+
+static int
+ctl_register(struct rte_swx_ctl_pipeline *ctl)
+{
+	struct rte_swx_ctl_pipeline_list *ctl_list;
+	struct rte_tailq_entry *te = NULL;
+
+	ctl_list = RTE_TAILQ_CAST(rte_swx_ctl_pipeline_tailq.head, rte_swx_ctl_pipeline_list);
+
+	rte_mcfg_tailq_write_lock();
+
+	TAILQ_FOREACH(te, ctl_list, next) {
+		struct rte_swx_ctl_pipeline *ctl_crt = (struct rte_swx_ctl_pipeline *)te->data;
+
+		if (!strncmp(ctl->info.name, ctl_crt->info.name, sizeof(ctl->info.name))) {
+			rte_mcfg_tailq_write_unlock();
+			return -EEXIST;
+		}
+	}
+
+	te = calloc(1, sizeof(struct rte_tailq_entry));
+	if (!te) {
+		rte_mcfg_tailq_write_unlock();
+		return -ENOMEM;
+	}
+
+	te->data = (void *)ctl;
+	TAILQ_INSERT_TAIL(ctl_list, te, next);
+	rte_mcfg_tailq_write_unlock();
+	return 0;
+}
+
+static void
+ctl_unregister(struct rte_swx_ctl_pipeline *ctl)
+{
+	struct rte_swx_ctl_pipeline_list *ctl_list;
+	struct rte_tailq_entry *te = NULL;
+
+	ctl_list = RTE_TAILQ_CAST(rte_swx_ctl_pipeline_tailq.head, rte_swx_ctl_pipeline_list);
+
+	rte_mcfg_tailq_write_lock();
+
+	TAILQ_FOREACH(te, ctl_list, next) {
+		if (te->data == (void *)ctl) {
+			TAILQ_REMOVE(ctl_list, te, next);
+			rte_mcfg_tailq_write_unlock();
+			free(te);
+			return;
+		}
+	}
+
+	rte_mcfg_tailq_write_unlock();
+}
+
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_free, 20.11)
 void
 rte_swx_ctl_pipeline_free(struct rte_swx_ctl_pipeline *ctl)
 {
 	if (!ctl)
 		return;
 
+	if (ctl->info.name[0])
+		ctl_unregister(ctl);
+
 	action_free(ctl);
 
 	table_state_free(ctl);
@@ -1155,6 +1274,7 @@ rte_swx_ctl_pipeline_free(struct rte_swx_ctl_pipeline *ctl)
 	free(ctl);
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_create, 20.11)
 struct rte_swx_ctl_pipeline *
 rte_swx_ctl_pipeline_create(struct rte_swx_pipeline *p)
 {
@@ -1420,6 +1540,12 @@ rte_swx_ctl_pipeline_create(struct rte_swx_pipeline *p)
 	if (status)
 		goto error;
 
+	if (ctl->info.name[0]) {
+		status = ctl_register(ctl);
+		if (status)
+			goto error;
+	}
+
 	return ctl;
 
 error:
@@ -1427,6 +1553,7 @@ error:
 	return NULL;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_table_entry_add, 20.11)
 int
 rte_swx_ctl_pipeline_table_entry_add(struct rte_swx_ctl_pipeline *ctl,
 				     const char *table_name,
@@ -1446,8 +1573,6 @@ rte_swx_ctl_pipeline_table_entry_add(struct rte_swx_ctl_pipeline *ctl,
 	CHECK(entry, EINVAL);
 	CHECK(!table_entry_check(ctl, table_id, entry, 1, 1), EINVAL);
 
-	CHECK(table->actions[entry->action_id].action_is_for_table_entries, EINVAL);
-
 	new_entry = table_entry_duplicate(ctl, table_id, entry, 1, 1);
 	CHECK(new_entry, ENOMEM);
 
@@ -1543,6 +1668,7 @@ rte_swx_ctl_pipeline_table_entry_add(struct rte_swx_ctl_pipeline *ctl,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_table_entry_delete, 20.11)
 int
 rte_swx_ctl_pipeline_table_entry_delete(struct rte_swx_ctl_pipeline *ctl,
 					const char *table_name,
@@ -1563,7 +1689,7 @@ rte_swx_ctl_pipeline_table_entry_delete(struct rte_swx_ctl_pipeline *ctl,
 	CHECK(!table_entry_check(ctl, table_id, entry, 1, 0), EINVAL);
 
 	/* The entry is found in the table->entries list:
-	 * - Move the existing entry from the table->entries list to to the
+	 * - Move the existing entry from the table->entries list to the
 	 *   table->pending_delete list.
 	 */
 	existing_entry = table_entries_find(table, entry);
@@ -1633,6 +1759,7 @@ rte_swx_ctl_pipeline_table_entry_delete(struct rte_swx_ctl_pipeline *ctl,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_table_default_entry_add, 20.11)
 int
 rte_swx_ctl_pipeline_table_default_entry_add(struct rte_swx_ctl_pipeline *ctl,
 					     const char *table_name,
@@ -1653,8 +1780,6 @@ rte_swx_ctl_pipeline_table_default_entry_add(struct rte_swx_ctl_pipeline *ctl,
 	CHECK(entry, EINVAL);
 	CHECK(!table_entry_check(ctl, table_id, entry, 0, 1), EINVAL);
 
-	CHECK(table->actions[entry->action_id].action_is_for_default_entry, EINVAL);
-
 	new_entry = table_entry_duplicate(ctl, table_id, entry, 0, 1);
 	CHECK(new_entry, ENOMEM);
 
@@ -1972,6 +2097,7 @@ table_abort(struct rte_swx_ctl_pipeline *ctl, uint32_t table_id)
 	table_pending_default_free(table);
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_selector_group_add, 21.08)
 int
 rte_swx_ctl_pipeline_selector_group_add(struct rte_swx_ctl_pipeline *ctl,
 					const char *selector_name,
@@ -1999,6 +2125,7 @@ rte_swx_ctl_pipeline_selector_group_add(struct rte_swx_ctl_pipeline *ctl,
 	return -ENOSPC;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_selector_group_delete, 21.08)
 int
 rte_swx_ctl_pipeline_selector_group_delete(struct rte_swx_ctl_pipeline *ctl,
 					   const char *selector_name,
@@ -2050,6 +2177,7 @@ rte_swx_ctl_pipeline_selector_group_delete(struct rte_swx_ctl_pipeline *ctl,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_selector_group_member_add, 21.08)
 int
 rte_swx_ctl_pipeline_selector_group_member_add(struct rte_swx_ctl_pipeline *ctl,
 					       const char *selector_name,
@@ -2109,6 +2237,7 @@ rte_swx_ctl_pipeline_selector_group_member_add(struct rte_swx_ctl_pipeline *ctl,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_selector_group_member_delete, 21.08)
 int
 rte_swx_ctl_pipeline_selector_group_member_delete(struct rte_swx_ctl_pipeline *ctl,
 						  const char *selector_name,
@@ -2362,6 +2491,7 @@ error:
 	return NULL;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_learner_default_entry_add, 21.11)
 int
 rte_swx_ctl_pipeline_learner_default_entry_add(struct rte_swx_ctl_pipeline *ctl,
 					       const char *learner_name,
@@ -2435,6 +2565,7 @@ learner_abort(struct rte_swx_ctl_pipeline *ctl, uint32_t learner_id)
 	learner_pending_default_free(l);
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_commit, 20.11)
 int
 rte_swx_ctl_pipeline_commit(struct rte_swx_ctl_pipeline *ctl, int abort_on_fail)
 {
@@ -2521,6 +2652,7 @@ rollback:
 	return status;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_abort, 20.11)
 void
 rte_swx_ctl_pipeline_abort(struct rte_swx_ctl_pipeline *ctl)
 {
@@ -2578,6 +2710,270 @@ mask_to_prefix(uint64_t mask, uint32_t mask_length, uint32_t *prefix_length)
 	return 0;
 }
 
+static int
+large_mask_to_prefix(uint8_t *mask, uint32_t n_mask_bytes, uint32_t *prefix_length)
+{
+	uint32_t pl, i;
+
+	/* Check input arguments. */
+	if (!mask || !n_mask_bytes || !prefix_length)
+		return -EINVAL;
+
+	/* Count leading bits of one. */
+	for (i = 0; i < n_mask_bytes * 8; i++) {
+		uint32_t byte_id = i / 8;
+		uint32_t bit_id = i & 7;
+
+		uint32_t byte = mask[byte_id];
+		uint32_t bit = byte & (1 << (7 - bit_id));
+
+		if (!bit)
+			break;
+	}
+
+	/* Save the potential prefix length. */
+	pl = i;
+
+	/* Check that all remaining bits are zeros. */
+	for ( ; i < n_mask_bytes * 8; i++) {
+		uint32_t byte_id = i / 8;
+		uint32_t bit_id = i & 7;
+
+		uint32_t byte = mask[byte_id];
+		uint32_t bit = byte & (1 << (7 - bit_id));
+
+		if (bit)
+			break;
+	}
+
+	if (i < n_mask_bytes * 8)
+		return -EINVAL;
+
+	*prefix_length = pl;
+	return 0;
+}
+
+static int
+char_to_hex(char c, uint8_t *val)
+{
+	if (c >= '0' && c <= '9') {
+		*val = c - '0';
+		return 0;
+	}
+
+	if (c >= 'A' && c <= 'F') {
+		*val = c - 'A' + 10;
+		return 0;
+	}
+
+	if (c >= 'a' && c <= 'f') {
+		*val = c - 'a' + 10;
+		return 0;
+	}
+
+	return -EINVAL;
+}
+
+static int
+hex_string_parse(char *src, uint8_t *dst, uint32_t n_dst_bytes)
+{
+	uint32_t i;
+
+	/* Check input arguments. */
+	if (!src || !src[0] || !dst || !n_dst_bytes)
+		return -EINVAL;
+
+	/* Skip any leading "0x" or "0X" in the src string. */
+	if ((src[0] == '0') && (src[1] == 'x' || src[1] == 'X'))
+		src += 2;
+
+	/* Convert each group of two hex characters in the src string to one byte in dst array. */
+	for (i = 0; i < n_dst_bytes; i++) {
+		uint8_t a, b;
+		int status;
+
+		status = char_to_hex(*src, &a);
+		if (status)
+			return status;
+		src++;
+
+		status = char_to_hex(*src, &b);
+		if (status)
+			return status;
+		src++;
+
+		dst[i] = a * 16 + b;
+	}
+
+	/* Check for the end of the src string. */
+	if (*src)
+		return -EINVAL;
+
+	return 0;
+}
+
+static int
+table_entry_match_field_read(struct table *table,
+			     struct rte_swx_table_entry *entry,
+			     uint32_t mf_id,
+			     char *mf_val,
+			     char *mf_mask,
+			     int *lpm,
+			     uint32_t *lpm_prefix_length_max,
+			     uint32_t *lpm_prefix_length)
+{
+	struct rte_swx_ctl_table_match_field_info *mf = &table->mf[mf_id];
+	uint64_t val, mask = UINT64_MAX;
+	uint32_t offset = (mf->offset - table->mf_first->offset) / 8;
+
+	/*
+	 * Mask.
+	 */
+	if (mf_mask) {
+		/* Parse. */
+		mask = strtoull(mf_mask, &mf_mask, 0);
+		if (mf_mask[0])
+			return -EINVAL;
+
+		/* LPM. */
+		if (mf->match_type == RTE_SWX_TABLE_MATCH_LPM) {
+			int status;
+
+			*lpm = 1;
+
+			*lpm_prefix_length_max = mf->n_bits;
+
+			status = mask_to_prefix(mask, mf->n_bits, lpm_prefix_length);
+			if (status)
+				return status;
+		}
+
+		/* Endianness conversion. */
+		if (mf->is_header)
+			mask = field_hton(mask, mf->n_bits);
+	}
+
+	/* Copy to entry. */
+	if (entry->key_mask)
+		memcpy(&entry->key_mask[offset], (uint8_t *)&mask, mf->n_bits / 8);
+
+	/*
+	 * Value.
+	 */
+	/* Parse. */
+	val = strtoull(mf_val, &mf_val, 0);
+	if (mf_val[0])
+		return -EINVAL;
+
+	/* Endianness conversion. */
+	if (mf->is_header)
+		val = field_hton(val, mf->n_bits);
+
+	/* Copy to entry. */
+	memcpy(&entry->key[offset], (uint8_t *)&val, mf->n_bits / 8);
+
+	return 0;
+}
+
+static int
+table_entry_action_argument_read(struct action *action,
+				 struct rte_swx_table_entry *entry,
+				 uint32_t arg_id,
+				 uint32_t arg_offset,
+				 char *arg_val)
+{
+	struct rte_swx_ctl_action_arg_info *arg = &action->args[arg_id];
+	uint64_t val;
+
+	val = strtoull(arg_val, &arg_val, 0);
+	if (arg_val[0])
+		return -EINVAL;
+
+	/* Endianness conversion. */
+	if (arg->is_network_byte_order)
+		val = field_hton(val, arg->n_bits);
+
+	/* Copy to entry. */
+	memcpy(&entry->action_data[arg_offset],
+	       (uint8_t *)&val,
+	       arg->n_bits / 8);
+
+	return 0;
+}
+
+static int
+table_entry_large_match_field_read(struct table *table,
+				   struct rte_swx_table_entry *entry,
+				   uint32_t mf_id,
+				   char *mf_val,
+				   char *mf_mask,
+				   int *lpm,
+				   uint32_t *lpm_prefix_length_max,
+				   uint32_t *lpm_prefix_length)
+{
+	struct rte_swx_ctl_table_match_field_info *mf = &table->mf[mf_id];
+	uint32_t offset = (mf->offset - table->mf_first->offset) / 8;
+	int status;
+
+	/*
+	 * Mask.
+	 */
+	if (!entry->key_mask)
+		goto value;
+
+	if (!mf_mask) {
+		/* Set mask to all-ones. */
+		memset(&entry->key_mask[offset], 0xFF, mf->n_bits / 8);
+		goto value;
+	}
+
+	/* Parse. */
+	status = hex_string_parse(mf_mask, &entry->key_mask[offset], mf->n_bits / 8);
+	if (status)
+		return -EINVAL;
+
+	/* LPM. */
+	if (mf->match_type == RTE_SWX_TABLE_MATCH_LPM) {
+		*lpm = 1;
+
+		*lpm_prefix_length_max = mf->n_bits;
+
+		status = large_mask_to_prefix(&entry->key_mask[offset],
+					      mf->n_bits / 8,
+					      lpm_prefix_length);
+		if (status)
+			return status;
+	}
+
+	/*
+	 * Value.
+	 */
+value:
+	/* Parse. */
+	status = hex_string_parse(mf_val, &entry->key[offset], mf->n_bits / 8);
+	if (status)
+		return -EINVAL;
+
+	return 0;
+}
+
+static int
+table_entry_large_action_argument_read(struct action *action,
+				       struct rte_swx_table_entry *entry,
+				       uint32_t arg_id,
+				       uint32_t arg_offset,
+				       char *arg_val)
+{
+	struct rte_swx_ctl_action_arg_info *arg = &action->args[arg_id];
+	int status;
+
+	status = hex_string_parse(arg_val, &entry->action_data[arg_offset], arg->n_bits / 8);
+	if (status)
+		return -EINVAL;
+
+	return 0;
+}
+
 static int
 token_is_comment(const char *token)
 {
@@ -2591,6 +2987,7 @@ token_is_comment(const char *token)
 
 #define RTE_SWX_CTL_ENTRY_TOKENS_MAX 256
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_table_entry_read, 20.11)
 struct rte_swx_table_entry *
 rte_swx_ctl_pipeline_table_entry_read(struct rte_swx_ctl_pipeline *ctl,
 				      const char *table_name,
@@ -2662,62 +3059,35 @@ rte_swx_ctl_pipeline_table_entry_read(struct rte_swx_ctl_pipeline *ctl,
 	for (i = 0; i < table->info.n_match_fields; i++) {
 		struct rte_swx_ctl_table_match_field_info *mf = &table->mf[i];
 		char *mf_val = tokens[1 + i], *mf_mask = NULL;
-		uint64_t val, mask = UINT64_MAX;
-		uint32_t offset = (mf->offset - table->mf_first->offset) / 8;
+		int status;
 
-		/*
-		 * Mask.
-		 */
 		mf_mask = strchr(mf_val, '/');
 		if (mf_mask) {
 			*mf_mask = 0;
 			mf_mask++;
-
-			/* Parse. */
-			mask = strtoull(mf_mask, &mf_mask, 0);
-			if (mf_mask[0])
-				goto error;
-
-			/* LPM. */
-			if (mf->match_type == RTE_SWX_TABLE_MATCH_LPM) {
-				int status;
-
-				lpm = 1;
-
-				lpm_prefix_length_max = mf->n_bits;
-
-				status = mask_to_prefix(mask, mf->n_bits, &lpm_prefix_length);
-				if (status)
-					goto error;
-			}
-
-			/* Endianness conversion. */
-			if (mf->is_header)
-				mask = field_hton(mask, mf->n_bits);
 		}
 
-		/* Copy to entry. */
-		if (entry->key_mask)
-			memcpy(&entry->key_mask[offset],
-			       (uint8_t *)&mask,
-			       mf->n_bits / 8);
-
-		/*
-		 * Value.
-		 */
-		/* Parse. */
-		val = strtoull(mf_val, &mf_val, 0);
-		if (mf_val[0])
+		if (mf->n_bits <= 64)
+			status = table_entry_match_field_read(table,
+							      entry,
+							      i,
+							      mf_val,
+							      mf_mask,
+							      &lpm,
+							      &lpm_prefix_length_max,
+							      &lpm_prefix_length);
+		else
+			status = table_entry_large_match_field_read(table,
+								    entry,
+								    i,
+								    mf_val,
+								    mf_mask,
+								    &lpm,
+								    &lpm_prefix_length_max,
+								    &lpm_prefix_length);
+		if (status)
 			goto error;
 
-		/* Endianness conversion. */
-		if (mf->is_header)
-			val = field_hton(val, mf->n_bits);
-
-		/* Copy to entry. */
-		memcpy(&entry->key[offset],
-		       (uint8_t *)&val,
-		       mf->n_bits / 8);
 	}
 
 	tokens += 1 + table->info.n_match_fields;
@@ -2773,7 +3143,7 @@ action:
 	for (i = 0; i < action->info.n_args; i++) {
 		struct rte_swx_ctl_action_arg_info *arg = &action->args[i];
 		char *arg_name, *arg_val;
-		uint64_t val;
+		int status;
 
 		arg_name = tokens[2 + i * 2];
 		arg_val = tokens[2 + i * 2 + 1];
@@ -2781,19 +3151,21 @@ action:
 		if (strcmp(arg_name, arg->name))
 			goto error;
 
-		val = strtoull(arg_val, &arg_val, 0);
-		if (arg_val[0])
+		if (arg->n_bits <= 64)
+			status = table_entry_action_argument_read(action,
+								  entry,
+								  i,
+								  arg_offset,
+								  arg_val);
+		else
+			status = table_entry_large_action_argument_read(action,
+									entry,
+									i,
+									arg_offset,
+									arg_val);
+		if (status)
 			goto error;
 
-		/* Endianness conversion. */
-		if (arg->is_network_byte_order)
-			val = field_hton(val, arg->n_bits);
-
-		/* Copy to entry. */
-		memcpy(&entry->action_data[arg_offset],
-		       (uint8_t *)&val,
-		       arg->n_bits / 8);
-
 		arg_offset += arg->n_bits / 8;
 	}
 
@@ -2815,6 +3187,7 @@ error:
 	return NULL;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_learner_default_entry_read, 21.11)
 struct rte_swx_table_entry *
 rte_swx_ctl_pipeline_learner_default_entry_read(struct rte_swx_ctl_pipeline *ctl,
 						const char *learner_name,
@@ -2967,6 +3340,7 @@ table_entry_printf(FILE *f,
 	fprintf(f, "\n");
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_table_fprintf, 20.11)
 int
 rte_swx_ctl_pipeline_table_fprintf(FILE *f,
 				   struct rte_swx_ctl_pipeline *ctl,
@@ -3017,6 +3391,7 @@ rte_swx_ctl_pipeline_table_fprintf(FILE *f,
 	return 0;
 }
 
+RTE_EXPORT_EXPERIMENTAL_SYMBOL(rte_swx_ctl_pipeline_selector_fprintf, 21.08)
 int
 rte_swx_ctl_pipeline_selector_fprintf(FILE *f,
 				      struct rte_swx_ctl_pipeline *ctl,
