@@ -194,40 +194,38 @@
  * - rte_regexdev_dequeue_burst()
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
+#include <rte_compat.h>
 #include <rte_common.h>
-#include <rte_config.h>
 #include <rte_dev.h>
-#include <rte_errno.h>
 #include <rte_mbuf.h>
-#include <rte_memory.h>
 
 #define RTE_REGEXDEV_NAME_MAX_LEN RTE_DEV_NAME_MAX_LEN
 
 extern int rte_regexdev_logtype;
+#define RTE_LOGTYPE_REGEXDEV rte_regexdev_logtype
 
-#define RTE_REGEXDEV_LOG(level, ...) \
-	rte_log(RTE_LOG_ ## level, rte_regexdev_logtype, "" __VA_ARGS__)
+#define RTE_REGEXDEV_LOG_LINE(level, ...) \
+	RTE_LOG_LINE(level, REGEXDEV, "" __VA_ARGS__)
 
 /* Macros to check for valid port */
 #define RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, retval) do { \
 	if (!rte_regexdev_is_valid_dev(dev_id)) { \
-		RTE_REGEXDEV_LOG(ERR, "Invalid dev_id=%u\n", dev_id); \
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid dev_id=%u", dev_id); \
 		return retval; \
 	} \
 } while (0)
 
 #define RTE_REGEXDEV_VALID_DEV_ID_OR_RET(dev_id) do { \
 	if (!rte_regexdev_is_valid_dev(dev_id)) { \
-		RTE_REGEXDEV_LOG(ERR, "Invalid dev_id=%u\n", dev_id); \
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid dev_id=%u", dev_id); \
 		return; \
 	} \
 } while (0)
 
 /**
+ * @warning
+ * @b EXPERIMENTAL: this API may change without prior notice.
+ *
  * Check if dev_id is ready.
  *
  * @param dev_id
@@ -237,6 +235,7 @@ extern int rte_regexdev_logtype;
  *   - 0 if device state is not in ready state.
  *   - 1 if device state is ready state.
  */
+__rte_experimental
 int rte_regexdev_is_valid_dev(uint16_t dev_id);
 
 /**
@@ -298,14 +297,14 @@ rte_regexdev_get_dev_id(const char *name);
  * backtracking positions remembered by any tokens inside the group.
  * Example RegEx is `a(?>bc|b)c` if the given patterns are `abc` and `abcc` then
  * `a(bc|b)c` matches both where as `a(?>bc|b)c` matches only abcc because
- * atomic groups don't allow backtracing back to `b`.
+ * atomic groups don't allow backtracking back to `b`.
  *
  * @see struct rte_regexdev_info::regexdev_capa
  */
 
 #define RTE_REGEXDEV_SUPP_PCRE_BACKTRACKING_CTRL_F (1ULL << 3)
 /**< RegEx device support PCRE backtracking control verbs.
- * Some examples of backtracing verbs are (*COMMIT), (*ACCEPT), (*FAIL),
+ * Some examples of backtracking verbs are (*COMMIT), (*ACCEPT), (*FAIL),
  * (*SKIP), (*PRUNE).
  *
  * @see struct rte_regexdev_info::regexdev_capa
@@ -611,6 +610,8 @@ struct rte_regexdev_info {
 	/**< Maximum payload size for a pattern match request or scan.
 	 * @see RTE_REGEXDEV_CFG_CROSS_BUFFER_SCAN_F
 	 */
+	uint16_t max_segs;
+	/**< Maximum number of mbuf segments that can be chained together. */
 	uint32_t max_rules_per_group;
 	/**< Maximum rules supported per group by this device. */
 	uint16_t max_groups;
@@ -1015,7 +1016,7 @@ rte_regexdev_rule_db_update(uint8_t dev_id,
  * @b EXPERIMENTAL: this API may change without prior notice.
  *
  * Compile local rule set and burn the complied result to the
- * RegEx deive.
+ * RegEx device.
  *
  * @param dev_id
  *   RegEx device identifier.
@@ -1242,7 +1243,6 @@ rte_regexdev_dump(uint8_t dev_id, FILE *f);
  * @see struct rte_regex_ops::matches
  */
 struct rte_regexdev_match {
-	RTE_STD_C11
 	union {
 		uint64_t u64;
 		struct {
@@ -1256,7 +1256,6 @@ struct rte_regexdev_match {
 			 */
 			uint16_t start_offset;
 			/**< Starting Byte Position for matched rule. */
-			RTE_STD_C11
 			union {
 				uint16_t len;
 				/**< Length of match in bytes */
@@ -1389,7 +1388,6 @@ struct rte_regex_ops {
 	 */
 
 	/* W3 */
-	RTE_STD_C11
 	union {
 		uint64_t user_id;
 		/**< Application specific opaque value. An application may use
@@ -1402,7 +1400,6 @@ struct rte_regex_ops {
 	};
 
 	/* W4 */
-	RTE_STD_C11
 	union {
 		uint64_t cross_buf_id;
 		/**< ID used by the RegEx device in order to support cross
@@ -1427,6 +1424,10 @@ struct rte_regex_ops {
 
 #include "rte_regexdev_core.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * @warning
  * @b EXPERIMENTAL: this API may change without prior notice.
@@ -1472,13 +1473,14 @@ rte_regexdev_enqueue_burst(uint8_t dev_id, uint16_t qp_id,
 	struct rte_regexdev *dev = &rte_regex_devices[dev_id];
 #ifdef RTE_LIBRTE_REGEXDEV_DEBUG
 	RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, -EINVAL);
-	RTE_FUNC_PTR_OR_ERR_RET(*dev->enqueue, -ENOTSUP);
+	if (dev->enqueue == NULL)
+		return -ENOTSUP;
 	if (qp_id >= dev->data->dev_conf.nb_queue_pairs) {
-		RTE_REGEXDEV_LOG(ERR, "Invalid queue %d\n", qp_id);
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid queue %d", qp_id);
 		return -EINVAL;
 	}
 #endif
-	return (*dev->enqueue)(dev, qp_id, ops, nb_ops);
+	return dev->enqueue(dev, qp_id, ops, nb_ops);
 }
 
 /**
@@ -1531,13 +1533,14 @@ rte_regexdev_dequeue_burst(uint8_t dev_id, uint16_t qp_id,
 	struct rte_regexdev *dev = &rte_regex_devices[dev_id];
 #ifdef RTE_LIBRTE_REGEXDEV_DEBUG
 	RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, -EINVAL);
-	RTE_FUNC_PTR_OR_ERR_RET(*dev->dequeue, -ENOTSUP);
+	if (dev->dequeue == NULL)
+		return -ENOTSUP;
 	if (qp_id >= dev->data->dev_conf.nb_queue_pairs) {
-		RTE_REGEXDEV_LOG(ERR, "Invalid queue %d\n", qp_id);
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid queue %d", qp_id);
 		return -EINVAL;
 	}
 #endif
-	return (*dev->dequeue)(dev, qp_id, ops, nb_ops);
+	return dev->dequeue(dev, qp_id, ops, nb_ops);
 }
 
 #ifdef __cplusplus
