@@ -5,14 +5,14 @@
 #ifndef _RTE_TELEMETRY_H_
 #define _RTE_TELEMETRY_H_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <stdint.h>
 #include <rte_compat.h>
 #include <rte_common.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** Maximum length for string used in object. */
 #define RTE_TEL_MAX_STRING_LEN 128
 /** Maximum length of string. */
@@ -136,22 +136,6 @@ rte_tel_data_add_array_int(struct rte_tel_data *d, int64_t x);
 int
 rte_tel_data_add_array_uint(struct rte_tel_data *d, uint64_t x);
 
- /**
- * Add a uint64_t to an array.
- * The array must have been started by rte_tel_data_start_array() with
- * RTE_TEL_UINT_VAL as the type parameter.
- *
- * @param d
- *   The data structure passed to the callback
- * @param x
- *   The number to be returned in the array
- * @return
- *   0 on success, negative errno on error
- */
-int
-rte_tel_data_add_array_u64(struct rte_tel_data *d, uint64_t x)
-	__rte_deprecated_msg("use 'rte_tel_data_add_array_uint' instead");
-
 /**
  * Add a container to an array. A container is an existing telemetry data
  * array. The array the container is to be added to must have been started by
@@ -249,25 +233,6 @@ int
 rte_tel_data_add_dict_uint(struct rte_tel_data *d,
 		const char *name, uint64_t val);
 
- /**
- * Add a uint64_t value to a dictionary.
- * The dict must have been started by rte_tel_data_start_dict().
- *
- * @param d
- *   The data structure passed to the callback
- * @param name
- *   The name the value is to be stored under in the dict
- *   Must contain only alphanumeric characters or the symbols: '_' or '/'
- * @param val
- *   The number to be stored in the dict
- * @return
- *   0 on success, negative errno on error, E2BIG on string truncation of name.
- */
-int
-rte_tel_data_add_dict_u64(struct rte_tel_data *d,
-		const char *name, uint64_t val)
-	__rte_deprecated_msg("use 'rte_tel_data_add_dict_uint' instead");
-
 /**
  * Add a container to a dictionary. A container is an existing telemetry data
  * array. The dict the container is to be added to must have been started by
@@ -337,15 +302,28 @@ typedef int (*telemetry_cb)(const char *cmd, const char *params,
 		struct rte_tel_data *info);
 
 /**
- * Used for handling data received over a telemetry socket.
+ * This telemetry callback is used when registering a telemetry command with
+ * rte_telemetry_register_cmd_arg().
  *
- * @param sock_id
- * ID for the socket to be used by the handler.
+ * It handles getting and formatting information to be returned to telemetry
+ * when requested.
+ *
+ * @param cmd
+ *   The cmd that was requested by the client.
+ * @param params
+ *   Contains data required by the callback function.
+ * @param arg
+ *   The opaque value that was passed to rte_telemetry_register_cmd_arg().
+ * @param info
+ *   The information to be returned to the caller.
  *
  * @return
- * Void.
+ *   Length of buffer used on success.
+ * @return
+ *   Negative integer on error.
  */
-typedef void * (*handler)(void *sock_id);
+typedef int (*telemetry_arg_cb)(const char *cmd, const char *params, void *arg,
+		struct rte_tel_data *info);
 
 /**
  * Used when registering a command and callback function with telemetry.
@@ -367,16 +345,28 @@ typedef void * (*handler)(void *sock_id);
 int
 rte_telemetry_register_cmd(const char *cmd, telemetry_cb fn, const char *help);
 
-
 /**
- * Get a pointer to a container with memory allocated. The container is to be
- * used embedded within an existing telemetry dict/array.
+ * Used when registering a command and callback function with telemetry.
+ *
+ * @param cmd
+ *   The command to register with telemetry.
+ * @param fn
+ *   Callback function to be called when the command is requested.
+ * @param arg
+ *   An opaque value that will be passed to the callback function.
+ * @param help
+ *   Help text for the command.
  *
  * @return
- *  Pointer to a container.
+ *   0 on success.
+ * @return
+ *   -EINVAL for invalid parameters failure.
+ * @return
+ *   -ENOMEM for mem allocation failure.
  */
-struct rte_tel_data *
-rte_tel_data_alloc(void);
+__rte_experimental
+int
+rte_telemetry_register_cmd_arg(const char *cmd, telemetry_arg_cb fn, void *arg, const char *help);
 
 /**
  * @internal
@@ -389,6 +379,17 @@ rte_tel_data_alloc(void);
 void
 rte_tel_data_free(struct rte_tel_data *data);
 
+/**
+ * Get a pointer to a container with memory allocated. The container is to be
+ * used embedded within an existing telemetry dict/array.
+ *
+ * @return
+ *  Pointer to a container.
+ */
+struct rte_tel_data *
+rte_tel_data_alloc(void)
+	__rte_malloc __rte_dealloc(rte_tel_data_free, 1);
+
 #ifdef __cplusplus
 }
 #endif
